/*
 * @(#)$Id: AppCleaner.java,v 1.1.1.1 2002/11/06 22:10:24 kris Exp $
 *
 */

package mokan.app;

import mokan.Kan;
import mokan.comm.LogicalNode;
import mokan.debug.Event;
import mokan.thread.AsynchCall;
import mokan.thread.ThreadID;

/**
 * Wait around for the application to exit so we can clean up after it
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:24 $
 * @author	Sandeep Bhatia
 * @author	Sean Brydon
 * @seemokan.app.Application
 */
final class AppCleaner implements Runnable {

    /**
     * The {@linkmokan.thread.ThreadID ThreadID} of the main application thread
     */
    private final ThreadID callID;

    /**
     * The {@linkmokan.thread.AsynchCall AsynchCall} executed by the main
     * application thread
     */
    private final AsynchCall call;

    /**
     * The application itself
     */
    private final Application myApp;

    /**
     * Whether to exit when the application finishes
     */
    private final boolean exit;

    /**
     * Create a new <code>AppCleaner</code>
     *
     * @param callID the <code>ID</code> of the main application thread
     * @param call the <code>AsynchCall</code> executed by the main
     *        application thread
     * @param thisApp the application itself
     * @param exit <code>true</code> if the system should exit when this
     *        application finishes
     */
    AppCleaner(final ThreadID callID, final AsynchCall call,
	       final Application thisApp, final boolean exit) {
	this.callID = callID;
	this.call   = call;
	this.myApp  = thisApp;
	this.exit   = exit;
	new Thread(this).start();
    }

    /**
     * Commence waiting for the application to exit so that we can clean up
     * after it
     */
    public void run() {
	// first wait for the future to return
	try {
	    call.join(callID.getFuture());
	    Event.DEBUG(Event.APP_END, "End of application: ", myApp);
	    if (Kan.verbose)
		System.out.println("The application exited.");
	} catch (Exception ex) {
	    // Ignore it; we're done anyway
	    ex.printStackTrace();
	}

	// Since main has returned, clean up the necessary data structures in
	// the various nodes, including the local one.  First get a list of
	// nodes to send this message to.

// 	final LogicalNode[] nodes = myApp.getNodes();
// 	final LogicalNode localNode = LogicalNode.getMyNode();
// 	final AppTearDownMsg msg = new AppTearDownMsg(localNode, myApp, exit);
// 	for (int i = 0; i < nodes.length; i++) {
// 	    // If the node is not local, send it an application teardown msg
// 	    if (nodes[i] != localNode) {
// 		msg.send(nodes[i]);
// 	    }
// 	}

// 	if (exit)
// 	    System.exit(0);
    }
}
