/*
 * @(#)$Id
 *
 *
 */

package mokan.app;

import java.io.*;
import mokan.comm.*;
import mokan.debug.Event;

/**
 *    A message passed between PhysicalNode to signal that a node wishes
 *    Join an application already running on the reciever.
 *    @author kristian kvilekval
 */
final class AppJoinMsg extends SysMessage implements Externalizable{


    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
     static final long serialVersionUID = -6378043029289521411L;





    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    private KanURL appurl;
    //private LogicalNode requester;
    private PhysicalNode requester;


    public AppJoinMsg() {
	super(false, true);
        appurl = null;
        requester = null;
    }
    /**
     * Create a new <code>AppJoin</code> to tell a remote node to start an
     * application
     *
     * @param app the application to start
     */
    public AppJoinMsg(final KanURL appurl, final PhysicalNode _requester) {
	super(false, true);
        this.appurl = appurl;
        this.requester = _requester;
    }


    public void handle() {
        Event.DEBUG (Event.MSG_HANDLE, "AppJoin.handle ", this);
        try {
            Application app = Application.getApplication(appurl);
            if (app != null) {
                LogicalNode ln = new LogicalNode(app, requester);
                app.insertNode (ln);

                AppJoinRsp reply = new  AppJoinRsp(this, LogicalNode.getMyNode(), true);
                KanComm.send (requester, reply);

                ApplicationEvent e =
                    new ApplicationEvent(ln, ApplicationEvent.INSERT_NODE);
                LogicalNode.setMyNode (app.getLocalNode());
                app.notifyApplicationListeners(e) ;
            }
            else{
              System.out.println("AppJoinMsg.handle: Unkown appurl " + appurl);
              AppJoinRsp reply = new AppJoinRsp(this, LogicalNode.getMyNode(), false);
              KanComm.send(requester, reply);
            }
        } catch (java.net.MalformedURLException me) {
            me.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
        }
    }


    /**
     * Save the state of the <code>AppJoin</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the send
     *             sequence number of this reply (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
	out.writeObject(appurl);
        out.writeObject(requester);
    }

    public void readExternal(final ObjectInput in)
	throws IOException, ClassNotFoundException {
	super.readExternal(in);
	appurl = (KanURL)in.readObject();
        requester = (PhysicalNode) in.readObject();
    }


    public String toString () {
        return "AppJoinMsg[" + appurl + ", " + requester + "]";
    }
};

