/*
 * @(#)$Id: AppJoinRsp.java,v 1.1.1.1 2002/11/06 22:10:24 kris Exp $
 *
 *
 */


package mokan.app;
import java.io.*;

import mokan.comm.SysReply;
import mokan.comm.LogicalNode;
import mokan.comm.KanURL;
import mokan.comm.KanComm;
import mokan.debug.Event;
import mokan.nameservice.Globals;

/**
 *  The <code>AppJoinRsp</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:24 $
 *  @author Kristian Kvilekval
 */
final class AppJoinRsp extends SysReply implements Externalizable{

    /* Private/Public Variables */

    /**
     *  A flag signifying that the join was accepted
     *  @serial boolean flag whether join was accepted
     */
    boolean accepted;

    /**
     *  The KanURL of the application
     */
    LogicalNode  joinNode;

    /**
     *  The default constructor
     */
    public AppJoinRsp () {
	super(false, true);
        accepted = true;
        joinNode = null;
    }

    public AppJoinRsp (AppJoinMsg msg, LogicalNode _joinNode, boolean _accepted) {
	super(msg, false, true);
        accepted = _accepted;
        joinNode = _joinNode;
    }

    public void handle() {
        Event.DEBUG (Event.MSG_HANDLE, "AppJoinRsp.handle ", this);

        try {
            Globals.app.insertNode (joinNode);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Wake up waiting task.
        super.handle();

        if(accepted == false) {
          throw new RuntimeException("AppJoinRsp.handle: AppJoin request not accepted.");
        }
    }

    /**
     * Save the state of the <code>AppJoin</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the send
     *             sequence number of this reply (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
	out.writeBoolean(accepted);
        out.writeObject(joinNode);
    }

    public void readExternal(final ObjectInput in)
	throws IOException, ClassNotFoundException {
	super.readExternal(in);
	accepted = in.readBoolean();
        joinNode = (LogicalNode)in.readObject();
    }


    public String toString () {
        return "AppJoinRsp[" + accepted + "]";
    }
};
