/*
 * @(#)$Id
 *
 *
 */

package mokan.app;

import java.io.*;
import mokan.comm.SysMessage;
import mokan.comm.LogicalNode;
import mokan.comm.KanURL;
import mokan.debug.Event;

/**
 *    A message to indicate that the sender is leaving the application
 *    group.
 *    @author Kristian Kvilekval
 */
final class AppLeaveMsg extends SysMessage implements Externalizable {

  /**
   * The <var>serialVersionUID</var> for this class.  If you change the
   * signature of this class (field names or types, method names, parameter
   * names or types, adding or removing fields or methods, etc.), you
   * <strong>must</strong> change this value.  The recommended way is to use
   * the <code>serialver</code> tool.
   */
  static final long serialVersionUID = -5399892052705685313L;

  /**
   * The list of persistent fields for this class.  This is empty, because
   * that is the fastest option for <code>Externalizable</code> classes when
   * we are not worried about versioning.
   */
  private static final ObjectStreamField[] serialPersistentFields =
      ObjectStreamClass.NO_FIELDS;

  private KanURL appurl;
  private LogicalNode requester;

  /**
   * Create a new <code>AppJoin</code> to tell a remote node to start an
   * application
   *
   * @param app the application to start
   */
  public AppLeaveMsg() {
    super(false, false);
    appurl = null;
    requester = null;
  }

  public AppLeaveMsg(LogicalNode local) {
    super(false, false);

    requester = local;
    appurl = local.getApplication().getURL();
  }

  public void handle() {
    Event.DEBUG(Event.MSG_HANDLE, "AppLeaveMsg.handle ", this);
    Application app = Application.getApplication(appurl);
    if (app != null) {
      if (app.containsNode(requester)) {
        ApplicationEvent e =
            new ApplicationEvent(requester, ApplicationEvent.REMOVE_NODE);

        app.notifyApplicationListeners(e);
        // Cancel any message going to requester (locks,etc).

        app.removeNode(requester);
      }
    }
  }

  /**
   * Save the state of the <code>AppJoin</code> instance to a stream (i.e.,
   * serialize it).
   *
   * @serialData The superclass data is written out, followed by the send
   *             sequence number of this reply (long).
   */
  public void writeExternal(final ObjectOutput out) throws IOException {
    super.writeExternal(out);
    out.writeObject(appurl);
    out.writeObject(requester);
  }

  public void readExternal(final ObjectInput in) throws IOException,
      ClassNotFoundException {
    super.readExternal(in);
    appurl = (KanURL) in.readObject();
    requester = (LogicalNode) in.readObject();
  }

  public String toString() {
    return "AppLeave[" + "]";
  }
};
