/*
 * @(#)$Id
 *
 *
 */

package mokan.app;

import java.io.*;
//import java.net.InetAddress;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Iterator;
//import java.util.HashMap;

import mokan.util.Hash2Map;
import mokan.comm.KanComm;
import mokan.comm.KanURL;
import mokan.comm.LogicalNode;
//import mokan.comm.MachinePort;
import mokan.comm.PhysicalNode;
import mokan.debug.Event;

import mokan.thread.AsynchCall;
import mokan.thread.ThreadID;
import mokan.thread.ThreadScheduler;
import mokan.nameservice.Globals;
import java.util.*;

/**
 * A data container for applications, holding the class name and parameters
 * used to invoke the application, as well as the logical nodes that are
 * participating in the application
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:24 $
 * @author	Sandeep Bhatia
 * @author	Sean Brydon
 * @author	Jerry James
 */
public final class Application {

  /**
   * A mapping from (PhysicalNode,class) keys to <code>Application</code>s
   */
  private static final Hash2Map apps = new Hash2Map();
  // We chose 11, because we don't expect there to be more than a few
  // applications running at any given time.  The 0.6f wastes some space,
  // but makes it likely that each app will be in its own bucket.

  /**
   * The known logical nodes participating in this application
   */
  private transient HashSet appNodes;

  /**
   *  Notify these on application events @see ApplicationEvent
   */
  private transient LinkedList appListeners;

  /**
   * The known local logical node
   */
  private transient LogicalNode localNode;

  /**
   *   Each Application URL associated with it
   */
  private final transient KanURL appURL;

  /**
   *  The identifying birthnod of the application
   */
  private LogicalNode birthNode;
  /**
   * The string corresponding to the fully qualified name of the
   * application.  This is a pseudo-final field.
   *
   * @serial a string value holding the fully qualified name of the
   *         application's main class
   */
  private String appName;

  /**
   * An array of {@link java.lang.String String} holding the application
   * arguments.  This is a pseudo-final field.
   *
   * @serial a string array holding the application arguments
   */
  private String[] appArgs;

  /**
   * The no-args constructor, needed for deserialization
   */
  public Application() {
    appNodes = null;
    localNode = null;
    appURL = null;
    appListeners = null;
    //appID = null;
  }

  /**
   *  Create an kan application from a kan url.
   * @param kanurl The url (birthplace,class, instance) of the app.
   */
  public Application(KanURL kurl) throws java.net.MalformedURLException,
      java.net.ConnectException {
    Event.DEBUG(Event.APPLICATION, "Application.Application: ", kurl);

    apps.put(kurl, this);
    appURL = kurl;
    appName = kurl.getAppName();
    appArgs = null;
    appNodes = new HashSet();

    //Modified by Huahai
    //PhysicalNode pn = KanComm.getPhysicalNodeByAddr (kurl.getAppHost());
    //String  classid = kurl.getAppName ();
    //apps.put (pn, classid, this);

    // Add the current node to the application.. Others must join.
//            new LogicalNode (this,
//                             KanComm.getPhysicalNode (kurl.getAppHost()));
//        addNode (birthNode);
//         if (birthNode.isLocal()) {
//             localNode = birthNode;
//         } else {
//             localNode = new LogicalNode (this, KanComm.getPhysicalNode());
//             addNode (localNode);
//         }
//         Event.DEBUG(Event.APPLICATION, "Application.getApp: App Table : " , apps );

  }

  /**
   *  Retrieve the aplication by name
   *  @param appName the name
   */
  public static Application getApplication(KanURL kurl) {

    Event.DEBUG(Event.APPLICATION, "Application.getApp: ", kurl);
    /*
         PhysicalNode pn = KanComm.getPhysicalNodeByAddr (kurl.getAppHost());
             String  classid = kurl.getAppName ();
             Application app = (Application)apps.get (pn, classid);
     */
    KanURL appUrl = kurl.getAppURL();
    Application app = (Application) apps.get(appUrl);
    if (app == null) {
      Event.DEBUG(Event.APPLICATION, "Application.getApp: App not found: ",
                  kurl);
      Event.DEBUG(Event.APPLICATION, "Application.getApp: App Table : ", apps);

      //app = new Application (kurl);
    }
    return app;
  }

  /**
   *  Get the URL representing this application.
   * @return KanURL
   */
  public KanURL getURL() {
    return appURL;
  }

  /** Insert a constructed node into the application) */
  public void insertNode(LogicalNode ln) {
    appNodes.add(ln);
  }

  /**
   *  Force the removal of the LogicalNode from the application
   */
  public void removeNode(LogicalNode node) {
    Event.DEBUG(Event.APPLICATION, "Application.removeNode: ", node);

    node.destroy();

    LogicalNode.remove(node);
    appNodes.remove(node);
  }

  public boolean containsNode(LogicalNode ln) {
    return appNodes.contains(ln);
  }

  /**
   *  Get the current Application Name
   */
  public String getName() {
    return appName;
  }

  /**
   *  Retrieve the local node
   */
  public LogicalNode getLocalNode() {
    return localNode;
  }

  public HashSet getNodes() {
    return appNodes;
  }

  /**
   * Start the application represented by this <code>AppInfo</code>
   *
   * @param appEntry the <code>Application</code> to pass to
   *        <code>main</code>
   */
  public void invoke(String[] appArgs) {
    Event.DEBUG(Event.APP_START, "Starting application: ", appName);

    localNode = new LogicalNode(this, KanComm.getPhysicalNode());
    //localNode  = insertNode (new KanURL (appURL.getNodeURL());

    final ThreadScheduler myScheduler =
        localNode.getLocalHandler().scheduler;
    final Object argObjects[] = {
        this, (appArgs == null) ? new String[0] : appArgs};
    final AsynchCall rootCall =
        new AsynchCall(localNode, appName, argObjects, myScheduler);
    final ThreadID rootThrdID = myScheduler.createLocThread(rootCall);

    /*
     * Create a thread to watch the AsynchCall so we can detect whether
     * main returned and do garbage collection later
     */
    new AppCleaner(rootThrdID, rootCall, this, true);

//         } catch (java.net.MalformedURLException url) {
//             Event.DEBUG(Event.APP_START, "Application.invoke: bad url", appURL);
//         }
  }

  /**
   * Join a running application represented by this <code>master</code>
   *
   * @param master A <code>KanURL</code> for the application @see KanURL
   */
  public void join(KanURL joinUrl) {
    Event.DEBUG(Event.APP_START, "Application.join: ", appName);

    try {
      // Make sure we are already in the application
      if (localNode == null) {
        localNode = new LogicalNode(this, Globals.localPhysicalNode);
        insertNode(localNode);
      }

      //Modified by Huahai
      //new AppJoinMsg (master.getAppURL(), localNode.getURL()).sendAndWait (KanComm.getPhysicalNodeByAddr(master.getAppHost()));
      String address = joinUrl.getAppHost();
      int port = joinUrl.getAppPort();
      PhysicalNode joinNode = KanComm.connect(address, port, null);

      AppJoinMsg joinMsg = new AppJoinMsg(this.appURL, Globals.localPhysicalNode);
      AppJoinRsp reply = (AppJoinRsp) joinMsg.sendAndWait(joinNode);

    } catch (java.net.MalformedURLException ue) {
      Event.DEBUG(Event.APP_START, "Application.join: bad url" + ue);
    } catch (IOException io) {
      Event.DEBUG(Event.APP_START, "Application.join: failed");
      io.printStackTrace();
    }
  }

  /**
   * The node will leave the application.
   * Application objects will be migrated off, while replicas
   * will be deleted.  Outstanding
   * @param waitfuture wait for outstanding asynchronous calls to complete.
   *
   */
  public static void leave(boolean waitfuture) {
    Event.DEBUG(Event.APP_LEAVE, "Application.leave ->");

    LogicalNode localNode = LogicalNode.getMyNode();
    Application app = localNode.getApplication();

    localNode.stop();

    AppLeaveMsg leave = new AppLeaveMsg(localNode);
    LogicalNode node;
    Iterator to = app.getNodes().iterator();
    while (to.hasNext()) {
      node = (LogicalNode) to.next();
      if (node.getPhysicalNode() != KanComm.getPhysicalNode()) {
        Event.DEBUG(Event.APP_END, "Application.leave: send->", node);
        try {
          leave.send(node.getPhysicalNode());
        } catch (IOException e) {
          // Ignore dead nodes.
        }
      }
    }

    app.removeNode(localNode);
    Event.DEBUG(Event.APP_LEAVE, "Application.leave <-");
  }

  /**
   * Return the number of logical nodes participating in this application
   *
   * @return the number of logical nodes participating in this application
   */
  public int numNodes() {
    return appNodes.size();
  }

  public void addApplicationListener(ApplicationListener l) {
    Event.DEBUG(Event.APP_LISTENER, "Application Listener Add:", l);
    if (appListeners == null) {
      appListeners = new LinkedList();
    }
    appListeners.add(l);
  }

  public void removeApplicationListener(ApplicationListener l) {
    Event.DEBUG(Event.APP_LISTENER, "Remove");
    if (appListeners != null) {
      appListeners.remove(l);
    }
  }

  public void notifyApplicationListeners(ApplicationEvent e) {
    if (appListeners == null) {
      return;
    }

    Iterator p = appListeners.iterator();
    while (p.hasNext()) {
      ( (ApplicationListener) p.next()).applicationEvent(e);
    }
  }

  public String toString() {
    return appURL.toString();
  }
}
