/*
 * @(#)AppReply.java AppReply.java,v 1.2 2002/11/22 23:26:06 kris Exp $
 *
 *
 */

package mokan.comm;

import java.io.*;

/**
 * Reply to a previously sent application message.
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:06 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
public class AppReply extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 2007007527043044200L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The sequence number of the message to which this is a reply.  This is a
     * pseudo-final field.
     *
     * @serial a long value holding the sequence number of the original message
     */
    protected long senderSeqNum;

    /**
     * Create a new <code>AppReply</code>; the deserializing constructor
     */
    public AppReply() {
        super(false, false);
    }

    /**
     * Create a new <code>AppReply</code>; the deserializing constructor for
     * subclasses
     *
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     * The no-args constructor, needed for deserialization
     */
    protected AppReply(final boolean needsReset, final boolean canBlock) {
        super(needsReset, canBlock);
    }

    /**
     * Create a new <code>AppReply</code>, and set up the sequence number of
     * the message to which this is a reply.
     *
     * @param msg the message to which this is a reply
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     */
    public AppReply(final AppMessage msg, final boolean needsReset,
                    final boolean canBlock) {
        super(msg.from, needsReset, canBlock);
        senderSeqNum = msg.SSN;
        to = msg.to;
    }
    
    /**
     * Take the receive action for this reply.  
     * The defualt is to initialize message fields and then
     * finish handling.   This is here so messages can be completely
     * handled before waking the threads that are waiting on them.
     * Derived method should use the style
     *    <code>handle() { replyBegin(); [actions...] ; replyEnd(); }
     *    </code>
     */
    public void handle() {
        replyBegin();
        replyEnd();
    }
    public void replyBegin() {
        super.handle();
    }


    /** This *must* be called at the end of message handling.  The
     * default action is to wakeup a waiting sender.  
     */
     public void replyEnd(){ 
         final Wakeup wake = (Wakeup)wakeupTable.get(senderSeqNum);
         // If an abort occurs during the handling of the message, the
         // sequence number will be removed. 
         // (@see AppMessage.sendAndWait).  Do the sensible and forget
         // about it.  We might want to either log or confirm here
         // that the message was aborted.
         if (wake != null) {
             wake.answerReceived(this);
         }
    }


    /**
     * Convert the <code>AppReply</code> to a string
     *
     * @return a string representation of this <code>AppReply</code>
     */
    public String toString() {
        return "AppReply(" + to + ',' + senderSeqNum + ')';
    }

    /**
     * Save the state of the <code>AppReply</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             send sequence number (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeLong(senderSeqNum);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {
	
        super.readExternal(in);
        senderSeqNum = in.readLong();
    }
}
