/*
 * @(#)$Id: GroupWakeup.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 */

package mokan.comm;

/**
 * A sleeping thread, waiting on communication from multiple nodes
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author	Jerry James
 */
final class GroupWakeup implements Wakeup {

    /**
     * Whether all replies so far have said yes
     */
    boolean consensus = true;

    /**
     * The number of replies received so far
     */
    private int counter = 0;

    /**
     * The number of replies to receive
     */
    private final int num;

    /**
     * Create a new sleeping thread, waiting for communication from multiple
     * nodes
     *
     * @param num number of replies to wait for
     */
    GroupWakeup(final int num) {
	this.num = num;
    }

    /**
     * Wait for all replies to be received
     *
     * @return <code>true</code> if all replies were yes; <code>false</code>
     *         if any reply was no
     * @exception InterruptedException if a transaction abort occurs while
     *         waiting
     */
    boolean await() throws InterruptedException {
	synchronized (this) {
	    while (counter != num && consensus) {
		wait();
	    }
	}
	return consensus;
    }

    /**
     * Tell this <code>GroupWakeup</code> that an answer was received
     *
     * @param msg the answer received
     */
    public void answerReceived(final Message msg) {
	final YesNoReply answer = (YesNoReply)msg;
	synchronized (this) {
	    consensus = consensus && answer.yes;
	    if (++counter == num || !consensus) {
		notify();
	    }
	}
    }

    /**
     * Create a string representation of this <code>GroupWakeup</code>
     *
     * @return a string representation of this <code>GroupWakeup</code>
     */
    public String toString() {
	return "GroupWakeup: " + counter + " of " + num;
    }
}
