/*
 * @(#)LocalLogicalNode.java LocalLogicalNode.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 */

package mokan.comm;

import mokan.obj.ObjectMgr;
import mokan.thread.ThreadScheduler;

/**
 * Data container for local {@link mokan.comm.LogicalNode LogicalNodes}.  This
 * class contains pointers to the data structures needed by each logical node,
 * namely the thread scheduler and object manager.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author	Sandeep Bhatia
 * @author	Sean Brydon
 * @author	Jerry James
 */
public final class LocalLogicalNode {

    /**
     * The thread scheduler for this logical node
     */
    public final ThreadScheduler scheduler;

    /**
     * The object manager for this logical node
     */
    public final ObjectMgr manager;

    /**
     * Create a new <code>LocalLogicalNode</code> for <var>parentNode</var>,
     * creating an associated
     * {@linkmokan.thread.ThreadScheduler ThreadScheduler} and
     * {@linkmokan.obj.ObjectMgr ObjectMgr}
     *
     * @param parentNode the containing logical node
     */
    LocalLogicalNode(final LogicalNode parentNode) {	
        scheduler = new ThreadScheduler(parentNode);
        manager = new ObjectMgr(parentNode);
    }
    public void destroy () {
        scheduler.destroy ();
    }

    /**
     * Set the value of allowObjectCreations.
     * @param v  Value to assign to allowObjectCreations.
     */
    public void allowObjectCreations(boolean  v) {
        manager.allowCreateObjects = v;
    }
}
