/*
 * @(#)$Id: MachinePort.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 */

package mokan.comm;

import java.net.InetAddress;
import java.net.UnknownHostException;
import mokan.Kan;
import mokan.nameservice.Globals;
/**
 * A machine identifier and a port number
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author      Kristian Kvilekval (based on a version by Jerry James).
 * @author	Jerry James
 */
public final class MachinePort {

    /**
     * The default port
     */
    private static  InetAddress me;
    static {
	try {
            me = InetAddress.getLocalHost();
	} catch (UnknownHostException ex) {
          me = null;
	}
    }

    /**
     * The machine represented by this object
     */
    public final InetAddress addr;

    /**
     * The port number represented by this object
     */
    int port;

    /**
     * A string representing this object
     */
    private final String name;

    /**
     * Create a new <code>MachinePort</code> from a machine name, optionally
     * followed by a colon and a port number
     *
     * @param machine a string of the form "machine" or "machine:port"
     * @exception UnknownHostException if the name cannot be resolved
     */
    public MachinePort(final String machine) throws UnknownHostException {
	final int index = machine.indexOf(':');
	InetAddress theAddr = null;
	if (index < 0) {
	    addr = InetAddress.getByName(machine);
	    port = Globals.DEFAULT_PORT;
	} else {
	    addr = InetAddress.getByName(machine.substring(0, index));
	    try {
		port = Integer.parseInt(machine.substring(index + 1));
	    } catch (NumberFormatException numEx) {
		System.err.print(machine.substring(index + 1));
		System.err.print(" is not a valid port number for ");
		System.err.println(machine.substring(0, index));
		System.err.print("Using the default port of ");
		System.err.println(Globals.DEFAULT_PORT);
                
                port = Globals.DEFAULT_PORT;
	    }
	}
	name = addr.getHostName() + ':' + port;
    }

    /**
     * Get the name of this machine
     *
     * @return a string containing the name of the machine
     */
    public String getName() {
	return addr.getHostName();
    }

    /**
     * Get the port number
     *
     * @return an <code>int</code> containing the port number
     */
    public int getPort() {
	return port;
    }

    /**
     * Determine whether this machine is the local one
     *
     * @return <code>true</code> if this <code>MachinePort</code> object
     * represents the local machine; <code>false</code> otherwise
     */
    public boolean isLocal() {
	return addr.equals(me);
    }

    public boolean equals(final MachinePort mp) {
	return port == mp.port && addr.equals(mp.addr);
    }

    // REVIEW: And maybe match if there is no port number in otherName?
    public boolean equals(final String otherName) {
	return otherName.equals(name);
    }

    public boolean equals(final Object obj) {
	return (obj instanceof String)
	    ? equals((String)obj)
	    : equals((MachinePort)obj);
    }

    // FIXME: Currently, this MUST be addr.hashCode() so that it matches up
    // with PhysicalNode, but we really need a way to add the port number in
    // somehow
    public int hashCode() {
	return addr.hashCode();
    }

    public String toString() {
	return name;
    }
}
