/*
 * @(#)Message.java Message.java,v 1.2 2002/11/22 23:26:06 kris Exp $
 *
 *
 */

package mokan.comm;

import java.io.*;
import mokan.debug.Event;

/**
 * The ultimate ancestor of all message objects.  This class simply asserts
 * that all subclasses have a <code>handle()</code> method that is called by
 * the message receiver, and defines 2 booleans for deciding whether to reset
 * the output stream before sending, and whether to use a separate thread for
 * handling the message.
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:06 $
 * @author	Jerry James
 */
public class Message implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -197935070893894516L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * <code>True</code> if the outgoing stream should be reset before sending
     * this message
     */
    protected final transient boolean reset;

    /**
     * <code>True</code> if the handling of this message might block the
     * handling thread.  This is a pseudo-final field.
     */
    public final transient boolean block;

    /**
     * Create a new message object
     *
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     */
    protected Message(final boolean needsReset, final boolean canBlock) {
        reset = needsReset;
        block = canBlock;
    }

    /**
     * Process this message
     */
    public void handle() {
        Event.DEBUG(Event.MSG_HANDLE, "Handling ", this);
    }

    /**
     * Save the state of the <code>Message</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData None
     */
    public void writeExternal(final ObjectOutput out) throws IOException { }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException { }
}
