/*
 * @(#)$Id: Network.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 *
 * @author Kristian G. Kvilekval
 */

package mokan.comm;

import java.util.ArrayList;
import java.util.Collection;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.net.UnknownHostException;

/**
 *   The base interface for Kan system network.  A network is
 *   an abstract collection of nodes that can communicate.
 *   The network is responsible for Connections/Disconnections
 *   and actually passing of messages.
 */
public interface Network  {

    /** Base class for Real network Address implementations
     *   A Node provides an Object Stream for now, which is how we
     *   send message throught the network, hthough we might need
     *   to rethink this in the future.
     *
     */
    public static class NodeID {
        public Network network;
        public ObjectOutputStream oos;
        public ObjectInputStream ois;
        public String getName ()    {return null;}
        public NodeID(Network _network, ObjectOutputStream _oos, ObjectInputStream _ois) {
          network = _network;
          oos = _oos;
          ois = _ois;
        }
    };

    public NodeID init ();
    public void   disable();
    public NodeID getLocalNodeID ();
    public NodeID find (String addr);
//    public NodeID connect (NodeID node) throws UnknownHostException;
//    public void   disconnect (NodeID node);

    public void send (NodeID node, Message m) throws IOException;
    public void send (Collection c, Message m);
    public Message recv (NodeID node) throws IOException;
    public void close (NodeID node) throws IOException ;

    /** Add <code>NetworkListener</code> l to those that
     * receive notification of network events
     */
    public void addNetworkListener (NetworkListener l);
    public void removeNetworkListener (NetworkListener l);
//    public void notifyAddNode (NodeID n);
//    public void notifyRemoveNode(NodeID n);
}
