/*
 * @(#)$Id: Reply.java,v 1.2 2002/11/22 23:26:06 kris Exp $
 *
 *
 */

package mokan.comm;

import java.io.*;

/**
 * Reply to a previously sent message.  This reply class is only for system
 * messages.  It does not contain <code>createThreadLocal()</code> in its
 * handle method.  For <code>AppMessage</code>, see <code>AppReply</code>.
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:06 $
 * @author	Sean Brydon
 * @author	Jerry James
 * @author	Sandeep Bhatia
 * @seemokan.comm.AppReply
 */
public abstract class Reply extends SysMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -4947718913150261125L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The sequence number of the message to which this is a reply.  This is a
     * pseudo-final field.
     *
     * @serial the sequence number of the message to which this is a reply
     */
    protected long senderSeqNum;

    /**
     * Create a new <code>Reply</code> object, for deserialization
     *
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     */
    protected Reply(final boolean needsReset, final boolean canBlock) {
	super(needsReset, canBlock);
    }

    /**
     * Create a new <code>Reply</code>, and set up the sequence number of the
     * message to which this is a reply.
     *
     * @param SSN the sequence number of the message to which this is a reply
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     */
    protected Reply(final long SSN, final boolean needsReset,
		    final boolean canBlock) {
	super(needsReset, canBlock);
	senderSeqNum = SSN;
    }

    /**
     * Take the receive action for this message.  The default action for
     * <code>Reply</code> is to wakeup a waiting sender.
     */
    public void handle() {
	super.handle();
	((Wakeup)wakeupTable.get(senderSeqNum)).answerReceived(this);
    }

    /**
     * Save the state of the <code>Reply</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the send
     *             sequence number of this reply (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
	out.writeLong(senderSeqNum);
    }

    public void readExternal(final ObjectInput in)
	throws IOException, ClassNotFoundException {
	
	super.readExternal(in);
	senderSeqNum = in.readLong();
    }
}
