/*
 * @(#)SingleWakeup.java SingleWakeup.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 */

package mokan.comm;

/**
 * A sleeping thread, waiting on communication from a single node
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author	Jerry James
 */
final class SingleWakeup implements Wakeup {

    /**
     * A single reply message
     */
    private Message reply = null;

    /**
     * Wait for a reply to be received
     *
     * @return the reply that was received
     * @exception InterruptedException if a transaction abort occurs while
     *         waiting
     */
    Message await() throws InterruptedException {
        synchronized (this) {
            while (reply == null) {
                wait();
            }
        }
        return reply;
    }

    /**
     * Tell this <code>SingleWakeup</code> that an answer was received
     *
     * @param answer the answer received
     */
    public void answerReceived(final Message answer) {
        reply = answer;
        synchronized (this) {
            notify();	// We know there is at most one thread waiting!
        }
    }

    public String toString() {
        return "SingleWakeup";
    }
}
