/*
 * @(#)$Id: SysReply.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 */


package mokan.comm;

import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

/**
 *  The <code>SysReply</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 *  @author Kristian Kvilekval
 */
public class SysReply extends SysMessage implements Externalizable{
    /**
     * The sequence number of the message to which this is a reply.  This is a
     * pseudo-final field.
     *
     * @serial a long value holding the sequence number of the original message
     */
    protected long senderSeqNum;

    /**
     * Create a new <code>SysReply</code>; the deserializing constructor
     */
    public SysReply() {
	super(false, false);
    }

    /**
     * Create a new <code>SysReply</code>; the deserializing constructor for
     * subclasses
     *
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     * The no-args constructor, needed for deserialization
     */
    protected SysReply(final boolean needsReset, final boolean canBlock) {
	super(needsReset, canBlock);
    }

    /**
     * Create a new <code>SysReply</code>, and set up the sequence number of
     * the message to which this is a reply.
     *
     * @param msg the message to which this is a reply
     * @param needsReset <code>true</code> if the outgoing stream should be
     *        reset before sending this message
     * @param canBlock <code>true</code> if the handling of this message might
     *        block the handling thread
     */
    public SysReply(final SysMessage msg, final boolean needsReset,
		    final boolean canBlock) {
	super(needsReset, canBlock);
	senderSeqNum = msg.SSN;
    }
    
    /**
     * Take the receive action for this reply.  The default action is to
     * wakeup a waiting sender.
     */
    public void handle() {
	super.handle();
	final Wakeup wake = (Wakeup)wakeupTable.get(senderSeqNum);
	wake.answerReceived(this);
    }

    /**
     * Convert the <code>SysReply</code> to a string
     *
     * @return a string representation of this <code>SysReply</code>
     */
    public String toString() {
	return "SysReply(" + ',' + senderSeqNum + ')';
    }

    /**
     * Save the state of the <code>SysReply</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             send sequence number (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
	out.writeLong(senderSeqNum);
    }

    public void readExternal(final ObjectInput in)
	throws IOException, ClassNotFoundException {
	
	super.readExternal(in);
	senderSeqNum = in.readLong();
    }
};
