/*
 * @(#)$Id
 *
 *
 */

package mokan.comm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.net.*;
//import java.net.Socket;
import java.io.*;

import mokan.Kan;
import mokan.util.IntHashMap;
import mokan.debug.Event;

import mokan.nameservice.*;

/**
 *    A KanNetwork based on TCP/IP connections.
 *    @author kristian kvilekval
 */
class TCPNetwork extends Thread implements Network {
  /**
   * The default port
   */

  private final int port;

  /**
   *  Set while network is listening for new connections.
   */
  private boolean listening;
  private ServerSocket listenSocket;

  /**
   * List of  @see NetworkListener waiting for @see NetworkEvent
   */
  private ArrayList listeners = new ArrayList(4);

  /**
   *    A TCP Node ID
   */
  static class TCPNodeID extends Network.NodeID {

    public Socket socket;
    public InetAddress address;
    public int port;

    public TCPNodeID(Network _network, ObjectOutputStream _oos,
                     ObjectInputStream _ois, Socket _socket,
                     InetAddress _address, int _port) {
      super(_network, _oos, _ois);
      socket = _socket;
      address = _address;
      port = _port;
    }

    public int hashCode() {
      return address.hashCode();
    }

    public boolean equals(Object n) {
      return ( (n instanceof TCPNodeID)
              && address.equals( ( (TCPNodeID) n).address));
//                     && ((TCPNodeID)n).port == port);

    }

    public String getName() {
      return address.getHostName();
    }

    public String toString() {
      return getName() + ":" + port;
    }
  };
  /**
   *   Local
   */
  private TCPNodeID localNode;

  /*
       public NodeID find (String addr) {
      try {
          TCPNodeID node = new TCPNodeID();
          node.network = this;
          node.name = addr;
          node.address = InetAddress.getByName (addr);
          node.port = DEFAULT_PORT;   //KK FIXME
          Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.find ", node);
          if (netNodes.containsKey (node)) {
              Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.find found node");
              return (TCPNodeID)netNodes.get (node);
          }
          Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.find ", addr);
          //KK TODO don't connect immeidately, but wait for a send.
          connect (node);
          return node;
      } catch (UnknownHostException uk) {
          return null;
      }
       }
       public final Network.NodeID connect (String addr) throws UnknownHostException {
      TCPNodeID node = new TCPNodeID();
      node.network = this;
      node.name = addr;
      node.address = InetAddress.getByName (addr);
      node.port = DEFAULT_PORT;   //KK FIXME
      if (netNodes.containsKey (node)) {
          Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.connect found node");
          return (TCPNodeID)netNodes.get (node);
      }
      Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.connect ", addr);
      connect (node);
      return node;
       }
   */

  public NodeID find(String addr) {
    throw new UnsupportedOperationException("TCPNetwork.find: Obsolete.");
  }

  /**
   * Connects to a node.
   * @param address The remote address
   * @param port The remote port
   * @param remoteID The uniqueID of the remote node, or null
   * @return A connected PhysicalNode, whether new or already existing
   */
  public final PhysicalNode connect(String address, int port, String remoteID) {
    Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.connect "
                + address + ":" + port);
    try {
      Socket socket = new Socket(address, port);
      socket.setTcpNoDelay(true);
      socket.setSoLinger(true, 2);
      ObjectOutputStream oos = new ObjectOutputStream(
          new BufferedOutputStream(socket.getOutputStream()));
      oos.flush();
      ObjectInputStream ois = new ObjectInputStream(
          new BufferedInputStream(socket.getInputStream()));

      TCPNodeID nid = new TCPNodeID(this, oos, ois, socket,
                                    InetAddress.getByName(address), port);

      Event.DEBUG(Event.SOCKET_OPEN, "Opened socket to ", nid);

      String localID = Globals.localPhysicalNode.getUniqueID();

      //When A connects to B,
      //1. A sends A's uniqueID
      //2. If A knows B's uniqueID
      //    1) A sends B's uniqueID
      //    2) A receives B's uniqueID
      //    3) A verifies that the two uniqueIDs are identical
      //
      //3. If A does not know B's uniqueID
      //    1) A sends ""
      //    2) A receives B's uniqueID
      //    3) A creates a PhysicalNode for B

      nid.oos.writeObject(localID);
      PhysicalNode pn;
      if (remoteID == null) {
        nid.oos.writeObject("");
        nid.oos.flush();
        String _remoteID = (String) nid.ois.readObject();
        pn = new PhysicalNode(nid, _remoteID);
        KanComm.uniqueID2node.put(_remoteID, pn);
        //return pn;
      } else {
        nid.oos.writeObject(remoteID);
        nid.oos.flush();
        String _remoteID = (String) nid.ois.readObject();
        if (!remoteID.equals(_remoteID)) {
          throw new RuntimeException(
              "TCPNetwork.connect ERROR: uniqueID not identical.");
        }
        pn = (PhysicalNode) KanComm.uniqueID2node.get(remoteID);
        pn.addNodeID(nid);
        pn.setConnected(true);
      }

      notifyAddNode(nid, pn);

      return pn;

      //new KanThread(node.ois, null);
    } catch (SocketException socketEx) {
      throw new RuntimeException(socketEx.getMessage());
    } catch (IOException ioEx) {
      throw new RuntimeException(ioEx.getMessage());
    } catch (ClassNotFoundException classEx) {
      throw new RuntimeException(classEx.getMessage());
    }
  }

  /**
   *    Listen for people creating joining the network
   */
  private final void listen(int port) throws IOException {

    listening = true;
    listenSocket = new ServerSocket(port);
    Event.DEBUG(Event.SOCKET_OPEN, "TCPNetwork.listen to port " + port);

    while(listening) {
      Socket socket = listenSocket.accept();
      InetAddress address = socket.getInetAddress();
      ObjectInputStream ois = new ObjectInputStream(
          new BufferedInputStream(socket.getInputStream()));
      ObjectOutputStream oos = new ObjectOutputStream
          (new BufferedOutputStream(socket.getOutputStream()));
      oos.flush();

      int remotePort = socket.getPort();
      TCPNodeID nid = new TCPNodeID(this, oos, ois, socket, address,
                                    remotePort);

      Event.DEBUG(Event.SOCKET_ACCEPT,
                  "TCPNetwork.listen Accepted socket from ", nid);

      try {
        String remoteID = (String) nid.ois.readObject();
        String _localID = (String) nid.ois.readObject();
        String localID = Globals.localPhysicalNode.getUniqueID();
        if (!_localID.equals("") && !_localID.equals(localID)) {
          throw new RuntimeException(
              "TCPNetwork.listen ERROR: uniqueID not identical.");
        }

        nid.oos.writeObject(localID);
        nid.oos.flush();

        PhysicalNode pn = (PhysicalNode) KanComm.uniqueID2node.get(remoteID);
        if (pn == null) {
          pn = new PhysicalNode(nid, remoteID);
          KanComm.uniqueID2node.put(remoteID, pn);
        } else {
          pn.addNodeID(nid);
          pn.setConnected(true);
        }

        // ?? DOes this need to be in it's own thread.
        notifyAddNode(nid, pn);

      } catch (IOException ioEx) {
        ioEx.printStackTrace();
      } catch (ClassNotFoundException classEx) {
        throw new RuntimeException(classEx.getMessage());
      }

    }
  }

  public void send(NodeID nid, Message message) throws IOException {
    Event.DEBUG(Event.MSG_SEND, "TCPNetwork.send ",
                message + " -> " + nid);
    TCPNodeID tcpNid = (TCPNodeID) nid;
    try {
      if (nid.oos != null) {
        synchronized (nid) {
          // See the comments below for this conditional
          if (message.reset) {
            nid.oos.reset();
          }
          nid.oos.writeObject(message);
          nid.oos.flush();
          //Event.DEBUG(Event.MSG_SEND, "Sent ", message);
        }
      } else {
        throw new RuntimeException("Unexpected ERROR!");
      }
    } catch (EOFException eof) {
      notifyRemoveNode(nid);
      throw eof;
    } catch (IOException ioe) {
      System.out.println("TCPNetwork.send: couldn't send " + message + " to " +
                         nid);
      ioe.printStackTrace();
    }
  }

  /*
   * The call to sendPipe.reset() above is ABSOLUTELY ESSENTIAL!!!  Do not
   * delete it, on pain of death, taxes, and rioting in the streets!  When
   * an object is serialized on the output stream, the stream keeps a handle
   * to the object.  If the object is subsequently serialized again, the
   * handle is sent, rather than reserializing the entire object (for
   * efficiency's sake).  This means that if the object is modified, the
   * MODIFICATIONS ARE NOT SENT!  The call to reset tells the stream to
   * discard its handle table and resend all objects.  It would be nicer if
   * we could flush individual objects out, but Java does not yet give us
   * that capability.
   */

  public void send(Collection c, Message m) {
    throw new UnsupportedOperationException("TCPNetwork.send");
  }

  public Message recv(NodeID nid) throws IOException {

    for (; ; ) {
      try {
        //Event.DEBUG (Event.MSG_RECEIVE, "TCPNetwork.recv from ", nid);
        Object o = nid.ois.readObject();
        Event.DEBUG(Event.MSG_RECEIVE, "TCPNetwork.recv " + o + " from " + nid);
        return (Message) o;

      } catch (IOException eof) {
        if (eof instanceof OptionalDataException) {
          continue;
        }
        notifyRemoveNode(nid);
        throw eof;
      } catch (ClassNotFoundException nf) {
        // LOG and continue;
      }
    }
  }

  /**
   *   Close the pathway through this network
   */
  public void close(NodeID n) throws IOException {
    Event.DEBUG(Event.MSG_RECEIVE, "TCPNetwork.close ", n);

    TCPNodeID nid = ( (TCPNodeID) n);
    if (nid.socket != null) {
      nid.socket.close();

    }
  }

  public TCPNetwork(int _port) {
    port = _port;
  }

  /**
   *    Initialize (restart?) the TCP/IP network.
   */
  public NodeID init() { // throws UnkownHostException,IOException{
    Event.DEBUG(Event.TCP_IP, "TCPNetwork.init");
    try {
      localNode = new TCPNodeID(this, null, null, null,
                                InetAddress.getLocalHost(), port);

      start();
      Event.DEBUG(Event.TCP_IP, "TCPNetwork.init", localNode);

      return localNode;
    } catch (UnknownHostException uh) {
      uh.printStackTrace();
      return null;
    }
  }

  public void disable() {
    Event.DEBUG(Event.TCP_IP, "TCPNetwork.disable:", localNode);
    try {
      listening = false;
      listenSocket.close(); // Should cause exception in listener.
      listenSocket = null;
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void run() {

    // Pull Local listener port from a default file or directory serice
    try {
      listen(port);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public Network.NodeID getLocalNodeID() {
    return localNode;
  }

  public void addNetworkListener(NetworkListener l) {
    listeners.add(l);
  }

  public void removeNetworkListener(NetworkListener l) {
    listeners.remove(l);
  }

  private void notifyAddNode(Network.NodeID nid, PhysicalNode pn) {
    NetworkEvent e = new NetworkEvent(this, nid, pn);
    Iterator p = listeners.iterator();
    while (p.hasNext()) {
      ( (NetworkListener) p.next()).networkAddNode(e);
    }
  }

  private void notifyRemoveNode(Network.NodeID nid) {
    NetworkEvent e = new NetworkEvent(this, nid, null);
    Iterator p = listeners.iterator();
    while (p.hasNext()) {
      ( (NetworkListener) p.next()).networkRemoveNode(e);
    }
  }
}
