/*
 * @(#)YesNoReply.java YesNoReply.java,v 1.2 2002/11/22 23:26:06 kris Exp $
 *
 *
 */

package mokan.comm;

import java.io.*;
import mokan.debug.Event;

/**
 * A reply that contains the answer "yes" or the answer "no"
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:06 $
 * @author	Jerry James
 */
public final class YesNoReply extends AppReply implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 8988194176784795024L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * <code>true</code> if we agree; <code>false</code> otherwise.  This is a
     * pseudo-final field.
     *
     * @serial <code>true</code> if the answer is yes
     */
    public boolean yes;

    /**
     * The no-args constructor, needed for deserialization
     */
    public YesNoReply() {
        super(false, false);
    }

    /**
     * Make a new <code>YesNoReply</code> with an agreement value
     *
     * @param orig the original message, to which this is a reply
     * @param agree <code>true</code> if the answer is "yes";
     *        <code>false</code> if the answer is "no"
     */
    public YesNoReply(final AppMessage orig, final boolean agree) {
        super(orig, false, false);
        yes = agree;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }


    /**
     * Convert the <code>YesNoReply</code> to a string
     *
     * @return a string representation of this <code>YesNoReply</code>
     */
    public String toString() {
        return "YesNoReply[[" + yes + ']';
    }

    /**
     * Save the state of the <code>YesNoReply</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the answer
     *             contained in this message (boolean).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeBoolean(yes);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {
	
        super.readExternal(in);
        yes = in.readBoolean();
    }
}
