/*
 * @(#)GuardExpr.java GuardExpr.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler;

import antlr.ASTFactory;
import java.util.ArrayList;
import mokan.compiler.expr.ExpressionAST;
import mokan.compiler.expr.VarAST;

/**
 * A data container that holds information on one guard expression in a class
 * currently being translated.  It is used to generate boolean methods
 * containing the guard expressions.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author	Jerry James
 */
final class GuardExpr {

    /**
     * The next guard expression in a linked list of such expressions
     */
    final GuardExpr next;

    /**
     * The expression itself
     */
    final ExpressionAST expr;

    /**
     * The number of the {@link mokan.obj.GuardCondition GuardCondition} for this
     * guard expression
     */
    final int number;

    /**
     * The parameters for the guard expression method
     */
    final ParameterAST[] params;

    /**
     * Create a new guard expression object
     *
     * @param astFactory the AST Factory currently in use
     * @param n the <code>GuardExpr</code> to link to
     * @param ex the expression to evaluate for the guard
     * @param num the number of the
     *        {@link mokan.obj.GuardCondition GuardCondition} for this guard
     */
    GuardExpr(final ASTFactory astFactory, final GuardExpr n,
	      final ExpressionAST ex, final int num) {
	next = n;
	expr = ex;
	number = num;

	// We don't care about declared or written variables ... there
	// shouldn't be any in a guard expression.  Just check out the
	// variables that are read and find the locals.
	final ArrayList list = new ArrayList();
	final VarAST[] vars = ex.getVarList().read;
	for (int i = 0; i < vars.length; i++) {
	    if (!vars[i].isField()) {
		list.add(vars[i]);
	    }
	}
	final VarAST[] realParams = new VarAST[list.size()];
	list.toArray(realParams);
	params = new ParameterAST[realParams.length];
	for (int i = 0; i < realParams.length; i++) {
	    final VarAST real = realParams[i];
	    final ModifierAST mods = new ModifierAST(real.getModifiers().mods);
	    final mokan.compiler.expr.TypeAST name =
		new mokan.compiler.expr.TypeAST(real.getTypeName().getName());
	    name.setFirstChild(real.getTypeName().getFirstChild());
	    params[i] = new ParameterAST(mods, name, real);
	    name.setNextSibling(real);
	    mods.setNextSibling(name);
	    params[i].setFirstChild(mods);
	}
    }
}
/* arch-tag: 9947db58-c3aa-425c-bb4a-faaed11bffc8 */