/*
 * @(#)$Id: GuardModifierAST.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler;

/**
 * An AST node that represents a (possibly empty) set of guard modifiers
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author Jerry James
 */
public final class GuardModifierAST extends KanAST implements KanTokenTypes {

    /**
     * The read modifier
     */
    public static final int READ = 0x00000001;

    /**
     * The write modifier
     */
    public static final int WRITE = 0x00000002;

    /**
     * The nonblocking modifier
     */
    public static final int NONBLOCKING = 0x00000004;

    /**
     * The blocking modifier
     */
    public static final int BLOCKING = 0x00000008;

    /**
     * The nested modifier
     */
    public static final int NESTED = 0x00000010;

    /**
     * The guard modifier values represented by this guard modifier AST
     */
    int mods;

    /**
     * Create a new guard modifier AST node
     *
     * @param modBits the modifier bits
     */
    public GuardModifierAST(final int modBits) {
	super();
	initialize(GUARD_MODIFIERS, "GUARD_MODIFIERS");
	mods = modBits;
    }

    /**
     * Determine whether this set of guard modifiers includes the
     * <code>read</code> modifier
     *
     * @return <code>true</code> if <code>read</code> is included
     */
    public boolean isRead() {
	return (mods & READ) != 0;
    }

    /**
     * Determine whether this set of guard modifiers includes the
     * <code>write</code> modifier
     *
     * @return <code>true</code> if <code>write</code> is included
     */
    public boolean isWrite() {
	return (mods & WRITE) != 0;
    }

    /**
     * Determine whether this set of guard modifiers includes the
     * <code>nonblocking</code> modifier
     *
     * @return <code>true</code> if <code>nonblocking</code> is included
     */
    public boolean isNonblocking() {
	return (mods & NONBLOCKING) != 0;
    }

    /**
     * Determine whether this set of guard modifiers includes the
     * <code>blocking</code> modifier
     *
     * @return <code>true</code> if <code>blocking</code> is included
     */
    public boolean isBlocking() {
	return (mods & BLOCKING) != 0;
    }

    /**
     * Determine whether this set of modifiers includes the
     * <code>nested</code> modifier
     *
     * @return <code>true</code> if <code>nested</code> is included
     */
    public boolean isNested() {
	return (mods & NESTED) != 0;
    }
}
/* arch-tag: badb734b-c17a-4dd9-9dfc-59ec886ef2e4 */