/*
 * @(#)ParameterAST.java ParameterAST.java,v 1.1.1.1 2002/11/06 22:10:25 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler;

import mokan.compiler.expr.VarAST;

/**
 * An AST node that represents a formal parameter
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:25 $
 * @author Jerry James
 */
public final class ParameterAST extends KanAST implements KanTokenTypes {

    /**
     * The modifiers for this parameter
     */
    private final ModifierAST mods;

    /**
     * The declared type of this parameter
     */
    private final mokan.compiler.expr.TypeAST typeAST;

    /**
     * The parameter name and trailing brackets
     */
    private final VarAST var;

    /**
     * Create a new parameter AST
     *
     * @param mod the modifiers for this parameter
     * @param type the type of this parameter
     * @param variable the parameter name and trailing brackets (if any)
     */
    public ParameterAST(final ModifierAST mod,
			final mokan.compiler.expr.TypeAST type,
			final VarAST variable) {
	setType(PARAMETER_DEF);
	mods = mod;
	typeAST = type;
	var = variable;
    }

    public void parseComplete() {
	context.mods = mods;
	context.type = typeAST;
	var.parseComplete();
    }

    /**
     * Get the modifiers for this variable
     *
     * @return the modifiers
     */
    public ModifierAST getModifiers() {
	return mods;
    }

    /**
     * Get the name of the declared type for this variable
     *
     * @return the type name
     */
    public mokan.compiler.expr.TypeAST getTypeName() {
	return typeAST;
    }

    /**
     * Get the parameter name and trailing brackets
     *
     * @return a variable declaration
     */
    public VarAST getParamName() {
	return var;
    }

    public String toString() {
	return var.toString();
    }
}
/* arch-tag: cacce68e-d00e-4603-98d0-547d604c8304 */