/*
 * @(#)IndexAST.java IndexAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.expr;

import antlr.Token;
import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents an index expression
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class IndexAST extends ExpressionAST implements KanTokenTypes {

    /**
     * The base of the index expression (i.e., the value being indexed)
     */
    private ExpressionAST base;

    /**
     * The index
     */
    private ExpressionAST index;

    /**
     * Create a new index expression AST
     *
     * @param token the token represented by this AST node
     */
    public IndexAST(final Token token) {
        super(token);
        setType(INDEX_OP);
    }

    public void parseComplete() {
        base = (ExpressionAST)getFirstChild();
        index = (ExpressionAST)base.getNextSibling();
        base.parseComplete();
        index.parseComplete();
    }

    protected Type computeType() {
        return base.retrieveType().getComponentType();
    }

    protected Type[] computeExceptions() {
        return Type.mergeTypeLists(base.getExceptionTypes(),
                                   index.getExceptionTypes());
    }

    protected Object computeValue() {
        return nonconstant;
    }

    public VarList getVarList() {
        return new VarList(base.getVarList(), index.getVarList());
    }

    /**
     * Get the base of this index expression; i.e., the value being indexed
     *
     * @return the base of the expression
     */
    public ExpressionAST getBase() {
        return base;
    }

    /**
     * Get the index of this index expression
     *
     * @return the index of the expression
     */
    public ExpressionAST getIndex() {
        return index;
    }

    public int transType() {
        final int bType = base.transType();
        if (bType == Transaction.NESTED)
            return bType;
        final int iType = index.transType();
        return (bType > iType) ? bType : iType;
    }
}
/* arch-tag: 93b3b44d-3586-453a-8b49-1752a0476358 */