/*
 * @(#)$Id: InitializerAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.expr;

import antlr.Token;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents a variable initializer
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class InitializerAST extends ExpressionAST {

    /**
     * The right-hand side of the initialization
     */
    private ExpressionAST rhs;

    /**
     * Create a new initializer AST
     *
     * @param token the token represented by this AST node
     */
    public InitializerAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	rhs = (ExpressionAST)getFirstChild();
	rhs.parseComplete();
    }

    protected Type computeType() {
	// The type of the initializer is the type of its rhs
	return rhs.retrieveType();
    }

    protected Type[] computeExceptions() {
	return rhs.getExceptionTypes();
    }

    protected Object computeValue() {
	return rhs.getValue();
    }

    public VarList getVarList() {
	return rhs.getVarList();
    }

    /**
     * Get the right-hand-side of this initializer
     *
     * @return the rhs of the expression
     */
    public ExpressionAST getRight() {
	return rhs;
    }

    public int transType() {
	return rhs.transType();
    }
}
/* arch-tag: 0de634e4-463c-429f-b6c0-8786e96ea5ca */