/*
 * @(#)$Id: ParenthesizedAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.expr;

import antlr.Token;
import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents a parenthesized expression
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ParenthesizedAST extends ExpressionAST
    implements KanTokenTypes {

    /**
     * The parenthesized expression
     */
    private ExpressionAST parenthesized;

    /**
     * Create a new parenthesized expression AST
     *
     * @param token the token represented by this AST node
     */
    public ParenthesizedAST(final Token token) {
	super(token);
	setType(PAREN_EXPR);
    }

    public void parseComplete() {
	parenthesized = (ExpressionAST)getFirstChild();
	parenthesized.parseComplete();
    }

    protected Type computeType() {
	return parenthesized.retrieveType();	// JLS 15.8.5
    }

    protected Type[] computeExceptions() {
	return parenthesized.getExceptionTypes();
    }

    protected Object computeValue() {
	return parenthesized.getValue();
    }

    public VarList getVarList() {
	return parenthesized.getVarList();
    }

    /**
     * Get the expression in the parentheses
     *
     * @return the parenthesized expression
     */
    public ExpressionAST getParenExpression() {
	return parenthesized;
    }

    public int transType() {
	return parenthesized.transType();
    }
}
/* arch-tag: d6833520-f65e-4445-8d39-ca544f45dc71 */