/*
 * @(#)$Id: StringLiteralAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.expr;

import antlr.Token;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents a literal string
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class StringLiteralAST extends ExpressionAST {

    /**
     * Create a new literal string AST
     *
     * @param token the token represented by this AST node
     */
    public StringLiteralAST(final Token token) {
	super(token);
	type = Type.stringType;		// JLS 15.8.1
	value = getText();
    }

    protected Type computeType() {
        return type;
    }

    protected Type[] computeExceptions() {
	return noTypes;
    }

    protected Object computeValue() {
	return value;
    }

    public VarList getVarList() {
	return new VarList();
    }

    public int transType() {
	return Transaction.NONBLOCKING;
    }
}
/* arch-tag: 835363fe-66ae-46fa-a296-ce086d3293eb */