/*
 * @(#)TypeAST.java TypeAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.expr;

import antlr.collections.AST;
import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents a type expression
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class TypeAST extends IdentifierAST implements KanTokenTypes {

    /**
     * Create a new type expression AST
     *
     * @param name the name of this type identifier
     */
    public TypeAST(final String name) {
	super();
	initialize(TYPE, name);
	setName(name);
    }

    protected Type computeType() {
	try {
	    return topLevel.getType(getName());
	} catch (ClassNotFoundException classEx) {
	    System.err.println("Could not find a type for " 
                               + getName()
                               + " in file " + topLevel);
	    return null;
	}
    }

    protected Type[] computeExceptions() {
	return noTypes;
    }

    protected Object computeValue() {
	return nonconstant;
    }

    public VarList getVarList() {
	return new VarList();
    }

    public int transType() {
	return Transaction.NONBLOCKING;
    }
}
/* arch-tag: 7b74b34b-d377-4efc-9837-ed136acc5775 */