/*
 * @(#)$Id: ClassAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.TypeAST;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents a type definition statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ClassAST extends StatementAST implements KanTokenTypes {

    /**
     * The class definition
     */
    private final TypeAST type;

    /**
     * Create a new type definition statement AST
     *
     * @param typeAST the type definition
     */
    public ClassAST(final TypeAST typeAST) {
	super();
	initialize(TYPE_STAT, "TYPE_STAT");
	type = typeAST;
    }

    public void parseComplete() {
	super.parseComplete();
	type.parseComplete();
    }

    protected Type[] computeExceptions() {
	return noTypes;
    }

    protected StatementAST[] computeControl() {
	return new StatementAST[] { next };
    }

    public VarList getVarList() {
	return new VarList();
    }

    /**
     * Get the type definition for this class
     *
     * @return the type definition
     */
    public TypeAST getTypeDef() {
	return type;
    }

    public int transType() {
	return Transaction.NONBLOCKING;
    }
}
/* arch-tag: 99853a7e-bc31-42c1-8d2a-190174e092cb */