/*
 * @(#)$Id: ContinueAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import antlr.collections.AST;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.compiler.expr.IdentifierAST;
import mokan.transact.Transaction;

/**
 * An AST node that represents a continue statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ContinueAST extends StatementAST {

    /**
     * The continue label
     */
    private IdentifierAST label;

    /**
     * Create a new continue statement AST
     *
     * @param token the token represented by this AST node
     */
    public ContinueAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	super.parseComplete();
	final AST maybeLabel = getFirstChild();
	if (maybeLabel instanceof IdentifierAST) {
	    label = (IdentifierAST)maybeLabel;
	} else {
	    // There is no label, so the closest inclosing for, while, or
	    // do-while statement is the target of this continue
	    control = new StatementAST[] { context.continueTarget() };
	}
    }

    protected Type[] computeExceptions() {
	return noTypes;
    }

    protected StatementAST[] computeControl() {
	// This is only called if label is non-null
	return new StatementAST[] { symTable.getLabel(label.getName()) };
    }

    public VarList getVarList() {
	return new VarList();
    }

    /**
     * Get the label at which control should continue
     *
     * @return the label, or <code>null</code> if there is not one
     */
    public IdentifierAST getLabel() {
	return label;
    }

    public int transType() {
	return Transaction.NONBLOCKING;
    }
}
/* arch-tag: af82bfa0-14a4-4ec8-97d9-49407c71e7bc */