/*
 * @(#)DoWhileAST.java DoWhileAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents a do-while statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class DoWhileAST extends StatementAST {

    /**
     * The condition test
     */
    private mokan.compiler.expr.ExpressionAST cond;

    /**
     * The statement
     */
    private StatementAST stmt;

    /**
     * Create a new do-while statement AST
     *
     * @param token the token represented by this AST node
     */
    public DoWhileAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	super.parseComplete();
	stmt = (StatementAST)getFirstChild();
	cond = (mokan.compiler.expr.ExpressionAST)stmt.getNextSibling()
	    .getNextSibling().getNextSibling();

	// Set up the proper break/continue context
	context.pushContinue(this);
	stmt.parseComplete();
	cond.parseComplete();
	context.popContinue();
    }

    protected Type[] computeExceptions() {
	return Type.mergeTypeLists(cond.getExceptionTypes(),
				   stmt.getExceptionTypes());
    }

    protected StatementAST[] computeControl() {
	return stmt.nextControlPoints();
    }

    public VarList getVarList() {
	return new VarList(cond.getVarList(), stmt.getVarList());
    }

    /**
     * Get the condition for the loop
     *
     * @return the condition
     */
    public mokan.compiler.expr.ExpressionAST getCondition() {
	return cond;
    }

    /**
     * Get the body of the loop
     *
     * @return the body
     */
    public StatementAST getBody() {
	return stmt;
    }

    public int transType() {
	return (cond.transType() == Transaction.NESTED ||
		stmt.transType() == Transaction.NESTED)
	    ? Transaction.NESTED
	    : Transaction.BLOCKING;
    }
}
/* arch-tag: 14a7ebad-4ae4-469d-ab6d-f89e93e945f4 */