/*
 * @(#)ExpressionAST.java ExpressionAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents an expression statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ExpressionAST extends StatementAST implements KanTokenTypes
{
    /**
     * The expression
     */
    private final mokan.compiler.expr.ExpressionAST expression;

    /**
     * Create a new expression statement AST
     *
     * @param expr the expression
     */
    public ExpressionAST(final mokan.compiler.expr.ExpressionAST expr) {
	super();
	initialize(EXPRESSION_STAT, "EXPRESSION_STAT");
	expression = expr;
    }

    public void parseComplete() {
	super.parseComplete();
	expression.parseComplete();
    }

    protected Type[] computeExceptions() {
	return expression.getExceptionTypes();
    }

    protected StatementAST[] computeControl() {
	return new StatementAST[] { next };
    }

    public VarList getVarList() {
	return expression.getVarList();
    }

    /**
     * Get the expression part of this statement
     *
     * @return the expression
     */
    public mokan.compiler.expr.ExpressionAST getExpression() {
	return expression;
    }

    public int transType() {
	return expression.transType();
    }
}
/* arch-tag: ed132462-978c-435a-bd64-35cbcb670de5 */