/*
 * @(#)GuardAST.java GuardAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.RecognitionException;
import antlr.Token;
import mokan.compiler.GuardModifierAST;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that holds information about a guard
 *
 * @version    $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author     Jerry James
 */
public final class GuardAST extends StatementAST {

    /**
     * The guard modifiers
     */
    private GuardModifierAST mods;

    /**
     * The condition test
     */
    private mokan.compiler.expr.ExpressionAST cond;

    /**
     * The statement
     */
    private StatementAST stmt;

    /**
     * Create a new <code>GuardAST</code>
     *
     * @param token the token to store in this AST node
     */
    public GuardAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	mods = (GuardModifierAST)getFirstChild();
	mods.parseComplete();
	cond = (mokan.compiler.expr.ExpressionAST)mods.getNextSibling()
	    .getNextSibling();
	cond.parseComplete();
	stmt = (StatementAST)cond.getNextSibling().getNextSibling();
	stmt.parseComplete();
    }

    protected Type[] computeExceptions() {
	return Type.mergeTypeLists(cond.getExceptionTypes(),
				   stmt.getExceptionTypes());
    }

    protected StatementAST[] computeControl() {
	return new StatementAST[] { next };
    }

    public VarList getVarList() {
	return new VarList(cond.getVarList(), stmt.getVarList());
    }

    public int transType() {
// 	return (cond.transType() == Transaction.NESTED ||
// 		stmt.transType() == Transaction.NESTED)
// 	    ? Transaction.NESTED
// 	    : Transaction.BLOCKING;
        if (mods.isNonblocking()) return Transaction.NONBLOCKING;
        if (mods.isBlocking()) return Transaction.BLOCKING;
        return  Transaction.NESTED;
        
    }

    /**
     * Determine whether the guard expression is trivially true
     *
     * @return <code>true</code> if the guard expression is trivially
     * satisfied; <code>false</code> otherwise
     */
    public boolean isTrivial() {
	final Object val = cond.getValue();
	return (val != null && (val instanceof Boolean) &&
		((Boolean)val).booleanValue());
    }

    public boolean isWriteTransaction () {
        return mods.isWrite();
    }
}
/* arch-tag: b051abf2-8a96-4ff7-8c65-23d704564c33 */