/*
 * @(#)$Id: LabelAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents a labeled statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class LabelAST extends StatementAST implements KanTokenTypes {

    /**
     * The labeled statement
     */
    private StatementAST labeled;

    /**
     * Create a new labeled statement AST
     *
     * @param token the token represented by this AST node
     */
    public LabelAST(final Token token) {
	super(token);
	setType(LABELED_STAT);
    }

    public void parseComplete() {
	super.parseComplete();
	labeled = (StatementAST)getFirstChild().getNextSibling();
	labeled.parseComplete();
    }

    protected Type[] computeExceptions() {
	return labeled.getExceptionTypes();
    }

    protected StatementAST[] computeControl() {
	return labeled.nextControlPoints();
    }

    public VarList getVarList() {
	return labeled.getVarList();
    }

    /**
     * Get the statement that is labeled
     *
     * @return the statement with this label
     */
    public StatementAST getStatement() {
	return labeled;
    }

    public int transType() {
	return labeled.transType();
    }
}
/* arch-tag: 13a8d79d-2c1d-4bd3-8e88-13667cebc943 */