/*
 * @(#)ReturnAST.java ReturnAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import antlr.collections.AST;
import mokan.compiler.KanTokenTypes;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents a return statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ReturnAST extends StatementAST implements KanTokenTypes {

    /**
     * The expression to return
     */
    private mokan.compiler.expr.ExpressionAST expr;

    /**
     * Create a new return statement AST
     *
     * @param token the token represented by this AST node
     */
    public ReturnAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	super.parseComplete();
	final AST retVal = getFirstChild();
	if (retVal.getType() != SEMI) {
	    expr = (mokan.compiler.expr.ExpressionAST)retVal;
	    expr.parseComplete();
	}
    }

    protected Type[] computeExceptions() {
	return (expr != null) ? expr.getExceptionTypes() : noTypes;
    }

    protected StatementAST[] computeControl() {
	return new StatementAST[] { nonlocal };
    }

    public VarList getVarList() {
	return (expr == null) ? new VarList() : expr.getVarList();
    }

    /**
     * Get the expression that is evaluated to produce the return value
     *
     * @return the return expression
     */
    public mokan.compiler.expr.ExpressionAST getReturnValue() {
	return expr;
    }

    public int transType() {
	return expr.transType();
    }
}
/* arch-tag: 75c1a867-bc6e-4f64-9fdc-d5c847c1274c */