/*
 * @(#)SynchronizedAST.java SynchronizedAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import mokan.compiler.Type;
import mokan.compiler.VarList;
import mokan.transact.Transaction;

/**
 * An AST node that represents a synchronized block
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class SynchronizedAST extends StatementAST {

    /**
     * The object to lock
     */
    private mokan.compiler.expr.ExpressionAST lock;

    /**
     * The statement
     */
    private CompoundAST body;

    /**
     * Create a new synchronized statement AST
     *
     * @param token the token represented by this AST node
     */
    public SynchronizedAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	super.parseComplete();
	lock =
	    (mokan.compiler.expr.ExpressionAST)getFirstChild().getNextSibling();
	body = (CompoundAST)lock.getNextSibling().getNextSibling();
	lock.parseComplete();
	body.parseComplete();
    }

    protected Type[] computeExceptions() {
	return Type.mergeTypeLists(lock.getExceptionTypes(),
				   body.getExceptionTypes());
    }

    protected StatementAST[] computeControl() {
	return body.nextControlPoints();
    }

    public VarList getVarList() {
	return new VarList(lock.getVarList(), body.getVarList());
    }

    /**
     * Get the object on which to acquire a lock
     *
     * @return the object to lock
     */
    public mokan.compiler.expr.ExpressionAST getLock() {
	return lock;
    }

    /**
     * Get the body of the <code>synchronized</code> statement
     *
     * @return the body
     */
    public StatementAST getBody() {
	return body;
    }

    public int transType() {
	return (body.transType() == Transaction.NESTED)
	    ? Transaction.NESTED
	    : Transaction.BLOCKING;
    }
}
/* arch-tag: 970eaa3b-3774-469f-818a-d49c028987bf */