/*
 * @(#)ThrowAST.java ThrowAST.java,v 1.1.1.1 2002/11/06 22:10:26 kris Exp $
 *
 * JParse: a freely available Java parser
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Jerry James
 * Email: james@eecs.ukans.edu, james@ittc.ukans.edu, jamesj@acm.org
 * Address: EECS Department
 *          415 Snow Hall
 *          University of Kansas
 *          Lawrence, KS  66045
 */
package mokan.compiler.stmt;

import antlr.Token;
import mokan.compiler.Type;
import mokan.compiler.VarList;

/**
 * An AST node that represents a throw statement
 *
 * @version $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:26 $
 * @author Jerry James
 */
public final class ThrowAST extends StatementAST {

    /**
     * The object to throw
     */
    private mokan.compiler.expr.ExpressionAST throwable;

    /**
     * Create a new throw statement AST
     *
     * @param token the token represented by this AST node
     */
    public ThrowAST(final Token token) {
	super(token);
    }

    public void parseComplete() {
	super.parseComplete();
	throwable = (mokan.compiler.expr.ExpressionAST)getFirstChild();
	throwable.parseComplete();
    }

    protected Type[] computeExceptions() {
	return new Type[] { throwable.retrieveType() };
    }

    protected StatementAST[] computeControl() {
	return new StatementAST[] { nonlocal };
    }

    public VarList getVarList() {
	return throwable.getVarList();
    }

    /**
     * Get the object to throw
     *
     * @return the expression to be evaluated to produce the object to throw
     */
    public mokan.compiler.expr.ExpressionAST getThrowable() {
	return throwable;
    }

    public int transType() {
	return throwable.transType();
    }
}
/* arch-tag: 569d334f-a9d7-464b-91e6-5ce0e85be3a5 */