/*
 * @(#)$Id: Key.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 *
 */

package mokan.example.dbtree;

import java.io.Serializable;

/**
 * A key for a distributed B-Tree.  This implementation uses integers as the
 * keys, but any object that implements the <code>Comparable</code> interface
 * can be used without modifying the rest of the code.
 *
 * @see java.lang.Comparable
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jerry James
 */
public final class Key implements Comparable, Serializable {

    /**
     * The maximum value of any key
     */
    private static final int MAX_KEY = Integer.MAX_VALUE;

    /**
     * The maximally valued key
     */
    public static final Key maxKey = new Key(MAX_KEY);

    /**
     * The value of this key
     */
    final int key;

    public Key() {
	key = MAX_KEY;
    }

    public Key(int k) {
	key = k;
    }

    public Key(Key k) {
	this(k.key);
    }

    public boolean equals(int k) {
	return (key == k);
    }

    public boolean equals(Integer k) {
	return (key == k.intValue());
    }

    public boolean equals(Key k) {
	return equals(k.key);
    }

    public boolean equals(Object obj) {
	return (obj instanceof Key) && equals((Key)obj);
    }

    public int compareTo(int k) {
	// Can't just return key - k because overflow/underflow give wrong sign
	if (key == k)
	    return 0;
	return (key > k) ? 1 : -1;
    }

    public int compareTo(Key k) {
	return compareTo(k.key);
    }

    public int compareTo(Object obj) throws ClassCastException {
	return compareTo((Key)obj);
    }

    public String toString() {
	return (key == MAX_KEY) ? "<MAX>" : String.valueOf(key);
    }
}
