package mokan.nameservice;

import mokan.comm.*;
import mokan.obj.*;
import java.util.*;
import java.io.*;
import mokan.transact.*;

/**
 * When a node is leaving, it sends this message to its neighbors to modify
 * the forwarding links. This message also indicates that the node has left.
 */
public class OwnerMapMsg extends AppMessage implements Externalizable {

  static final long serialVersionUID = 9166876757915541256L;

  /** The node that sends this message, which is going to leave. */
  LogicalNode sender;

  /**
   * The owner map of the sender
   */
  HashMap ownerMap;

  /**
   * The no-args constructor, needed for deserilization
   */
  public OwnerMapMsg() {
    super(true, false);
    ownerMap = null;
    sender = null;
  }

  public OwnerMapMsg(LogicalNode sender, HashMap ownerMap) {
    super(true, true); // reset and/or can block?
    this.sender = sender;
    this.ownerMap = ownerMap;
  }

  protected OwnerMapMsg(LogicalNode sender, HashMap ownerMap,
                           boolean needsReset, boolean canBlock) {
    super(needsReset, canBlock);
    this.sender = sender;
    this.ownerMap = ownerMap;
  }

  protected OwnerMapMsg(LogicalNode sender, HashMap ownerMap,
                           LogicalNode from, boolean needsReset,
                           boolean canBlock) {
    super(from, needsReset, canBlock);
    this.sender = sender;
    this.ownerMap = ownerMap;
  }

  public void handle() {
    super.handle();

    // update the local forwarder map
    Globals.nameService.updateOwnerMap(sender, ownerMap);

    // remove the sender node from the neighbors
    Globals.neighbors.remove(sender);

    //TransactionID.abortTransactions (null, new mokan.app.LeaveException());

    // remove the sender node from the application
    //Globals.app.removeNode(sender);
    KanComm.close(sender.getPhysicalNode());
    LogicalNode.remove (sender);
    Globals.app.getNodes().remove(sender);

  }

  public void writeExternal(final ObjectOutput out) throws IOException {
    super.writeExternal(out);

    //Write sender
    out.writeObject(sender);

    //Write ownerMap
    int size = ownerMap.size();
    out.writeInt(ownerMap.size());
    Iterator it = ownerMap.keySet().iterator();
    while(it.hasNext()) {
      ObjectID oid = (ObjectID) it.next();
      LogicalNode owner = (LogicalNode)ownerMap.get(oid);
      out.writeObject(oid);
      out.writeObject(owner);
    }
  }

  public void readExternal(final ObjectInput in) throws IOException,
      ClassNotFoundException {

    super.readExternal(in);

    //Read sender
    sender = (LogicalNode) in.readObject();

    //Read ownerMap
    int size = in.readInt();
    ownerMap = new HashMap(size);
    for(int i = 0; i < size; i++) {
      ObjectID oid = (ObjectID) in.readObject();
      LogicalNode owner = (LogicalNode) in.readObject();
      ownerMap.put(oid, owner);
    }
  }

}