package mokan.nameservice;

import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.obj.*;
import java.io.*;

public class QueryHomeMsg extends AppMessage implements Externalizable {
  private ObjectID objID;
  private LogicalNode requester;

  static final long serialVersionUID = -5844658485489810243L;
  private static final ObjectStreamField[] serialPersistentFields =
      ObjectStreamClass.NO_FIELDS;

  /**
   * the deserializing constructor.
   */
  public QueryHomeMsg() {
    super(false, false);
  }

  public QueryHomeMsg(ObjectID oid, LogicalNode requester) {
    super(false, false);
    objID = oid;
    this.requester = requester;
  }

  protected QueryHomeMsg(boolean needsReset, boolean canBlock, ObjectID oid, LogicalNode requester) {
    super(needsReset, canBlock);
    objID = oid;
    this.requester = requester;
  }

  public void handle() {
    super.handle();

    // If this node is the home node, i.e., owner==local, then send a reply
    // message to the requester, otherwise forward the message to the owner.

    LogicalNode local = LogicalNode.getMyNode();
    LogicalNode owner = Globals.nameService.getOwner(objID);
    if(owner == null) {
      throw new NameServiceException("Unkown owner of " + objID);
    }
    if(local.equals(owner)) {
      QueryHomeReply reply = new QueryHomeReply(this, owner);
      reply.send(requester);
    }
    else {
      this.send(owner);
    }

  }

  public void writeExternal(final ObjectOutput out) throws IOException {
    super.writeExternal(out);
    out.writeObject(objID);
    out.writeObject(requester);
  }

  public void readExternal(final ObjectInput in) throws IOException,
      ClassNotFoundException {

    super.readExternal(in);
    objID = (ObjectID) in.readObject();
    requester = (LogicalNode) in.readObject();
  }

}