/*
 * @(#)CopyObjMsg.java CopyObjMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.obj;

import java.io.*;
import mokan.comm.LogicalNode;
import mokan.comm.AppMessage;
import mokan.debug.Event;

import mokan.nameservice.*;

/**
 * Request to get a copy of a <code>KanObject</code>
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
final class CopyObjMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -1770429528331958146L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ID</code> of the object to copy.  This is a pseudo-final
     * field.
     *
     * @serial the <code>ID</code> of the object to copy
     */
    ObjectID objID;

    /**
     * The no-args constructor, needed for deserialization
     */
    public CopyObjMsg() {
        super(false, true);
    }

    /**
     * Create a new <code>CopyObjMsg</code>
     *
     * @param objID the <code>ID</code> of the object to copy
     */
    CopyObjMsg(final ObjectID objID) {
        this(objID, null);
    }

    /**
     * Create a new <code>CopyObjMsg</code> from a specific logical node
     *
     * @param objID the <code>ID</code> of the object to copy
     * @param node the node sending this message
     */
    CopyObjMsg(final ObjectID objID, final LogicalNode node) {
        super(node, false, true);
        this.objID = objID;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * This message might block while trying to release the lock, so let
     * another thread take over handling requests
     *
     * @return <code>true</code>, to indicate that processing of this message
     *         might block
     */
    public boolean canBlock() {
        return true;
    }

    /**
     * Process the request to copy the object
     */
    public void handle() {

        super.handle();
        Event.DEBUG(Event.OBJECT_COPY, "Copy Request for "+objID+" from "+from);
        final KanObject kObj = objID.getLocKanObject();
        new CopyObjReply(this, kObj.copy(from)).send(from);
    }

    /**
     * Convert the <code>CopyObjMsg</code> to a string
     *
     * @return the string representation of this <code>CopyObjMsg</code>
     */
    public String toString() {
        return "CopyObjMsg(" + objID + ')';
    }

    /**
     * Save the state of the <code>CopyObjMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             identifier for the object to be copied (ObjectID).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
    }
}
