/*
 * @(#)CopyObjReply.java CopyObjReply.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.obj;

import java.io.*;
import mokan.comm.AppReply;
import mokan.debug.Event;

import mokan.nameservice.*;

/**
 * Reply to a <code>CopyObjMsg</code> with a copy of the requested object
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
final class CopyObjReply extends AppReply implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 6424848737605965955L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /*
     * A copy of the requested object.  This is a pseudo-final field.
     *
     * @serial a copy of the requested object
     */
    private GlobalObjCopy copy;

    /**
     * The <code>ObjectID</code> for the requested object.  This is a
     * pseudo-final field.
     *
     * @serial the <code>ObjectID</code> for the requested object
     */
    private ObjectID objID;

    /**
     * The <code>KanObject</code> created on the receiving side
     */
    transient KanObject kObj;

    /**
     * The no-args constructor, needed for deserialization
     */
    public CopyObjReply() {
        super(true, false);
    }

    /**
     * Create a new <code>CopyObjReply</code>
     *
     * @param msg the message to which this is a reply
     * @param copy a copy of the requested object
     */
    CopyObjReply(final CopyObjMsg msg, final GlobalObjCopy copy) {
        super(msg, true, false);
        this.copy = copy;
        objID = msg.objID;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }


    /**
     * Process the reply containing the object copy
     */
    public void handle() {
        super.replyBegin();

        Event.DEBUG(Event.OBJECT_COPY, "Copy reply for "+objID+" gave "+copy);
        //Pseudo-final handling
        final GlobalObjCopy myCopy = copy;
        if (copy != null) {
            final ObjectMgr toObjManager = to.getLocalHandler().manager;

            kObj = new KanObject(objID, myCopy.gObj, myCopy.protocol,
                                 toObjManager, myCopy.version);
        }
        super.replyEnd();
    }

    /**
     * Convert the <code>CopyObjReply</code> to a string
     *
     * @return the string representation of this <code>CopyObjReply</code>
     */
    public String toString() {
        return "CopyObjReply(" + objID + ')';
    }

    /**
     * Save the state of the <code>CopyObjReply</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, then the copied object
     *             (GlobalObjCopy), followed by the identifier of the copied
     *             object (ObjectID).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(copy);
        out.writeObject(objID);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        copy = (GlobalObjCopy)in.readObject();
        objID = (ObjectID)in.readObject();
    }
}
