/*
 * @(#)CreateObjMsg.java CreateObjMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.obj;

import java.io.*;
import mokan.comm.LogicalNode;
import mokan.comm.AppMessage;
import mokan.debug.Event;
import mokan.nameservice.*;

/**
 * Ask a remote node to create a global object
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
final class CreateObjMsg extends AppMessage implements Externalizable {

  /**
   * The <var>serialVersionUID</var> for this class.  If you change the
   * signature of this class (field names or types, method names, parameter
   * names or types, adding or removing fields or methods, etc.), you
   * <strong>must</strong> change this value.  The recommended way is to use
   * the <code>serialver</code> tool.
   */
  static final long serialVersionUID = 6090160351546710234L;

  /**
   * The list of persistent fields for this class.  This is empty, because
   * that is the fastest option for <code>Externalizable</code> classes when
   * we are not worried about versioning.
   */
  private static final ObjectStreamField[] serialPersistentFields =
      ObjectStreamClass.NO_FIELDS;

  /**
   * The name of the class to instantiate.  This is a pseudo-final field.
   *
   * @serial the name of the class to instantiate
   */
  private String className;

  /**
   * The number of the constructor to call.  These numbers are assigned by
   * the Kan compiler.  This is a pseudo-final field.
   *
   * @serial the number of the constructor to call
   */
  private int consNum;

  /**
   * The arguments to give to the constructor.  This is a pseudo-final
   * field.
   *
   * @serial the arguments to give to the constructor
   */
  private Object[] args;

  /**
   * The no-args constructor, needed for deserialization
   */
  public CreateObjMsg() {
    super(true, true);
  }

  /**
   * Create a new <code>CreateObjMsg</code>
   *
   * @param from the originating <code>LogicalNode</code>
   * @param className the name of the class to instantiate
   * @param consNum the constructor number to call
   * @param args the constructor arguments
   */
  CreateObjMsg(final LogicalNode from, final String className,
               final int consNum, final Object[] args) {
    super(from, true, true);
    this.className = className;
    this.consNum = consNum;
    this.args = args;
    Event.DEBUG(Event.MSG_CREATE, "Created ", this);
  }

  /**
   * Process the request to create a global object
   */
  public void handle() {
    super.handle();
    CreateObjReply reply;
    try {
      final ObjectMgr objMgr = to.getLocalHandler().manager;
      final ObjectID newObjID =
          objMgr.createObject(className, consNum, args);
      reply = new CreateObjReply(this, newObjID);

      // Added by Huahai
      System.out.println("CreateObjMsg: " + className + "," + newObjID + "," +
                         LogicalNode.getMyNode());
      Globals.nameService.addOwner(newObjID, LogicalNode.getMyNode());
    } catch (Exception classEx) {
      Event.DEBUG(Event.MSG_CREATE, "Exception : ", this);
      classEx.printStackTrace();
      reply = new CreateObjReply(this, classEx);
    }
    reply.send(from);
  }

  /**
   * Convert the <code>CreateObjMsg</code> to a string
   *
   * @return a string representation of this <code>CreateObjMsg</code>
   */
  public String toString() {
    return ("CreateObjMsg((" + from + ",to=" + to + "),"
            + className + ',' + consNum + "): " + args);
  }

  /**
   * Save the state of the <code>CreateObjMsg</code> instance to a stream
   * (i.e., serialize it).
   *
   * @serialData The superclass data is written out, then the name of the
   *             class to create a new instance of (String), then the number
   *             of the constructor to invoke (int), followed by the
   *             constructor arguments (Object[]).
   */
  public void writeExternal(final ObjectOutput out) throws IOException {
    super.writeExternal(out);
    out.writeUTF(className);
    out.writeInt(consNum);
    out.writeObject(args);
  }

  public void readExternal(final ObjectInput in) throws IOException,
      ClassNotFoundException {

    super.readExternal(in);
    className = in.readUTF();
    consNum = in.readInt();
    args = (Object[]) in.readObject();
  }
}
