/*
 * @(#)$Id: GlobalObj.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 */

package mokan.obj;

import java.io.Serializable;

/**
 * An interface that must be implemented by all global objects in the Kan
 * system.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jerry James
 */
public interface GlobalObj extends Serializable {

    /**
     * Retrieve the stub for this object
     */
    public ObjectID kanGetObjID();

    /**
     * Make a copy of this object.  This is essentially the same thing as
     * Object.clone(), but that method is protected.  This one is public,
     * thereby defeating a security mechanism that was put into place for our
     * own good.
     *
     * @return a copy of the object
     */
    public GlobalObj kanClone();

    /**
     * Set the fields of this object.  This method is always called from
     * inside of synchronized code, so no thread will ever observe partial
     * results.  The intent is that the object will copy the fields of obj
     * into its own.
     *
     * @param obj an object of the same class whose fields will be used to set 
     *        the fields of this object
     */
    public void kanSetObject(GlobalObj obj);
    // TODO: Make diffs possible, by indicating which fields to copy

    /**
     * Call a numbered method on this object.  This functionality allows us to
     * avoid using costly reflection to call methods, but requires some
     * special support from the Kan compiler.
     *
     * @param methodNum the number of the method to invoke
     * @param args the arguments to the method
     * @return what the called method returns (wrapped, if of primitive type),
     *        or <code>null</code> for <code>void</code> methods.
     * @exception Exception if the called method throws an exception
     */
    public Object kanCallMethod(int methodNum, Object[] args) throws Exception;

    /**
     * Call a numbered guard method on this object.  This functionality allows
     * us to avoid using costly reflection to evaluate guards, but requires
     * some special support from the Kan compiler.
     *
     * @param guardNum the number of the guard to evaluate
     * @param args the arguments to the guard method
     * @return a boolean value, representing the guard evaluation
     */
    public boolean kanCallGuard(int guardNum, Object[] args);
}
