/*
 * @(#)GlobalObjCopy.java GlobalObjCopy.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 */

package mokan.obj;

import java.io.*;

/**
 * A copy of a global object, together with its version number, in a form
 * suitable for transmission over the network
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 */
public final class GlobalObjCopy implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 6967271977134494919L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * A copy of a global object.  This is a pseudo-final field.
     *
     * @serial a copy of a global object
     */
    GlobalObj gObj;

    /**
     * The version number of this object.  This is a pseudo-final field.
     *
     * @serial the version number of this object
     */
    long version;

    /**
     *  Protocol expected by home node
     */
    int protocol;

    /**
     * The no-args constructor, needed for deserialization
     */
    public GlobalObjCopy() { }

    /**
     * Create a copy of a global object
     *
     * @param gObj the copy of the global object
     * @param version the version number of this copy
     */
    public GlobalObjCopy(final GlobalObj gObj, final long version, int prot) {
        this.gObj    = gObj;
        this.version = version;
        this.protocol = prot;
    }

    /**
     * Convert the <code>GlobalObjCopy</code> to a string
     *
     * @return a string representation of this <code>GlobalObjCopy</code>
     */
    public String toString() {
        return "GlobalObjCopy(" + version + ')';
    }

    /**
     * Save the state of the <code>GlobalObjCopy</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The copied user object is written out (GlobalObj), followed
     *             by its version number (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        out.writeObject(gObj);
        out.writeLong(version);
        out.writeInt(protocol);
    }

    public void readExternal(final ObjectInput in)
        throws ClassNotFoundException, IOException {

        gObj = (GlobalObj)in.readObject();
        version = in.readLong();
        protocol = in.readInt();
    }
}
