/*
 * @(#)GuardCondition.java GuardCondition.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.obj;

import java.io.*;
import java.lang.reflect.Method;

/**
 * A representation of a guard (a predicate over the state of an object)
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jing Wang
 * @author	Jerry James
 */
public final class GuardCondition implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 6686601412282981482L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * A number that uniquely identifes <var>meth</var> in its declaring
     * class.  This is a pseudo-final field.
     */
    private int methNum;

    /**
     * The arguments to pass to the guard expression (representing local
     * thread state).  This is a pseudo-final field.
     *
     * @serial the arguments to pass to the guard expression
     */
    private Object[] args;

    /**
     * The no-args constructor, needed for deserialization
     */
    public GuardCondition() { }

    /**
     * Create a new <code>GuardCondition</code>
     *
     * @param methNum a unique number for <var>meth</var> in its declaring
     *        class
     * @param args the arguments to pass to the guard expression
     */
    public GuardCondition(final int methNum, final Object[] args) {
	this.methNum = methNum;
	this.args = args;
    }

    /**
     * Determine whether the guard condition is satisfied
     *
     * @param obj the object on which to test the guard
     * @return <code>true</code> if the guard is satisfied on <var>obj</var>;
     *         <code>false</code> otherwise
     */
    public boolean isSatisfied(final GlobalObj obj) {
	return obj.kanCallGuard(methNum, args);
    }

    /**
     * Produce a string representation of this <code>GuardCondition</code>
     *
     * @return a string representation of this <code>GuardCondition</code>
     */
    public String toString() {
	final StringBuffer buf = new StringBuffer();
	buf.append("kanGuard");
	buf.append(methNum);
	buf.append('(');
	if (args != null && args.length > 0) {
	    buf.append(args[0]);
	    for (int i = 1; i < args.length; i++) {
		buf.append(',');
		buf.append(args[i]);
	    }
	}
	buf.append(')');
	return buf.toString();
    }

    /**
     * Save the state of the <code>GuardCondition</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The guard number for its containing object is written out
     *             (int), followed by the arguments to the method (Object[]).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	out.writeInt(methNum);
	out.writeObject(args);
    }

    public void readExternal(final ObjectInput in)
	throws ClassNotFoundException, IOException {

	methNum = in.readInt();
	args = (Object[])in.readObject();
    }
}
