/*
 * @(#)ObjectMgr.java ObjectMgr.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 *
 */

package mokan.obj;

import java.lang.reflect.Method;
import mokan.comm.LogicalNode;
import mokan.transact.TransactionUserException;
import mokan.nameservice.*;

/**
 * The object manager is responsible for tracking the owners and replicas of
 * distributed objects.  Its primary purpose is to provide a way of invoking
 * methods on distributed objects, regardless of the current owner and replica
 * locations.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Suk Y. Lee
 * @author	Jerry James
 */
public final class ObjectMgr {

    /**
     * The signature of the constructor caller in <code>ObjectID</code>
     */
    private static final Class[] makeArgs =
	{ int.class, LogicalNode.class, Object[].class };

    /**
     * The logical node for which this manages objects
     */
    final LogicalNode parentNode;

    /**
     * The wrapped index number of this logical node
     */
//    private final Integer parentIndex;

    /**
     *   Allowing remote objects to be instated here
     */
    public boolean allowCreateObjects = true;

    /**
     * Create a new <code>ObjectMgr</code>
     *
     * @param parent the node for which this manager manages objects
     */
    public ObjectMgr(final LogicalNode parent) {
        parentNode = parent;
//	parentIndex = new Integer(parent.getIndex());
    }

    /**
     * Create a new global object from a request by another node
     *
     * @param className the name of the class of object to create
     * @param consNum the constructor number to call
     * @param args constructor arguments
     * @exception Exception if the class file is missing or the Kan stubs need
     *        to be regenerated
     */
    ObjectID createObject(final String className, final int consNum,
                          final Object[] args) throws Exception {


        if (!allowCreateObjects) throw new mokan.app.LeaveException();

        try {
            final Class objClass = Class.forName(className + "$KanID");
            final Object[] realArgs =
                {  new Integer(consNum), parentNode, args };
            //final String name = className.replace('.', '$');
            final Method meth =
                objClass.getDeclaredMethod("kanCreate", makeArgs);
            return (ObjectID)meth.invoke(null, realArgs);
        } catch (Exception ex) {
            System.err.println(
                "I could not find a constructor or a class on a remote node!\n"
                +"This indicates that the required class is missing on the remote node\n"
                +"or that you need to regenerate the Kan stub objects for your program.");
            ex.printStackTrace();
            throw ex;
        }
    }

    /**
     * Create a new global object on another logical node
     *
     * @param objID the stub object for the new global object
     * @param objClass the class of the object to create
     * @param consNum the constructor number to call
     * @param args constructor arguments
     */
    public void createRmtObject(final ObjectID objID, final Class objClass,
                                final int consNum, final Object[] args) {
        // Send a request to the home-node-to-be
        final CreateObjMsg msg =
            new CreateObjMsg(parentNode, objClass.getName(), consNum, args);
        final CreateObjReply reply =
            (CreateObjReply)msg.sendAndWait(objID.getNode());
        if (reply.ex != null) {
            throw new TransactionUserException(reply.ex);
        } else {
            objID.update(reply.objID);

            //Added by Huahai
            Globals.nameService.putOwner(objID, objID.getNode());
        }
    }


    public void createLocObject(final ObjectID objID, final Class objClass,
                                final int consNum, final Object[] args) {
        throw new RuntimeException("Obsolete method");
    }


}
