/*
 * @(#)ReplicaDropMsg.java ReplicaDropMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.obj;

import java.io.*;
import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.debug.Event;

/**
 * Tell the home node that I'm getting rid of a replica
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 */
public final class ReplicaDropMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -9067916762535678909L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ID</code> of the object whose replica is being dropped.  This
     * is a pseudo-final field.
     *
     * @serial the <code>ID</code> of the object whose replica is being
     *         dropped
     */
    private ObjectID objID;

    /**
     * The no-args constructor, needed for deserialization
     */
    public ReplicaDropMsg() {
        super(false, false);
    }

    /**
     * Create a new replica drop notification message
     *
     * @param objID the <code>ID</code> of the object whose replica is being
     *        dropped
     */
    public ReplicaDropMsg(final ObjectID objID) {
        super(null, false, false);
        this.objID = objID;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * Handle this message by removing the sender from the list of replica
     * holders for the indicated object
     */
    public void handle() {
        super.handle();
        objID.getLocKanObject().protocol.dropReplica(from);
    }

    /**
     * Convert the <code>ReplicaDropMsg</code> to a string
     *
     * @return a string representation of this <code>ReplicaDropMsg</code>
     */
    public String toString() {
        return "ReplicaDropMsg[" + objID + ']';
    }

    /**
     * Save the state of the <code>ReplicaDropMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             identifier for the object whose replica is being deleted
     *             (ObjectID).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
    }
}
