/*
 * @(#)ReplicaMsg.java ReplicaMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 */

package mokan.obj;

import java.io.*;
import mokan.comm.AppMessage;
import mokan.debug.Event;

import mokan.nameservice.*;

/**
 * Create a replica on a remote node
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
final public class ReplicaMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 6834695984374190737L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ObjectID</code> for the copied object.  This is a
     * pseudo-final field.
     *
     * @serial the <code>ObjectID</code> for the copied object
     */
    private ObjectID objID;

    /**
     * The object itself.  This is a pseudo-final field.
     *
     * @serial the object itself
     */
    private GlobalObj objCopy;

    /**
     * The version number of this object.  This is a pseudo-final field.
     *
     * @serial the version number of this object
     */
    private long version;

    /** The local protocol number that we are using
     */
    private int protocol;

    /**
     * The no-args constructor, needed for deserialization
     */
    public ReplicaMsg() {
        super(true, false);
    }

    /**
     * Create a new <code>ReplicaMsg</code>
     *
     * @param objID the <code>ObjectID</code> of the object to replicate
     * @param objCopy a copy of the object
     * @param version the version number of this object
     */
    public ReplicaMsg(final ObjectID objID,
               final GlobalObj objCopy,
               final long version,
               final int  protocol) {
        super(objID.getNode(), true, false);
        this.objID = objID;
        this.objCopy = objCopy;
        this.version = version;
        this.protocol = protocol;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    public void handle() {

        super.handle();
        final ObjectMgr toObjMgr = to.getLocalHandler().manager;
        new KanObject(objID, objCopy, protocol, toObjMgr, version);
        Event.DEBUG(Event.OBJECT_REPLICATE, "Replica created on " + to +
                    " for ", objID);
    }

    /**
     * Convert the <code>ReplicaMsg</code> to a string
     *
     * @return a string representation of this <code>ReplicaMsg</code>
     */
    public String toString() {
        return "ReplicaMsg(" + objID + "): " + version;
    }

    /**
     * Save the state of the <code>ReplicaMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, then the ID of the
     *             object being replicated (ObjectID), the object itself
     *             (GlobalObj), followed by the version number of the object
     *             (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
        out.writeObject(objCopy);
        out.writeLong(version);
        out.writeInt(protocol);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
        objCopy = (GlobalObj)in.readObject();
        version = in.readLong();
        protocol = in.readInt();
    }
}
