/**
 * ReplicaRequestMsg
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.obj;


import java.io.*;
import mokan.comm.AppMessage;
import mokan.debug.Event;

import mokan.nameservice.*;



/**
 *  The <code>ReplicaRequestMsg</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
final class ReplicaRequestMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 0L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ObjectID</code> for the copied object.  This is a
     * pseudo-final field.
     *
     * @serial the <code>ObjectID</code> for the copied object
     */
    private ObjectID objID;

    /**
     * The no-args constructor, needed for deserialization
     */
    public ReplicaRequestMsg() {
        super(true, false);
    }

    /**
     * Create a new <code>ReplicaMsg</code>
     *
     * @param objID the <code>ObjectID</code> of the object to replicate
     * @param objCopy a copy of the object
     * @param version the version number of this object
     */
    ReplicaRequestMsg(final ObjectID objID) {
        super(objID.getNode(), true, false);
        this.objID = objID;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }


    /**   Send the requesting node a replica */
    public void handle() {

        super.handle();
        //LogicalNode requester = from;
        KanObject kanObj = objID.getLocKanObject();
        Event.DEBUG(Event.OBJECT_REPLICATE, "Replica request for ", objID);

        kanObj.protocol.addReplica (from);
        new ReplicaRespMsg(this, objID, kanObj).send(from);

    }

    /**
     * Convert the <code>ReplicaMsg</code> to a string
     *
     * @return a string representation of this <code>ReplicaMsg</code>
     */
    public String toString() {
        return "ReplicaRequestMsg(" + objID + "): ";
    }

    /**
     * Save the state of the <code>ReplicaMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, then the ID of the
     *             object being replicated (ObjectID), the object itself
     *             (GlobalObj), followed by the version number of the object
     *             (long).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
    }
}
