/**
 * ReplicaRespMsg
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.obj;

import java.io.Externalizable;
import java.io.ObjectStreamField;
import java.io.ObjectStreamClass;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

import mokan.comm.LogicalNode;
import mokan.comm.AppMessage;
import mokan.comm.AppReply;
import mokan.debug.Event;

import mokan.nameservice.*;


/**
 *  The <code>ReplicaRespMsg</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class ReplicaRespMsg extends AppReply implements Externalizable {


    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 1795414288035014435L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ObjectID</code> for the copied object.  This is a
     * pseudo-final field.
     *
     * @serial the <code>ObjectID</code> for the copied object
     */
    private ObjectID objID;

    /**
     * A copy of a global object.  This is a pseudo-final field.
     *
     * @serial a copy of a global object
     */
    GlobalObj gObj;

    /**
     * The version number of this object.  This is a pseudo-final field.
     *
     * @serial the version number of this object
     */
    long version;


    /**
     * The <code>KanObject</code> created on the receiving side
     */
    transient KanObject kObj;


    public ReplicaRespMsg(final AppMessage orig, ObjectID oid, KanObject kobj){
        super(orig, false, false);
        objID = oid;
        gObj = kobj.obj;
        version = kobj.version;
    }


    /**
     * Process the reply containing the object copy
     */
    public void handle() {
        super.handle();
    }


    /**
     * Save the state of the <code>YesNoReply</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the answer
     *             contained in this message (boolean).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
        out.writeObject(gObj);
        out.writeLong(version);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
        gObj = (GlobalObj)in.readObject();
        version = in.readLong();
    }

}
