/*
 * tag: kkvilekval@acm.org Sat Nov 24 22:07:37 2001 
 */

package mokan.prefetch.analysis;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

import java.util.LinkedList;
import java.util.Arrays;
import java.util.Date;

import klogger.Logger;
import mokan.prefetch.util.RTClass;

import mokan.prefetch.analysis.classanalysis.ClassAnalysis;
import mokan.prefetch.analysis.classanalysis.NaiveClassAnalysis;
import mokan.prefetch.analysis.classanalysis.RTAClassAnalysis;

import mokan.prefetch.analysis.shapeanalysis.Analysis;
import mokan.prefetch.analysis.shapeanalysis.ShapeAnalysis;
import mokan.prefetch.analysis.shapeanalysis.CallAnalysis;

import mokan.prefetch.analysis.instrumentor.AnnotateAccesses;


/**
 *  The <code>Precache</code> description.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */

public class Main {

    /* Private/Public Variables */

    //public static boolean allowSystemClasses = false;
    //public static boolean allowAllClasses = false;
    public static String pathPrefix = ",,app-annotated/";
    public static String annotationPrefix = "app-annotated/";
    

    /**
     *  The default constructor
     */
    public Main () {
    }

    public static void main(String[] args) {

        String apppaths = ".";  // Don't load any extra classes
        String classpath = ".";
        boolean externalgraphs = false;
        LinkedList argv = new LinkedList(Arrays.asList(args));

        while (!argv.isEmpty()) {
            String arg  = (String)argv.removeFirst();
            if (arg.equals ("--analysis-safe")) {
                RTClass.setSystemClassLevel (RTClass.JAVAX);
                pathPrefix = ",,safe-annotated/";
            } else 
            if (arg.equals ("--analysis-full")) {
                RTClass.setSystemClassLevel (RTClass.NONE);
                pathPrefix = ",,full-annotated/";
            } else 
            if (arg.equals ("--external-graphs")) {
                externalgraphs = true;

            } else 
            if (arg.equals("--cp")) {
                // Top level directory where application classes are
                classpath = (String)argv.removeFirst();
            } else 
            if (arg.equals("--load")) {
                // Application classes
                apppaths = (String)argv.removeFirst();
            } else 
            if (arg.startsWith("-")) {
                System.out.println ("--analysis-{safe,full}, --cp <dir> --load <dir>, --external-graphs");
                System.exit(0);
            } else {
                argv.addFirst(arg);
                break;
            }
        }
        String mainclassname = (String)argv.removeFirst();
        annotationPrefix = pathPrefix+mainclassname+'/';
        System.setProperty ("MAIN", mainclassname);
        System.setProperty ("TIME", new Date().toString().replace(' ', '-'));

        /** Configure the logging classes based on the file */
        Logger.configure ("analysis.properties");

        ClassAnalysis cla;
        //cla = new NaiveClassAnalysis (classpath, apppaths, mainclassname);
        cla = new  RTAClassAnalysis (classpath, apppaths, mainclassname);

        Analysis aa = new ShapeAnalysis (cla, mainclassname);
        aa.visitProgram();

        CallAnalysis ca = new CallAnalysis (cla, aa.getMethods());
        ca.visitProgram();


        AnnotateAccesses an = new AnnotateAccesses(cla, aa.getMethods(),
                                                   externalgraphs);
        an.annotateProgram();

        if (externalgraphs) {
            String gfname  = pathPrefix + mainclassname+".sg";
            try {
            // Shape graph.
            FileOutputStream ostream = new FileOutputStream(gfname);
            ObjectOutputStream p = new ObjectOutputStream(ostream);
            p.writeObject (aa.getMethods());
            p.flush();
            ostream.close();

            // Class(type) graph
            gfname  = pathPrefix + mainclassname+".tg";
            ostream = new FileOutputStream(gfname);
            p = new ObjectOutputStream(ostream);
            p.writeObject (cla.getStaticClassGraph());
            p.flush();
            ostream.close();
            
            // call graph
            gfname  = pathPrefix + mainclassname+".cg";
            ostream = new FileOutputStream(gfname);
            p = new ObjectOutputStream(ostream);
            p.writeObject (cla.getStaticCallGraph());
            p.flush();
            ostream.close();


            } catch (java.io.IOException io) {
                System.err.println ("Couldn't save graphs in " + gfname);
                io.printStackTrace();
            }
        }
    }

};
