/**
 * Callsite
 * tag: kkvilekal@acm.org 2002/08/26 23:48:24
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.analysis.classanalysis;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;


import de.fub.bytecode.Constants;
import de.fub.bytecode.generic.InstructionHandle;
import de.fub.bytecode.generic.InvokeInstruction;

import mokan.prefetch.util.RTMethod;

/**
 *  The <code>Callsite</code> description.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
class Callsite {

    final static int VIRTUAL   = 0x01; // InvokeInterface or Virtual
    final static int INTERFACE = 0x02; // InvokeInterface
    final static int SYSTEM    = 0x04; // System Targets

    /* Private/Public Variables */
    RTMethod caller;            // Back pointer to method
    int instr;                  // position in instruction list
    String target;              // message signature.
    String handleclass;         // handle type 
//     boolean virtualcall;        // Multiple targets (virtual or interface)
//     boolean interfacecall;      // call is an interface.
    int flags;                  // 
    RTMethod[] callees;         // callee found.
    /**
     *  The default constructor
     */
    Callsite (RTMethod m, int i, String h, String t, boolean v) {
        caller = m;
        instr=i; target = t;handleclass=h;
        flags = 0;

        setFlag (VIRTUAL, v);
    }

    private void setFlag (int f, boolean x) {
        flags = x? (flags | f) : (flags & ~f);
    }
    public final boolean isVirtual() {
        return (flags & VIRTUAL) != 0;
    }
    public final boolean isInterface() {
        return (flags & INTERFACE) != 0;
    }
    public final boolean isSystemCall () {
        return (flags & SYSTEM) != 0;
    }
    public final void setVirtual (boolean x){
        setFlag (VIRTUAL, x);
    }
    public final void setInterface (boolean x){
        setFlag (INTERFACE, x);
    }
    public final void setSystemCall (boolean x) {
        setFlag (SYSTEM, x);
    }

    /** Find and return the call sites for the given method.
     *  The method is searched and any invoke instruction
     *  is stored as a call site 
     *  NOTE: is new a callsite ?
     */
//     static Callsite[] getCallSites (final RTMethod m) {
//         ArrayList calls = new ArrayList ();
//         InstructionHandle[] ihs = m.getInstructions();

//         for (int ii=0; ii < ihs.length; ii++) {
//             if (ihs[ii].getInstruction() instanceof InvokeInstruction) {
//                 InvokeInstruction instr = (InvokeInstruction)ihs[ii].getInstruction();
//                 String target = (instr.getMethodName(m.getConstantPoolGen())
//                                  +instr.getSignature(m.getConstantPoolGen()));
//                 // Record that Method m actually call the some function.
// 		String handleclass = instr.getClassName (m.getConstantPoolGen());
//                 //log.info ("Found call " + handleclass + "->" + target);
// //              String signature = instr.getSignature(m.getConstantPoolGen());
//                 boolean vcall = !(instr.getTag()== Constants.INVOKESPECIAL
//                                   || instr.getTag()==Constants.INVOKESTATIC );
//                 Callsite cs =  new Callsite (m, ii, handleclass, target,vcall);
//                 calls.add (cs);
//             }
//         }
//         return  (Callsite[])calls.toArray(new Callsite[calls.size()]);
//     }


    public int compareTo (Object o) {
        return (instr - ((Callsite)o).instr);
    }

    /**
     *  @return a string representation of this <code>Callsite</code> 
     */
    public String toString () {
        return handleclass + '.'+target;
    }
}
