/**
 * ClassAnalysis
 * tag: kkvilekal@acm.org 2002/08/15 20:59:41
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.analysis.classanalysis;

import java.util.Collection;
import java.util.List;
import java.util.Set;
import mokan.prefetch.util.RTMethod;
import mokan.prefetch.util.RTClass;

import agl2.Bigraph;

/**
 *  The <code>ClassAnalysis</code> is an interface to 
 *  the general 
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public interface ClassAnalysis {

    /** Return the class structure given a full class name */
    RTClass getClass (String name);

    /** Get all classes  
    /** @return Set<RTClass> */
    public Collection getLoadedClasses ();

    /** Detemine if a class has a descendant class */
    public boolean isAncestorClass(RTClass ancestor, RTClass descendant);

    /** Find the minimum target set based on the context and 
     *  base object handle.
     *  @param context  The context of the call 
     *  @param 
     *  @return Set<RTMethod> 
    */
    RTMethod[] getTargetSet (RTMethod m, int instr);

    /** the current classgraph 
     *  @return Digraph<RTClass> 
     */
    Bigraph getStaticClassGraph ();

    /** the current callgrpah 
     * @return Digrpah<RTMethod>
     */
    Bigraph getStaticCallGraph ();


//    public RTMethod[] getInvokeTarget (RTMethod m, int instr);
//    public boolean    isAncestorClass (RTClass base, RTClass derived);
}
