/**
 * ClassSet
 * tag: kkvilekal@acm.org 2002/08/26 19:01:05
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.analysis.classanalysis;

import java.io.File;
import java.io.FilenameFilter;

import java.util.Enumeration;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.StringTokenizer;

import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;


import de.fub.bytecode.Constants;
import de.fub.bytecode.classfile.JavaClass;
import de.fub.bytecode.classfile.ConstantPool;
import de.fub.bytecode.classfile.Constant;
import de.fub.bytecode.classfile.ConstantUtf8;
import de.fub.bytecode.classfile.ConstantClass;

import de.fub.bytecode.ClassPath;

import mokan.prefetch.util.Assert;
import mokan.prefetch.util.RTClass;
import mokan.prefetch.util.RTMethod;
import mokan.prefetch.util.WorkSet;

import klogger.Logger;
/**
 *  The <code>ClassSet</code> create a set of class based on 
 *  reachable classes. 
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class ClassSet extends AbstractMap {
    final static Logger log = 
      Logger.getLogger("prefetch.classanalysis.ClassSet");

    int count = 0;

    /**
     * A map for all Classes, the ones we're going to package.  Store
     * class name against the JavaClass. From the JavaClass we get the
     * bytes to create the jar.
     */
    TreeMap allClasses = new TreeMap();

    /**
     * We start at the root classes, put them in here, then go through
     * this list, putting dependent classes in here and from there
     * into allClasses. Store class names against class names of their
     * dependents
     */
    TreeMap dependents = new TreeMap();

    /** Special instance of java.lang.Object */
    private RTClass classObject ;

    /** The entire classpath to be used (serves as marker). */
    private String  classloadpath;
    /** Application classes are taken from this directory in classpath */
    private String  apploadpath;



    /**
     * add given class to dependents (from is where its dependent from)
     * some fiddeling to be done because of array class notation
     */  
    private void addClassString(String clas,String from){

        // must check if it's an arrary (start with "[")
        if(clas.startsWith("[")) {
            if(clas.length() == 2 ) {
                // it's an array of built in type, ignore
                return;
            }
            if( 'L' == clas.charAt(1) ){
                // it's an array of objects, the class name is between [L and ;
                // like    [Ljava/lang/Object;
                addClassString(clas.substring(2,clas.length()-1),from);
                return;
            }
            if( '[' == clas.charAt(1) ){
                // it's an array of arrays, call recursive
                addClassString(clas.substring(1),from);
                return ;
            }
            //throw new IOException("Can't recognize class name =" + clas);
        }

//        if( !RTClass.isSystemClass (clas) && allClasses.get(clas) == null) {
        clas = clas.replace ('/', '.');

        log.trace ("processing: " + clas +" referenced by " + from);
        if( allClasses.get(clas) == null) {
            dependents.put(clas,from);
            //      System.out.println("       yes" );
        } else {
            //      System.out.println("       no" );
        }
    }

    


    /** 
     *Add this class to allClasses. Then go through all its dependents
     * and add them to the dependents list if they are not in allClasses
     */
    private void addDependents( RTClass clz ) {
        JavaClass clazz = clz.getJavaClass();
        //String name = clazz.getClassName().replace('.', '/');
        String name = clazz.getClassName();
        allClasses.put( name , clz );
        ConstantPool pool = clazz.getConstantPool();
        for( int i = 1 ; i < pool.getLength() ; i++){
            Constant cons =  pool.getConstant(i);
            //System.out.println("("+i+") " + cons );
            if( cons!=null && cons.getTag() == Constants.CONSTANT_Class ){
                int idx = ((ConstantClass)pool.getConstant(i)).getNameIndex();
                String clas = ((ConstantUtf8)pool.getConstant(idx)).getBytes();
                addClassString(clas,name);
            }
        }
    }


    private String resolveClassname (String clas) {
        // must check if it's an arrary (start with "[")
        if(clas.startsWith("[")) {
            if(clas.length() == 2 ) {
                // it's an array of built in type, ignore
                return null;
            }
            if( 'L' == clas.charAt(1) ){
                // it's an array of objects, the class name is between [L and ;
                // like    [Ljava/lang/Object;
                return resolveClassname(clas.substring(2,clas.length()-1));
            }
            if( '[' == clas.charAt(1) ){
                // it's an array of arrays, call recursive
                return resolveClassname(clas.substring(1));
            }
            //throw new IOException("Can't recognize class name =" + clas);
        }
        return clas.replace ('/', '.');
    }

    public String[] getDependents (RTClass clz) {
        ArrayList names = new ArrayList();
        
        JavaClass clazz = clz.getJavaClass();
        //String name = clazz.getClassName().replace('.', '/');
        String name = clazz.getClassName();
        allClasses.put( name , clz );
        ConstantPool pool = clazz.getConstantPool();
        for( int i = 1 ; i < pool.getLength() ; i++){
            Constant cons =  pool.getConstant(i);
            //System.out.println("("+i+") " + cons );
            if( cons!=null && cons.getTag() == Constants.CONSTANT_Class ){
                int idx = ((ConstantClass)pool.getConstant(i)).getNameIndex();
                String clas = ((ConstantUtf8)pool.getConstant(idx)).getBytes();
                clas = resolveClassname (clas);
                if (clas != null)
                    names.add (clas);
            }
        }
        return (String[])names.toArray(new String[names.size()]);
    }


    private void loadClassdir (File classdir) {
        File[] classfiles = classdir.listFiles (new FilenameFilter () {
                public boolean accept(File dir, String filename) {
                    return filename.endsWith(".class");
                }
            });

        for (int f=0; f < classfiles.length; f++) {
            //if (!RTClass.isSystemClass (classfiles[f]){
                RTClass clz =new RTClass(classfiles[f]);
                log.trace ("preloading class : " + clz.getClassName());
                addDependents (clz);
            //}
        }
    }
    private void loadJar (File jarfile) {
        log.warn ("ClassSet", "loadJar", "Jar files are not loaded (yet)");
    }
    
    /** Load the classes in path .. null will load all classes in 
     *  system CLASSPATH 
    */
    private void loadClasspath (String path) { 
        String class_path = path;
        if (path == null) class_path = System.getProperty("java.class.path");

        log.info ("loading all classes in " + path);
        StringTokenizer tok = new StringTokenizer(class_path, File.pathSeparator);
        while (tok.hasMoreTokens()) {
            File  pathentry = new File(tok.nextToken());

            if (pathentry . isDirectory() ) {
                loadClassdir (pathentry);
            } else if (pathentry.getName().endsWith (".jar")) {
                loadJar (pathentry);
            }
        }
    }
 
    private int pumpClasses (Collection added) {
        RTClass clz;
        count = 0;
        while(!dependents.isEmpty() ){
            String name = (String)dependents.firstKey();
            String from = (String) dependents.remove(name);
            if(allClasses.get(name) == null){
                clz = new RTClass (name);
                if (added != null) added.add (clz);
                addDependents(clz);
                count++;
            }
        }
        return count;
    }
    /** Default constructor */
    public ClassSet () {
        this (null, null);
    }

    /** Initialize the ClassSet
     *  @param classpath a classpath string where class files are found 
     *  @param apppath All class files from appath are loaded
     */
    public ClassSet (String classpath, String apppath) {

        if (classpath == null) 
            classpath = System.getProperty("java.class.path");

        this.classloadpath = classpath;
        this.apploadpath = apppath;

        //loadClasspath (classpath);

        //count = pumpClasses (null);
        log.info ("ClassSet added " + count);
    }
    /** Load a class and all classes refered to by it. 
     */
//     public int loadClass (String clname, Collection added) {
//         if (allClasses.get (clname) == null) {
//             RTClass clz = new RTClass (clname);
//             addDependents (clz);
//             return pumpClasses(added);
//         }
//         return 0;
//     }

    /** Return a the class or null if it has yet been loaded */
    public RTClass findClass (String clname) {
        return (RTClass)allClasses.get (clname);
    }
    /** Load the class and only the class */
    public RTClass loadClass (String name) {
        RTClass cl = (RTClass)allClasses.get(name);
        if (cl == null) {
            cl = new RTClass (name);
            allClasses.put (name, cl);
        }
        return cl;
    }


//     public RTClass getClass (String cl) {
//         return (RTClass)allClasses.get (cl);
//     }
//     public boolean hasClass (String cl) {
//         return allClasses.containsKey (cl);
//     }

    public RTClass getObjectClass () {
        if (classObject == null) {
            classObject = loadClass ("java.lang.Object");
        }
        return classObject;
    }

     public Set entrySet () {
         return allClasses.entrySet();
     }
     public Set keySet () {
         return allClasses.keySet();
     }
     public Collection values () {
         return allClasses.values();
     }

    private void listClassDir (String top, File dir, ArrayList names) 
        throws java.io.IOException {
        
        File[] entries = dir.listFiles ();
        String classname="";
        for (int fi=0; fi < entries.length; fi++) {
            if (entries[fi].isDirectory())
                listClassDir (top, entries[fi], names);
            else if (entries[fi].getName().endsWith(".class")){
                classname = entries[fi].getCanonicalPath();

                // Strip off ".class"
                classname = classname.substring(0, classname.length()-6);
                // Strip away everything before the real class
                // Remove the ending .class
                int spot = classname.indexOf (top);
                //System.out.println (classname);
                //System.out.println (top);
                if (spot >= 0 ) {
                    spot += top.length();
                    classname = classname.substring (spot+1);
                    classname = classname.replace('/', '.');
                    names.add (classname);
                } else {
                    log.warn ("Couldn't find classname of " + 
                              classname+ " using " + top);
                }
            }
        }
    }

    private void listJar (ZipFile jar, ArrayList names){
        Enumeration e = jar.entries();
        while (e.hasMoreElements()) {
            ZipEntry ze = (ZipEntry)e.nextElement();

            if (ze.isDirectory()) continue;
            
            String classname = ze.getName();
            classname = classname.substring (0, classname.length() - 6);
            classname = classname.replace ('/','.');

            names.add (classname);

        }
    }

//    public String[] listClasses () {
    public ArrayList listClasses () {

        ArrayList classnames = new ArrayList();

        try {
            String cp = new File (classloadpath).getCanonicalPath();

            log.message ("searching for classes in " + apploadpath);
            StringTokenizer tok = new StringTokenizer(apploadpath, 
                                                      File.pathSeparator);
            while (tok.hasMoreTokens()) {
                File  pathentry = new File(tok.nextToken());

                log.message (pathentry.toString());
                if (pathentry . isDirectory() ) {
                    listClassDir (cp, pathentry, classnames);
                } else {
                    listJar (new ZipFile(pathentry), classnames);
                }
            }
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }

        //return (String[])classnames.toArray (new String[classnames.size()]);
        return classnames;
    }

}
