/**
 * NaiveClassAnalysis
 * tag: kkvilekal@acm.org 2002/08/25 22:21:20
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.analysis.classanalysis;

import java.util.Map;
import java.util.Set;
import java.util.HashMap;
import java.util.Arrays;
import java.util.ArrayList;

import mokan.prefetch.util.WorkSet;
import mokan.prefetch.util.RTClass;
import mokan.prefetch.util.RTMethod;

import agl2.Bigraph;
import agl2.alg.DepthFirstSearch;
import agl2.alg.AbstractGraphVisitor;

import klogger.Logger;

/**
 *  The <code>NaiveClassAnalysis</code> loads all available classes.
 *  It uses a form of Class Hierarchy analysis to determine 
 *  a call graph.  It loads all depandent classes based on scanning
 *  of the reachable classfiles.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class NaiveClassAnalysis  extends AbstractClassAnalysis {



    /** Find all real method targets for this callsite. */
    RTMethod[] getVirtualTargets (final Callsite cs, 
                                  final String handle, 
                                  final String target) {
        final ArrayList methods = new ArrayList();

        DepthFirstSearch dfs = new DepthFirstSearch (classgb.getBigraph());
        log.debug ("dfs looking for " + handle + '.'+ target);
        AbstractGraphVisitor add_matching_method = new AbstractGraphVisitor () {
                public void finishVertex (Object v) {
                    RTClass clz = (RTClass)v;
                    //log.debug ("visit " + clz);
                    if (clz.isInterface()) return;
                    if (clz.isSystemClass()) return;
                    RTMethod m = findMethod (clz, target);
                    if (m == null) return;
                    if (m.isAbstract()) return;
                    if (m.isNative()) {
                        cs.setSystemCall(true);
                        return;
                    }
                    
                    if (!methods.contains (m)) methods.add (m);
                }
            } ;

        RTClass rt = classes.findClass (handle);
        dfs.apply (rt, add_matching_method);
        return (RTMethod[])methods.toArray(new RTMethod[methods.size()]);
    }

    boolean visitMethod (RTMethod method) {

        CallsiteVisitor cv = new CallsiteVisitor (method);
        method.visitInstructions (cv);
        method2callsites.put (method, cv.getCallsites());
        return true;
    }

    
    public NaiveClassAnalysis (String classpath, String apppath, String mainclassname){
        log.entering ("NaiveClassAnalysis", "NaiveClassAnalysis");

         buildGraphs(classpath, apppath, mainclassname);
//         buildClassSet(classpath, apppath, mainclassname);
//         buildClassgraph(mainclassname);
//         buildCallgraph(mainclassname);

        log.message ("classgraph nodes:" + classgraph.getVertices().size() +
                  " edges:" + classgraph.getEdges().size());

        log.message ("callgraph nodes:" + callgraph.getVertices().size() +
                  " edges:" + callgraph.getEdges().size());


        log.exiting ("NaiveClassAnalysis", "NaiveClassAnalysis" );
    }

}
