/**
 * RTAClassAnalysis
 * tag: kkvilekal@acm.org 2002/08/27 04:35:20
 */
package mokan.prefetch.analysis.classanalysis;

import java.util.Map;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;

import mokan.prefetch.util.WorkSet;
import mokan.prefetch.util.RTClass;
import mokan.prefetch.util.RTMethod;

import agl2.Bigraph;
import agl2.BigraphBuilder;
import agl2.alg.DepthFirstSearch;
import agl2.alg.AbstractGraphVisitor;

import klogger.Logger;

/**
 *  The <code>RTAClassAnalysis</code> description.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class RTAClassAnalysis extends AbstractClassAnalysis{

    final static Logger log = 
      Logger.getLogger("prefetch.classanalysis");

    HashSet instantiated;
    boolean rta = true;

    RTMethod[] visitSpecialCall (final Callsite cs, 
                                 final String handle, 
                                 final String target) {
        // Everybody is newed 
        if (rta && target.equals(NEWINSTANCE)){
            log.warn ("newinstance: in " + cs.caller.getFullname() );
                throw new UnsafeAnalysisException ("Class.newInstance");
        }
        return super.visitSpecialCall (cs, handle, target);
    }

    RTMethod[] getVirtualTargets(final Callsite cs, 
                                 final String handle, 
                                 final String target){
        DepthFirstSearch dfs = new DepthFirstSearch (classgb.getBigraph());
        final ArrayList methods = new ArrayList();

        RTClass rt = classes.findClass (handle);
        dfs.apply (rt, new AbstractGraphVisitor () {
                public void finishVertex (Object v) {
                    RTClass clz = (RTClass)v;
//                      log.debug ("examining " + v 
//                                 +" instantiated="+instantiated.contains(clz)
//                                 +" interface=" + clz.isInterface()
//                                 +" systemClass="+ clz.isSystemClass()
//                          );
                    
                    if (rta && !instantiated.contains (clz)) return;
                    if (clz.isInterface()) return;
                    if (clz.isSystemClass()) return;

                    RTMethod m = findMethod (clz, target);
//                     log.debug (" is null" + (m == null) );

                    if (m == null) return;
//                     log.debug ( "is abstract"+ m.isAbstract());
                    if (m.isAbstract()) return;
                    if (m.isNative()) {
                        cs.setSystemCall(true);
                        return;
                    }
                
                    if (!methods.contains (m)) methods.add (m);
                }
            });

        return (RTMethod[])methods.toArray(new RTMethod[methods.size()]);
    }

    boolean visitMethod (RTMethod method) {
        // Method will have been visited by loading
//         RTAMethodVisitor cv = 
//             new RTAMethodVisitor (method, classes, instantiated);
//         method.visitInstructions (cv);
//         method2callsites.put (method, cv.getCallsites());
        return true;
    }

    boolean visitClass (RTClass cl) {
        if (cl.isSystemClass ()) return false;
        // Scan the class for any methods that contain
        // a "new instruction and add these to the instantiated set.
        RTMethod[] methods = cl.getMethods();
        for (int mi=0; mi < methods.length; mi++){
            RTAMethodVisitor cv = 
                new RTAMethodVisitor (methods[mi], classes, instantiated);
            methods[mi].visitInstructions (cv);
            method2callsites.put (methods[mi], cv.getCallsites());
        }
        return true;
    }

    public RTAClassAnalysis(String classpath,String apppath,String mainname){
        log.entering ("RTAClassAnalysis", "RTAClassAnalysis");

        log.message("RTAClassAnalysis:claspath="+classpath
                    +" apppath="+apppath
                    +" mainname="+mainname);

        try {
            instantiated = new HashSet();
            buildGraphs(classpath, apppath, mainname);
        } catch (UnsafeAnalysisException e) {
            rta = false;
            log.warn ("restarting RTAClassAnaysis");
            buildGraphs(classpath, apppath, mainname);
        }

        log.message ("classgraph nodes:" + classgraph.getVertices().size() +
                  " edges:" + classgraph.getEdges().size());

        log.message ("callgraph nodes:" + callgraph.getVertices().size() +
                  " edges:" + callgraph.getEdges().size());

        log.exiting ("RTAClassAnalysis", "RTAClassAnalysis" );
    }
}
