/**
 * RTAMethodVisitor
 * tag: kkvilekal@acm.org 2002/08/27 06:26:14
 */
package mokan.prefetch.analysis.classanalysis;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashSet;


import de.fub.bytecode.Constants;
import de.fub.bytecode.generic.InstructionHandle;
import de.fub.bytecode.generic.AllocationInstruction;
import de.fub.bytecode.generic.InvokeInstruction;
import de.fub.bytecode.generic.CPInstruction;
import de.fub.bytecode.generic.Type;
import de.fub.bytecode.generic.ArrayType;
import de.fub.bytecode.generic.ObjectType;

import mokan.prefetch.util.RTMethod;
import mokan.prefetch.util.RTClass;

import klogger.Logger;

/**
 *  The <code>RTAMethodVisitor</code> tracks new'ed types used in
 *  RTA class analysis.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class RTAMethodVisitor extends CallsiteVisitor {

    final static Logger log = 
      Logger.getLogger("prefetch.classanalysis");

    ClassSet classes;
    HashSet newed;
    
    RTAMethodVisitor (RTMethod m, ClassSet classes, HashSet newed) {
        super (m);
        this.classes = classes;
        this.newed = newed;
    }

    public void visitAllocationInstruction(AllocationInstruction ai){
        if (ai instanceof CPInstruction) {
            Type type=((CPInstruction)ai).getType(method.getConstantPoolGen());

            if (type instanceof ArrayType) 
                type = ((ArrayType)type).getBasicType();
            if (type instanceof ObjectType) {
                String typename  = ((ObjectType)type).getClassName();
                RTClass clz = classes.findClass (typename);

                while (clz != null) {
                    newed.add (clz);
                    // Add super classes to instantiated
                    log.debug ("instantiated " + clz);
                    clz = classes.findClass( clz.getSuperclassName() );
                    if (clz == classes.getObjectClass()) break;
                }
            }
        }
    }
}
