/*
 * 
 *
 * tag: kkvilekval@acm.org Wed Nov 14 10:01:23 2001 (AnnotateAccesses)
 *
 */


package mokan.prefetch.analysis.instrumentor;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.LinkedList;
import java.io.IOException;

import de.fub.bytecode.Constants;
import de.fub.bytecode.classfile.JavaClass;
import de.fub.bytecode.classfile.Field;
import de.fub.bytecode.classfile.Method;
import de.fub.bytecode.classfile.Code;
import de.fub.bytecode.classfile.ConstantPool;
import de.fub.bytecode.classfile.ConstantClass;
import de.fub.bytecode.classfile.ConstantUtf8;
import de.fub.bytecode.classfile.Utility;
import de.fub.bytecode.generic.ConstantPoolGen;
import de.fub.bytecode.generic.MethodGen;
import de.fub.bytecode.generic.InstructionList;
import de.fub.bytecode.generic.InstructionHandle;
import de.fub.bytecode.generic.InvokeInstruction;
import de.fub.bytecode.generic.FieldInstruction;
import de.fub.bytecode.generic.ReturnInstruction;
import de.fub.bytecode.generic.AllocationInstruction;
import de.fub.bytecode.generic.Type;
import de.fub.bytecode.generic.ReferenceType;
import de.fub.bytecode.generic.AALOAD;
import de.fub.bytecode.generic.ALOAD;
import de.fub.bytecode.generic.AASTORE;
import de.fub.bytecode.generic.DUP;
import de.fub.bytecode.generic.DUP2;
import de.fub.bytecode.generic.DUP2_X1;
import de.fub.bytecode.generic.DUP_X2;
import de.fub.bytecode.generic.PUSH;
import de.fub.bytecode.generic.POP;
import de.fub.bytecode.generic.GETSTATIC;
import de.fub.bytecode.generic.GETFIELD;
import de.fub.bytecode.generic.PUTFIELD;
import de.fub.bytecode.generic.INVOKESPECIAL;
import de.fub.bytecode.generic.INVOKEVIRTUAL;
import de.fub.bytecode.generic.INVOKESTATIC;
import de.fub.bytecode.generic.NEW;

import mokan.prefetch.analysis.Main;
import mokan.prefetch.analysis.classanalysis.ClassAnalysis;
//REM import precache.callgraph.CallGraph;
//REM import precache.classgraph.ClassGraph;
import mokan.prefetch.analysis.shapeanalysis.MethodTable;
import mokan.prefetch.analysis.shapeanalysis.MethodState;


import klogger.Logger;

import mokan.prefetch.util.RTMethod;
import mokan.prefetch.util.RTClass;

/**
 *  The <code>AnnotateAccesses</code> description.
 *  @version $Revision: 1.4 $, $Date: 2003/02/06 03:20:23 $
 *  @author Kristian Kvilekval
 */
public class AnnotateAccesses {

    private final static Logger log =Logger.getLogger("prefetch.instrumentor");


    /* Private/Public Variables */
//REM    private ClassGraph classgraph;
//REM    private CallGraph  callgraph;
    private ClassAnalysis classanalysis;
    private MethodTable methods;
    boolean external;
    //private boolean allowSystemClasses;

    /** Index into ConstantPool of precache.runtime.AccessTable Methods */
    private int entry;
    private int exit;
    private int access;
    private int prefetch;
    private int alloc;
    private int transBegin;
    private int transEnd;

    /** some statistics */
    int total_methods;
    int annotated_methods;
    int global_references;
        
    /** allow classes to be found on the fly */
    LinkedList workq;
    HashSet    visited;


    /**
     *  The default constructor
     */
    public AnnotateAccesses (ClassAnalysis ca, MethodTable a,
                             boolean external) {
//REM        callgraph = calls;
//REM        classgraph = cg;
        classanalysis = ca;
        
        methods = a;
        this.external = external;
        //allowSystemClasses = Precache.allowSystemClasses;
    }

    /** Annotate the method by recording the entry, field access 
     *  and the cahcing descisions
     */
    public RTMethod annotateMethod (RTMethod m, ConstantPoolGen cp){
        log.entering ("Annotate", "method");
        String clname = m.getDefiningClass().getClassName() /*+ "_pc"*/;

        Code   code  = m.getMethod().getCode();
        int    flags = m.getMethod().getAccessFlags();
        String name  = m.getMethod().getName();
        boolean analyzed = true;
        
        // Sanity check
        if(m.isNative() || m.isAbstract() || (code == null)){
            log.debug( m + "is native, abstract or has no code");
            return m;
        }
        
        if (! methods.hasShape (m) ) {
            // This method did not appear in the callgraph
            // MAYBE should put some osrt of check that the method
            // is never entered????
            log.warn("No shape available for " + m);

            if (classanalysis.getStaticCallGraph().hasVertex (m))
                log.warn ("call graph is null for " + m + "->not analyzed");
            analyzed = false;
        }

        // NOTE  :  recent upgrade makes this method fail
        // NOTE  :  need to find out why
        if (name.equals ("class$")){
            log.warn("class$ found");
            return m;
        }

        MethodState ms=null;
        if (analyzed) {
            ms = methods.getState (m);
            if (ms == null )
                log.warn ("annotating "+name + " but has no state");
            else {
                log.debug ("patching " + name + " level=" + ms.callLevel);

                // Test for globals 
                HashMap gm = ms.getMethodGlobals ();
                if (gm != null) {
                    //log.warn ("found global references in " + m);
                    Iterator gi = gm.keySet().iterator ();
                    while (gi.hasNext()){
                        String gl = (String)gi.next();
                        int sp1 = gl.lastIndexOf('%');
                        int sp0 = gl.lastIndexOf('%',sp1-1);
                        String fclz = gl.substring(0,sp0);
                        String fname = gl.substring(sp0+1, sp1);
                        String fsig = gl.substring (sp1+1);

                        if (!RTClass.isSystemClass(fclz)){
                            int cpindex= cp.lookupFieldref(fclz,fname,fsig);
                            if (cpindex < 0) {
                                log.warn ("cl="+fclz + " field="+fname+" sig="+fsig);
                                log.warn("   global = "+gl+"="+cpindex);
                            }
                        }
                        global_references++;
                    }
                }
            }
        }

        /* Create instruction list to be inserted at method start.
         */
        MethodGen           mg  = m.getMethodGen();
        InstructionList     il  = mg.getInstructionList();
        InstructionHandle[] ihs = il.getInstructionHandles();

        // Create the entry patch.
        //   The entry patch pushes the self pointer and any
        //   arguments... and globals

        if (ihs.length == 0) {
            log.severe (m + " has no instructions: " + il);
            log.severe (m + " has instruction list:" + il.getLength());
            log.severe (m + " has code array: " + m.getMethod().getCode().getCode().length);
            return m;
        }

        // Scan for the special method transactBegin..
        if (!analyzed) return m;

        boolean found = false;
        for (int j=0; j < ihs.length; j++) {
            if (ihs[j].getInstruction() instanceof INVOKESTATIC) {
                INVOKESTATIC invoke=((INVOKESTATIC)ihs[j].getInstruction()); 
                String targetname = invoke.getName(cp);
                log.message("Checking staticinvoke : "+ targetname);
                if (targetname.equals ("guardBegin")){
                    log.message("Found guard begin in" + m);
                    found = true;
                    break;
                }
                //String staticname = invoke.getClassName(cp);
                //RTClass invoked = classanalysis.getClass (staticname);
            }
        }
        if (!found) return m;

        log.info ("Annotate", "method", "Found transactBegin in " + m);

        InstructionList patch  = new InstructionList();

        // For the moment add check the shape graph at
        // initial entry instead of at each callsite (bogda's idea)
        int li = 0;             // arg list index
        String localShape;
        if (!m.isStatic()) {
            // Push the self variable
            if (external) 
                localShape = Integer.toString(li);
            else 
                localShape = methods.getLocalShape (m,li);
            if (localShape.length()!=0){
                patch.append(new ALOAD(li));
                patch.append(new PUSH(cp, localShape));
                patch.append(new INVOKESTATIC(prefetch));
                log.trace("shape " + localShape);
            }
            li = 1;
        }

        // Push the argument values
        Type[] argtypes = mg.getArgTypes();
        for (int ai=0; ai < argtypes.length; ai++,li++) {
//          for (int ai=0; ai < m.getMaxLocals(); ai++,li++) {

            // Don't push objects of systems classes
            if (RTClass.isSystemClass(argtypes[ai].toString()))
                continue;

        
             if (argtypes[ai] instanceof ReferenceType) {
                 if (external) 
                     localShape = Integer.toString(li);
                 else 
                     localShape = methods.getLocalShape (m,li);
                if (localShape.length()!=0){
                    patch.append(new ALOAD(li));
                    patch.append(new PUSH(cp, localShape));
                    patch.append(new INVOKESTATIC(prefetch));
                    log.trace ("shape " + localShape);
                }
             } else if (argtypes[ai] == Type.DOUBLE) {
                 // Double are two words wide.
                 li++;
             }
        }
        // Push any references to globals
        HashMap gm = ms.getMethodGlobals ();
        if (gm != null) {
            //log.warn ("found global references in " + m);
            Iterator gi = gm.keySet().iterator ();
            li = ms.locals.size();
            while (gi.hasNext()){
                String gl = (String)gi.next();
                int sp1 = gl.lastIndexOf('%');
                int sp0 = gl.lastIndexOf('%',sp1-1);
                String fclz = gl.substring(0,sp0);
                String fname = gl.substring(sp0+1, sp1);
                String fsig = gl.substring (sp1+1);

                if (!RTClass.isSystemClass(fclz)){
                    int cpindex= cp.lookupFieldref(fclz,fname,fsig);
                    if (cpindex>=0){
                        if (external) {
                            ms.locals.push(ms.getGlobalReference(gl));
                            localShape = Integer.toString(li++);
                        } else 
                            localShape = methods.getLocalShape (m,li);
                        log.trace("global var="+gl+':'+localShape);

                        patch.append(new GETSTATIC(cpindex));
                        patch.append(new PUSH (cp, localShape));
                        patch.append(new INVOKESTATIC(prefetch));
                    }
                    else {
                        log.warn ("cl="+fclz + " field="+fname+" sig="+fsig );
                        log.warn("   global = "+gl+"="+cpindex);
                    }
                    global_references++;
                }
            }
        }
        patch.append(new PUSH (cp, m.toString()));
        patch.append(new PUSH (cp, analyzed));
        patch.append(new INVOKESTATIC(entry));

        // Add the entry patch 
        //   If this is a class constructor we must wait until 
        //   we have invoked the constructor of the base class.
        //   Otherwise we just insert the patch as the first
        //   set of instructions.
        if(name.equals("<init>")) {
            // First let the super or other constructor be called
            for(int j=1; j < ihs.length; j++) {

                // Find an invokespecial of that is a superclass
                // of rthe current class. 
                // NOTE:  this may still fail if we are given something
                // like   B -> D
                //  D() { super (new B()) } 
                //  Here we would have an invoke to call B() before
                //  we would have invoke for the super (*) 
                if(ihs[j].getInstruction() instanceof INVOKESPECIAL) {
                    INVOKESPECIAL invoke=((INVOKESPECIAL)ihs[j].getInstruction()); 
                    String specialname = invoke.getClassName(cp);
                    RTClass invoked = classanalysis.getClass (specialname);
                    
                    if (classanalysis.isAncestorClass(invoked, m.getDefiningClass())) {
                        // <init> is acting funny (since we dump 
                        // things into a hashmap based cache sometime
                        // the hashCode method cannot be called because
                        // the object is not initialized yet.
                        patch.append(patch.insert(new ALOAD(0)), 
                                     new INVOKESTATIC(alloc));
                        il.append(ihs[j], patch); 

                        break;
                    }
                }
            }
        } else
            il.insert(ihs[0], patch);


        //   Analyze the entire method for instructions of interest
        //   Annotate those instruction to call the runtime so
        //   that we may check the accuracy of the prefetches created
        //   above.
        for (int j=0; j < ihs.length; j++) {
            // Patch the field instructions with a call to the entry
            InstructionHandle newtarget = null;

//             if(ihs[j].getInstruction() instanceof FieldInstruction) {
//                 FieldInstruction fi=(FieldInstruction)ihs[j].getInstruction();
//                 if (fi.getFieldType(cp) instanceof ReferenceType) {
//                     if (fi instanceof GETFIELD){
//                         // BCEL insert eats patch
//                         patch = new InstructionList();
//                         patch.append (new DUP());
//                         patch.append (new PUSH(cp, false));
//                         patch.append (new INVOKESTATIC(access));
//                         log.trace ("getfield at "+j);
//                         newtarget = il.insert (ihs[j], patch);
//                     } else if (fi instanceof PUTFIELD) {
//                         // BCEL insert eats wpatch
//                         patch = new InstructionList();
//                         patch.append (new DUP2());
//                         patch.append (new POP());
//                         patch.append (new PUSH(cp, true));
//                         patch.append (new INVOKESTATIC(access));
//                         newtarget = il.insert (ihs[j], patch);
//                         log.trace ("putfield at "+j);
//                     }                                        
//                     // Check that the class is either visted or
//                     // will be visited otherwise add this to workq
//                     String classname = fi.getClassName(cp);
//                     RTClass clz = classanalysis.getClass (classname);
//                     if (clz != null && !clz.isSystemClass()) {
//                         if (!visited.contains (clz))
//                             workq.addLast (clz);
//                     }else {
//                         if (!clz.isSystemClass())
//                             log.severe("refered to class:"
//                                        +classname+ " will not annotated");
//                     }
//                 }
//             } else
//             if (ihs[j].getInstruction() instanceof AALOAD) {
//                 patch = new InstructionList();
//                 patch.append (new DUP2());
//                 patch.append (new POP());
//                 patch.append (new PUSH(cp, false));
//                 patch.append (new INVOKESTATIC(access));
//                 log.trace ("aaload at "+j);
//                 newtarget = il.insert (ihs[j], patch);
//             }else
//             if (ihs[j].getInstruction() instanceof AASTORE) {
//                 patch = new InstructionList();
//                 patch.append (new DUP2_X1());
//                 patch.append (new POP());
//                 patch.append (new POP());
//                 patch.append (new DUP_X2());
//                 patch.append (new PUSH(cp, true));
//                 patch.append (new INVOKESTATIC(access));
//                 log.trace ("aastore at "+j);
//                 newtarget = il.insert (ihs[j], patch);
//             } else
//             if(ihs[j].getInstruction() instanceof AllocationInstruction) {
//                 if (ihs[j].getInstruction() instanceof NEW) {
//                     // Search forward for the invoke static
//                     Type clstype = ((NEW)ihs[j].getInstruction()).getType(cp);
//                     il.append(ihs[j], new DUP());
//                     for (int i=j+1; i < ihs.length; i++) {
//                         if(ihs[i].getInstruction() instanceof INVOKESPECIAL){
//                             Type nc =((INVOKESPECIAL)ihs[i].getInstruction()).getClassType(cp);
//                             if (clstype.equals (nc)) {
//                                 il.append(ihs[i], new INVOKESTATIC(alloc));
//                                 break;
//                             }
//                         }
//                     }
//                 } else {
//                     patch = new InstructionList();
//                     patch.append(new DUP());
//                     patch.append(new INVOKESTATIC(alloc));
//                     il.append (ihs[j],patch);
//                 }
//                 log.trace ("allocation at "+j);
//             } else
            if(ihs[j].getInstruction() instanceof ReturnInstruction) {
                patch  = new InstructionList();
                patch.append(new PUSH (cp, m.toString()));
                patch.append(new INVOKESTATIC(exit));
                newtarget  = il.insert (ihs[j], patch);
             } 
//           else
//             if(ihs[j].getInstruction() instanceof InvokeInstruction) {
//                 String invname=((InvokeInstruction)ihs[j].getInstruction()).getMethodName(cp);
//                 if (invname .equals ("newInstance")) {
//                     log.warn ("Calling " +invname + " in " + name);
//                     patch = new InstructionList();
//                     patch.append(new DUP());
//                     patch.append(new INVOKESTATIC(alloc));
//                     il.append (ihs[j],patch);
//                 }
                
//             }

            if (newtarget != null)
                il.redirectBranches(ihs[j], newtarget);
        }
        //if(code.getMaxStack() < 3)
        //mg.setMaxStack(code.getMaxStack()+4);
        mg.setMaxStack();

        m.setMethod( mg.getMethod() );

        il.dispose(); // Reuse instruction handles

        log.exiting ("Annotate", "method");
    
        return m;
    }


    /** Annotate the class with expected accesses.
     *  Write the new classfiles to the repository *
     */
    public void annotateClass (RTClass clz) {
        if (visited.contains (clz)) return;

        String class_name = clz.getClassName() /*+ "_pc"*/;
        ConstantPoolGen cp = clz.getConstantPoolGen();
        JavaClass javaclass = clz.getJavaClass();

        visited.add (clz);

        // Reset the class name in the constant pool
//         int index = javaclass.getClassNameIndex();
//         index = ((ConstantClass)cp.getConstant(index)).getNameIndex();
//         cp.setConstant(index, new ConstantUtf8(class_name.replace('.', '/')));
        // Add the constants we will access during runtime.
        entry = cp.addMethodref("mokan.prefetch.runtime.AccessTable", 
                                "methodEntry",
                                "(Ljava/lang/String;Z)V");
        exit = cp.addMethodref("mokan.prefetch.runtime.AccessTable", 
                               "methodExit",
                               "(Ljava/lang/String;)V");
        access = cp.addMethodref("mokan.prefetch.runtime.AccessTable", 
                                 "access",
                                 "(Ljava/lang/Object;Z)V");
        prefetch = cp.addMethodref("mokan.prefetch.runtime.AccessTable", 
                                   "prefetch",
                                   "(Ljava/lang/Object;Ljava/lang/String;)V");
        alloc = cp.addMethodref("mokan.prefetch.runtime.AccessTable",
                                "alloc",
                                "(Ljava/lang/Object;)V");
        // Change to public access for all fields
        Field[] fields = javaclass.getFields();
        int flags;
        for (int fi=0; fi < fields.length; fi++) {
            flags = fields[fi].getAccessFlags();
            // Clear both PROTECED and PRIVATE
            flags &= ~(Constants.ACC_PROTECTED|Constants.ACC_PRIVATE);
            flags |= Constants.ACC_PUBLIC;
            fields[fi].setAccessFlags(flags);
            //log.trace ("field is now " + fields[fi]);
        }
        // Change the class access rights also..
        flags = javaclass.getAccessFlags();
        flags &= ~(Constants.ACC_PROTECTED|Constants.ACC_PRIVATE);
        flags |= Constants.ACC_PUBLIC;
        javaclass.setAccessFlags (flags);


        // Patch all methods.
        RTMethod[] rtmethods = clz.getMethods();
        
        for(int j=0; j < rtmethods.length; j++) {// Directly use array
            if (log.isEnabled (Logger.TRACE)) {
                log.trace ("Annotate", "class", "Annotating " +rtmethods[j]);
                log.trace (RTMethod.dumpListing (rtmethods[j]));
            }
            try {
                rtmethods[j] = annotateMethod(rtmethods[j], cp);
            } catch (RuntimeException e) {
                log.severe ("exception "+e+" while processing " + rtmethods[j]);
                e.printStackTrace();
            }
        }        
        clz.setMethods (rtmethods);

        /* Finally dump it back to a file.
         */
        javaclass.setConstantPool(cp.getFinalConstantPool());
        try { 
            javaclass.dump(Main.annotationPrefix
                           + class_name.replace('.','/')+".class");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public void annotateProgram () {
        
        // Visit each class in the program
        visited = new HashSet ();
        workq = new LinkedList();
        int count = 0;
        RTClass rtclz;

        int total_classes = classanalysis.getLoadedClasses().size();
        Iterator ci = classanalysis.getLoadedClasses().iterator();
        while (ci.hasNext()) {
            rtclz = (RTClass)ci.next();
            if (!rtclz.isSystemClass()){
                annotateClass (rtclz);
                count++;
            }
        }
        
        /*

        Iterator topiter = callgraph.getTopologicalSCCIterator();
        while (topiter.hasNext()) {
            Set scc =  ((Set)topiter.next());
            Iterator mi = scc.iterator();
            while (mi.hasNext()) {
                rtclz = ((RTMethod)mi.next()).getDefiningClass();
                
                if (!visited.contains (rtclz)) {
                    visited.add (rtclz);
                    if (rtclz.isSystemClass() ) 
                        continue;
                    workq.addLast (rtclz);
                    //annotateClass (m.getDefiningClass());
                    count++;
                }
            }
        }
        int total_classes = visited.size();
        
        visited.clear();
        while (!workq.isEmpty()) {
            rtclz = (RTClass)workq.removeFirst();
            if (!visited.contains(rtclz))
                annotateClass(rtclz);
        }
        */

        log.message ("annotated="+count 
                     + " classes of total " + total_classes
                     + " global refs=" + global_references);

        log.exiting ("AnnotateAccesses", "visitProgram");
    }
};
