/*
 * 
 *
 * tag: kkvilekval@acm.org Wed Nov 14 09:45:48 2001 (AccessFlowVisitor)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.Iterator;
import java.util.List;
import java.util.HashMap;

import klogger.Logger;

/**
 *  The <code>AccessFlowVisitor</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class AccessFlowVisitor {

    private final static Logger log = Logger.getLogger ("prefetch.analysis.StackVisitor");

    private final static int LASTNODES = 64;
    private HashMap bb2state;
    private StackVisitor stackVisitor;
    private int visitcount;
    private int maxcount;
    private BlockNode[] lastnodes; // Debugging

    public AccessFlowVisitor (StackVisitor sv) {
        bb2state = new HashMap();
        stackVisitor = sv;
        lastnodes = new BlockNode[LASTNODES];
    }

    public void clearState () {
        bb2state.clear();
        visitcount = 0;
        maxcount = 10000;
    }


    public void addSuccessors (BlockNode b, List workQ) {
        Iterator succs = b.successors();
        while (succs.hasNext()) {
            BlockNode succ = (BlockNode)succs.next();
            if (merge (b, succ))
                if (!workQ.contains (succ))
                    workQ.add (succ);
        }
    }

    public boolean merge (BlockNode from, BlockNode to) {
        // Initialize the next local/stack with the from
        BlockState f = (BlockState)bb2state.get (from);
        BlockState t = (BlockState)bb2state.get (to);
        boolean newblock = false;

        log.debug ("FlowVisitor","merge",  "Blocks " + from + "->" + to);

        if (t == null) { 
            t = new BlockState (f);
            bb2state.put (to, t);
            newblock = true;
        }
        log.debug("FlowVisitor", "merge", "Merging "+from+f.toString());
        log.debug("FlowVisitor", "merge", "  into " + to + t.toString());
        
        if (to.isExceptionHandler()/* && !p.isExceptionHandler()*/)
            t.mergeHandler (f);
        else
            t.merge (f);
        
        log.debug("FlowVisitor", "merge", "Block " + to + " became" + t);
        return (newblock || t.changed());
    }

        
    public void visitBlock (BlockNode n, List workQ){
        BlockState state = (BlockState)bb2state.get(n);
        log.trace("FlowVisitor", "visit", n+ " before "+ state);

        if (state != null) state.reset();
        stackVisitor.visitBlock (n, state);
        bb2state.put (n, state = stackVisitor.currentState());
        addSuccessors (n, workQ);

        log.trace("FlowVisitor", "visit", n+" after "+state);

        lastnodes[(visitcount++) % LASTNODES] = n;
        if (visitcount > maxcount) {
            log.warn ("FlowVisitor", "visit", "too many visits" + visitcount);
            //log.warn ("FlowVisitor", "visit", "blocks" + bb2state);
            int last = visitcount % LASTNODES;
            int pos  = last;
            while (--pos != last) {
                if (pos < 0) pos = LASTNODES-1;
                log.warn("FlowVisitor", "visit", "visited " + lastnodes[pos]);
            } 
            
            throw new RuntimeException();
        }

        // If state has changes then add successors to the work Set.
        // For now visit everybody once.
        //             Iterator succs = n.successors();
        //             while (succs.hasNext()) {
        //                 BlockNode succ = (BlockNode)succs.next();
        //                 if (!visited.contains (succ) && !workQ.contains(succ))
        //                     workQ.add (succ);
        //             }
        //             succs = n.getHandlers().iterator ();
        //             while (succs.hasNext()) {
        //                 BlockNode succ = (BlockNode)succs.next();
        //                 if (!visited.contains (succ))
        //                     workQ.add (succ);
        //             }
    }

};
