/*
 * 
 *
 * tag: kkvilekval@acm.org Sat Nov 10 14:47:24 2001 (BlockFactory)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

//import de.fub.bytecode.generic.DefaultVisitor;
import de.fub.bytecode.generic.MethodGen;
import de.fub.bytecode.generic.CodeExceptionGen;
import de.fub.bytecode.generic.Instruction;
import de.fub.bytecode.generic.InstructionList;
import de.fub.bytecode.generic.InstructionHandle;
import de.fub.bytecode.generic.BranchInstruction;
import de.fub.bytecode.generic.ReturnInstruction;
import de.fub.bytecode.generic.IfInstruction;
import de.fub.bytecode.generic.JsrInstruction;
import de.fub.bytecode.generic.Select;
import de.fub.bytecode.generic.ExceptionThrower;
import de.fub.bytecode.generic.ATHROW;
import de.fub.bytecode.generic.RET;

//import de.fub.bytecode.generic.BranchHandle;

import mokan.prefetch.util.RTMethod;
import agl.DirectedGraphBuilder;

import klogger.Logger;

/**
 *  The <code>BlockFactory</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class BlockFactory {

    private final static Logger log = Logger.getLogger ("prefetch.analysis.BlockFactory");
    
 
    /* Private/Public Variables */
    /** Map from instructio handle index to block */
    private HashMap  blocks;
    private int blocknum;
    private DirectedGraphBuilder builder;


    /**
     *  The default constructor
     */
    public BlockFactory () {
        blocks = new HashMap();
    }

    public BlockFactory (DirectedGraphBuilder gb) {
        blocks = new HashMap();
        builder = gb;
    }

    public BlockNode[] generate (RTMethod method){

        log.entering ("BlockFactory", "generate", method);

        MethodGen mg = method.getMethodGen();
        //  Keep a list of all blocks founbd in order 
        ArrayList blockarr = new ArrayList();

        blocknum = 0; // Rest the blocknum to 0
        InstructionList     il  = mg.getInstructionList();
        InstructionHandle[] ihs = il.getInstructionHandles();
        
        // Create block at the exception starts.
        // Sometime these do not occur at natural block
        // boundaries, therefor we create them first
        // and they will be discovered as we pass over
        // them later.
        CodeExceptionGen[] ehs = method.getExceptionHandlers();
        for(int i=0; i < ehs.length; i++) {
            CodeExceptionGen  c  = ehs[i];
            InstructionHandle sh = c.getStartPC(); 
            getBlock(sh);
        }
        // The first block begins at the first instruction.
        BlockNode startBlock= getBlock (ihs[0]);
        BlockNode current = null;
        BlockNode next;
        for(int i=0; i < ihs.length; i++) {
            // If we pass by a defined block then it's the current block.
            next = (BlockNode)blocks.get (ihs[i]);
            if (next != null) {
                // Finish the current block (if not the first)
                next.setTag (blocknum++);
                next.setStart (i);

                if (current != null) {
                    current.setEnd (i-1);
                }
                current = next;
                blockarr.add(current);
            }

            // Check for instuction that start blocks
            //  Current branches and return statement
            Instruction inst = (Instruction)ihs[i].getInstruction();
            if (inst instanceof ExceptionThrower) 
                current.setFlag(BlockNode.THROWS, true);

            // Tag the instruction from the most broad to the least.
            if(inst instanceof BranchInstruction) {
                // GotoInstruction, IfInstruction, JsrInstruction, Select 
                BranchInstruction bi = (BranchInstruction)inst;
                // Special cases LOOKUPSWITCH and TABLESWITCH   
                if(bi instanceof Select) { 
                    InstructionHandle[] targets = ((Select)bi).getTargets();
                    
                    for(int j=0; j < targets.length; j++) {
//                         BlockNode.addEdge (current, getBlock (targets[j]));
                        if (builder != null) {
                            builder.addEdge (current, getBlock (targets[j]));
                        }
                        //log.trace ("select:" + current + "->"+getBlock (targets[j]));
                    }
                }
                // Add and edge to the jmp location (select, goto, if)
//                 BlockNode.addEdge (current, getBlock (bi.getTarget()));
                if (builder != null) {
                    builder.addEdge(current, getBlock (bi.getTarget()));
                }
                //log.trace ("branch target:" + current + "->"+getBlock(bi.getTarget()));
                // And create a new block after the jmp
                BlockNode after = getBlock (ihs[i].getNext());
                if (bi instanceof IfInstruction) {
//                     BlockNode.addEdge (current, after);
                    if (builder != null) {
                        builder.addEdge(current, after);
                    }
                    //log.trace ("ifelse:" + current + "->"+after);
                } else if (bi instanceof JsrInstruction) {
                    // The target has already linked blocks
                    // Record the fact that this is a place for a
                    // return
                    BlockNode target = getBlock (bi.getTarget());
                    target.addJsr (after);
                }
            } else if (inst instanceof ReturnInstruction) {
                // A return ends a block.
                getBlock(ihs[i].getNext());
            } else if (inst instanceof RET) {
                // A return ends a block.
                Iterator jsrs = current.getJsrs().iterator ();
                while (jsrs.hasNext()) {
                    BlockNode jsrTarget = (BlockNode)jsrs.next();
                    builder.addEdge(current, jsrTarget);
                    //log.trace ("jsr:" + current + "->"+jsrTarget);
                }
                getBlock(ihs[i].getNext());
                
            } else if (inst instanceof ATHROW) {
                // A Throw end a block
                getBlock(ihs[i].getNext());
            } else if ((i+1) < ihs.length && blocks.containsKey (ihs[i+1])){
                next = (BlockNode)blocks.get (ihs[i+1]);
                // Check for try block starts 
                
                // Total hack for exception handlers since
                // sometimes then don't begin at natural blocks.
                // Since we create a fake block above 
                // we need to link the previous block to the
                // current when at a block boundary and
                // we didn't meet the criteria above (branh or return).
//                 BlockNode.addEdge(current, next);
                builder.addEdge(current, next);
                //log.trace("block end:" + current +"->"+next);
            }
            
        }
            // Finish up the last block.
        current.setEnd (ihs.length-1);

        builder.dump (System.out);

        // Handle the exception handlers ..
        // The startPC should always be in the beginning of block
        // Find the blocks between the startPC and the endPC 
        // and make an edge from that block
        // to the handlerPC 
        //CodeExceptionGen[] ehs = method.getExceptionHandlers();
        for(int i=0; i < ehs.length; i++) {
            CodeExceptionGen  c  = ehs[i];
            InstructionHandle ih = c.getStartPC(); 
            InstructionHandle eh = c.getEndPC(); 
            InstructionHandle hh = c.getHandlerPC();
            BlockNode eblock    = (BlockNode)blocks.get (hh);
            eblock.setFlag(BlockNode.HANDLER, true);

            current = null;
            while (ih != eh) {
                next = (BlockNode)blocks.get (ih);
                // At a block change.
                if (next != null && current != next){
                    current = next;
                    // Some blocks never throw so why bother with them.
                    if (current.isThrower()){
//                          BlockNode.addExceptionEdge (current, eblock);
                        builder.addEdge(current, eblock);
                        //log.trace("handler " + current +"->"+eblock);
                    }
                }
                ih = ih.getNext();
            }
        }

        BlockNode[] rblocks = 
            (BlockNode[])blockarr.toArray(new BlockNode[blockarr.size()]);

        if (log.isEnabled(Logger.TRACE)) {
            log.trace (BlockFactory.toString (rblocks));
        }

        log.exiting ("BlockFactory", "generate");
        
        return rblocks;
    }
    
    /** Find or create the a basic block at this instruction */
    BlockNode getBlock (InstructionHandle ih) {
        BlockNode bn = (BlockNode)blocks.get (ih);
        if(bn == null) {
            bn = new BlockNode ();
            blocks.put (ih, bn);
            if (builder != null) builder.addVertex (bn);
        }
        return bn;
    }

    public static void dumpCFG (BlockNode start) {

        java.util.Stack bbstack = new java.util.Stack();
        java.util.Stack done = new java.util.Stack();

        bbstack.push (start);
        while (!bbstack.empty()) {
            BlockNode bb = (BlockNode)bbstack.pop();
            if (!done.contains (bb)) {
                done.push (bb);
                bbstack.addAll (bb.getSuccessors());
                System.out.print ("block " + bb + "->" );
                Iterator bi = bb.getSuccessors().iterator();
                while (bi.hasNext()) 
                    System.out.print ((BlockNode)bi.next() + " ");
                System.out.println ("");
            }
        }
    }

    public static String  toString (BlockNode[] blocks) {

        StringBuffer buf = new StringBuffer();
        for (int ib=0; ib < blocks.length; ib++) {
            BlockNode bb = blocks[ib];
            buf.append ("block "); buf.append( bb ); buf.append( "->" );
            Iterator bi = bb.getSuccessors().iterator();
            while (bi.hasNext()) 
                buf.append ((BlockNode)bi.next() + " ");

            // Show any exception handler blocks.
            bi = bb.getHandlers().iterator();
            if (bi.hasNext()) {
                buf.append ("handlers ");
                while (bi.hasNext()) 
                    buf.append ((BlockNode)bi.next() + " ");
                //buf.append ('\n');
            }
            buf.append ('\n');
        }

        return buf.toString();
    }
};
