/*
 * 
 *
 * tag: kkvilekval@acm.org Wed Sep 26 14:52:03 2001 (BlockNode)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;


import java.util.List;
import java.util.Set;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import mokan.prefetch.util.RTMethod;

import agl.DirectedGraphNode;

/**
 *  The <code>BlockNode</code> represent the basic block for section of code.
 *  
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class BlockNode implements  DirectedGraphNode {

    public final static int HANDLER = 0x001;
    public final static int THROWS = 0x002;

    /* Private/Public Variables */

    /** Block Belongs to method */
    RTMethod method;
    int      properties;

    /** A list of previous blocks */
    HashSet pred;
    /** A list subsequent blocks */
    HashSet succ;
    /** A list exceptio handler blocks for this block */
    ArrayList handlers;

    /** Jsr instruction branch to me so record them here */
    ArrayList jsrs;

    /** Unique ID naming the node. */
    public int  blocknum;
    /** an Index into the code of the method (Handles in MethodGen) */
    public int start;
    public int end;
    
    /**
     *  The default constructor
     */
    public BlockNode () {
        pred = new HashSet();
        succ = new HashSet();
        handlers = new ArrayList();
        jsrs = new ArrayList();
        properties = 0;
        blocknum = -1;
    }

    public void addPredecessor (BlockNode prev){
        if (!pred.contains(prev)) pred.add (prev);
    }
    public void addSuccessor (BlockNode next) {
        if (!succ.contains(next)) succ.add (next);
    }

    public void setFlag (int flag, boolean x) {
        if (x) properties|=flag;
        else   properties&= ~(flag);
    }
    public boolean isExceptionHandler () {
        return ((properties&HANDLER)!=0);
    }
    public boolean isThrower () {
        return ((properties&THROWS)!=0);
    }

    public static void addEdge (BlockNode from, BlockNode to) {
        from.addSuccessor (to);
        to.addPredecessor (from);
    }

    public static void addExceptionEdge (BlockNode from, BlockNode to) {
        to.addPredecessor (from);
        if (!from.handlers.contains(to)) from.handlers.add (to);
    }
    public void addJsr (BlockNode me) {
        jsrs . add (me);
    }
    public ArrayList getJsrs () {
        return jsrs;
    }

    public void setTag (int block) { blocknum = block;}
    public void setStart (int x) { start = x; }
    public void setEnd (int x) { end = x; }


    /** Return an Iterator<BlockNode> of the successors */
    public Iterator successors () {
        return succ.iterator();
    }

    /** Return an Iterator<BlockNode> of the predecessors */
    public Iterator predecessors () {
        return pred.iterator();
    }

    public int countPredecessors () {
        return pred.size();
    }
    public int countSuccessors () {
        return succ.size();
    }
    public Set getSuccessors () {
        return succ;
    }
    public Set getPredecessors () {
        return pred;
    }
    public List getHandlers () {
        return handlers;
    }

    /** Standard visitor pattern */
    public void accept (BlockVisitor v) { v.visit(this); }

    /**
     *  @return a string representation of this <code>BlockNode</code> 
     */
    public String toString () {
        return  (isExceptionHandler()?"eb":"bb") +blocknum+"["+start+":"+end+"]";
        //return  Integer.toString(blocknum);
    }

    /** Graph Interface **/

    public void addInEdge (Object to) {
        addPredecessor ((BlockNode)to);
    }
    public void addOutEdge (Object to) {
        addSuccessor ((BlockNode)to);
    }
    public void remInEdge (Object to) {
        throw new UnsupportedOperationException("BlockNode.remInEdge");
    }
    public void remOutEdge (Object to) {
        throw new UnsupportedOperationException("BlockNode.remOutEdge");
    }
    public boolean hasInEdge (Object to) {
        return pred.contains (to);
    }
    public boolean hasOutEdge (Object to) {
        return succ.contains (to);
    }
    


};
