/*
 * 
 *
 * tag: kkvilekval@acm.org Sun Nov 11 16:58:18 2001 (BlockState)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import klogger.Logger;


/**
 *  The <code>BlockState</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class BlockState {
    private final static Logger log = Logger.getLogger ("prefetch.analysis.BlockState");


    /* Private/Public Variables */
    private ShapePool pool;
    public VarStack initial_locals;
    public VarStack initial_stack;

    public VarStack locals;
    public VarStack stack;


    /**
     *  The default constructor
     */
    public BlockState (BlockState p) {
        pool   = p.pool;
//         locals = (VarStack)p.locals.clone();
//         stack  = (VarStack)p.stack.clone();
    }
    /** Entry point for StackVisitor on first block of method */
    public BlockState (ShapePool p, VarStack locals, VarStack stack){
        this.pool = p;
        this.initial_locals = locals;
        this.initial_stack =  stack;
        this.locals = copyStack(this,locals);
        this.stack = copyStack(this, stack);
    }

    public void reset () {
        pool.resetChanged ();

        locals = copyStack(this, initial_locals);
        stack  = copyStack(this, initial_stack);

    }
    public boolean changed () {
        return pool.isChanged();
    }
    private VarStack copyStack (BlockState as, VarStack vs) {
        VarStack ns = (VarStack)vs.clone();
/*
        VarStack ns = new VarStack ();
        Iterator vi = vs.iterator();
        while (vi.hasNext()){
            RTVariable ov = (RTVariable)vi.next();
            if (ov  != null) {
                RTVariable nv = createVariable();
                ns.push (nv);
                AccessState.unifyState (this, nv, as, ov);
            } else
                ns.push (null);
        }
*/
        return ns;
    }

    /** Unify all the variable in the locals and stack 
     *  We are merging the states as follows :
     *  {succ.state} = {succ.entryState} U  {pred.state}
     *  If entrState is null then this is first
     *  time entering OR we have only one(1) predeccessor
     *  and have no need to save the entryState for merging.
     */
    public  void merge (BlockState pred) {
        
        // This is the first merge
        if (initial_stack == null) {
            locals = initial_locals = copyStack(pred, pred.locals);
            stack = initial_stack = copyStack(pred, pred.stack);
            return;
        }

//         if (entrystack == null) {
//             entrystack = (VarStack)pred.stack.clone();
//             entrylocals = (VarStack)pred.locals.clone();
//             stack = (VarStack)entrystack.clone();
//             locals = (VarStack)entrylocals.clone();
//             return;
//         }

        try { 
            int locs = initial_locals.size();
            for (int l=0; l < locs; l++) {
//                 RTVariable v1 = entrylocals.getAt (l);
                RTVariable v1 = initial_locals.getAt (l);
                RTVariable v2 = pred.locals.getAt (l);
                if (v1 == null || v2 == null) continue;
                pool.unifyState (v1, v2);
            }
            
            int stak = initial_stack.size();
            for (int s=0; s < stak; s++) {
//                 RTVariable v1 = entrystack.getAt (s);
                RTVariable v1 = initial_stack.getAt (s);
                RTVariable v2 = pred.stack.getAt (s);
                if (v1 == null || v2 == null) continue;
                pool.unifyState (v1,v2);
            }

            

        } catch (RuntimeException e) {
            log.severe ("BlockState","merge", "predlocals:" + pred.locals + "predstack:" + pred.stack);
            log.severe ("BlockState","merge", "locals:" + locals + "stack:" + stack);
            throw e;
        }
    }

    public  void mergeHandler (BlockState pred) {
        if (initial_stack == null) {
            locals = initial_locals = copyStack(pred, pred.locals);
            stack  = initial_stack = new VarStack();
            initial_stack.push (pool.createVariable());
            return;
        }

//         if (stack == null) {
//             stack = new VarStack();
//             entrystack.push (createVariable());

//             entrylocals = (VarStack)pred.locals.clone();
//             stack = (VarStack)entrystack.clone();
//             locals = (VarStack)entrylocals.clone();
//             return;
//         } 

        int locs = initial_locals.size();
        for (int l=0; l < locs; l++) {
            RTVariable v1 = initial_locals.getAt (l);
            RTVariable v2 = pred.locals.getAt (l);
            if (v1 == null || v2 == null) continue;
            pool.unifyState (v1, v2);
        }
    }
        
    

    /**
     *  @return a string representation of this <code>BlockState</code> 
     */
    public String toString () {
        return "[locals:" + locals + " stack:" + stack + "]";
    }
};
