/**
 * CallAnalysis
 * 
 * tag: kkvilekval@acm.org Mon Jun 24 17:58:20 2002 (CallAnalysis)
 * Copyright kkvilekval@acm.org  See COPYRIGHT for details.
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Arrays;


//import pretime.classgraph.ClassGraph;
//import pretime.callgraph.CallGraph;
import mokan.prefetch.util.RTMethod;
import mokan.prefetch.util.RTClass;
import klogger.Logger;

import agl2.Bigraph;
import agl2.alg.AllPaths;
import agl2.alg.TopologicalSort;
import agl2.alg.AbstractGraphVisitor;

import mokan.prefetch.analysis.classanalysis.ClassAnalysis;


/**
 *  The <code>CallAnalysis</code> extracts data from the static
 *  call tree in order to determine whether prefetching should
 *  be done.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:35 $
 *  @author Kristian Kvilekval
 */
public class CallAnalysis implements Analysis{

    /* Private/Public Variables */
    private final static Logger log = Logger.getLogger ("prefetch.analysis.CallAnalysis");

    /** This class is a forward block visitor.
     *  It visits each block and creates 
     */

    /* Private/Public Variables */
    private Bigraph classgraph;
    private Bigraph callgraph;
    private MethodTable methods;


    int maxHeight;

    /**
     *  Create an instance of the analsis.
     */
    public CallAnalysis(ClassAnalysis ca, MethodTable table) {
        classgraph=ca.getStaticClassGraph();
        callgraph=ca.getStaticCallGraph();
        methods = table;
    }

    private void allPathsMethod () {
        AllPaths ap = new AllPaths (callgraph);

        ap.apply (callgraph.getRoot(), new AbstractGraphVisitor () {
                final Bigraph graph = callgraph;
                int callHeight;

                public void examineVertex (Object o) {
                    Set  vertices = graph.getAdjacentVertices(o);

                    // If node is a leaf then reset the height to 0.
                    if (vertices.size() == 0) {
                        callHeight = 0;
                        MethodState ms = methods.getState ((RTMethod)o);
                        //ms.callLevel = 0;
                    }
                }

                public void finishVertex (Object o) {
                    int tallest = -1;
                    MethodState ms;
                    Iterator kids = graph.getAdjacentVertices(o).iterator();
                    while (kids.hasNext()){
                        // Find the parent's height and 
                        RTMethod method = (RTMethod)kids.next();
                        ms = methods.getState (method);
                        
                        if (ms == null) {
                            log.warn ("No State for " +method);
                            continue;
                        }
                        if (ms.callLevel > tallest){
                            tallest  = ms.callLevel;
                            log.debug ("CallVisitor: " + ms + " height="+ms.callLevel);
                            if (callHeight > maxHeight){ 
                                maxHeight = callHeight;
                                log.debug ("CallVisitor:  maxheight="+maxHeight);
                            }
                        }
                    }
                    ms = methods.getState ((RTMethod)o);
                    ms.callLevel = tallest+1;
                }
            });
    }

    private void topologicalSortMethod () {

        Iterator ni = TopologicalSort.nodeIterator (callgraph, null);
        MethodState ms;
        while (ni.hasNext()) {
            RTMethod method = (RTMethod)ni.next();
            ms = methods.getState (method);
            if (ms == null) {
                log.warn ("No State for " +method);
                continue;
            }

            Set kids = callgraph.getAdjacentVertices (method);
            if (kids.size() == 0) {
                ms.callLevel = 0;
                continue;
            }

            int tallest = 0;
            Iterator ki = kids.iterator();
            while (ki.hasNext()) {

                RTMethod kid = (RTMethod)ki.next();
                MethodState ks;
                ks = methods.getState (kid);
                        
                if (ks == null) {
                    log.warn ("No State for " +method);
                    continue;
                }
                if (ks.callLevel > tallest){
                    tallest  = ks.callLevel;
                    log.debug ("CallVisitor: " + ks + " height="+ks.callLevel);
                    if (tallest > maxHeight){ 
                        maxHeight = tallest;
                        log.debug ("CallVisitor:  maxheight="+maxHeight);
                    }
                }
            }
            ms.callLevel = tallest+1;
        }
    }


    /** Visit the static the call graph and label the method with 
     *  the miniumum height.  We will use this to exclude prefetching
     *  from the lower nodes of the tree where most objects should
     *  have been prefetched by upper levels. 
     */
    public void visitProgram () {

        log.entering ("CallAnalysis","visitProgram");
        // The call visitor does 
//         CallVisitor cv = new CallVisitor (callgraph, methods);
//         cv.apply (callgraph.getRoot());

        topologicalSortMethod();

        log.info ("max callheight="+maxHeight);


        log.exiting ("CallAnalysis","visitProgram");
    }
    public MethodTable getMethods() {
        return methods;
    }
};
