/**
 * CallVisitor
 * 
 * tag: kkvilekval@acm.org Fri Jun 28 17:59:03 2002 (CallVisitor)
 * Copyright kkvilekval@acm.org  See COPYRIGHT for details.
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.Iterator;
import java.util.Set;
import java.util.HashMap;

import klogger.Logger;

import agl2.BidirectedGraph;

import mokan.prefetch.util.RTMethod;
//import pretime.callgraph.CallGraph;

/**
 *  The <code>CallVisitor</code> visits static call graph implementing
 *  the callgraph prefetching analysis.   We simply visit the graph
 *  using depth first search and label each node (call) with it's maximum
 *  height in the call graph.   We can use this number to filter proper 
 *  prefetches.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:35 $
 *  @author Kristian Kvilekval
 */
public class CallVisitor  {
    private final static Logger log = Logger.getLogger ("prefetch.analysis.CallAnalysis");

    /* Private/Public Variables */
    private final static Object WHITE = new Object();
    private final static Object GRAY = new Object();
    private final static Object BLACK = new Object();


    int currentHeight;
    int maxHeight;
    MethodTable methods;
    BidirectedGraph  graph;
    HashMap        colorMap;

    /**
     *  The default constructor
     */
    public CallVisitor (BidirectedGraph cg, MethodTable mt) {
        graph = cg;
        methods=mt;
        maxHeight = 0;
        colorMap = new HashMap ();
    }
    public void visit (Object u) {
        //visitor.discoverVertex (u);
        colorMap.put (u, GRAY);
        log.info ("visit " + u);

        Set  vertices = graph.getAdjacentVertices(u);

        // If node is a leaf then reset the height to 0.
        if (vertices.size() == 0) 
            currentHeight = 0;
        else {

            Iterator adjl = vertices.iterator();
            log.info (u + " kids=" +vertices.size());
            while (adjl.hasNext()) {
                Object adj = adjl.next();
                Object color = colorMap.get (adj);
                if (color == WHITE) {
                    // Predecessor p[v] = u for (u,v)
                    visit (adj);
                } else if (color == GRAY) {
                    //  Back Edge
                    currentHeight = 0;
                } else if (color == BLACK) {
                    // Cross or forward edge
                } else {
                    log.warn("No color associated with " +adj);
                }

            }
        }
        colorMap.put (u, BLACK);

        // Set any parent of the current node to the next higher level
        System.out.println ("checking parents = "+graph.getAdjacentInVertices(u).size());
        currentHeight++;
        Iterator parent = graph.getAdjacentInVertices(u).iterator();
        while (parent.hasNext()){
            // Find the parent's height and 
            MethodState ms = methods.getState ((RTMethod)parent.next());

            if (ms.callLevel < currentHeight){
                ms.callLevel = currentHeight;
                log.info ("CallVisitor: " + ms + " height="+ms.callLevel);
                if (currentHeight > maxHeight){ 
                    maxHeight = currentHeight;
                    log.info ("CallVisitor:  maxheight="+maxHeight);
                }
            }
        }
    }
    public void apply (Object root) {
        Iterator ni = graph.getVertices().iterator();
        while (ni.hasNext()) {
            Object v = ni.next();
            colorMap.put (ni, WHITE);
            //visitor.initializeVertex (ni);
        }

        if (root != null) {
            visit (root);
            return;
        }
        // 
        ni = graph.getVertices().iterator();
        while (ni.hasNext()) {
            Object v = ni.next();
            Object c = colorMap.get (ni);
            if (c == WHITE) 
                visit (v);
        }        
    }
};
