/*
 * MethodState
 *
 * tag: kkvilekval@acm.org Tue Dec  4 14:33:58 2001 (MethodState)
 *
 */
package mokan.prefetch.analysis.shapeanalysis;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import mokan.prefetch.util.RTMethod;
import klogger.Logger;

import de.fub.bytecode.generic.Type;
import de.fub.bytecode.generic.ReferenceType;

/**
 *  The <code>MethodState</code> manages all collected
 *  state for some method.
 *  @version $Revision: 1.3 $, $Date: 2003/02/06 03:20:23 $
 *  @author Kristian Kvilekval
 */

public class MethodState implements Serializable {

    //    private final static Logger log = Logger.getLogger ("prefetch.analysis.MethodState");

    private static HashMap global2variable = new HashMap();
    private static ShapePool globalPool = new ShapePool();

    /* Private/Public Variables */
    private RTMethod method;
    private ShapePool pool;
    private HashMap globalRefs;

    /** locals, rtReturn, rtException are valid only in pool */
    public VarStack locals;
    public RTVariable rtReturn;
    public RTVariable rtException;

    /** Lowest call level in callgraph that the program was called at.
     *  0 means that the method is a leaf
     *  1 means that the method calls only leafs
     *  2            ""         calls at least one method of hieght 1
     *  ...
     */
    public int callLevel;

    /**
     *  The default constructor
     */
    public MethodState (ShapePool p, RTMethod m) {
        pool = p; method = m; 
        callLevel = -1;        // Unitialized.
    }

    public String getName () {
        return method.getFullname();
    }
    public ShapePool getPool() {
        return pool;
    }
    public RTVariable getReturn () {
        return rtReturn;
    }
    public RTVariable getException () {
        return rtException;
    }
    public RTVariable getLocal(int i) {
        return locals.getAt (i);
    }
    public RTVariable getGlobalReference(String name) {
        if (globalRefs == null) {
            globalRefs = new HashMap();
        }
        RTVariable lv = (RTVariable)globalRefs.get(name);
        if (lv == null) {
            // Copy the reference from the global pool;
            RTVariable gv = (RTVariable)global2variable.get(name);
            if (gv != null) {
                lv = pool.getExternal(globalPool, gv);
            }
            globalRefs.put(name, lv);
        }
        return lv;
    }
    public void putGlobalReference (String name, RTVariable lv) {
        globalRefs.put(name, lv);
        // Copy the shape and replace it in the global pool.
        RTVariable gv = globalPool.getExternal(pool, lv);
        global2variable.put(name, gv);
    }
    public void saveGlobalReferences() {
        //Scan our global reference and save them to the global 
        // pool.
        if (globalRefs != null) {
            Iterator gi = globalRefs.entrySet().iterator();
            while (gi.hasNext()) {
                Map.Entry ge = (Map.Entry)gi.next();
                putGlobalReference ((String)ge.getKey(),
                                    (RTVariable)ge.getValue());
            }
        }
    }
    public HashMap getMethodGlobals () {
        return globalRefs;
    }
    /**
     *  @return a string representation of this<code>MethodState</code> 
     */
    public String toString () {
        String x =  method.toString() ;
        if (locals != null) 
            x = x + "[locals: " + locals + "; level:" + callLevel +"]";
        return x;
    }

    public String getLocalShape (int i) {
//         try {
            if (locals.getAt (i) != null) {
                String base = "self";
                if (method.isStatic ()) base = "l0";
                if (i > 0) base = "l"+i;
                return pool.getShapeString (locals.getAt (i), base);

            }
            return "";
//         } catch (RuntimeException e) {
//             log.severe ("MethodState", "getLocalShape",
//                         "exception arg="+ i + " locals=" + locals);
//             throw e;
//         }
    }

    public void logShapes (Logger log, int level) {

        int li = 0;
        if (!method.isStatic ()) {
            RTVariable v = locals.getAt (li);
            String localShape = getLocalShape (li);
                log.log(level, "ShapeAnalysis", "visitMethod", 
                         "shape of "+v+"->" + localShape);
                li ++;
        }
        Type[] argtypes = method.getMethodGen().getArgTypes();
        for (int ai=0; ai < argtypes.length; ai++,li++) {
            if (argtypes[ai] instanceof ReferenceType) {
                RTVariable v = locals.getAt (li);
                String localShape = getLocalShape (li);
                if (localShape.length()!=0){
                    log.log(level, "ShapeAnalysis", "visitMethod", 
                             "shape of "+v+"->" + localShape);
                }
            }
        }

        if (rtReturn != null) 
            log.log(level, "ShapeAnalysis", "visitMethod", 
                     "return "+rtReturn+"->"+pool.getShapeString (rtReturn, "ret"));
    }

    public void logStatistics(Logger log){
    }

};
