/*
 * 
 *
 * tag: kkvilekval@acm.org Sat Oct  6 13:23:43 2001 (RTAccess)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;
//import java.util.StringBuffer;

/**
 *  The <code>RTAccess</code> represents an runtime access to a 
 *  field.  By Follwoing the links of access to RTVariable
 *  you can build up an express for the compile time access.
 *  V a -f1> Access[R,8] -> v2 -f2> Access[W, 10]
 *  You can reconstruct that  
 *         a.f1 read at and a.f1.f2 was written at 10.
 *
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class RTAccess {

    public static final int CREATE = 0;
    public static final int READ   = 1;
    public static final int WRITE  = 2;
    public static final char[] ACCESS= new char[] { 'C', 'R', 'W' };

    /* Private/Public Variables */
    /** Count from beginning of routine */
    public static class  Access {
        int position;
        int access;
        RTVariable variable;
        Access (int p, int t) { position = p; access = t; }
        Access (int p, int t, RTVariable r) { 
            position = p; access = t; variable = r; 
        }
        public String toString () {
            return "acc["+variable+":"+ACCESS[access]+":"+position+"]";
        }
    }
    /** Debug name */
    //public String name;

    /** Map between field names and access */
    private HashMap field2access;

    /** Record an access to the variable */
    public void tagAccess (String name, int count, int type) {
        // For now record only the first access encountered 
        //   ?? maybe should lookat count value ??
        if (field2access.get (name) == null) 
            field2access.put (name, new Access (count, type));
    }
    public void tagAccess (String name, int count, int type, RTVariable rv) {
        // For now record only the first access encountered 
        //   ?? maybe should lookat count value ??
        if (field2access.get (name) == null) 
            field2access.put (name, new Access (count, type, rv));
    }

    /**
     *  The default constructor
     */
    public RTAccess () {
        field2access = new HashMap();
    }

    
    /** Retrieve the variable denoted by the field name fname*/
    public RTVariable getVariable (String fname) {
        Access a = (Access)field2access.get (fname);
        return a.variable;
    }
    /** Set the field name to be a link to variable V */
    public RTVariable setVariable (String fname, RTVariable v) {
        Access a = (Access)field2access.get (fname);

        RTVariable prev = a.variable;
        a.variable = v;

        return prev;
    }

    /** Pull all the fields in from other 
     * NOTE: shouldn't we choose the smaller edge.
     */
    public void mergeFields (RTAccess other) {
        if (this == other) return;

        Iterator fi = other.field2access.keySet().iterator ();
        while (fi.hasNext()) {
            String fname = (String)fi.next();

            // Hmm this looks funny 
            // Is there someway to make these do the 
            // merge right here instead of doing where I 
            // being called from(AccessState)???
            // Some state object perhaps??
            if (field2access.containsKey (fname)) 
                continue;

            field2access.put (fname, other.field2access.get (fname));
        }
    }


    /** Return the set of field names 
     *  type Set <String> 
     */
     public Set getFields () {
         return field2access.keySet ();
     }

    public Access getAccess (String fname ) {
        return (Access)field2access.get (fname);
    }


    /**
     *  @return a string representation of this <code>RTAccess</code> 
     */
    public String toString () {
        return "ra" + field2access;

    }


};
