/*
 * 
 *
 * tag: kkvilekval@acm.org Mon Nov 12 19:19:48 2001 (ShapePaths)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Collections;
import java.util.Comparator;
import java.util.StringTokenizer;

import klogger.Logger;

import agl.AbstractGraphVisitor;
import mokan.prefetch.util.ObjIntMap;
import mokan.prefetch.util.Assert;

/**
 *  The <code>ShapePaths</code> construct path expressions for a
 *  variable in the shape graph..  The shape graph may contain cycles
 *  and these will represent by looping constructs.  The Alogrtihm
 *  explores all paths through the graph by examining the edges.  It
 *  keeps a current path expression and a stack representing the nodes
 *  on the path it has visited.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class ShapePaths {

    private final static Logger log = Logger.getLogger ("prefetch.analysis.ShapePath");

    /* Private/Public Variables */
    private static class TooManyShapesException extends RuntimeException {
        
    }


    public static class PathCollector extends AbstractGraphVisitor {

        ShapePool  pool;
        ShapeGraph graph;
        /** the list of paths and the nodes on those paths */
        ArrayList nodepaths;
        /** the list of paths and the edges on those paths */
        ArrayList edgepaths;
        /** the list of paths and the edges on those paths */
        ArrayList strpaths;
        /** current path we are building */
        LinkedList path;
        /* edges in current path */
        LinkedList edges;
        /* count of non-cyclic edges of nodes */ 
        ObjIntMap    counts; 
        /** stack of string reps for the path being visited */
        LinkedList  pathexprs;
        int         visitedcount = 0;

        public PathCollector (ShapePool pool) {
            this.pool  = pool;
            graph = pool.gb;
            nodepaths = new ArrayList();
            edgepaths = new ArrayList();
            strpaths = new ArrayList();
            path  = new LinkedList();
            edges = new LinkedList();
            counts = new ObjIntMap();
            pathexprs = new LinkedList();
            pathexprs.addLast ("");
        }
        
        /** Visit all edges adding to the path.. When 
         *  a node has no kids it's the end of path
         */
        public void visit (Object o) {

            String pathexpr = (String)pathexprs.getLast() ;

            log.debug ("path visiting " + o);

            visitedcount++;

            if (visitedcount > 10000) {
                log.warn ("big tree:" + o);
            }
            if (path.contains (o)) {
                log.trace ("recursive : removing edge from " + o);
                counts.putInt (o, counts.getInt(o)-1);

                // Change the path expression to represent the loop.
                // Find the common prefix by walking down the stack
                // looking for this node.  We are able to look back 
                // through the path expressions find the corresponding
                // path.  We then use the two path path to find the 
                // common prefix and modiy the current path expression.

                ListIterator ni = path.listIterator ();
                ListIterator pi = pathexprs.listIterator ();
                String header ="";
                String newpath = pathexpr;
                Object node;

                while (ni.hasNext()) {
                    node = ni.next(); 
                    header = (String)pi.next();
                     if (node == o) break;
                }
//                 while (ni.hasPrevious()) {
//                     node = ni.previous();
//                     header = (String)pi.previous();
//                     if (node == o) break;
//                 }

                log.trace("recursive path: " + header + '[' +newpath + '-'
                          + header +']');
                int prefix_len = 0;

                Assert.assertion (!pathexpr.equals(header));
                
                while (prefix_len < header.length() 
                       && prefix_len < newpath.length()
                       && newpath.charAt (prefix_len)==header.charAt(prefix_len))
                    prefix_len++;
                if (prefix_len>0 && header.charAt(prefix_len-1) =='.')prefix_len--;
                pathexpr = (header.substring(0, prefix_len) +".["
                            + header.substring (prefix_len) 
                            + newpath.substring(prefix_len)+ ".]");
                
                if (pathexpr.indexOf ("[.]") >=0) {
                    log.severe("constructed bad expression: header{"+header+"}  prepath{"+newpath+"} final{"+pathexpr+"}");
                }
                //pathexpr = simplerPath (pathexpr);
                pathexprs.removeLast();
                pathexprs.addLast (pathexpr);

                log.debug ("recursive path:" + pathexpr);
            }
            
            path.add(o);
            
            LinkedList adj = new LinkedList (graph.getAdjacentEdges(o));
            Collections.sort (adj, new Comparator () {
                    public int compare(Object a, Object b){
                        return ((ShapeEdge)a).mincount - ((ShapeEdge)b).mincount;
                    }
                });

            log.trace ("node " + o +':' + adj);
            Iterator  ei = adj.iterator();
            while (ei.hasNext()) {
                ShapeEdge e = (ShapeEdge)ei.next();
                RTVariable b = pool.getVariable(e.getSink());
                if (!edges.contains (e)) {
                    counts.putInt (o, counts.getInt(o)+1);


                    log.debug("following edge:"+o+'.'+e+'.'+b );

                    pathexprs.addLast (pathexpr + '.' + e.field+'.'+b);
                    edges.add(e);
                    visit (b);
                    edges.removeLast();
                    pathexprs.removeLast();
                } else {
                    log.debug ("skiping path edge: "+o+'.'+e+'.'+b );
                }
                
            }
            
            //log.trace (o+" has "+counts.getInt(o) + " edges");
            if (counts.getInt(o) == 0) {
                log.trace ("endpoint found:" + pathexpr);
                nodepaths.add (new ArrayList(path));
                edgepaths.add (new ArrayList(edges));
                strpaths .add (simplerPath (pathexpr));
                if (strpaths.size() > 100) throw new TooManyShapesException();
            }
            
            path.removeLast();
        }
    }
    public static String simplerPath (String path) {
        // Remove .X.[.X.] == .[X}
        // Remove .[X.].X == .[X} from the expr

        Assert.assertion (path.indexOf("[.]") == -1);
        String simpler = path;
        boolean modified = true;
        while (modified) {
            StringTokenizer  st = new StringTokenizer(simpler,".");
            modified = false;
            while (st.hasMoreTokens()) {
                String tok = st.nextToken();
                
                if (tok.charAt(0) == '[' || tok.charAt(0) == ']') continue;
                
                String check = '.' + tok + ".[." + tok + ".]";
                int found = simpler.indexOf (check);
                if (found >=0) {
                    modified = true;
                } else {
                    check =  ".[." + tok + ".]." + tok;
                    found = simpler.indexOf (check);
                    if (found >= 0) modified = true;
                }
                if (modified) {
                    simpler = (simpler.substring (0, found)
                               +  ".[." + tok + ".]"
                               + simpler.substring (found + check.length()));
                    break;

                }
            }
        }
        Assert.assertion (simpler.indexOf("[.]") == -1);

        return simpler;
    }

    static boolean  checkPath(String simpler) {

        StringTokenizer  st = new StringTokenizer(simpler,".");
        boolean modified = false;
        while (st.hasMoreTokens()) {
            String tok = st.nextToken();
            
            if (tok.charAt(0) == '[' || tok.charAt(0) == ']') continue;
            
            String check = '.' + tok + ".[." + tok + ".]";
            int found = simpler.indexOf (check);
            if (found >=0) {
                modified = true;
            } else {
                check =  ".[." + tok + ".]." + tok;
                    found = simpler.indexOf (check);
                    if (found >= 0) modified = true;
            }
            if (modified) {
                return false;
            }
        }
        return true;
    }


    public static ArrayList allpaths (ShapePool p, Object root) {
        PathCollector pc = new PathCollector(p);

        try { 
            pc.visit (p.getVariable (root));
        } catch (TooManyShapesException e) {
            log.warn ("too many shapes");
        }

        //System.out.println (pc.nodepaths);
        //System.out.println (pc.edgepaths);
        //System.out.println (pc.strpaths);
        log.info ("ShapePaths", "allpaths", "found " + pc.strpaths.size());
        return pc.strpaths;
    }
};
