/*
 * 
 *
 * tag: kkvilekval@acm.org Mon Nov 12 17:47:18 2001 (ShapePool)
 *
 */


package mokan.prefetch.analysis.shapeanalysis;

import java.io.Serializable;

import java.util.Set;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Comparator;
import java.util.Arrays;
import java.util.Collections;


import klogger.Logger;

import mokan.prefetch.util.Pair;
import mokan.prefetch.util.UnionFind;

import agl.GraphListener;
import agl.GraphEvent;
import agl.Paths;

/**
 *  The <code>ShapePool</code> is a group of related shape graphs.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */
public class ShapePool  implements   GraphListener, Serializable {
    private final static transient Logger log = Logger.getLogger ("prefetch.analysis.ShapePool");

    private static class VarComparator implements Comparator {
        public int compare (Object o1, Object o2) {
            return (((Integer)((Pair)o1).o1).intValue() 
                    - ((Integer)((Pair)o2).o1).intValue());
        }
        public boolean equals (Object comp) {
            return  (this == comp); 
        }
    }

    /* Private/Public Variables */
    public  ShapeGraph gb;
    private UnionFind nodes;
    private boolean changed;
    /** Map copied external to local copies. This is usuefull when we
     * have already copied a variable over from another pool and we
     * would like to it's corersponding shape in the current pool.
     * Since this is a map<RTVariable, RTVariable> each variable must
     * be unique.
     */
    private HashMap externals;

    /**
     *  The default constructor
     */
    public ShapePool () {
        gb = new ShapeGraph();
        nodes = new UnionFind();
        externals = new HashMap();
        gb.addListener (this);
        changed = false;
    }
//     void finalize () {
//         gb.remGraphListener (this);
//     }
    /** Create a new variable in this shape pool */
    public RTVariable createVariable () {
        return new RTVariable();
    }

    private ShapeEdge getEdge(RTVariable v, String name){

        Set edges = gb.getAdjacentEdges (v);
        return findEdge(edges, name);
    }
    private ShapeEdge findEdge(Set edges, String name){
        Iterator ei = edges.iterator(); 
        while (ei.hasNext()) {
            ShapeEdge se = (ShapeEdge)ei.next();
            if (name.equals (se.field))
                return se;
        }
        return null;
    }

    public void resetChanged () {
        changed = false;
    }
    public boolean isChanged() {
        return changed;
    }
    public void graphEvent (GraphEvent ge) {
        changed = true;
    }

    public RTVariable getVariable (Object v) {
        return (RTVariable)nodes.find (v);
    }

    private static Set setIntersect (Set s1, Set s2) {
        HashSet ret = new HashSet();
//         if (s1.size() > s2.size)() {
//             Set t = s1; s1 = s2; s2 = t; 
//         }
        Iterator si = s1.iterator();
        while (si.hasNext()) {
            Object o = si.next();
            if (s2.contains (o))
                ret.add (o);
        }
        return ret;
    }
    
    public RTVariable accessAllocate (int count) {
        RTVariable v = createVariable();
//         ShapeEdge  se = new  ShapeEdge("<create>",count);
//         se.access = "C";
//         gb.addEdge (v, createVariable(), se);
//         return (RTVariable)se.getSource();
        return v;
    }


    public RTVariable accessField (RTVariable v, String f, int count, boolean w) {
        ShapeEdge se = getEdge (v = getVariable(v), f);
        if (se == null) {
            se = new ShapeEdge (f, count);
            se.access = w?"W":"R";
            gb.addEdge (v, createVariable(), se);
        }
        
        RTVariable b =  (RTVariable)getVariable (se.getSink());
        log.debug ("field " + v + '.'+f+'.'+b);
        return b;
    }
    public RTVariable accessArray (RTVariable v, String expr, int count) {
        ShapeEdge se = getEdge (v = getVariable(v), "<array>");
        if (se == null) {
            se = new ShapeEdge ("<array>", count);
            gb.addEdge (v, createVariable(), se);
        }
        RTVariable b = (RTVariable)getVariable(se.getSink());
        return b;
    }


    // Given two variables (roots of shape graphs), unify the 
    // graphs by merging matching nodes and edges.

    public void unifyState (RTVariable v1, RTVariable v2) {
        LinkedList workQ = new LinkedList ();
        workQ.add (v1); workQ.add(v2);

        while (!workQ.isEmpty()) {
            v1 = getVariable(workQ.removeFirst());
            v2 = getVariable(workQ.removeFirst());

            log.debug ("processing " + v1 +"<->" + v2);
            if (v1 == v2) continue;
            if (v1 ==  null || v2 == null) {
                log.severe ("ShapePool", "unify", "v1="+v1+" v2="+v2);
                continue;
            }
            if (nodes.sameSet (v1, v2)) continue;


            nodes.union(v1, v2);
            RTVariable  rep = (RTVariable)nodes.find(v1);

            log.debug("union " + v1 +"<->"+v2 +"=" + rep);
            // Check the maps for conflicts (same field)
            // and add these to the workQ.
//             ShapeEdge edge;
//             Set v1Fields = gb.getAdjacentEdges(v1);
//             Set v2Fields = gb.getAdjacentEdges(v2);
//             log.debug ("fields:" + v1 + ":"+v1Fields);
//             log.debug ("fields:" + v2 + ":"+v2Fields);

            // Scan for the common edges
            // Add these (endpoints) to the work list for further
            // unification.   

            //RTVariable v1b, v2b, repb;
            if (v1 != rep) {
                unifyNodeEdges(rep, v1, workQ);
//            } else if (v2 != rep) {
            }else {
                unifyNodeEdges(rep, v2, workQ);
            }
        }
    }
    
    // Unify a single nodes edges by discovering which 
    // are missing and copying them and scheduling unification
    // of common edges.
    private void unifyNodeEdges(RTVariable rep, RTVariable v, LinkedList workQ) {
        RTVariable vb,repb;
        Set vEdges = gb.getAdjacentEdges(v);
        Set repEdges = gb.getAdjacentEdges(rep);

        Iterator ci = vEdges.iterator();
        while (ci.hasNext()){
            ShapeEdge vedge = (ShapeEdge)ci.next();
            ShapeEdge repEdge = findEdge (repEdges, vedge.field);
            vb =  getVariable(vedge.getVertexB());
            log.debug("checking field:"+toString(v)+'.'+vedge+'.'+toString(vb));
            if (repEdge == null) {
                gb.addEdge (rep, vb, vedge);
                log.debug("add field:"+rep+":"+vedge.field+"->"+vb);
            }else {
                repEdge.unify(vedge);
                // Add the endpoints of the edge into the workque.
                repb = getVariable(repEdge.getVertexB());
                workQ.add(vb);
                workQ.add(repb);

                log.debug("unify field:"+repEdge.field+" " + repb +"="+vb);
            }
        }
        log.debug("remove fields:" + v);
        gb.remEdges (v);
    }



    public RTVariable getExternal (final ShapePool formalPool, 
                                   RTVariable formal) {
        formal = formalPool.getVariable (formal);
        RTVariable nf = (RTVariable)externals.get(formal);
        if (nf == null) {
            nf = copyShape (formalPool, formal);
            //externals.put (formal, nf);
        }
        return getVariable(nf);
    }

    /* Copy a vairiable from any shape pool, creating a new variable graph */
    public RTVariable copyShape (final ShapePool pool, RTVariable rv) {
        RTAccess  ra, na;
        RTVariable nv = createVariable();
        LinkedList workQ = new LinkedList ();
        //HashMap  old2new = new HashMap();
        
        RTVariable copied = nv;

        workQ.add (nv);
        workQ.add (rv);

        while (!workQ.isEmpty()) {
            nv = getVariable(workQ.removeFirst());
            rv = pool.getVariable(workQ.removeFirst());

            // nv is just created.
            // Save the association between these vars
            //old2new.put (rv, nv);
            externals.put (rv, nv);
            log.debug ("copied " + rv +"->" + nv);

            // For each field in rv add a field to nv
            Iterator fi = pool.gb.getAdjacentEdges(rv).iterator();
            while (fi.hasNext()) {
                //Copy eache edge maintain relationships between nodes.

                RTVariable of, nf;
                ShapeEdge e = (ShapeEdge)fi.next();
                
                of = pool.getVariable (e.getVertexB());

                log.debug ("old edge " + rv+'.'+e.field+'.'+of);

                //if (old2new.containsKey(of)) 
                if (externals.containsKey(of)) {
                    // We have mapped this variable before .. there is no need
                    // to search it edges.
                    //nf = (RTVariable)old2new.get (of);
                    nf = getVariable(externals.get (of));
                } else {
                    // Otherwise keep following.
                    nf = createVariable();
                    workQ.add (nf);
                    workQ.add (of);

                    externals.put (of, nf);
                }
                // Create the new edge 
                gb.addEdge (nv, nf, e.copy());
                log.debug ("new edge " + nv+'.'+e.field+'.'+nf);
            }
        }
        return copied;
    }

    /** Produce s staring representing the shapes of variable V,
     *  with  base is used to prefix all shape strings.  This version
     *  sorts the elements of the string by expected count.
     *  The string is made of access fields [RW]:<count>:<expr>
     *  seperated by commas.
     *  @param v a variable with in the shape graph.
     *  @param the base string (self, arg0, etc)..
     */
    public String getShapeStringOLD (RTVariable v, String base) {
        // display the shape graph of V using a base name of base
        StringBuffer sb = new StringBuffer();
        HashSet visited = new HashSet ();
        LinkedList workQ = new LinkedList();
        HashMap    v2fqn = new HashMap();
        ArrayList  varlist = new ArrayList();

        int distance = 0;
        
        v.distance = distance;
        workQ. add (new Pair (v, base));
        
        while (!workQ.isEmpty()) {
            Pair p = (Pair)workQ.removeFirst();
            v = (RTVariable)getVariable (p.o1);
            base = (String)p.o2;

            distance = v.distance;

            if (v == null) continue;
            if (visited.contains (v)) {
                // We have a recursive datatype 
                // Mark it?
                log.warn ("ShapePool", "getShape", "recursive node:"+v);
                // Not true.. could be two different paths
                continue;
            }

            visited.add (v);

            Iterator fi = gb.getAdjacentEdges(v).iterator ();
            
            while (fi.hasNext()) {
                ShapeEdge edge = (ShapeEdge)fi.next();
                String fqn = base + "." + edge.name();

                RTVariable sub = getVariable (edge.getSink());
                sub.distance = distance + edge.mincount;
                workQ.add (new Pair (sub, fqn));
                varlist.add (new Pair (new Integer(sub.distance), 
                                       (edge.access
                                        +":" + (distance+edge.mincount)
                                        +":" + fqn)));
            }
        }

        Pair[] vars = (Pair[])varlist.toArray(new Pair[varlist.size()]);
        Arrays.sort (vars,  new VarComparator());
        //System.out.println (Arrays.asList(vars));
        for (int ri=0; ri < vars.length; ri++) {
            Pair p   = (Pair)vars[ri];
            if (sb.length() != 0) sb.append (',');
            sb.append (p.o2);
        }
        return sb.toString();
    }

//     public String getShapeString1 (RTVariable va, String base) {
//         // Do a depth first prioritized search of the graph
//         // collecting path data was we go.  A path is defined
//         // from the variable v to any node that has 
//         // 0 outoging edges (excluding self loops).

//         HashSet visited = new HashSet ();
//         HashMap v2path = new HashMap();
//         LinkedList workQ = new LinkedList();
//         ArrayList path = new ArrayList();
//         workQ.add (va);
//         v2path.put (va, base);
//         while (!workQ.isEmpty()) {
//             va   = (RTVariable)workQ.removeLast();
//             String path =  (String)v2path.get(va);
//             visited.add (va);
//             int edgecount = 0;
//             Iterator fi = gb.getAdjacentEdges(v).iterator ();
//             while (fi.hasNext()) {
//                 // Need to pick edges in order..
//                 ShapeEdge edge = (ShapeEdge)fi.next();
//                 String newpath = path + '.' + edge.field;
//                 edgecount++;
//                 RTVariable vb = getVariable (edge.getSink());
//                 if (visited.contains (vb)) {
//                     String dest = (String)v2path.get(vb);
//                     // destpath = destprefix [ newpath - destprefix ]
//                     String destpath = dest + '[' + newpath.substring(dest.length()) + ']';
                    
//                     v2path.put (vb, destpath);
//                     edgecount --;
//                     continue;
//                 }
//                 workQ.add (vb);
//                 v2path.put (vb, newpath);
//             }
//         }
//     }

    public void dfsshape (RTVariable va, LinkedList currentpath,
                          HashSet visited,
                          HashMap v2path, ArrayList endpoints) {

        currentpath.addLast (va);
        String path = (String)v2path.get(va);

        LinkedList edges = new LinkedList (gb.getAdjacentEdges(va));
        Collections.sort (edges, new Comparator () {
                public int compare(Object a, Object b){
                    return ((ShapeEdge)a).mincount - ((ShapeEdge)b).mincount;
                }
            });
                
        log.debug ("dfsvisit: " + va + edges);

        Iterator adjl = edges.iterator();
        int edgecount = 0;
        visited.add (va);
        
        while (adjl.hasNext()) {
            ShapeEdge edge = (ShapeEdge)adjl.next();
            RTVariable vb = getVariable(edge.getSink());
            String newpath ;

            if (path != null ) 
                newpath = path + '.' + edge.field;
            else 
                newpath = '.' + edge.field;
            edgecount++;

            log.debug ("building path " + newpath +"->"+vb);

            if (currentpath.contains (vb)) {
                String dest = (String)v2path.get(vb);
                if (dest == null) dest = "";

                // Find the common prefix of the path
                // Combine using the dest path and current path
                // destpath = destprefix [ destsuffix . newpath - destprefix ]
                log.debug("recursive path: " + dest + '[' +newpath + '-'
                          + dest +']');

                edgecount --;
                String destpath;
                int prefix_len = 0;
                    
                while (prefix_len < dest.length() 
                       && prefix_len < newpath.length()
                       && newpath.charAt (prefix_len)==dest.charAt(prefix_len))
                    prefix_len++;
                if (prefix_len > 0 && dest.charAt (prefix_len-1) =='.')prefix_len--;
                destpath = (dest.substring(0, prefix_len) +".["
                            + dest.substring (prefix_len) 
                            + newpath.substring(prefix_len)+ ".]");

                log.debug ("recursive path:" + destpath);
                    
                v2path.put (vb, destpath);
                continue;
            } 

            log.debug ("visited  does not have " + vb);
            v2path.put(vb, newpath);
            dfsshape (vb, currentpath, visited, v2path, endpoints);
        }
        log.debug("finishing: " + va + ':'+ path +':' +edgecount);
        //if (edgecount == 0) {
             endpoints.add (v2path.get(va));
             //}
        currentpath.removeLast();
    }
    /*
    public boolean dfsshape (RTVariable va, 
                             LinkedList currentpath,
                             String newpath) {

        boolean looped = false;
        String path = (String)v2path.get(va);

        if (nodepath.contains (va)) {
            // If the endpoint is in the path then 
            // we entered a loop and visiting the node 

            String dest = (String)v2path.get(va);
            if (dest == null) dest = "";
            
            // Find the common prefix of the path
            // Combine using the dest path and current path
            // destpath = destprefix [ destsuffix . newpath - destprefix ]
            log.debug("recursive path: " + dest + '[' +newpath + '-'
                      + dest +']');

                String destpath;
                int prefix_len = 0;
                    
                while (prefix_len < dest.length() 
                       && prefix_len < newpath.length()
                       && newpath.charAt (prefix_len)==dest.charAt(prefix_len))
                    prefix_len++;
                if (prefix_len > 0 && dest.charAt (prefix_len-1) =='.')prefix_len--;
                destpath = (dest.substring(0, prefix_len) +".["
                            + dest.substring (prefix_len) 
                            + newpath.substring(prefix_len)+ ".]");



            looped = true;
        }

        nodepath.add(va);

        LinkedList edges = new LinkedList (gb.getAdjacentEdges(va));
        Collections.sort (edges, new Comparator () {
                public int compare(Object a, Object b){
                    return ((ShapeEdge)a).count - ((ShapeEdge)b).count;
                }
            });
                
        log.debug ("dfsvisit: " + va + edges);

        Iterator adjl = edges.iterator();
        int edgecount = 0;
        
        while (adjl.hasNext()) {
            ShapeEdge edge = (ShapeEdge)adjl.next();
            RTVariable vb = getVariable(edge.getSink());
            String newpath;
        
            edgecount++;

            if (!edgepath.contains (edge)) {
                edgepath.addLast (edge);
                if (dfsshape (vb, edgepath, nodepath)) 
                    edgecount--;
                edgepath.removeLast (edge);
            }
        }
        if (edgecount == 0) {
             endpoints.add (v2path.get(va));
        }
        nodepath.removeLast ();

        return looped;
    }
    */




    public String getShapeString (RTVariable va, String base) {


        String shapes = "";
        Iterator ei = ShapePaths.allpaths (this, va).iterator();

        
        /*
          ArrayList endpoints = new ArrayList();
          HashSet   visited   = new HashSet ();
          HashMap   v2path    = new HashMap ();
          LinkedList nodepath     = new LinkedList();
          dfsshape (getVariable(va),nodepath,visited, v2path, endpoints);
          Iterator ei = endpoints.iterator();
        */
        while (ei.hasNext()) {
            String path = (String) (ei.next());
            if (path != null) 
                shapes = shapes + base+path + ',';
        }

        log.debug("newshape:" + shapes);

        return shapes;
    }


    /** Produce a graph suitable for drawing from the shape graph.
     *  Nodes are labelled with the unified values and edges are
     *  labelled with the field names that connect them.  The output
     *  of this function may be used the system found at
     *  http://www.research.att.com/sw/tools/graphviz/ 
     *
     *  Example use:
     *   System.out.println ("digraph G{")
     *   System.out.println ("label =" + name)
     *   root= GetVariable(..);
     *   System.out.println (pool.graph (root));
     *   System.out.println ("}")
     *  
     */
    public String graph (RTVariable va) {
        StringBuffer sb = new StringBuffer ();

        sb.append (getVariable(va) +"[label=\"" + getVariable(va) + ":root\"];\n");
        Iterator ni = gb.getVertices().iterator();
        while (ni.hasNext()) {
            Object n = ni.next();
            Iterator adjs = gb.getAdjacentEdges (n).iterator();
            while (adjs.hasNext()) {
                ShapeEdge e = (ShapeEdge)adjs.next();
                sb.append(getVariable(n)); 
                sb.append ("->");
                sb.append(getVariable(e.getSink())); 
                sb.append ("[ label = \"" + e +':' + e.mincount + ','+e.maxcount + "\" ]");
                sb.append(";\n");
            }
        }
        return sb.toString ();

    }


    public String toString (RTVariable rv) {
        return rv.toString() + '('+getVariable (rv) +')';
    }

};
