/*
 * 
 *
 * tag: kkvilekval@acm.org Wed Nov 14 09:47:52 2001 (StackVisitor)
 *
 */
package mokan.prefetch.analysis.shapeanalysis;

import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.HashMap;

import de.fub.bytecode.Constants;
import de.fub.bytecode.generic.*;

//REM import precache.callgraph.CallGraph;
import mokan.prefetch.analysis.classanalysis.ClassAnalysis;

import mokan.prefetch.util.Assert;
import mokan.prefetch.util.RTMethod;
import klogger.Logger;

/**
 *  The <code>BlockVisitor</code> description.
 *   
 *  @version $Revision: 1.4 $, $Date: 2003/02/06 03:20:23 $
 *  @author Kristian Kvilekval
 */
public class StackVisitor extends DefaultVisitor{

    private final static Logger log = Logger.getLogger ("prefetch.analysis.StackVisitor");

    /* Private/Public Variables */
    /** Current method */
    private RTMethod method;
    /** Map from method to MethodState for calls */
    private MethodTable methods;
    // REM /** Callgraph of program */
    // REM private CallGraph callgraph;
    /** Rsults of class analysis */
    private ClassAnalysis classanalysis;
    /** The # of accesses in the block used in ordering accesses*/
    private int accessCount;
    /** The Block offset .. used in ordering accesses 
     ** instr offset = instruction - blockstart + blockoffset 
     **/
    private int blockStart;
    private int blockOffset;
    private int instruction;

    /** Keep track of the state of the dataflow */
    private BlockState  currentBlock;
    private MethodState currentMethod;
    private ShapePool   current;
    
    public VarStack locals;
    public VarStack stack;

    /**
     *  The default constructor
     */
    public StackVisitor(MethodTable methods, ClassAnalysis ca) {
        this.methods = methods;
        // REM this.callgraph = cg;
        this.classanalysis = ca;


//         current = new AccessState (m.getMaxLocals(), m.getMaxStack());
//          locals = current.locals;
//          stack  = current.stack;

    }



    /** initialize the parameters and stack */
    private void initLocals (VarStack loc) {
        // 'this' parameter
        if (!method.isStatic ())
            loc.push (create(null));

        //  Create a stack for the formal parameters
        Type[] ty = method.getMethodGen().getArgTypes();
        for (int ti =0; ti < ty.length; ti++) {
            if (ty[ti] instanceof ReferenceType) 
                loc.push (create (null));
            else
                loc.push (ty[ti].getSize());
        }
        // Fill in extra slots ??
        loc.push (method.getMaxLocals() - loc.size());
    } 
    public void prepareMethod (RTMethod m, ShapePool p) {

        this.method = m;
        this.locals = new VarStack (m.getMaxLocals());
        this.current = p;

        initLocals (locals);

        MethodState ms = new MethodState (p, m);
        ms.locals = locals;
        methods.putState (m, ms);
    }


    public void initMethod (RTMethod m) {
        this.method = m;
        this.instruction = 0;
        this.accessCount = 0;
        
        currentMethod = (MethodState)methods.getState (m);
        current = currentMethod.getPool();

        log.trace ("StackVisitor", "method" , "Begin " + m.toString());
//         current = new AccessState (m.getMaxLocals(), m.getMaxStack());
        locals = currentMethod.locals;
        stack  = new VarStack (m.getMaxStack());
    }

    public void finalMethod (RTMethod m) {
        // Transfer the current block state the method state
        // ???Should we merge all returning(ending) blocking
        // into a single block before doing this????
        currentMethod.locals = locals;
        methods.putState (m, currentMethod);

        // Merge global reference back into global state.
        currentMethod.saveGlobalReferences();

        log.trace ("StackVisitor", "method" , "End " + m.toString());
    }
    

    /** Visit the block b with current state set to */
    public void visitBlock (BlockNode b, BlockState s) {
        // Get the instruct list and visit each instruction

        if (s == null) {        // First block has null state
            s = new BlockState(currentMethod.getPool(), locals, stack);
        }
        locals = s.locals;
        stack  = s.stack;
        currentBlock = s;

//         if (b.isExceptionHandler ()) {
//             // Ecxeption handlers are entered with 
//             // one element on the stack..
//             s.stack = stack = (VarStack)s.entrystack.clone();
//         }

        // Set the blockOffset to the min of predecessor blocks end
        // intruction counts.  This permits parallel Blocks to compete
        // for spots in the cache.
        blockOffset = 0;
        Iterator pi = b.predecessors();
        if (pi.hasNext()){
            blockOffset = Integer.MAX_VALUE;
            while (pi.hasNext()) {
                BlockNode pred = (BlockNode)pi.next();
                if (pred.end < blockOffset) 
                    blockOffset = pred.end;
            }
        }

        MethodGen mg = method.getMethodGen();
        InstructionList     il  = mg.getInstructionList();
        InstructionHandle[] ihs = il.getInstructionHandles();

        VarStack debugstack=null;
        VarStack debuglocals=null;
        try {
            blockStart = b.start;
            for (int ii = b.start ; ii <= b.end; ii++) {
                // The next two debug lines can be removed at ANY time
                // Just used in the case of errors.
                debuglocals = (VarStack)locals.clone();
                debugstack = (VarStack)locals.clone();
                int stackbefore = stack.size();
                instruction = ii;
                ihs[ii].accept (this);
                //log.trace ("instr " + ii +  " stack=" +stack.size());
                log.trace ( "instr " + ii +" current[locals:" + locals + " stack:" + stack + "]");
            }
        } catch (Exception e) {
            log.severe ("Exception while processing " + instruction + " in " + method);
            log.severe (" before[locals:" + debuglocals + " stack:" + debugstack + "]");
            log.severe (" current[locals:" + locals + " stack:" + stack + "]");
            e.printStackTrace();
            Runtime.getRuntime().exit(0);
            
        }
        
    }

    public BlockState currentState () {
        return currentBlock;
    }

    /** Currently used for debugging 
    public String methodShapes () {
        StringBuffer sb = new StringBuffer();

        MethodGen mg = method.getMethodGen();
        Type[] argtypes = mg.getArgTypes();

        // Argument Names are unavailable.. Boo (see BCEL methodGen).
        int ai = 0;
        if (!method.isStatic()) {
            sb.append (current.getShapeString ((RTVariable)locals.get(0), 
                                              "self"));
            ai = 1;
            sb.append(',');
        }

        for (int ap=0; ap < argtypes.length; ap++, ai++) {
            if (argtypes[ap] instanceof ReferenceType) {
                RTVariable arg = (RTVariable)locals.get (ai);
                if (arg != null)
                    sb.append (current.getShapeString (arg, "arg"+ap));
            }
        }
        return sb.toString();
    }

    */

    private RTVariable create(Instruction obj) {
        return current.createVariable();
    }

    /** Create an allocated variable in the current context 
     *  We mark this with a create access.
     */
    private RTVariable allocate (Instruction obj) {
//TIME        return  current.accessAllocate(++accessCount);
        return  current.accessAllocate(instruction);
    }
    
    /** record the use of an array  */
    private RTVariable arrayUse(RTVariable v, boolean write, 
                                Instruction obj) {
//TIME        return current.accessArray(v, null, ++accessCount);
        return current.accessArray(v, null, instruction);
    }

    /** record the use of a field */
    private RTVariable fieldUse(RTVariable v, boolean write, 
                                FieldInstruction obj) {
        String fname = obj.getFieldName (method.getConstantPoolGen());
        
        // TODO 
        //
        // Constuct a field name from the declaring class of the
        // targeted instruction.  Such that the field name will now be
        //    fname = "declclass:fname"
        // This should solve two issues:
        // 1.  Stack position used by diffent classes and the
        //     have the field name
        //     Ex.  if () { 
        //            Iterator x = type1.iterator();
        //              x.next  // type1.routine
        //          } else { 
        //            Iterator x = type2.iterator();
        //              x.next  // type2.routine
        //          }
        // 2. while using shape graphs we will be able to
        //    cache reflection results.

        String tname = obj.getClassName(method.getConstantPoolGen()).toString();

        fname = tname + ':'+ fname;

//        return current.accessField (v, fname, ++accessCount, write);
        return current.accessField (v, fname, instruction, write);
    }

    private RTVariable staticFieldUse(boolean write, FieldInstruction obj) {

        String fullname = (obj.getClassName(method.getConstantPoolGen())
                           +'%'+obj.getFieldName(method.getConstantPoolGen())
                           +'%'+obj.getSignature(method.getConstantPoolGen()));

        // When we use a static field we must get the current
        // Shape this methods  globalmap.  Each pool sees a
        // different set of global variables . Why ?  when 
        // Entering a method we want to know the created by 
        // the method 
        RTVariable field = (RTVariable)currentMethod.getGlobalReference(fullname);
        if (field == null) { 
            field = current.createVariable();
            currentMethod.putGlobalReference (fullname, field);
        }
        return field;
    }

    /**  alias two variables.  newalias recieves the values 
     *   of value.
     */
    private void  alias(RTVariable newalias, RTVariable value) {
        // Push all state informatio from value into newalias

        current.unifyState (newalias, value);
        
        log.debug ("aliased "+ newalias+ " = " + value);
    }

    /**
     *  Create a merge of shape graphs based on pre-calculated 
     *  formals and list of actuals.  The list of actuals contains
     *  only reference type  arguments.
     * @param obj : the instruction
     * @param actual : list of RVariable actuals.
     * @return the RTVariable resulting from merging the results.
     */
    private RTVariable mergeCallsite (InvokeInstruction obj, List actuals) {
        
        // REM RTMethod[] targets = callgraph.getCallsAt (method, instruction);
        RTMethod[] targets = classanalysis.getTargetSet (method, instruction);
        

        // Each target is a possible end point of this call.
        // Actually only one is chosen at runtime and this
        // is introduces quite a bit on inaccuracy into the
        // analysis.
        log.trace ("StackVisitor","mergeCallsite","found "+targets.length + " targets");

        // Each target state will be merged  into the current state.
        RTVariable ret=null;

        for (int mi=0; mi < targets.length; mi++) {
            MethodState callee = methods.getState (targets[mi]);

            if (callee == null) {
                // we have not visited this method yet.  We will have
                // to revisit this.
                log.severe ("StackVisitor", "mergeCallSite", 
                            "callee unavailable:" + targets[mi]);
                continue;
            }

            boolean sameSCC = (current == callee.getPool());

            // Unify the arguments of this method with the state of the 
            // target (importing the shapes of the target into this
            // scope).
            
            Iterator ai = actuals.iterator();
            int pi = 0;
            log.debug ("calling " + callee);

            while (ai.hasNext()) {

                RTVariable actual = (RTVariable)ai.next();
                RTVariable formal = (RTVariable)callee.locals.get(pi++);

                if (actual == null || formal == null) continue;

                log.debug ("unify actual="+current.toString(actual)
                           +" formal="+callee.getPool().toString(formal));

                // Merging graphs of the same Component
                // Just merge everything together.  This 
                // takes care of the recursive case too.
                if (!sameSCC) 
                    formal = current.getExternal (callee.getPool(), formal);

                alias (actual, formal);
                
                if (log.isEnabled(Logger.DEBUG))
                    log.debug ("actual "+current.getShapeString(actual,actual.toString()));
            }

            // Unify the return values if any 
            if (callee.rtReturn != null) {
                // Allocate the first return var for unifying.
                //if (ret == null) ret = create(obj);

                log.debug ("unify ret=" + ret + " formal=" + callee.rtReturn);

                RTVariable rtRet = callee.rtReturn;
                if (!sameSCC) 
                    rtRet = current.getExternal(callee.getPool(), rtRet);

                if (ret == null) 
                    ret = rtRet;

                alias (ret, rtRet);

                if (log.isEnabled(Logger.DEBUG))
                    log.debug ("ret "+current.getShapeString(ret,ret.toString()));
            }
        }
        return ret;
    }

    private void handle (Instruction obj) {
        stack.pop (obj.consumeStack(method.getConstantPoolGen()));
        stack.push (obj.produceStack(method.getConstantPoolGen()));
    }

  public void visitStackInstruction(StackInstruction obj) { }
  public void visitLocalVariableInstruction(LocalVariableInstruction obj) { }
  public void visitBranchInstruction(BranchInstruction obj) { }
  public void visitLoadClass(LoadClass obj) { }
  public void visitFieldInstruction(FieldInstruction obj) { }
  public void visitIfInstruction(IfInstruction obj) { }
  public void visitConversionInstruction(ConversionInstruction obj) { }
  public void visitPopInstruction(PopInstruction obj) { }
  public void visitJsrInstruction(JsrInstruction obj) { }
  public void visitGotoInstruction(GotoInstruction obj) { }
  public void visitStoreInstruction(StoreInstruction obj) { }
  public void visitTypedInstruction(TypedInstruction obj) { }
  public void visitSelect(Select obj) { }
  public void visitInstructionTargeter(InstructionTargeter obj) { }
  public void visitUnconditionalBranch(UnconditionalBranch obj) { }
  public void visitPushInstruction(PushInstruction obj) { }
  public void visitArithmeticInstruction(ArithmeticInstruction obj) { }
  public void visitCPInstruction(CPInstruction obj) { }
  public void visitInvokeInstruction(InvokeInstruction obj) { 
      // Pull the actual parameters off the stack
      //handle(obj);
      LinkedList actuals = new LinkedList();

      Type[] ty = obj.getArgumentTypes( method.getConstantPoolGen() );
      for( int pi=ty.length-1; pi>=0; pi-- ) {

          if( ty[pi] instanceof ReferenceType ) {
              actuals.addFirst( stack.pop() );
          } else {
              int sz = ty[pi].getSize();
              stack.pop (sz);
              while (sz-- > 0) 
                  actuals.addFirst(null);
          }
      }								
      // get the object parameter
      if (!(obj instanceof INVOKESTATIC)) {
          actuals.addFirst(stack.pop());
      }

      RTVariable callret = mergeCallsite (obj, actuals);

      Type rt = obj.getReturnType( method.getConstantPoolGen() );
      if( rt instanceof ReferenceType ) {
          //Assert.assertion (callret != null, "no method return");
          if (callret == null) callret = create(obj);
          stack.push ( callret );
      } else {
          stack.push (rt.getSize());
      }

      // Merge the variables of the 
  }
  public void visitArrayInstruction(ArrayInstruction obj) { 
      switch (obj.getTag ()) {
      /*case Constants.AALOAD:*/ case Constants.BALOAD: case Constants.CALOAD: 
      case Constants.DALOAD: case Constants.FALOAD: 
      case Constants.IALOAD: case Constants.LALOAD: case Constants.SALOAD: {
          RTVariable index = stack.pop (); 
          RTVariable array = stack.pop();
          stack.push( obj.produceStack (method.getConstantPoolGen()));
          arrayUse (array, false, obj);
      } break;

     /*case Constants.AASTORE:*/ case Constants.BASTORE: case Constants.CASTORE: 
      case Constants.DASTORE: case Constants.FASTORE: 
      case Constants.IASTORE: case Constants.LASTORE: case Constants.SASTORE:{ 
          stack.pop( obj.consumeStack (method.getConstantPoolGen()) - 1);
          RTVariable array = stack.pop ();
          arrayUse (array, true, obj);
      } break;
      }
  }
  public void visitAllocationInstruction(AllocationInstruction obj) {
      stack.pop(((Instruction)obj).consumeStack (method.getConstantPoolGen()));
      stack.push (allocate((Instruction)obj));
  }
  public void visitReturnInstruction(ReturnInstruction obj) {}
  public void visitFieldOrMethod(FieldOrMethod obj) { }
  public void visitConstantPushInstruction(ConstantPushInstruction obj) {}
  public void visitCodeConstraint(CodeConstraint obj) { }
  public void visitExceptionThrower(ExceptionThrower obj) { }
  public void visitCompoundInstruction(CompoundInstruction obj) { }
  public void visitLoadInstruction(LoadInstruction obj) { }
  public void visitVariableLengthInstruction(VariableLengthInstruction obj) { }
  public void visitStackProducer(StackProducer obj) { }
  public void visitStackConsumer(StackConsumer obj) { }

    // STANDARD OPCODES BELOW

  public void visitAALOAD(AALOAD obj) {
      RTVariable index = stack.pop();
      RTVariable array = stack.pop();
      RTVariable val = arrayUse (array, false, obj);
      stack.push (val);
  }
  public void visitAASTORE(AASTORE obj) {
      RTVariable value = stack.pop();
      RTVariable index = stack.pop();
      RTVariable arrob = stack.pop ();
      RTVariable arrref = arrayUse (arrob, true, obj);
      alias (value, arrref);
  }
  public void visitACONST_NULL(ACONST_NULL obj) {
      // Will probably be used as a variable in future.
      stack.push (create (obj));
  }
  public void visitALOAD(ALOAD obj) {
      int index = obj.getIndex (); 
      stack.push ( locals.getAt (index) );
  }
  public void visitANEWARRAY(ANEWARRAY obj) {}
  public void visitARETURN(ARETURN obj) {
      RTVariable rt = stack.pop();
      if (currentMethod.rtReturn == null) 
          currentMethod.rtReturn = rt;
      else
          alias (currentMethod.rtReturn, rt);
  }
  public void visitARRAYLENGTH(ARRAYLENGTH obj) {
      RTVariable array = stack.pop (); 
      arrayUse (array, false, obj);
      stack.push(null);
  }
  public void visitASTORE(ASTORE obj) {
      int index = obj.getIndex (); 
      locals.setAt (index, stack.pop());
  }
  public void visitATHROW(ATHROW obj) { 
      // Same result.
      //RTVariable v = stack.pop();
      //stack.push (v);
  }
  public void visitBALOAD(BALOAD obj) {}
  public void visitBASTORE(BASTORE obj) {}
  public void visitBIPUSH(BIPUSH obj) { handle (obj); }
  public void visitCALOAD(CALOAD obj) {}
  public void visitCASTORE(CASTORE obj) {}
    public void visitCHECKCAST(CHECKCAST obj) { /*objectref->objectref*/ }
  public void visitD2F(D2F obj) { handle (obj); }
  public void visitD2I(D2I obj) { handle (obj); }
  public void visitD2L(D2L obj) { handle (obj); }
  public void visitDADD(DADD obj) { handle (obj); }
  public void visitDALOAD(DALOAD obj) {}
  public void visitDASTORE(DASTORE obj) {}
  public void visitDCMPG(DCMPG obj) { handle (obj); }
  public void visitDCMPL(DCMPL obj) { handle (obj); }
  public void visitDCONST(DCONST obj) { handle (obj); }
  public void visitDDIV(DDIV obj) { handle (obj); }
  public void visitDLOAD(DLOAD obj) { handle (obj); }
  public void visitDMUL(DMUL obj) { handle (obj); }
  public void visitDNEG(DNEG obj) { handle (obj); }
  public void visitDREM(DREM obj) { handle (obj); }
  public void visitDRETURN(DRETURN obj) { handle (obj); }
  public void visitDSTORE(DSTORE obj) { handle (obj); }
  public void visitDSUB(DSUB obj) { handle (obj); }
  public void visitDUP(DUP obj) {
      RTVariable x = stack.pop(); stack.push(x); stack.push (x);
  }
  public void visitDUP2(DUP2 obj) {
      RTVariable w1 = stack.pop(); RTVariable w2 = stack.pop(); 
      stack.push (w2); stack.push(w1); stack.push(w2); stack.push (w1);
  }
  public void visitDUP2_X1(DUP2_X1 obj) {
      RTVariable w1 = stack.pop(); RTVariable w2 = stack.pop(); RTVariable w3 = stack.pop(); 
      stack.push (w2); stack.push(w1); stack.push(w3); stack.push(w2); stack.push (w1);}
  public void visitDUP2_X2(DUP2_X2 obj) {
      RTVariable w1 = stack.pop(); RTVariable w2 = stack.pop(); 
      RTVariable w3 = stack.pop(); RTVariable w4 = stack.pop(); 
      stack.push (w2); stack.push(w1); stack.push(w4); stack.push(w3); stack.push(w2); stack.push (w1);
  }
  public void visitDUP_X1(DUP_X1 obj) {
      RTVariable w1 = stack.pop(); RTVariable w2 = stack.pop(); 
      stack.push (w1); stack.push(w2); stack.push(w1);
  }
  public void visitDUP_X2(DUP_X2 obj) {
      RTVariable w1 = stack.pop(); 
      RTVariable w2 = stack.pop(); 
      RTVariable w3 = stack.pop(); 
      stack.push(w1); stack.push(w3); stack.push(w2); stack.push (w1);
  }
  public void visitF2D(F2D obj) { handle (obj); }
  public void visitF2I(F2I obj) { handle (obj); }
  public void visitF2L(F2L obj) { handle (obj); }
  public void visitFADD(FADD obj) { handle (obj); }
  public void visitFALOAD(FALOAD obj) {}
  public void visitFASTORE(FASTORE obj) {}
  public void visitFCMPG(FCMPG obj) { handle (obj); }
  public void visitFCMPL(FCMPL obj) { handle (obj); }
  public void visitFCONST(FCONST obj) { handle (obj); }
  public void visitFDIV(FDIV obj) { handle (obj); }
  public void visitFLOAD(FLOAD obj) { handle (obj); }
  public void visitFMUL(FMUL obj) { handle (obj); }
  public void visitFNEG(FNEG obj) { handle (obj); }
  public void visitFREM(FREM obj) { handle (obj); }
  public void visitFRETURN(FRETURN obj) { handle (obj); }
  public void visitFSTORE(FSTORE obj) { handle (obj); }
  public void visitFSUB(FSUB obj) { handle (obj); }
  public void visitGETFIELD(GETFIELD obj) { 
      Type type = obj.getFieldType (method.getConstantPoolGen()); 
      RTVariable ref = stack.pop(); 

      if (type instanceof ReferenceType) {
//           RTVariable value = create (obj);
//           alias(value, field);
          RTVariable field = fieldUse (ref, false, obj);
          stack.push (field);
      }else {
          stack.push (obj.produceStack(method.getConstantPoolGen()));
      }
  }
  public void visitGETSTATIC(GETSTATIC obj) {
//       handle (obj);
       Type type = obj.getFieldType (method.getConstantPoolGen()); 
       if (type instanceof ReferenceType) {
           RTVariable v = staticFieldUse (false, obj);
           stack.push (v);
       } else {
           stack.push (obj.produceStack(method.getConstantPoolGen()));
       }
  }
  public void visitGOTO(GOTO obj) { handle (obj); }
  public void visitGOTO_W(GOTO_W obj) { handle (obj); }
  public void visitI2B(I2B obj) { handle (obj); }
  public void visitI2C(I2C obj) { handle (obj); }
  public void visitI2D(I2D obj) { handle (obj); }
  public void visitI2F(I2F obj) { handle (obj); }
  public void visitI2L(I2L obj) { handle (obj); }
  public void visitI2S(I2S obj) { handle (obj); }
  public void visitIADD(IADD obj) { handle (obj); }
  public void visitIALOAD(IALOAD obj) {}
  public void visitIAND(IAND obj) { handle (obj); }
  public void visitIASTORE(IASTORE obj) {}
  public void visitICONST(ICONST obj) { handle (obj); }
  public void visitIDIV(IDIV obj) { handle (obj); }
  public void visitIFEQ(IFEQ obj) { handle (obj); }
  public void visitIFGE(IFGE obj) { handle (obj); }
  public void visitIFGT(IFGT obj) { handle (obj); }
  public void visitIFLE(IFLE obj) { handle (obj); }
  public void visitIFLT(IFLT obj) { handle (obj); }
  public void visitIFNE(IFNE obj) { handle (obj); }
  public void visitIFNONNULL(IFNONNULL obj) { handle (obj); }
  public void visitIFNULL(IFNULL obj) { handle (obj); }
  public void visitIF_ACMPEQ(IF_ACMPEQ obj) { handle (obj); }
  public void visitIF_ACMPNE(IF_ACMPNE obj) { handle (obj); }
  public void visitIF_ICMPEQ(IF_ICMPEQ obj) { handle (obj); }
  public void visitIF_ICMPGE(IF_ICMPGE obj) { handle (obj); }
  public void visitIF_ICMPGT(IF_ICMPGT obj) { handle (obj); }
  public void visitIF_ICMPLE(IF_ICMPLE obj) { handle (obj); }
  public void visitIF_ICMPLT(IF_ICMPLT obj) { handle (obj); }
  public void visitIF_ICMPNE(IF_ICMPNE obj) { handle (obj); }
  public void visitIINC(IINC obj) { handle (obj); }
  public void visitILOAD(ILOAD obj) { handle (obj); }
  public void visitIMPDEP1(IMPDEP1 obj) { handle (obj); }
  public void visitIMPDEP2(IMPDEP2 obj) { handle (obj); }
  public void visitIMUL(IMUL obj) { handle (obj); }
  public void visitINEG(INEG obj) { handle (obj); }
  public void visitINSTANCEOF(INSTANCEOF obj) { handle (obj); }
  public void visitINVOKEINTERFACE(INVOKEINTERFACE obj) {
      // visitInvokeInstruction(InvokeInstruction obj) { 
  }
  public void visitINVOKESPECIAL(INVOKESPECIAL obj) {
      // visitInvokeInstruction(InvokeInstruction obj) { 
  }
  public void visitINVOKESTATIC(INVOKESTATIC obj) {
      // visitInvokeInstruction(InvokeInstruction obj) { 
  }
  public void visitINVOKEVIRTUAL(INVOKEVIRTUAL obj) { 
      // visitInvokeInstruction(InvokeInstruction obj) { 
  }
  public void visitIOR(IOR obj) { handle (obj); }
  public void visitIREM(IREM obj) { handle (obj); }
  public void visitIRETURN(IRETURN obj) { handle (obj); }
  public void visitISHL(ISHL obj) { handle (obj); }
  public void visitISHR(ISHR obj) { handle (obj); }
  public void visitISTORE(ISTORE obj) { handle (obj); }
  public void visitISUB(ISUB obj) { handle (obj); }
  public void visitIUSHR(IUSHR obj) { handle (obj); }
  public void visitIXOR(IXOR obj) { handle (obj); }
  public void visitJSR(JSR obj) { handle (obj); }
  public void visitJSR_W(JSR_W obj) { handle (obj); }
  public void visitL2D(L2D obj) { handle (obj); }
  public void visitL2F(L2F obj) { handle (obj); }
  public void visitL2I(L2I obj) { handle (obj); }
  public void visitLADD(LADD obj) { handle (obj); }
  public void visitLALOAD(LALOAD obj) {}
  public void visitLAND(LAND obj) { handle (obj); }
  public void visitLASTORE(LASTORE obj) {}
  public void visitLCMP(LCMP obj) { handle (obj); }
  public void visitLCONST(LCONST obj) { handle (obj); }
  public void visitLDC(LDC obj) { 
      Type rt = obj.getType( method.getConstantPoolGen() );
      if( rt instanceof ReferenceType ) {
          stack.push ( create(obj) );
              
      } else {
          stack.push (rt.getSize());
      }
  }
  public void visitLDC2_W(LDC2_W obj) {
      Type rt = obj.getType( method.getConstantPoolGen() );
      if( rt instanceof ReferenceType ) {
          stack.push ( create(obj) );
              
      } else {
          stack.push (rt.getSize());
      }
  }
  public void visitLDIV(LDIV obj) { handle (obj); }
  public void visitLLOAD(LLOAD obj) { handle (obj); }
  public void visitLMUL(LMUL obj) { handle (obj); }
  public void visitLNEG(LNEG obj) { handle (obj); }
  public void visitLOOKUPSWITCH(LOOKUPSWITCH obj) { handle (obj); }
  public void visitLOR(LOR obj) { handle (obj); }
  public void visitLREM(LREM obj) { handle (obj); }
  public void visitLRETURN(LRETURN obj) { handle (obj); }
  public void visitLSHL(LSHL obj) { handle (obj); }
  public void visitLSHR(LSHR obj) { handle (obj); }
  public void visitLSTORE(LSTORE obj) { handle (obj); }
  public void visitLSUB(LSUB obj) { handle (obj); }
  public void visitLUSHR(LUSHR obj) { handle (obj); }
  public void visitLXOR(LXOR obj) { handle (obj); }
  public void visitMONITORENTER(MONITORENTER obj) { handle (obj); }
  public void visitMONITOREXIT(MONITOREXIT obj) { handle (obj); }
  public void visitMULTIANEWARRAY(MULTIANEWARRAY obj) {}
  public void visitNEW(NEW obj) {}
  public void visitNEWARRAY(NEWARRAY obj) {}
  public void visitNOP(NOP obj) { handle (obj); }
  public void visitPOP(POP obj) { handle (obj); }
  public void visitPOP2(POP2 obj) { handle (obj); }
  public void visitPUTFIELD(PUTFIELD obj) {
      Type type = obj.getFieldType (method.getConstantPoolGen()); 
      if (type instanceof ReferenceType) {
          RTVariable value = stack.pop(); 
          RTVariable ref = stack.pop(); 
          RTVariable field = fieldUse (ref, true, obj);
          alias (field, value);
      }else {
          stack.pop (obj.consumeStack (method.getConstantPoolGen()) - 1); 
          RTVariable ref = stack.pop(); 
          // field is unused as it is a basic type;
          //RTVariable field = fieldUse (ref, true, obj);
      }
  }
  public void visitPUTSTATIC(PUTSTATIC obj) {
       Type type = obj.getFieldType (method.getConstantPoolGen()); 
        if (type instanceof ReferenceType) {
            RTVariable value = stack.pop(); 
            RTVariable field = staticFieldUse (true, obj);
            alias (field, value);
        }else {
           stack.pop (obj.consumeStack ()); 
           //RTVariable ref = stack.pop(); 
       }
  }
  public void visitRET(RET obj) { handle(obj); }
  public void visitRETURN(RETURN obj) { handle (obj); }
  public void visitSALOAD(SALOAD obj) {}
  public void visitSASTORE(SASTORE obj) {}
  public void visitSIPUSH(SIPUSH obj) { handle (obj); }
  public void visitSWAP(SWAP obj) { handle (obj); }
  public void visitTABLESWITCH(TABLESWITCH obj) { handle (obj); }
};
