/*
 * 
 *
 * tag: kkvilekval@acm.org Fri Nov  9 17:33:46 2001 (AccessTable)
 *
 */


package mokan.prefetch.runtime;


import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.StringTokenizer;

import mokan.prefetch.util.xml.XMLWriter;


/**
 *  The <code>AccessTable</code> is the entry point for 
 *  an annotated program to record it's accesses.  The table
 *  is responsible for forwarding the annotated calls to 
 *  the proper caching modules.
 *  <p>
 *  
 *  
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class AccessTable {

    /* Private/Public Variables */

    /** List of Cache  strategies (AccessCache) to apply access to */
    private static AccessStrategy dispatch;
    /** Flag used to prevent rentering the module */
    private static boolean profiling = false;
    /** Flag used to control whether the clear call functions */
    private static boolean clearing = false;

    /** Must be called before program runs */
   public static void setStrategy(AccessStrategy as) {
       dispatch = as;
    }
    public static void setProfiling(boolean x) {
        profiling = x;
    }
    public static void setClearing(boolean x) {
        clearing = x;
    }

    /**
     *  The default constructor is private .. use getAccessTable
     */
//     private AccessTable () {
//     }

    /** Record the entry of the method.  This is called by each annotated method
     *  after it has listed all it's primary accessed references.
     *  @param name  fqn of the method.
     */
    public static void methodEntry (String name, boolean analyzed){
//         if (profiling) {
//             profiling = false;
            //if (AccessRun.print) System.out.println ("entering " + name);
            dispatch.methodEntry (name, analyzed);
//             profiling = true;
//         }
    }
    /** Record the exit of the method.  Called before each return from
     *  an annotated method.
     *  @param name  fqn of the method.
     */
    public static void methodExit (String name){
//         if (profiling) {
//             profiling = false;
            //if (AccessRun.print) System.out.println ("exiting " + name);
            dispatch.methodExit (name);
//             profiling = true;
//         }
    }

    /**Record that an object was accessed.   Each load or store 
     * in the method is preceded by a call to <code>access</code>
     * with the object and it's effect (write/read).
     * @param rtobj the object.
     * @param acctype read or write
     */ 
       
    public static void access(Object rtobj, boolean iswrite) {
        if (rtobj == null) return;
        if (profiling){
            profiling = false;
            dispatch.access(rtobj, iswrite);
            profiling = true;
        }
    }

    /**Should be prefetched.  Before entry all top
     * level objects references with there associated access graph.
     * If graphs are internal (no longer supproted) then the access
     * is a string representation of the shap graph.
     * If graphs are external then the String is integer index of
     * method shapegraph in the methodState.
     * @param rtobj the object.
     * @param access A ',' seperated string of fields
     */ 
    public static void prefetch (Object rtobj, String access) {
        if (rtobj == null) return;
//         if (profiling) {
//             profiling = false;
            dispatch.preload(rtobj, access);
//             profiling = true;
//         }
    }
    /** Record that an object was allocated.  After each new statement
     *  the <code>alloc</code> is caled with the created object.
     */
    public static void alloc (Object rtobj) {
        if (profiling) {
            profiling = false;
            dispatch.alloc (rtobj);
            profiling = true;
        }
    }


    public static long transactBegin(Object[] os){
        long x = 0;
//         if (profiling) {
//             profiling = false;
            x = dispatch.transactBegin(os);
//             profiling = true;
//         }
        return x;
    }

    public static void transactEnd(){
//         if (profiling) {
//             profiling = false;
            dispatch.transactEnd();
//             profiling = true;
//         }
    }


    /** Start a strategy .. Never called from annotated code */
    public static void start () {
        dispatch.start ();
    }
    /** Stop a strategy .. Never called from annotated code */
    public static void stop () {
        dispatch.stop ();
    }
    /** Clear the cache.. this is called directly from Benchmark programs
     *  after initialization to simulate starting with a cold cache 
     */
    public static void clear(){
        if (clearing)
            dispatch.clear();
    }

    public static void dumpXML (XMLWriter aw) {
            dispatch.dumpXML(aw);
    }
};
