/*
 * 
 *
 * tag: kkvilekval@acm.org Fri Nov  9 17:34:10 2001 (DispatchStrategy)
 *
 */


package mokan.prefetch.runtime;

import java.util.ArrayList;

import mokan.prefetch.util.xml.XMLWriter;

/**
 *  Dispatch an AccessStrategy command to a set of AccessStrategies
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
class DispatchStrategy extends AccessStrategy {

    private ArrayList strats = new ArrayList();
    private AccessStrategy[] strategies;


    public DispatchStrategy () {
        super("DISPATCH");
    }

    public void addStrategy(AccessStrategy as) {
        strats.add (as);
    }

    public void start () {
        strategies = (AccessStrategy[])strats.toArray(new AccessStrategy[strats.size()]);
        for (int si=0; si < strategies.length; si++)
            strategies[si].start();
    }
    public void stop () {
        for (int si=0; si < strategies.length; si++)
            strategies[si].stop();
    }
    public void clear () {
        for (int si=0; si < strategies.length; si++)
            strategies[si].clear();
    }
    public void methodEntry(String name, boolean analyzed){
        for (int si=0; si < strategies.length; si++)
            strategies[si].methodEntry(name, analyzed);
    }
    public void methodExit(String name){
        for (int si=0; si < strategies.length; si++)
            strategies[si].methodExit(name);
    }

    public void alloc(Object rtobj){
        for (int si=0; si < strategies.length; si++)
            strategies[si].alloc(rtobj);
    }
    public void access(Object rtobj, boolean iswrite){
        for (int si=0; si < strategies.length; si++)
            strategies[si].access(rtobj,iswrite);
    }
    public void preload (Object rtobj, String access){
        for (int si=0; si < strategies.length; si++)
            strategies[si].preload(rtobj,access);
    }
    public boolean contains (Object rtobj) {
        for (int si=0; si < strategies.length; si++)
            strategies[si].contains(rtobj);
        return false;
    }

    public long transactBegin(Object[] os) {
        long x=0;
        for (int si=0; si < strategies.length; si++)
             x += strategies[si].transactBegin(os);
        return x;
    }
    public void transactEnd() {
        for (int si=0; si < strategies.length; si++)
            strategies[si].transactEnd();
    }

    public void dumpXML  (XMLWriter p){
        for (int si=0; si < strategies.length; si++)
            strategies[si].dumpXML(p);
    }
};
