/*
 * tag: kkvilekval@acm.org Fri Jan 11 19:08:04 2002 (PrecacheStrategy)
 */
package mokan.prefetch.runtime;

import java.util.LinkedList;
import mokan.prefetch.util.xml.XMLWriter;
import klogger.Logger;


/**
 *  The <code>PrecacheStrategy</code> is the top lovel strategy object
 *  for prefetching objects.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */
class PrecacheStrategy extends AccessStrategy {

    private final static Logger log = Logger.getLogger("mokan.prefetch.Prefetch");

    private Prefetcher prefetcher;
    /** Cache size in Objects.. The Cache will accept K objects
     *  every time it runs 
     */
    private int lookahead  = 0 ;
    private int cachesize  = 0 ;

    /** The method interval allows caching to be run less often.
     *  There is a trade off between caching performance and
     *  and the # of time we run to precache objects. 
     */

    private int preload_attempts;
    private int allocs_during_disconnect;
    private int disconnected_cache_hit;
    private int disconnected_cache_miss;
    private int disconnected_access;
    private int access_counter;
    private int distance;

    /** Hack the call stack end logging code so it does freak 
     *  freak out when we turn sampling on
     */
    private int call_level = 0;	

    /** A precaching strategy 
     *  @param cachesize The size in objects the backing cache store
     *  @param interval  Specify how often to run the precaching.   A larger
     *                   interval will spend less time precaching.
     *  @param objects   A each precaching point cache Kobjects.  Actually
     *                   cache upto K objects of each shape graph passed
     *                   to the method.
     *  @param printlevel 0 - no printing, 1- summary, 2-trace
     *  @param mincallheight  @see precache.analysis.CallAnalysis
     *  @param maxcallheight  @see precache.analysis.CallAnalysis
     *  @param externalShapes  (use external shape graphs .. true).
     */
    PrecacheStrategy(int lookahead,
                     int mincallheight,
                     int maxcallheight,
                     String externalShapes){
        //super ("PRE(" + cachesize +','+interval+','+objects+')');
        super ("PRE");
        //comm.addListener (this);

        this.lookahead = (lookahead!=0)?lookahead:Integer.MAX_VALUE;
        this.cachesize = cachesize;

        preload_attempts = 0;
        if (maxcallheight < 0)
            maxcallheight = Integer.MAX_VALUE;
        if (mincallheight < 0)
            mincallheight = Integer.MAX_VALUE;

        prefetcher = new PrefetchExternal (lookahead, externalShapes,
                                           mincallheight,maxcallheight);
    }

    /** Method entry is called after all prefetch statements 
     * have been seen.  We preload all the object in the prefetch
     * lost 
     */
    public  void methodEntry (String name, boolean analyzed){
        call_level ++;
        
        System.out.println ("methodEntry:"+ name);
        
        // No prefetching while disconnected
//        if (!comm.isConnected ()) return;
        
        preload_attempts++;
        log.debug("PrecacheStrategy", "methodEntry","");
        prefetcher.prefetchWaiting(name);
        prefetcher.clearWaiting();
        access_counter = 0;
    }

    /** Called on exit from the method .. allows the strategy to cleanup */
    public  void methodExit (String name){
        //log.debug ("MethodExit", name);
    }

    /** stop the stratgey.. called at the end of the simulation */
    public void stop(){
    }
    /** clear the cache and prepare for simulation */
    public void clear(){
    }

    public void alloc (Object rtobj){
    }

    public  void access(Object rtobj, boolean iswrite){
    }

    /** Preloading will queue an object and it's shape graph 
     * to be searched on the next method entry.
     */
    public void preload (Object rtobj, String access) {
        if (rtobj != null) {
            prefetcher.addWaiting (rtobj, access);
        }
    }


    public boolean contains (Object rtobj) {
        return false;
    }

    public void dumpXML  (XMLWriter p ){
        p.startTag("strategy", new Object[] { "name", ident });
//         p.printTagged("interval", methodInterval);
//         p.printTagged("lookahead", lookahead);
//         p.printTagged("allocations", cache.allocations());
        p.printTagged("disconnectaccess", disconnected_access);
        p.printTagged("disconnecthits", disconnected_cache_hit);
        p.printTagged("disconnectmisses", disconnected_cache_miss);
        p.printTagged("discalloc", allocs_during_disconnect);
//         p.printTagged ("hits",((double)cache.hits()) / cache.accesses());
//         p.printTagged ("accesses",cache.accesses());
//         p.printTagged ("preloadattempts",preload_attempts);
        prefetcher.dumpXML(p);
//        cache.dumpXML (p);
//        comm.dumpXML(p);
        p.endTag("strategy");
    }
};

