/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Jan 22 14:23:47 2002 (PrefetchExternal)
 *
 */


package mokan.prefetch.runtime;

import java.io.FileInputStream;
import java.io.ObjectInputStream;

import java.lang.reflect.Field;
import java.lang.reflect.Array;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collections;
import java.util.Comparator;

import mokan.prefetch.analysis.shapeanalysis.RTVariable;
import mokan.prefetch.analysis.shapeanalysis.ShapePool;
import mokan.prefetch.analysis.shapeanalysis.ShapeGraph;
import mokan.prefetch.analysis.shapeanalysis.ShapeEdge;
import mokan.prefetch.analysis.shapeanalysis.MethodState;
import mokan.prefetch.analysis.shapeanalysis.MethodTable;

import klogger.Logger;

/**
 *  The <code>PrecaheVisitor</code> description.
 *  @version $Revision: 1.3 $, $Date: 2003/02/06 03:20:23 $
 *  @author Kristian Kvilekval
 */
public class PrefetchExternal extends Prefetcher {

    /* Private/Public Variables */
    /** The ShapeGraphs of all procedures are stored in the
     * MethodTable. */
    private MethodTable methods;
    /** The current (fetching) methods state (ShapeGraph + locals) */
    MethodState         mstate;
    /** The Shape Pool/Graph  */
    ShapePool           pool;
    ShapeGraph          graph;
    /** The BFS search of the shape graph.. nodes to be visited */
    PriorityQueue           fetchq;
    /** Node + Object that we have seen (to prevent looping) */
    HashSet             seenq;
    final static ObjVar searchov = new ObjVar();
    /** Number of time we have tried to fetch */
    int                 fetchattempt;

    

    private Object getFieldVal (Object rtobj, String fieldtag ) {
        Class rtclass = rtobj.getClass();
        try {
            Field fob = rtclass.getField (fieldtag);
            rtobj = fob.get (rtobj);

        } catch (IllegalAccessException f) {
            //System.out.println("illegal access:"+rtclass +"."+ fieldtag);
            rtobj = null;
        } catch (NoSuchFieldException f) {
            // Thrown when ther object rtpartial does
            // not contain the field <fieldtag>
            
            // If no such field pazienza..
            // System.out.println ("no field " + fieldtag);
            // while(fields.hasMoreTokens()) fields.nextToken();

            //System.out.println ("No field:" + rtclass +"."+ fieldtag);
            rtobj = null;
        }
        return rtobj;
    }

    public PrefetchExternal (int look, String externalShapes,
                             int minch, int maxch) {
        this.lookahead = look;
        this.fetchq = new PriorityQueue();
        this.seenq =     new HashSet();
        this.mincallheight = minch;
        this.maxcallheight = maxch;
        this.actualmaxcall = 0;
        String gfname  = externalShapes;


        try {
            FileInputStream istream = new FileInputStream(gfname);
            ObjectInputStream p = new ObjectInputStream(istream);
            methods = (MethodTable)p.readObject ();
            istream.close();
        } catch (java.io.IOException io) {
            log.severe ("Reading graphs in " + gfname);
            io.printStackTrace();
        } catch (ClassNotFoundException mc) {
            log.severe ("No shapes in  " + gfname);
        }
    }

    public boolean prefetch_start (String name){
        Assert.assertion(methods!=null, "NULL methods");
        Assert.assertion(name!=null, "NULL name");

        super.prefetch_start(name);

        mstate = methods.getState (name);

        if (actualmaxcall < mstate.callLevel) {
            actualmaxcall = mstate.callLevel;
        }
        if (mstate != null) {
            if (mincallheight <= mstate.callLevel 
                && mstate.callLevel <= maxcallheight) {
                pool   = mstate.getPool();
                graph  = pool.gb;
                fetchq.clear();
                seenq.clear();
                fetchattempt = 0;
                return true;
            } 
            log.debug ("PrefetchExternal", "prefetch",
                       "Skipping " + name + " at level " + mstate.callLevel);
        } 
        return false;
    }
    public void prefetch_end (String name){
        fetchq.clear();
        seenq.clear();

        super.prefetch_end(name);
    }
    /** Begin the loading of a shape graph */
    public void preload_object (Object rtobj, String access) {
        // Add this top level object to the queue
        int localpos = Integer.valueOf (access).intValue();
        RTVariable root = pool.getVariable (mstate.getLocal(localpos));

        if (root == null) {
            log.warn ("PrefetchExternal", "preload_object", 
                      "preload of object with no graph");
            return;
        }
            
        add_fetchq (rtobj, root, 0);

        prefetch_objects();
    }
    // Add an (obj,var, dist)  tuple to the fetchq if we haven't seen the exact same tuple before
    private void add_fetchq (Object rtobj, RTVariable v, int d) {

        if (rtobj != null && v != null) {
            searchov.rtobj = rtobj;
            searchov.rtvar = v;
            searchov.hash();
            if (!seenq.contains (searchov)){
                ObjVar ov = new ObjVar(rtobj, v, d);
                fetchq.putLast (ov);
                if (debug > 1) 
                    System.out.print('.');
            }
        }
    }

    void  prefetch_objects (){
        Object loaded;
        while (!fetchq.isEmpty() && preloaded < lookahead) {

            ObjVar ov = (ObjVar)fetchq.getFirst ();
            
            Object rtobj = ov.rtobj;
            RTVariable n = ov.rtvar;
            int distance = ov.distance;

            if (log.isEnabled(Logger.DEBUG))
                log.debug ("PrefetchExternal", "prefetch", orep(rtobj) +" at " + pool.toString(n) + "  distance:"+ distance); 
            seenq.add (ov);

            // Explore the useful edges.


//             LinkedList adj = new LinkedList (graph.getAdjacentEdges(n));
//             Collections.sort (adj, new Comparator () {
//                     public int compare(Object a, Object b){
//                         return ((ShapeEdge)a).count - ((ShapeEdge)b).count;
//                     }
//                 });
//             Iterator  ei = adj.iterator();
            Iterator ei = graph.getAdjacentEdges(n).iterator();
            while (ei.hasNext()) {
                ShapeEdge e = (ShapeEdge)ei.next();
                RTVariable b = pool.getVariable(e.getSink());

//                 if (debug > 2) {
//                     System.err.println("examining " +orep(rtobj) + "." + e );
//                 }

                if (e.field.equals("<array>")) {
                    Object arrobj = rtobj;
                    if (arrobj instanceof Object[]){
                        int arrlen = Array.getLength(arrobj);
                        int index = 0;
                        while (index<arrlen && preloaded < lookahead) {
                            loaded = Array.get (arrobj, index);
                            if (loaded != null) {
                                addPreloadQ (loaded);
                                add_fetchq(loaded, b, distance+e.mincount);
                            }
                            index++;
                        }
                    }
                } else {
                    loaded = getFieldVal (rtobj, e.field);
                    // if successfull .. keep reading.
                    if (loaded != null) {
                        addPreloadQ (loaded);
                        add_fetchq(loaded, b, distance+e.mincount);
                    }
                }
            }
        }
    }

    private static class ObjVar implements Comparable  {
        Object      rtobj;
        RTVariable  rtvar;
        int         distance;
        int         hashcode;

        ObjVar(){}
        ObjVar(Object rtobj, RTVariable v, int d){
            this.rtobj = rtobj;
            this.rtvar =     v;
            this.distance = d;
            hash();
        }

        public void hash() {
            this.hashcode = (System.identityHashCode(rtobj) >> 8) ^ rtvar.hashCode();
        }
        public boolean equals (Object e) {
            return ((ObjVar)e).rtobj == rtobj && ((ObjVar)e).rtvar == rtvar;
        }
        public int hashCode () {
            return hashcode;
        }
        public int compareTo (Object o) {
            return distance - ((ObjVar)o).distance;
        }
    }


};
