/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Jan 22 05:13:26 2002 (Prefetcher)
 *
 */


package mokan.prefetch.runtime;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;

import mokan.obj.ObjectID;
import mokan.prefetch.util.IdentityHashMap;
import mokan.prefetch.util.xml.XMLWriter;
import klogger.Logger;

/**
 *  The <code>Prefetcher</code> is a common point for internal and 
 *   external prefetchers.
 *   External Interface:
 *     addWaiting (Object rtobj, String accesskey) 
 *     prefetchWaiting (String method, Cache, Comm)
 * 
 *   Internel Interface:
 *     
 *  @version $Revision: 1.3 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */
public abstract class Prefetcher {

    protected final static Logger log = Logger.getLogger ("mokan.prefetch.Prefetcher");
    

    private  static class Entry {
        Object o;
        public Entry(Object rtobj) { o = rtobj; }
        public boolean equals (Object o) {
            return this.o == ((Entry)o).o;
        }
        public int hashCode () {
            return System.identityHashCode(o);
        }

        public String toString () {
            return o.getClass().toString() +':' +System.identityHashCode(o);
        }
    }
    private Entry   search = new Entry (null);
    private IdentityHashMap newset;
    //private HashMap newset;
    private ArrayList accessList;
    private int       accessIndex;

    /* Private/Public Variables */
//     private ArrayList oldset;
//     private ArrayList newset;
    private Queue prefetchWaiting;

    /** how many objects have actuall been preloaded */
    protected int preloaded;
    /** The current lookahead limit */
    protected int lookahead;
    protected int debug;

    /** maintain some stats */
    protected int prefetch_attempts;
    protected int prefetch_length;
    protected int prefetch_loads;
    protected int prefetch_max;
    protected int prefetch_total;


    /** CallHeight only prefetch when entering routines in callheight range*/
    protected int mincallheight;
    protected int maxcallheight;
    protected int actualmaxcall;
    

    public Prefetcher () {
        prefetchWaiting = new Queue();
//         oldset = new ArrayList ();
//         newset = new ArrayList ();

        newset = new IdentityHashMap(300*1024);
        //        newset = new HashMap(300*1024);
        accessList = new ArrayList (300*1024);
    }
    public void setDebuglevel (int d) {
        debug = d;
    }
    /** Add and object and a graph to the queue of
     * this prefetcher.  The objects will be loaded 
     *  when  prefetchWaiting is called
     */
    public void addWaiting (Object rtobj, String access) {
        prefetchWaiting.putLast (rtobj);
        prefetchWaiting.putLast (access);
    }
    /** load all the waiting objects loaded in the queue This routine
     *  relies on the derived class to do most of the work.  
     *
     * It relies on:
     *  prefetch_start : Tell the prefetcher that we
     *                   begining a set of prefetches (clear structures)
     *  preload_object : Load the object graph into the newset table
     *
     *  preload_newset : load object found in newset into the cache
     *                   if they are not already in transfer or there.
     *  prefect_end    : clean up 
     */

    public void prefetchWaiting (String name) {
        if (prefetch_start(name)) {
            if (debug > 2) System.out.print('+');
            log.trace ("Prefetcher", "prefetchWaiting", "Entering: " + name);

            preloaded = 0;
            while (!prefetchWaiting.isEmpty ()) {
                Object rtobj = prefetchWaiting.getFirst();
                String access = (String) prefetchWaiting.getFirst();
                preload_object (rtobj, access);
            }
            // Preload object not already in cache.
            //int count = preload_newset (cache, comm);

            prefetch_end(name);
            if (debug > 2) System.out.print('-');
            log.trace ("Prefetcher", "prefetchWaiting", "Exiting: " + name);
        }
    }
    /** remove any left over object from the waiting queue */
    public void clearWaiting() {
        prefetchWaiting.clear();
    }

    public int getDistance (Object rtobj){
        int distance = 0;
//         int index = accessIndex;

//         while (index < accessList.size()) {
//             if (accessList.get(index) == rtobj) {
//                 break;
//             }
//             index ++ ;
//         }
//         distance = index - accessIndex;
//         accessIndex = index;
        return distance;
    }


    /** called for each object to preloaded, determines
     *  whether this object has already been scheduled 
     *  for loading and if not, adds it to the set of object
     *  to be fetched .. CALLED from derived class 
     */
    protected void addPreloadQ (Object rtobj) {
        boolean found = false;
//        search.o = rtobj; 
//        if (!newset.containsKey (search)){
        if (!(rtobj instanceof mokan.obj.ObjectID))
            return;


        if (!newset.containsKey (rtobj)){
            log.debug ("Prefetcher", "addPreloadQ", "Preloading object:"+orep(rtobj));
            newset.put (rtobj, rtobj);
            //newset.put (new Entry(rtobj), rtobj);
            accessList.add (rtobj);
            preloaded++;

            if (preloaded % 1024==0) {
                System.out.print ('.');
            }
        }
    }

    /**  Load the prefetch objects freom the prefetch queue into the
     *  given cache.
     *  This routine check if an object is already in the cache 
     *  or 
     */
    protected int  preload_newset () {
        // Any in oldset that have not been touched
        // and are not in the new set are candidates 
        // for immediate eviction.  We dispose of these
        // before loading the newset.

        int oi=0;
        prefetch_attempts++;
        prefetch_length = accessList.size();
        if (prefetch_max < prefetch_length) prefetch_max = prefetch_length;
        prefetch_total += prefetch_length;

        //Comm.clearPrefetches();

        Iterator ei = accessList.iterator();
        while (ei.hasNext()){
            Object rtobj = ei.next();
//            if (!cache.isCached(rtobj)){
            if (true /*FIXME */){
                //comm.readStore (rtobj);
                //cache.preload (rtobj);
                prefetch_loads++;
                oi++;
                log.debug ("preloading : "+orep(rtobj));
                ObjectID objID = (ObjectID)rtobj;
                if (!objID.isLocalValid()) {
                    objID.getObjectCopy();
                }
            }
        }

        log.trace ("Prefetcher", "preload", 
                   "preloaded "+oi+" from "+prefetch_length+" candidates");

        return oi;
    }

    public  static String orep(Object o) {
        return AccessStrategy.orep (o);
    }

    /** Schedule an object and graph for prefetching
     * The acutual prefetcher will need to overload 
     * and define the interpretatiopn of the access string 
     * @param rtobj The root object of the graph
     * @param access graph representation.
     */
    public abstract void preload_object(Object rtobj, String access);
    /**  Begin a prefetching of a group of object..
        @param name the method name
     */
    public boolean prefetch_start (String name){ 
        accessList.clear();
        return false; 
    }
    /** Complete set of prefetching objects 
     *  @param name the method name
     */
    public void prefetch_end (String name) {
        newset.clear();
    }

    public void dumpXML (XMLWriter p ){
        p.startTag ("prefetch");
        p.printTagged ("prefetch-calls", prefetch_attempts);
        p.printTagged ("prefetch-maxqueue", prefetch_max);
        p.printTagged ("prefetch-avgqueuelength", ((double)prefetch_total)/prefetch_attempts);
        p.printTagged ("prefetch-avgloads", ((double)prefetch_loads)/prefetch_attempts);
        p.printTagged ("callmin", mincallheight);
        p.printTagged ("callmax", maxcallheight);
        p.printTagged ("maxcall", actualmaxcall);
        p.endTag ("prefetch");
    }
};
