/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Jan 22 10:58:45 2002 (PriorityQueue)
 *
 */

package mokan.prefetch.runtime;
import java.util.Comparator;
import java.util.Collection;
import java.util.Iterator;
import java.util.Arrays;


/**
 *  The <code>PriorityQueue</code> keep a queue of ordered objects .. 
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */
public class PriorityQueue  implements Cloneable {

    protected Object[]   buffer;
    //protected int begin;
    //protected int end;
    protected int count;
    


    /* Private/Public Variables */
    Comparator comp;

    private void growQueue() {
        int newsize = buffer.length * 2; 
        if (newsize == 0) newsize = 1;
        Object [] nb = new Object[newsize];
        System.arraycopy (buffer, 0, nb, 0, buffer.length);
        buffer = nb;
    }            
    private void addLast (Object x) {
        if (count+1>buffer.length){
            growQueue();
        }
        buffer[count++] = x;
    }

    private final int compare(Object k1, Object k2) {
	return ((Comparable)k1).compareTo(k2);
    }

    private final void swap (int i, int j) {
            Object t = buffer[i];
            buffer[i] = buffer[j];
            buffer[j] = t;
    }
    private  void   heapify (int x){
        int smallest = x;
        int l = 2* (x)+1;
        int r = 2* (x)+2;
        if (r < count && compare(buffer[r], buffer[x]) < 0) 
            smallest = r;
        if (l < count && compare(buffer[l], buffer[smallest]) < 0)
            smallest = l;
        if (smallest != x) {
            swap (x, smallest);
            heapify (smallest);
        }
    }
    private  void   heapify_comparator (int x){
        int smallest = x;
        int l = 2* (x+1)-1;
        int r = 2* (x+1);
        if (l < count && comp.compare(buffer[l], buffer[x]) < 0) 
            smallest = l;
        if (r < count && comp.compare(buffer[r], buffer[smallest]) < 0)
            smallest = r;
        if (smallest != x) {
            swap (x, smallest);
            heapify_comparator (smallest);
        }
    }

    public PriorityQueue () {
        this(32);
    }
    public PriorityQueue (int initial) {
        buffer = new Object[initial];
        count = 0;
    }
    
    public PriorityQueue (Comparator c) {
        this ();
        comp = c;
    }

    public PriorityQueue (Collection c) {
        this(c.size());
        Iterator ci = c.iterator();
        while (ci.hasNext()) {
            putLast (ci.next());
        }
    }
    public Object clone() {
        PriorityQueue nq = new PriorityQueue (buffer.length);
        System.arraycopy (buffer, 0, nq.buffer, 0, buffer.length);
        nq.count = count;
        return nq;
    }
    public final void clear () {
        count = 0;
    }

    public final boolean isEmpty () {
        return count == 0; 
    }
    public int indexOf (Object o) {
        int index=0;
        while (index != count) {
            if (buffer[index++] == o)
                return index-1;
        }
        return -1;
    }
    public final Object first () {
        return buffer [0];
    }

    public final Object getFirst (){
        if (count == 0) throw new java.util.NoSuchElementException();

        Object min = buffer[0];
        try {
            buffer[0] = buffer[--count];
            if (comp != null) 
                heapify_comparator (0);
            else 
                heapify (0);
        }catch (RuntimeException e) {
            System.out.println (this.toString());
            throw e;
        }
        return min;
    }

    public final void putLast (Object a){
        addLast (null);
        int x = count-1;
        while (x > 0 && compare (a, buffer[(x-1)/2]) < 0) {
            buffer[x] = buffer[(x-1)/2]; // parent(x)
            x = (x-1)/2;
        }
        buffer[x] = a;
    }

    public final boolean remove (Object o) {
        int spot = indexOf (o);
        
        if (spot == -1 ) return false;

        buffer[spot] = buffer[--count];
        
        if (comp != null) 
            heapify_comparator (spot);
        else 
            heapify (spot);
        return true;
    }

    public String internalRep () {
        StringBuffer sb = new StringBuffer();
        for (int index=0; index < count; index++){
            sb.append (buffer[index].toString());
            sb.append (' ');
        }
        return sb.toString();
    }

    public String toString () {
        PriorityQueue nq = (PriorityQueue)clone();
        StringBuffer sb = new StringBuffer();
        for (int index=0; index < count; index++){
            sb.append (nq.getFirst());
            sb.append (' ');
        }
        return sb.toString();
    }

    public static void main(String[] args) {
        PriorityQueue pq = new PriorityQueue (4);

        System.out.println("new queue" + pq);

        
        System.out.println ("add 10, 20, 0");
        pq.putLast (new Integer (10));
        System.out.println("10 ->" + pq);
        pq.putLast (new Integer (20));
        System.out.println("20 ->" + pq);
        pq.putLast (new Integer (0));
        System.out.println("0 ->" + pq);

        System.out.println ("Should get : 0, 10, 20");
        

        System.out.println ("Remove min -> " + pq.getFirst());
        System.out.println("Got :" + pq);
        System.out.println ("add 0,30,40");
        
        pq.putLast (new Integer (0));
        System.out.println("0 ->" + pq);
        pq.putLast (new Integer (30));
        System.out.println("30 ->" + pq);
        pq.putLast (new Integer (40));
        System.out.println("40 ->" + pq);

        System.out.println ("Should get : 0, 10, 20, 30, 40");

        pq.clear();

        Integer a;

        pq.putLast (new Integer (10));
        System.out.println("10 ->" + pq);
        pq.putLast (a = new Integer (20));
        System.out.println("20 ->" + pq);
        pq.putLast (new Integer (0));
        System.out.println("0 ->" + pq);
        pq.putLast (new Integer (0));
        System.out.println("0 ->" + pq);
        pq.putLast (new Integer (40));
        System.out.println("40 ->" + pq);
        pq.putLast (new Integer (30));
        System.out.println("30 ->" + pq);
  
        System.out.println("Got :" + pq);
        System.out.println ("Should get : 0, 0, 10, 20, 30, 40");

        pq.remove (a);
        System.out.println("Remove 20 -> " +pq);
        System.out.println ("Should get : 0, 0, 10, 30, 40");

    }



};
