/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Jan 22 10:54:29 2002 (Queue)
 *
 */


package mokan.prefetch.runtime;

import java.util.Iterator;
import java.util.Arrays;

/**
 *  The <code>Queue</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class Queue {

    /* Private/Public Variables */
    protected Object[]   buffer;
    protected int begin;
    protected int end;
    protected int count;

    private void growQueue() {
        Object [] nb = new Object[buffer.length*2];
        if (begin < end) {
            System.arraycopy (buffer, 0, nb, 0, buffer.length);
            
        } else {
            int ep = buffer.length-begin;
            System.arraycopy (buffer, begin, nb, 0, ep);
            System.arraycopy (buffer, 0, nb, ep, end);
        }
        begin = 0; end = buffer.length;
        buffer = nb;
    }

    public Queue(int initial) {
        buffer = new Object[initial];
        begin = end = 0;
        count = 0;
    }

    public Queue() {
        this(8);
    }
    public void clear () {
        begin = end = count = 0;
        Arrays.fill (buffer, null);
    }

    public  Object getFirst(){
        if (count == 0) throw new java.util.NoSuchElementException();

        Object x = buffer[begin++];
        if (begin == buffer.length) {
            begin = 0;
        }
        count--;
        return x;
    }
    public  Object getLast(){
        if (count == 0) throw new java.util.NoSuchElementException();

        if (end == 0) {
            end = buffer.length;
        }
        Object x = buffer[--end];
        count--;
        return x;
    }
    public void putFirst (Object x) {
        if (count+1 > buffer.length){
            growQueue();
        }
        if (begin == 0) 
            begin = buffer.length;

        buffer[--begin] = x;
        count++;
    }
    public void putLast (Object x) {
        if (count+1 > buffer.length){
            growQueue();
        }
        buffer[end++] = x;
        if (end == buffer.length) end = 0;
        count++;
    }
    public Object first () {
        if (count == 0) throw new java.util.NoSuchElementException();
        return buffer[begin];
    }
    public Object last () {
        if (count == 0) throw new java.util.NoSuchElementException();
        if (end == 0) return buffer[buffer.length-1];
        return buffer[end-1];
    }

    public Object get (int index) {
        if (index < count) {
            return buffer[(begin + index) % buffer.length];
        }
        throw new java.util.NoSuchElementException();
    }

    public int indexOf (Object o) {
        int index=0;
        int b = begin, e = end;
        while (index != count) {
            if (buffer[b++] == o)
                return index;
            if (b == buffer.length) b = 0;
            index++;
        }
        return -1;
    }

    public boolean contains(Object o) {
        int b = begin, e = end;
        int index = 0;
        while (index != count) {
            if (buffer[b++].equals (o))
                return true;
            if (b == buffer.length) b = 0;
            index++;
        }
        return false;
    }

    public boolean containsExactly(Object o) {
        int b = begin;
        int index = 0;
        while (index !=count) {
            if (buffer[b++] == o)
                return true;
            if (b == buffer.length) b = 0;
            index++;
        }
        return false;
    }

    public int  length(){
        return count;
    }
    public boolean isEmpty() {
        return count == 0;
    }

    public boolean remove (Object o) {
        throw new UnsupportedOperationException();
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append ('[');
        int index=0;
        int b = begin, e = end;
        while (index != count) {
            sb.append (buffer[b++]);
            sb.append (',');
            if (b == buffer.length) b = 0;
            index++;
        }
        sb.append(']');
        sb.append(begin);
        sb.append(':');
        sb.append(end);
        sb.append(':');
        sb.append(count);
        return sb.toString();
    }
    public String internalRep () {
        StringBuffer sb = new StringBuffer();
        sb.append("count:");
        sb.append(count);
        sb.append("begin:");
        sb.append (begin);
        sb.append("end:");
        sb.append (end);
        sb.append ('[');
        for (int i=0; i < buffer.length; i++) {
            sb.append(buffer[i]);
            sb.append (',');
        }
        sb.append (']');
        return sb.toString();
    }

    public class QueueIterator implements Iterator {
        private int pos;
        private int index;

        public QueueIterator () {
            pos = begin;
            index = 0;
        }
        public boolean hasNext() {
            return index < count;
        }

        public Object next() {
            if (pos == end) throw new java.util.NoSuchElementException();
            Object x = buffer[pos++];
            index++;
            if (pos == buffer.length) pos = 0;
            return x;
        }

        public void remove () {
            throw new UnsupportedOperationException();
        }
    }


    public Iterator iterator () { return new QueueIterator (); }

};
