/**
 * ShowShape
 * tag: kkvilekal@acm.org 2002/11/14 00:55:31
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.tools;


import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.PrintStream;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collections;
import java.util.Comparator;

import mokan.prefetch.util.RTMethod;

import agl2.Bigraph;
import agl2.VertexGraph;
import agl2.Graphs;


/**
 *  The <code>GraphShape</code> generates a directed graph 
 *  description file suitable for the GraphViz suite (dot)
 *  The first argument is to a program shape file (ext .sg).
 *  @version $Revision: 1.1 $, $Date: 2003/02/06 03:20:23 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class GraphCalls {
    /* Private/Public Variables */


    Bigraph calls;
    PrintStream out;


    public String nodeString (Object o) {

        RTMethod m = (RTMethod)o;

        String name = m.getFullname();

        // Strip off Package names 
        int spot = name.lastIndexOf ('.'); // Class.Method point
        int pspot = name.lastIndexOf('.', spot-1);
        if (pspot >= 0) {
            name = name.substring (pspot+1);
        }

        name = name.replace ('.','_');
        name = name.replace ('$','_');
        name = name.replace ('<','_');
        name = name.replace ('>','_');

        return name;
    }

    public String graphViz (VertexGraph g) {
        StringBuffer sb = new StringBuffer ();

        sb.append ("digraph G {");
        Iterator ni = g.getVertices().iterator();
        while (ni.hasNext()) {
            Object n = ni.next();
            Iterator adjs = g.getAdjacentVertices (n).iterator();
            String s = nodeString (n);
            while (adjs.hasNext()) {
                sb.append (s); 
                sb.append ("->");
                sb.append (nodeString(adjs.next())); 
                sb.append(";\n");
            }
        }
        sb.append ("}");
        return sb.toString ();
    }


    public void GraphAll () {
        Object r = calls.getRoot ();

        out.print (graphViz (calls));

    }

    public GraphCalls (Bigraph g, String outname) {
        calls = g;
        try {
        out = new PrintStream (new FileOutputStream (outname));
        }catch (java.io.IOException e){
            System.err.println ("Couldn't open " + outname);
        }
    }

    public static void main(String[] args) {

        String graphfile = args[0];
        Bigraph  calls=null;
        try {
            FileInputStream istream = new FileInputStream(graphfile);
            ObjectInputStream p = new ObjectInputStream(istream);
            calls = (Bigraph)p.readObject ();
            istream.close();


        } catch (java.io.IOException io) {
            System.err.println  ("Reading graphs in " + graphfile);
            io.printStackTrace();
        } catch (ClassNotFoundException mc) {
            System.err.println ("No shapes in  " + graphfile);
        }

        GraphCalls gs = new GraphCalls (calls, "calls.out");

        System.out.println (calls.properties());

        gs.GraphAll();
    }

}
