/**
 * ShowShape
 * tag: kkvilekal@acm.org 2002/11/14 00:55:31
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.prefetch.tools;


import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.PrintStream;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collections;
import java.util.Comparator;

import mokan.prefetch.analysis.shapeanalysis.RTVariable;
import mokan.prefetch.analysis.shapeanalysis.ShapePool;
import mokan.prefetch.analysis.shapeanalysis.ShapeGraph;
import mokan.prefetch.analysis.shapeanalysis.ShapeEdge;
import mokan.prefetch.analysis.shapeanalysis.MethodState;
import mokan.prefetch.analysis.shapeanalysis.MethodTable;


import agl.Graphs;

/**
 *  The <code>GraphShape</code> generates a directed graph 
 *  description file suitable for the GraphViz suite (dot)
 *  The first argument is to a program shape file (ext .sg).
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class GraphShape {
    /* Private/Public Variables */

    MethodTable methods;
    MethodState mstate;
    ShapePool   pool;
    PrintStream out;
    String name;

     int variables;
     int graph_count; 
     int method_count;

    public void GraphVariable (int spot) {
        
        RTVariable root = pool.getVariable (mstate.getLocal(spot));
        if (root != null){
            out.println ("digraph G {");
            out.println ("label = \""+name+':'+spot +"\";");
            out.println ("label = \"callevel="+ mstate.callLevel+"\";");
            out.print (pool.graph (root));
            out.println ("}");
        }
    }

    public void GraphMethod (String name){
        this.mstate = methods.getState (name);
        this.name = mstate.getName();
        this.pool = mstate.getPool();
        int local_sz = mstate.locals.size();

        for (int a=0; a < local_sz; a++) {
            GraphVariable (a);
        }
    }

    public void GraphAll () {
        Iterator mi = methods.keySet().iterator();
        while (mi.hasNext()){
            String name = (String)mi.next();
            GraphMethod (name);
        }
    }

    public GraphShape (MethodTable t, String outname) {
        methods = t;
        try {
        out = new PrintStream (new FileOutputStream (outname));
        }catch (java.io.IOException e){
            System.err.println ("Couldn't open " + outname);
        }
    }


    private static void usage () {
            System.err.println("usage: java pretime.GraphShape [-stats] shapegraph-file");
            System.exit(0);
    }


    public static void main(String[] args) {

        String shapefile=null;
        MethodTable methods=null;
        boolean stats = false;

        if (args.length < 1) {
            usage();
        }
        String arg;
        for (int a=0; a < args.length; a++) {
            if (args[a].startsWith("-stats")) {
                stats = true;
            } else {
                shapefile = args[a];
            }
        }
       

        if (shapefile == null) {
            usage();
        }
        try {
            FileInputStream istream = new FileInputStream(shapefile);
            ObjectInputStream p = new ObjectInputStream(istream);
            methods = (MethodTable)p.readObject ();
            istream.close();


        } catch (java.io.IOException io) {
            System.err.println  ("Reading graphs in " + shapefile);
            io.printStackTrace();
        } catch (ClassNotFoundException mc) {
            System.err.println ("No shapes in  " + shapefile);
        }

        GraphShape gs = new GraphShape (methods, "shapes.out");


        gs.GraphAll();
        if (stats) 
            gs.printStats();
    }

    public void printStats (){
        System.out.println ("Methods =" +  methods.keySet().size());
        Iterator mi = methods.keySet().iterator();
        int graph_count = 0;

        while (mi.hasNext()){
            String name = (String)mi.next();
            mstate = methods.getState (name);
            int local_sz = mstate.locals.size();
            graph_count += local_sz;
        }

        System.out.println ("Graphs =" + graph_count);

    }

}
