/*
 * 
 *
 * tag: kkvilekval@acm.org Thu Oct 18 16:01:22 2001 (NestedIterator)
 *
 */


package mokan.prefetch.util;

import java.util.Iterator;
import java.util.Collection;
import java.util.NoSuchElementException;

/**
 *  The <code>NestedIterator</code> allows a set to be iterated over
 *  calling another iterator function.  
 *
 * Example Use: Iterator over a * set of containers calling iterator
 * on each one.  S = * Set<Set<Object>> Iterator ni = new
 * NestedIterator (S.iterator(), * new NestedIterator());
 *  
 *  
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class NestedIterator implements Iterator {

    /* Private/Public Variables */
    Iterator top;
    Functor mapFun;
    Iterator nested;

    public static class Functor {
        Iterator getIterator(Object o) {
            return null;
        }
    }

    /**
     *  The default constructor
     */
    public NestedIterator () {
    }

    public NestedIterator  (Iterator c, Functor f) {
        top = c; 
        mapFun = f;
        if (top.hasNext())
            nested = mapFun.getIterator (top.next());
    }

    Iterator getIterator (Object o) {
        return ((Collection)o).iterator();
    }


    public boolean hasNext() {
        return (nested != null && nested.hasNext()) || top.hasNext();
    }
    public Object  next() {
        while (!nested.hasNext()) {
            if (top.hasNext()) 
                nested = mapFun.getIterator (top.next());
            else
                throw new NoSuchElementException();
        }
        return nested.next();
    }
    public void remove () {
        nested.remove();
    }
};





    
