/*
 * 
 *
 * tag: kkvilekval@acm.org Wed Nov  7 01:39:43 2001 (RTMethod)
 *
 */

package mokan.prefetch.util;

import java.io.Serializable;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;


import java.util.HashMap;
import java.util.StringTokenizer;

import de.fub.bytecode.Repository;
import de.fub.bytecode.Constants;

import de.fub.bytecode.classfile.JavaClass;
import de.fub.bytecode.classfile.Method;
import de.fub.bytecode.classfile.ConstantPool;

import de.fub.bytecode.generic.ConstantPoolGen;
import de.fub.bytecode.generic.MethodGen;
import de.fub.bytecode.generic.CodeExceptionGen;
import de.fub.bytecode.generic.LineNumberGen;
import de.fub.bytecode.generic.LocalVariableGen;

import de.fub.bytecode.generic.ObjectType;

import de.fub.bytecode.generic.InstructionList;
import de.fub.bytecode.generic.InstructionHandle;

import de.fub.bytecode.generic.Instruction;
import de.fub.bytecode.generic.BranchInstruction;
import de.fub.bytecode.generic.IfInstruction;
import de.fub.bytecode.generic.Select;
import de.fub.bytecode.generic.BranchHandle;

import de.fub.bytecode.generic.TABLESWITCH;

/**
 *  The <code>RTMethod</code> description.
 *  @version $Revision: 1.2 $, $Date: 2003/01/24 18:23:17 $
 *  @author Kristian Kvilekval
 */
public class RTMethod implements Serializable {

    /** The class that this methods belongs to */
    private RTClass cl;

    /** Cached values of BCEL elements */
    private transient Method    method;
    private transient MethodGen methodgen;

    /** Cached fully qualified <class>.<name>(parameters)<ret>  */
    private String fqname ;
    /** Cached signature <shortname>(parameters)<return> */
    private String fullsignature;
    /** classname.methodname */
    private String methodname;

    /**
     *  The default constructor
     */
    public RTMethod (RTClass cls, Method m) {
        this.cl = cls;
        this.method = m;
        fullsignature = m.getName() + m.getSignature();
        fqname = cl.getClassName() + '.' + fullsignature;
        methodname = cl.getClassName()+'.'+m.getName();
    }
    public RTMethod (String name) {
        methodname = fqname = fullsignature = name;
    }


    public boolean isAbstract() {
        return method.isAbstract(); 
    }
    public boolean isNative() {
        return method.isNative(); 
    }
    public boolean isInterface() {
        return method.isInterface(); 
    }
    public boolean isFinal() {
        return method.isFinal(); 
    }
    public boolean isStatic() {
        return method.isStatic(); 
    }
    public boolean isSystemMethod () {
        return cl.isSystemClass();
    }
    public RTClass getDefiningClass () {
        return cl;
    }
    public ConstantPool getConstantPool () {
        return cl.getConstantPool();
    }
    public ConstantPoolGen getConstantPoolGen () {
        return cl.getConstantPoolGen();
    }
    public String getSignature () {
	return fullsignature;
    }
    /** Simple name for method  i.e. println */
    public String getName () {
        return method.getName();
    }
    public String getFullname () {
        return methodname;
    }
    public Method getMethod() {
        return method;
    }
    public void setMethod(Method m) {
        method = m;
    }
    public int getMaxLocals() {
        return getMethodGen().getMaxLocals();
    }
    public int getMaxStack() {
        return getMethodGen().getMaxStack();
    }
    public CodeExceptionGen[]  getExceptionHandlers () {
        return getMethodGen().getExceptionHandlers();
    }
    public MethodGen getMethodGen () {
        if (methodgen == null) {
            methodgen = new MethodGen (method, cl.getClassName(), 
                                       cl.getConstantPoolGen());

        }
        return methodgen;
    }
    public InstructionHandle[] getInstructions() {
        InstructionList il =  getMethodGen().getInstructionList();
        InstructionHandle[] handles = il.getInstructionHandles();
        return handles;
    }


    public void visitInstructions (RTInstructionVisitor v) {
        MethodGen           mg  = getMethodGen();
        InstructionList     il  = mg.getInstructionList();
        if (il != null) {
            Instruction[]       ihs = il.getInstructions();
            for(int i=0; i < ihs.length; i++) {
                //log.info ("instr " + ihs[i]);
                v.setPosition (i);
                ihs[i].accept (v);
            }
        }
    }

    /**
     *  @return a string representation of this <code>RTMethod</code> 
     */
    public String toString () {
        return fqname;
    }

    public static String dumpListing (RTMethod m) {

        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        PrintStream     out = new PrintStream(stream);

        int label_counter = 0;

//         out.println(".limit stack " + code.getMaxStack());
//         out.println(".limit locals " + code.getMaxLocals());
        if(m.isNative() ){
            out.println (m + " is native");
            return stream.toString();
        }
        if ( m.isAbstract()){
            out.println (m + " is abstract");
            return stream.toString();
        }
        if ( m.isInterface()){
            out.println (m + " is an interface");
            return stream.toString();
        }


        MethodGen           mg  = m.getMethodGen();
        InstructionList     il  = mg.getInstructionList();
        InstructionHandle[] ihs = il.getInstructionHandles();

        /* Pass 1: Give all referenced instruction handles a symbolic name, i.e. a
         * label.
         */
        LabelMap map = new LabelMap();

        for(int i=0; i < ihs.length; i++) {
            if(ihs[i] instanceof BranchHandle) {
                BranchInstruction bi = (BranchInstruction)ihs[i].getInstruction();
	
                if(bi instanceof Select) { // Special cases LOOKUPSWITCH and TABLESWITCH
                    InstructionHandle[] targets = ((Select)bi).getTargets();
	  
                    for(int j=0; j < targets.length; j++)
                        map.put(targets[j], "Label" + label_counter++ + ":");
                }

                InstructionHandle ih = bi.getTarget();
                map.put(ih, "Label" + label_counter++ + ":");
            }
        }

         LocalVariableGen[] lvs = mg.getLocalVariables();
         for(int i=0; i < lvs.length; i++) {
             InstructionHandle ih = lvs[i].getStart();
             map.put(ih, "Label" + label_counter++ + ":");
             ih = lvs[i].getEnd();
             map.put(ih, "Label" + label_counter++ + ":");	
         }
    
         CodeExceptionGen[] ehs = mg.getExceptionHandlers();
         for(int i=0; i < ehs.length; i++) {
             CodeExceptionGen  c  = ehs[i];
             InstructionHandle ih = c.getStartPC();
            map.put(ih, "Label" + label_counter++ + ":");	
             ih = c.getEndPC();
             map.put(ih, "Label" + label_counter++ + ":");	
             ih = c.getHandlerPC();
             map.put(ih, "Label" + label_counter++ + ":");	
         }

        LineNumberGen[] lns = mg.getLineNumbers();
        for(int i=0; i < lns.length; i++) {
            InstructionHandle ih = lns[i].getInstruction();
            map.put(ih, ".line " + lns[i].getSourceLine());
        }
 
        /* Pass 2: Output code.
         */
        for(int i=0; i < lvs.length; i++) {
            LocalVariableGen l = lvs[i];
            out.println(".var " + l.getIndex() + " is " + l.getName() + " " +
                        l.getType().getSignature() + 
                        " from " + map.get(l.getStart()) + 
                        " to " + map.get(l.getEnd()));
        }

        out.print("\n");
    
        for(int i=0; i < ihs.length; i++) {
            InstructionHandle ih   = ihs[i];
            Instruction       inst = ih.getInstruction();
            String            str  = (String)map.get((Object)ih);
      
            if(str != null)
                out.println(str);

            //out.print (ih.getPosition());
            out.print (i);
            if(inst instanceof BranchInstruction) {
                if(inst instanceof Select) { // Special cases LOOKUPSWITCH and TABLESWITCH
                    Select              s       = (Select)inst;
                    int[]               matchs  = s.getMatchs();
                    InstructionHandle[] targets = s.getTargets();
                    
                    if(s instanceof TABLESWITCH) {
                        out.println("\ttableswitch " + matchs[0] + " " +
                                    matchs[matchs.length - 1]);
	    
                        for(int j=0; j < targets.length; j++)
                            out.println("\t\t" + map.get(targets[j]));

                    } else { // LOOKUPSWITCH
                        out.println("\tlookupswitch ");

                        for(int j=0; j < targets.length; j++)
                            out.println("\t\t" + matchs[j] + " : " + map.get(targets[j]));
                    }

                    out.println("\t\tdefault: " + map.get(s.getTarget())); // Applies for both
                } else {
                    BranchInstruction bi  = (BranchInstruction)inst;
                    ih  = bi.getTarget();
                    str = map.get(ih);
                    out.println("\t" + Constants.OPCODE_NAMES[bi.getTag()] + " " + str);
                }
            }
            else
                out.println("\t" + inst.toString(m.getConstantPool()));
        }
    
        out.print("\n");

        for(int i=0; i < ehs.length; i++) {
            CodeExceptionGen c = ehs[i];
            ObjectType caught = c.getCatchType();
            String class_name = (caught == null)?  // catch any exception, used when compiling finally
                "all" : caught.getClassName().replace('.', '/');

            out.println(".catch " + class_name + " from " +
                        map.get(c.getStartPC()) + " to " + map.get(c.getEndPC()) +
                        " using " + map.get(c.getHandlerPC()));
        }

        return stream.toString();
    }

    private static class LabelMap  extends HashMap{


        public final String get(InstructionHandle ih) {
            String str = new StringTokenizer((String)get((Object)ih), "\n").nextToken();
            return str.substring(0, str.length() - 1);
    }
    
        public final void put(InstructionHandle ih, String line) {
            String str = (String)get((Object)ih);
            
            if(str == null)
                put((Object)ih, (Object)line);
            else {
                if(line.startsWith("Label") || str.endsWith(line)) // Already have a label in the map
                    return;
                
                put((Object)ih, (Object)(str + "\n" + line)); // append
            }
        }     
        
    }



};
