/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Nov  6 14:06:24 2001 (UnionFind)
 *
 */
package mokan.prefetch.util;

import java.io.Serializable;

import java.util.Set;
import java.util.Collections;
import java.util.Iterator;
import java.util.AbstractSet;
import java.util.HashMap;
import java.util.NoSuchElementException;

/**
 *  The <code>UnionFind</code> Discjoint Set code with some set interfaces
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Unknown  Borrowed from Jeff Bogda Extended by Kristian Kvilekval
 */
public class UnionFind implements Serializable {
    // node representation.
    private static class Node implements Serializable  {
        Node parent;
        Node next;
        Node last;
        final Object element;
        int rank;
        Node(Object element) {
            this.parent = this; this.element = element; this.rank = 0;
            this.next  = null; this.last = this;
        }
    }

    private final HashMap elmap = new HashMap();
	
    /** Creates a <code>DisjointSet</code>. */
    public UnionFind() { }
	
    /** Unites the dynamic sets that contain <code>o1</code> and
     *  <code>o2</code>, say S1 and S2, into a new set that is the
     *  union of these two sets.  The two sets are assumed to be
     *  disjoint prior to the operation.  The representative of the
     *  resulting set is the representative of either S1 or S2; if
     *  both S1 and S2 were previously singletons, the representative
     *  of S1 union S2 is the representative of S2. */
    public void union(Object o1, Object o2) {
        Node x = (Node) elmap.get(o1);
        Node y = (Node) elmap.get(o2);
        if (x==null) x=_make_set(o1);
        if (y==null) y=_make_set(o2);
        //Assertion.assertion(!x.equals(y), "Sets assumed to be disjoint");
        _union(x, y);
    }

    /** Returns the representative of the (unique) set containing
     *  <code>o</code>. */
    public Object find(Object o) {
        Node x = (Node) elmap.get(o);
        if (x==null) return o;
        return _find_set(x).element;
    }

    /** Determines if there is a set of more than one element containing
     *  <code>o</code>. */
    public boolean contains(Object o) {
        return elmap.containsKey(o);
    }

    // these are the routines according to CLR
    private Node _make_set(Object o) {
        //Assertion.assertion(!elmap.containsKey(o));
        Node x = new Node(o);
        elmap.put(o, x);
        return x;
    }

    private Node _find_set(Node x) {
        if (x.parent != x)
            x.parent = _find_set(x.parent);
        return x.parent;
    }

    private void _union(Node x, Node y) {
        _link(_find_set(x), _find_set(y));
    }

    private void _link(Node x, Node y) {
        if (x.rank < y.rank){
            x.parent = y;
            y.last.next = x;
            y.last = x.last;
        } else {
            y.parent = x;
            x.last.next = y;
            x.last = y.last;

            if (x.rank == y.rank)
                y.rank++;
        }
    }

    public boolean sameSet( Object element1, Object element2 ) {
        return find( element1 ).equals( find( element2 ));
    }

    public String toString () {
        MultiMap mm = new MultiMap ();
        Iterator mi = elmap.keySet().iterator();
        while (mi.hasNext()) {
            Object v = mi.next();
            Object r = find (v);
            mm.add (r, v);
        }
        return mm.toString();
    }


    public Set members (Object o) {
        final Node x = (Node) elmap.get(o);
        if (x != null) 
            return new AbstractSet () {
                    public Iterator iterator () {
                        return new SetIterator (_find_set(x));
                    }
                    public int size() {
                        Node n = _find_set(x);
                        // Uggh count the list;
                        int count = 1;
                        while (n.next != null){
                            n = n.next;
                            count++;
                        }
                        return count;
                    }
                };
        return Collections.EMPTY_SET;
    }

    private static class SetIterator implements Iterator {
        Node current;
        SetIterator (Node root) {
            
        }
        public boolean hasNext() {
            return current != null;
        }
        public Object next() {
            if (current != null) {
                Object o = current.element;
                current = current.next;
                return o;
            }
            throw new java.util.NoSuchElementException();
        }
        public void remove () {
            throw new UnsupportedOperationException("SetIterator");
      }
    }
}
