/*
 * 
 *
 * tag: kkvilekval@acm.org Tue Nov 27 14:01:15 2001 (XMLWriter)
 *
 */


package mokan.prefetch.util.xml;

import java.io.Writer;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.util.Arrays;


/**
 *  The <code>AccessWriter</code> description.
 *  @version $Revision: 1.1.1.1 $, $Date: 2002/12/19 23:48:36 $
 *  @author Kristian Kvilekval
 */
public class XMLWriter extends Writer  {

    /* Private/Public Variables */

    char[] indent;
    int    count;
    boolean bol;
    Writer out;
    

    /**
     *  The default constructor
     */
    public XMLWriter (Writer out) {
        this.out= out;
        count = 0;
        indent = new char[0];
        bol = true;
    }
    public XMLWriter (OutputStream out) {
        this(new OutputStreamWriter(out));
    }

    void inc() {
        if (count < 80){
            count+=2;
            indent = new char[count];
            Arrays.fill (indent, ' ');
        }
    }
    void dec() {
        if (count >0){
            count-=2;
            indent = new char[count];
            Arrays.fill (indent, ' ');
        }
    }

    private void writeEsc (String s) {
        try {
        int c = 0, l = s.length();
        while (c < l) {
            char x = s.charAt (c++);
            switch (x) {
            case '&':
                out.write("&amp;");
                break;
            case '<':
                out.write("&lt;");
                break;
            case '>':
                out.write("&gt;");
                break;
            default:
                if (x > '\u007f') {
                    out.write("&#");
                    out.write(Integer.toString(x));
                    out.write(';');
                } else {
                    out.write(x);
                }
            }
        }
        } catch (IOException e) {
        }
    }
    private void writeEsc (char s[]) {
        try {
        int c = 0, l = s.length;
        while (c < l) {
            char x = s[c++];
            switch (x) {
            case '&':
                out.write("&amp;");
                break;
            case '<':
                out.write("&lt;");
                break;
            case '>':
                out.write("&gt;");
                break;
            default:
                if (x > '\u007f') {
                    out.write("&#");
                    out.write(Integer.toString(x));
                    out.write(';');
                } else {
                    out.write(x);
                }
            }
        }
        } catch (IOException e) {
        }
    }


    private void writeTag (String tag) {
        // Left for namespace support and such..
        write(tag);
    }

    public void close() {
        try {
        out.close();
        } catch (IOException e) {
        }
    }
    public void flush() {
        try {
        out.flush();
        } catch (IOException e) {
        }
    }

    public void write(int c) {
        try {
            if (bol) out.write(indent);
            out.write(c);  
            bol= false;
            if (c == '\n') bol = true;
        } catch (IOException e) {
        }
    }

    
    public void write(char[] s) {
        try {
        if (bol) out.write (indent);
        out.write(s);
        bol=false;
        } catch (IOException e) {
        }
    }
    public void write(char s[], int off, int len) {
        try {
        if (bol) out.write (indent);
        out.write(s,off,len);
        bol=false;
        } catch (IOException e) {
        }
    }
    public void write(String s) {
        try {
        if (bol) out.write (indent);
        out.write(s);
        bol=false;
        } catch (IOException e) {
        }
    }
    public void write(String s, int off, int len) {
        try {
        if (bol) out.write (indent);
        out.write(s, off, len);
        bol=false;
        } catch (IOException e) {
        }
    }
    public void writeln() {
        try {
            out.write('\n'); out.flush();
        bol=true;
        } catch (IOException e) {
        }
    }
    public void writeln(int c) {
        try {
            out.write(c); out.write('\n'); out.flush();
        bol=true;
        } catch (IOException e) {
        }
    }
    public void writeln(String s) {
        try {
            out.write(s); out.write('\n'); out.flush();
        bol=true;
        } catch (IOException e) {
        }
    }

    public void print(boolean b) {
        write (b?"true":"false");
    }
    public void print(char c) {
        switch (c) {
        case '&':
            write("&amp;");
            break;
        case '<':
            write("&lt;");
            break;
        case '>':
            write("&gt;");
            break;
        default:
            if (c > '\u007f') {
                write("&#");
                write(Integer.toString(c));
                write(';');
            } else {
                write(c);
            }
        }
    }
    public void print(char[] s) {
        writeEsc (s);
    }

    public void print(double d) {
        write(Double.toString(d));
    }
    public void print(float f) {
        write (Float.toString (f));
    }
    public void print(int i) {
        write (Integer.toString(i));
    }
    public void print(long l) {
        write(Long.toString(l));
    }
    public void print(Object obj) {
        if (obj != null)
            writeEsc (obj.toString());
    }
    public void print(String s) {
        if (s != null)
            writeEsc (s);
    }
    public void println(char[] s) {
        writeEsc (s);
        writeln();
    }
    public void println(boolean b) {
        write (b?"true":"false");
        writeln();
    }
    public void println(char c) {
        writeln();
    }
    public void println(double d) {
        write(Double.toString(d));
        writeln();
    }
    public void println(float f) {
        write (Float.toString (f));
        writeln();
    }
    public void println(int i) {
        write (Integer.toString(i));
        writeln();
    }
    public void println(long l) {
        write(Long.toString(l));
        writeln();
    }
    public void println(Object obj) {
        if (obj != null)
            writeEsc (obj.toString());
        writeln();
    }
    public void println(String s) {
        if (s != null) 
            writeEsc (s);
        writeln();
    }
    public void println () {
        writeln();
    }


    public void startTag(String t){
        write('<');
        writeTag(t);
        write('>');
        inc();
    }
    public void startTag(String t, String o){
        write('<');
        writeTag(t);
        write(' '); write (o);
        writeln('>');
        inc();
    }

    public void startTag(String t, Object[] args){
        write('<');
        writeTag(t);
        for (int i=0; i < args.length; i+=2) {
            write(' ');
            write(args[i].toString());
            write ('=');
            write ('"');
            write (args[i+1].toString());
            write ('"');
        }
        writeln('>');
        inc();
    }


    public void endTag(String t){
        dec();

        write("</");
        writeTag(t);
        writeln('>');
        bol = true;
    }

    /** Print a single tagged value <tag /> */
    public void printTag(String t){
        write('<');
        write(t);
        write("/>");
    }

    /** Print a single tagged value <tag dkd="llsls" /> */
    public void emptyTag(String t, Object[] args){
        write('<');
        write(t);
        for (int i=0; i < args.length; i+=2) {
            write(' ');
            write(args[i].toString());
            write ('=');
            write ('"');
            write (args[i+1].toString());
            write ('"');
        }
        writeln("/>");
    }
    public void printTagVal(String tag, int v){
        write('<');
        write(tag);
        write(" value=\"");
        print(v);
        writeln("\" />");
    }

    public void printTagVal(String tag, long v){
        write('<');
        write(tag);
        write(" value=\"");
        print(v);
        writeln("\" />");
    }

    public void printTagVal(String tag, double v){
        write('<');
        write(tag);
        write(" value=\"");
        print(v);
        writeln("\" />");
    }

    public void printTagVal(String tag, String v){
        write('<');
        write(tag);
        write(" value=\"");
        print(v);
        writeln("\" />");
    }



    /** Print a tagged vcalue <tag>value</tag> */
    public void printTagged (String tag, String v) {
        startTag (tag);print (v); endTag(tag);
    }
    public void printTagged (String tag, int v) {
        startTag (tag);print (v); endTag(tag);
    }
    public void printTagged (String tag, long v) {
        startTag (tag);print (v); endTag(tag);
    }
    public void printTagged (String tag, double v) {
        startTag (tag);print (v); endTag(tag);
    }

};
