/*
 * @(#)$Id: Driver.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test;

import java.util.Random;
import mokan.*;
import mokan.obj.*;
import mokan.thread.*;
import mokan.transact.*;

/**
 * Run tests on the Kan system.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Suk Y. Lee
 * @author	Jerry James 
 */
public class Driver implements GlobalObj, Cloneable {
    /**
     * Probability of a write operation.
     */
    private static float probLocal, probPush, probPop, probTransfer;
    private ObjectID objID;
    private Random randObj;
    private Random randMeth;
    private Random randLocal;
    private int numMethod;
    private int currRmtOID = 0;
    private int currRmtOIDcnt = 51;
    private int numNodes;
    private int numObjects;

    public Driver(ObjectID oid, Float localRatio, Float updateRatio,
		  Integer nMethInv, Integer nNodes, Integer nObjects) {
	probLocal = localRatio.floatValue();
	probPush = updateRatio.floatValue();
	probPop = (float)0.5 * probPush;
	probTransfer = (float)0.25 * probPop;
	objID = oid;
	randObj   = new Random();
	randMeth  = new Random(randObj.nextLong());
	randLocal = new Random(randMeth.nextLong());
	numNodes = nNodes.intValue();
	numMethod = nMethInv.intValue() / numNodes;
	numObjects = nObjects.intValue();
    }

    public void startSim(Integer meInt, ObjectID[] oids) {
	long startTime = System.currentTimeMillis();
	Thread thisThread = Thread.currentThread();

	int me = meInt.intValue();
	Future tid[] = new Future[numMethod];
	float f;
	int nextOid1;
	int nextOid2;
	int nextOid3;
	int cntPop = 0;
	int cntPush = 0;
	int cntTop = 0;

	Object args1[][] = new Object[numMethod][1];
	Object args2[][] = new Object[numMethod][2];

	for (int i = 0; i < numMethod; i++) {
	    try {
		thisThread.sleep(100);
	    } catch(InterruptedException ex) {
		System.err.println("startSim's sleep() was interrrupted!");
	    }

	    try {
		f = randMeth.nextFloat();
		boolean didIt = false;
		/*
		if (f < probTransfer3) {
		    nextOid1 = nextOID(me);
		    nextOid2 = nextOID(me);
		    while(nextOid2 == nextOid1)
			nextOid2 = nextOID(me);
		    nextOid3 = nextOID(me);
		    while(nextOid3 == nextOid1 || nextOid3 == nextOid2)
			nextOid3 = nextOID(me);
		    args2[i][0] = oids[nextOid2];
		    args2[i][1] = oids[nextOid3];
		    tid[i] = KanSystem.invokeMethod(oids[nextOid1],
						    "transfer3", args2[i],
						    false);
		    System.err.println("Driver invoked transfer3.");
		} else */
		if (f < probTransfer) {
		    nextOid1 = nextOID(me);
		    nextOid2 = nextOID(me);
		    while(nextOid2 == nextOid1)
			nextOid2 = nextOID(me);
		    args1[i][0] = oids[nextOid2];
		    while (!didIt) {
			try {
			    tid[i] =
				KanSystem.invokeMethod(oids[nextOid1],
						       "transfer2", args1[i],
						       true);
			    System.err.println("Driver invoked transfer2.");
			    didIt = true;
			} catch (TransactionException te) {
			}
		    }
		} else if (f < probPop) {
		    while (!didIt) {
			try {
			    tid[i] =
				KanSystem.invokeMethod(oids[nextOID(me)],
						       "pop", null, true);
			    cntPop++;
			    System.err.println("Driver invoked pop.");
			    didIt = true;
			} catch (TransactionException te) {
			}
		    }
		} else if (f < probPush) {
		    args1[i][0] = new String("XXX");
		    while (!didIt) {
			try {
			    tid[i] =
				KanSystem.invokeMethod(oids[nextOID(me)],
						       "push", args1[i], true);
			    cntPush++;
			    System.err.println("Driver invoked push.");
			    didIt = true;
			} catch (TransactionException te) {
			}
		    }
		} else {
		    while (!didIt) {
			try {
			    tid[i] =
				KanSystem.invokeMethod(oids[nextOID(me)],
						       "top", null, true);
			    cntTop++;
			    System.err.println("Driver invoked top.");
			    didIt = true;
			} catch (TransactionException te) {
			}
		    }
		}
	    } catch(Exception e) {
		System.err.println("startSim caught an exception.");
		e.printStackTrace();
	    }
	}
	System.err.println("Joining " + numMethod + " threads for exit.");
	for (int i = 0; i < numMethod; i++) {
	    System.err.println("Waiting for thread " + tid[i]);
	    try {
		tid[i].join();
	    } catch (TransactionException te) {
		System.err.println("Caught exception while joining with " +
				   tid[i]);
	    }
	}

	System.out.println("Finished startSim! with " + cntPop + " " + cntPush
			   + " " + cntTop);
	System.err.println("Elapsed time: " +
			   (System.currentTimeMillis() - startTime));
    }

    private int nextOID(int me) {
	// if (numNodes == 1)
	    // return (Math.abs(randObj.nextInt()) % numObjects);
	int ret;
	float f = randLocal.nextFloat();

	if (f < probLocal) {
	    ret = numNodes * (Math.abs(randObj.nextInt()) %
			      (numObjects / numNodes)) + me; 
	    if (ret % numNodes != me) {
		System.err.println("Driver.nextOID(): not local.");
		System.exit(0);
	    }
	    return ret;
	} else {
	    ret = Math.abs(randObj.nextInt()) % numObjects;
	    while(me == (ret % numNodes))
		ret = Math.abs(randObj.nextInt()) % numObjects;
	    return ret;
	}
    }

    public String toString() {
	return "Driver " + objID;
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	Driver d = (Driver)obj;
	probLocal = d.probLocal;
	probPush = d.probPush;
	probPop = d.probPop;
	probTransfer = d.probTransfer;
	randObj = d.randObj;
	randMeth = d.randMeth;
	randLocal = d.randLocal;
	numMethod = d.numMethod;
	currRmtOID = d.currRmtOID;
	currRmtOIDcnt = d.currRmtOIDcnt;
	numNodes = d.numNodes;
	numObjects = d.numObjects;
    }
}
