/*
 * @(#)$Id: GStack.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test;

import java.util.*;
import mokan.*;
import mokan.obj.*;
import mokan.thread.*;
import mokan.transact.TransactionException;

/**
 * A basic stack implementation with some enhancements.  We support the usual
 * push and pop methods with atomic transactions.  We also provide two methods 
 * that use nested transactions: transfer2 (move an item from the top of one
 * stack to the top of another stack), and transfer3 (push an item, then
 * transfer2 it to another stack).
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jing Wang
 * @author	Suk Y. Lee
 * @author	Jerry James
 */
public class GStack implements GlobalObj, Cloneable {
    public ObjectID objID;
    public Stack ss;

    public GuardCondition gc = new GuardCondition("notEmpty", null);

    public GStack() {
	objID = null;
	ss = new Stack();
    }

    public GStack(ObjectID oid) {
	objID = oid;
	ss = new Stack();
    }

    public GStack(ObjectID oid, Integer n) {
	Random rand = new Random();

	objID = oid;
	ss = new Stack();
	for (int i = 0; i < n.intValue(); i++)
	    ss.push(Long.toString(rand.nextLong()));
    }

    public boolean notEmpty() {
	return (!ss.isEmpty());
    }

    public Object top() throws TransactionException {
	KanSystem.guardBegin(objID, gc, false);
	try {
	    return ss.peek();
	} catch (Exception ex) {
	    KanSystem.abort(ex);
	    return null;	// Doesn't matter; exception will be thrown
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public Object pop() throws TransactionException {
	KanSystem.guardBegin(objID, gc, true);
	try {
	    return ss.pop();
	} catch (Exception ex) {
	    KanSystem.abort(ex);
	    return null;	// Doesn't matter; exception will be thrown
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void push(String obj) throws TransactionException {
	KanSystem.guardBegin(objID, null, true);
	try {
	    ss.push(obj);
	} catch (Exception ex) {
	    KanSystem.abort(ex);
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void transfer2(ObjectID oid) throws TransactionException {
	KanSystem.guardBegin(objID, gc, true, true, true);
	try {
	    String ret = (String)KanSystem.invokeMethod(oid, "pop", null,
							false).joinObject();
	    Object args[] = new Object[1];
	    args[0] = ret;
	    KanSystem.invokeMethod(oid, "push", args, false).join();
	} catch (Exception ex) {
	    KanSystem.abort(ex);
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void transfer3(ObjectID oid1, ObjectID oid2)
	throws TransactionException {
	
	boolean done;

	do {
	    done = true;
	    KanSystem.guardBegin(objID, gc, true, true, true);
	    try {
		Object ret = ss.pop();
		Object args1[] = new Object[1];
		Object args2[] = new Object[1];
		args1[0] = new String((String)ret);
		KanSystem.invokeMethod(oid1, "push", args1, false).join();
		args2[0] = oid2;
		KanSystem.invokeMethod(oid1, "transfer2", args2, false).join();
	    } catch (Exception ex) {
		done = false;
		KanSystem.abort(ex);
	    } finally {
		KanSystem.guardEnd();
	    }
	} while (!done);
    }

    public String toString() {
	return ss.toString();
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	GStack stack = (GStack)obj;
	ss = stack.ss;
    }
}
