package mokan.test.WebCache;

import java.io.IOException;
import java.net.*;
import java.util.Hashtable;
import java.util.LinkedList;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.transact.TransactionException;
import mokan.obj.NoSuchKanObjectException;
import mokan.thread.Future;


public class Proxy
implements Runnable, GlobalObj, Cloneable
{
    public static boolean DEBUG = true;
    
    ObjectID objID;
    static final int MAX_CACHE_SIZE = 100000;

    Hashtable webCache = new Hashtable(); // the actual ObjectID cache
    LinkedList lastUsed = new LinkedList(); // the most recently used few

    ServerSocket clientReqSocket;
    Socket httpReqSocket;

    int cacheSize = 0;
    Thread t;
    ObjectID[] otherProxies;

    public Proxy(ObjectID myID)
    throws IOException
    {
	objID = myID;
	webCache = new Hashtable();
	lastUsed = new LinkedList();
	clientReqSocket = new ServerSocket(2000);
	t = new Thread(this);
	if (DEBUG) {
	    System.out.println("Proxy() : New Proxy Created");
	}
    }
    
    public void run()
    {
	// this guy's job is to accept http requests and search for them in
	// local hashtable. If not, search concurrently in other hashtables.
	// If still not found make a request to the 'toons' server
	if (DEBUG) {
	    System.out.println("Proxy.run() : Proxy Thread running");
	}
	String requestURL;
	try {
	    while (true) {
		// first get the socket and start the response thread
		if (DEBUG) {
		    System.out.println("Proxy.run() : " +
				       "Waiting for user request");
		}
		Socket clientReq = clientReqSocket.accept();
		new ServiceThread(clientReq, this);
	    }
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    }
    
    public CacheID searchCache(String fileName)
    {
	return ((CacheID)webCache.get(fileName));
    }

    public boolean searchOtherProxies(String fileName)
    {
	// for now I'm going to search these guys one by one.
	// Later do them concurrently
	boolean found = false;
	try {
	    int currProxy = 0;
	    CacheID requestedCacheID;
	    Object[] argSearch = new Object[1];
	    argSearch[0] = fileName;
	    int fileSize = 0;
	    while (!found) {
		Future fSrchCache = 
		    KanSystem.invokeMethod(otherProxies[currProxy++],
					   "searchCache", 
					   argSearch, false);
		requestedCacheID = (CacheID)fSrchCache.joinObject();
		if (requestedCacheID != null) {
		    found = true;
		    // enter this guy into the hashTable
		    fileSize = requestedCacheID.getSize();

		    evictVictims(new Integer(fileSize));
		    webCache.put(fileName, requestedCacheID);
		}
		if (currProxy >= otherProxies.length) {
		    break;
		}
	    } // end while...
	    if (!found) {
		// means none of the other proxies have it
		// ask the 'toons' web server
		evictVictims(new Integer(fileSize));
		found = fetchNewObject(fileName);
	    }
	}
	catch (TransactionException te) {
	    te.printStackTrace();
	}
	/*
	catch (NoSuchKanObjectException nskoe) {
	    nskoe.printStackTrace();
	}
	catch (NoSuchMethodException nsme) {
	    nsme.printStackTrace();
	}
	*/
	return (found);
    }
    
    // accessor methods
    public int getCacheSize()
    {
	return (cacheSize);
    }
    
    public void setCacheSize(Integer newSize)
    {
	cacheSize = newSize.intValue();
    }

    public Hashtable getWebCache()
    {
	return (webCache);
    }

    public LinkedList getLastUsed()
    {
	return (lastUsed);
    }
    
    public void setProxy(ObjectID[] otherProxies)
    {
	if (DEBUG) {
	    System.out.println("Proxy.setProxy() : " +
			       "Receiving info. about other caches");
	}
	this.otherProxies = otherProxies;
	// now that we know about the other proxies, we can start up our
	// request thread
	t.start();
    }

    public void evictVictims(Integer spaceRequired)
    {
	System.out.println("Proxy : EvictVictims() not implemented!");
    }

    public boolean fetchNewObject(String URL)
    {
	System.out.println("Proxy : FetchNewObject() not implemented!");
	return (false);
    }
    
    // GlobalObj functions
    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// Do nothing; this object has no state to set
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    return searchCache((String)args[0]);
	case 1:
	    return new Boolean(searchOtherProxies((String)args[0]));
	case 2:
	    return new Integer(getCacheSize());
	case 3:
	    setCacheSize((Integer)args[0]);
	    return null;
	case 4:
	    return getWebCache();
	case 5:
	    return getLastUsed();
	case 6:
	    setProxy((ObjectID[])args[0]);
	    return null;
	case 8:
	    evictVictims((Integer)args[0]);
	    return null;
	case 9:
	    return new Boolean(fetchNewObject((String)args[0]));
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }
}

