package mokan.test.WebCache;

import mokan.app.Application;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.thread.Future;

public class WebCache 
implements GlobalObj, Cloneable
{
    
    public static boolean DEBUG = true;
    
    public static void main(Application app, String[] args)
    throws Exception
    {
	int totalMachines = app.numNodes();
	ObjectID[] proxies = new ObjectID[totalMachines];

	// first instantiate the proxies - and get their threads running
	for (int currMach = 0; currMach < totalMachines; currMach++) {
	    proxies[currMach] = KanSystem.newKanObject(Proxy.class, null,
						       false, currMach);
	}
	
	// now tell everyone about everyone else
	informProxies(totalMachines, proxies);
	Object monitor = new Object();
	synchronized(monitor) {
	    monitor.wait(); // indefinetly
	}
    }

    private static void informProxies(int totalMachines, ObjectID[] proxies)
    throws Exception
    {
	// now let every proxy know about every other
	if (DEBUG) {
	    System.out.println("WebCache.informProxies() : " +
			       "Searching all proxies");
	}
	ObjectID otherProxies[] = new ObjectID[totalMachines - 1];

	for (int myProxyNum = 0; myProxyNum < totalMachines; myProxyNum++) {
	    
	    int currProxyNum = 0;
	    for (int otherProxyNum = 0; otherProxyNum < totalMachines;
		 otherProxyNum++) {
		// instantiate proxy array to pass on
		if (otherProxyNum != myProxyNum) {
		    otherProxies[currProxyNum++] = proxies[otherProxyNum];
		}
	    }
	    
	    // ok pass the proxy array to this guy
	    Object[] argSetProxy = new Object[1];
	    argSetProxy[0] = otherProxies;
	    Future fSetProxies = KanSystem.invokeMethod(proxies[myProxyNum],
						       "setProxy", 
						       argSetProxy, false);
	    fSetProxies.join();
	} // end for(myProxyNum...)
	if (DEBUG) {
	    System.out.println("WebCache.informProxies() : " +
			       totalMachines + " proxies informed");
	}
    }

    // GlobalObj functions
    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// Do nothing; this object has no state to set
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    return main((Application)args[0], (String[])args[1]);
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }
}
