/*
 * @(#)$Id: WorkLoad.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test;

import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.thread.Future;

/**
 * Set up the system to run tests.  The actual tests are in the Driver class.
 * This class just creates the distributed objects and then launches one
 * Driver per node.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Suk Y. Lee
 * @author	Jerry James
 */
public class WorkLoad implements GlobalObj, Cloneable {
    /**
     * Total number of objects in the system.
     */
    public static int numObjects;

    /**
     * Total number of nodes in the system.
     */
    public static int numNodes;

    /**
     * Number of method invocations in the system.
     */
    public static int numMethInv;

    /**
     * Probability of an update occuring.
     */
    public static float updateRatio = 0.2F;

    /**
     * Probability of a local access.
     */
    public static float localRatio;

    public ObjectID objID;

    public static void main(Application app, String argv[]) {
	numNodes	      = app.numNodes();
	numObjects	      = numNodes * 2;
	numMethInv	      = numNodes * 10;
	localRatio	      = 1.0F / numNodes;
	ObjectID oids[]       = new ObjectID[numObjects];
	ObjectID driverOids[] = new ObjectID[numNodes];
	Future tids[]         = new Future[numNodes];
	Object args[][]       = new Object[numNodes][2];

	for (int i = 0; i < argv.length; i++) {
	    System.err.println("Option #" + i + ": " + argv[i]);
	    if (argv[i].charAt(0) == '-') {
		switch (Character.toLowerCase(argv[i].charAt(1))) {
		case 'l':
		    localRatio =
			(Float.valueOf((argv[i].length() > 2)
				       ? argv[i].substring(2)
				       : argv[++i])).floatValue();
		    break;
		case 'u':
		    updateRatio =
			(Float.valueOf((argv[i].length() > 2)
				       ? argv[i].substring(2)
				       : argv[++i])).floatValue();
		    break;
		default:
		    System.err.println("Unrecognized option: " + argv[i]);
		}
	    } else {
		System.err.println("Unrecognized option: " + argv[i]);
	    }
	}

	try {
	    System.out.println("Creating stacks");
	    Object args2[] = { new Integer((int)(30 * updateRatio + 1)) };
	    for (int i = 0; i < numObjects; i++) {
		oids[i] = KanSystem.newKanObject(GStack.class, args2, true,
						 i % numNodes);
	    }
	    System.out.println("Creating Drivers");
	    Object[] arg = { new Float(localRatio), new Float(updateRatio),
			     new Integer(numMethInv), new Integer(numNodes),
			     new Integer(numObjects) };
	    for (int i = 0; i < numNodes; i++)
		driverOids[i] = KanSystem.newKanObject(Driver.class, arg,
						       false, i);

	    long startTime = System.currentTimeMillis();

	    System.out.println("Gentlemen, start your engines!");
	    for (int i = 0; i < numNodes; i++) {
		args[i][0] = new Integer(i);
		args[i][1] = oids;
		tids[i] = KanSystem.invokeMethod(driverOids[i], "startSim",
						 args[i], true);
	    }
	    System.out.println("Joining threads");
	    for (int i = 0; i < numNodes; i++)
		tids[i].join();

	    System.out.println("WorkLoad finished the simulation.");
	    System.out.println("Elapsed time: " +
			       (System.currentTimeMillis() - startTime));
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// Do nothing
    }
}
