/*
 * @(#)$Id: AsyncWorkLoad.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.thread.Future;

import java.util.Vector;
/**
 * Set up the system to run tests.  The actual tests are in the Driver class.
 * This class just creates the distributed objects and then launches one
 * Driver per node.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
public class AsyncWorkLoad implements GlobalObj, Cloneable {

    public static final int SMALL_SIM  = 0; //simulation types
    public static final int MEDIUM_SIM = 1;
    public static final int BIG_SIM    = 2;


    public static final int NUM_CALLS = 1000; //default number of method calls to make on each object
    public static final int NUM_OBJS_PER_NODE = 20;
    /**
     * Total number of objects in the system.
     */
    public static int numObjects;

    /**
     * Total number of nodes in the system.
     */
    public static int numNodes;

    /**
     * Number of method invocations in the system.
     */
    public static int numMethInv;

    public ObjectID objID;

    public static void main(Application app, String argv[]) {

	Object[] argsSim = new Integer[2]; //args to Driver.simulation method
	long startTime = 0;
	numNodes = app.numNodes();
	System.out.println("WorkLoad :: NumNodes = " + numNodes);

	if (argv.length != 3) {
	    //System.out.println("Usage : Workload <num of Object> [simulation type]");
	    numObjects = NUM_OBJS_PER_NODE * numNodes;
	    argsSim[0] = new Integer(SMALL_SIM);  //type of Simulation
	    argsSim[1] =  new Integer(NUM_CALLS); //number method calls to make
	}
	else {
	    numObjects = Integer.parseInt(argv[0]);
	    //numMethInv = numNodes;
	    argsSim[0] = new Integer(argv[1]);   //type of Simulation
	    argsSim[1] = new Integer(argv[2]);
	}

	ObjectID oids[]       = new ObjectID[numObjects]; // Object IDs of all 'TestObjects' in the system
	int oidNodes[]        = new int[numObjects];      // map of oids to nodes
	ObjectID driverOids[] = new ObjectID[numNodes];   // Object IDs of all 'Drivers' in the system
	Future tids[]         = new Future[numNodes];     // Future for each 'startSim' method to be invoked
	// is object or Vector better ?
	Vector[] testObjID    = new Vector[numNodes];     // Array of Vectors. Each vector contains the objectIDs 
	                                                  // of all testobjects on that node
	
	for (int currNode = 0; currNode < numNodes; currNode++) {
	    testObjID[currNode] = new Vector();
	}

	// create the TestObjects on all nodes in a round robin
	// fashion
	try {
	    //System.out.println("WorkLoad:: Creating Test Objects");
	    Vector[] argsTestObject = null;
	    for (int i = 0; i < numObjects; i++) {
		oids[i] = KanSystem.newKanObject(TestObject.class, argsTestObject, true,
						 i % numNodes);
		oidNodes[i] = (i % numNodes);
	    }

	    // create the driver objects and pass to each the objectIDs
	    // of objects NOT on his local node

	    //System.out.println("WorkLoad:: Creating Drivers");
	    for (int currObject = 0; currObject < numObjects; currObject++) {
		testObjID[oidNodes[currObject]].addElement(oids[currObject]);
	    }
	    Object [] wrapper = new Object[1];

	    for (int currNode = 0; currNode < numNodes; currNode++) {
		int slot = 0;
		Vector[] argsDriver = new Vector[numNodes - 1];
		for (int thisNode = 0; thisNode < numNodes; thisNode++) {
		    
		    if (thisNode != currNode) { //get all remote objs
			argsDriver[slot++] = testObjID[thisNode];
		    }
		}
		wrapper[0]= argsDriver;
		driverOids[currNode] = KanSystem.newKanObject(asyncDriver.class,
							      wrapper,
							      true, currNode);
	    }
	    
	    //startTime = System.currentTimeMillis();
	 	   
	    //System.out.println("Gentlemen, start your engines!");
	    for (int currNode = 0; currNode < numNodes; currNode++) {
		tids[currNode] = KanSystem.invokeMethod(driverOids[currNode],
							"startSim", argsSim, true);
	    }

	    // wait for these invoked methods to join
	    //System.out.println("Joining threads");
	    for (int i = 0; i < numNodes; i++)
		tids[i].join();

	    //System.out.println("WorkLoad finished the simulation.");
	    //System.out.println("Elapsed time: " +
	    // (System.currentTimeMillis() - startTime));
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// Do nothing
    }
}
