/*
 * @(#)$Id: DetailDriver.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.io.*;
import java.util.Random;
import java.util.Vector;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.thread.Future;
import mokan.util.Timer;

/**
 * Run tests on the Kan system.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jerry James
 */
public final class DetailDriver
    implements Cloneable, GlobalObj, Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = 508536512190003919L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    public ObjectID objID; 

    // pseudo-final
    int numNodes;

    // pseudo-final
    Vector[] remoteObjects;

    transient final Random rand = new Random();

    // Deserializing constructor
    public DetailDriver() {
	numNodes = 0;
	remoteObjects = null;
	objID = null;
    }

    public DetailDriver(ObjectID oid, Vector[] remoteObjects) {
	numNodes = remoteObjects.length;
	this.remoteObjects = remoteObjects;  
	this.objID = oid;
    }

    private ObjectID chooseNext() {
	int currNode = rand.nextInt(numNodes);
	Vector objs = remoteObjects[currNode];
	int size = objs.size();
	return (size > 0)
	    ? ((ObjectID)objs.elementAt(rand.nextInt(size)))
	    : null;
    }

    public long[][] startSim(Integer simType, Integer numMethodCalls) {
	Object[] args = null;
	short methodNum;

	switch (simType.intValue()) {
	case WorkLoad.INT_ARRAY:
	    methodNum = 1;
	    args = new Object[1];
	    int[] array = new int[32];
	    for (int i = 0; i < 32; i++) {
		array[i] = rand.nextInt();
	    }
	    args[0] = array;
	    break;
	case WorkLoad.NODE_ARG:
	    methodNum = 2;
	    args = new Object[1];
	    args[0] = new Tree(rand);
	    break;
	case WorkLoad.TREE_ARG:
	    methodNum = 2;
	    args = new Object[1];
	    args[0] = Tree.makeTree(rand);
	    break;
	default:
	    methodNum = 0;
	    break;
	}

	int numCalls = numMethodCalls.intValue();
	long[][] timings = new long[numCalls][];

	try {
	    for (int i = 0; i < numCalls; i++) {
		//choose object to invoke method on
		ObjectID nextOID = chooseNext();
		long[] times = timings[i] = new long[12];
		long startTime = times[0] = Timer.currentTimeNanos();
		if (nextOID != null)
		    KanSystem.invokeMethod(nextOID, methodNum, args,
					   times).join();
		times[0] = Timer.currentTimeNanos() - startTime;
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	return timings;
    }

    public String toString() {
	return "DetailDriver " + objID;
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// There's no state to copy, so this is a null method
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    return startSim((Integer)args[0], (Integer)args[1]);
	case 1:
	    return toString();
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
	out.writeInt(numNodes);
	out.writeObject(remoteObjects);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (ObjectID)in.readObject();
	numNodes = in.readInt();
	remoteObjects = (Vector[])in.readObject();
    }
}
