/*
 * @(#)$Id: DetailWork.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.util.Vector;
import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.ObjectID;
import mokan.thread.Future;

/**
 * Set up the system to run tests.  The actual tests are in the
 * <code>DetailDriver</code> class.  This class just creates the distributed
 * objects and then launches one <code>DetailDriver</code> per node.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
public final class DetailWork {

    public static final int NULL_ARG  = 0;
    public static final int INT_ARRAY = 1;
    public static final int NODE_ARG  = 2;
    public static final int TREE_ARG  = 3;

    /**
     * Default number of method calls to make on each object
     */
    public static final int NUM_CALLS = 1000;
    public static final int NUM_OBJS_PER_NODE = 1;

    /**
     * Total number of objects in the system.
     */
    public static int numObjects;

    /**
     * Total number of nodes in the system.
     */
    public static int numNodes;

    /**
     * Number of method invocations in the system.
     */
    public static int numMethInv;

    public static void main(Application app, String argv[]) {
	Object[] argsSim = new Integer[2]; //args to DetailDriver.sim method
	numNodes = app.numNodes();
	System.out.println("DetailWork :: NumNodes = " + numNodes);

	if (argv.length != 3) {
	    numObjects = NUM_OBJS_PER_NODE * numNodes;
	    argsSim[0] = new Integer(NULL_ARG);  //type of Simulation
	    argsSim[1] = new Integer(NUM_CALLS); //number method calls to make
	} else {
	    numObjects = Integer.parseInt(argv[0]);
	    argsSim[0] = new Integer(argv[1]);   //type of Simulation
	    argsSim[1] = new Integer(argv[2]);
	}

	// Object IDs of all 'TestObjects' in the system
	ObjectID[] oids       = new ObjectID[numObjects];
	// map of oids to nodes
	int[] oidNodes        = new int[numObjects];
	// Object IDs of all 'DetailDrivers' in the system
	ObjectID[] driverOids = new ObjectID[numNodes];
	// Future for each 'startSim' method to be invoked
	Future[] futs         = new Future[numNodes];
	// is object or Vector better ?
	// Array of Vectors. Each vector contains the objectIDs 
	// of all testobjects on that node
	Vector[] testObjID    = new Vector[numNodes];

	for (int currNode = 0; currNode < numNodes; currNode++) {
	    testObjID[currNode] = new Vector();
	}

	// Create the TestObjects on all nodes in a round robin fashion
	try {
	    for (int i = 0; i < numObjects; i++) {
		int node = i % numNodes;
		oids[i] =
		    KanSystem.newKanObject(TestObject.class, null, true, node);
		oidNodes[i] = node;
	    }

	    // Create the driver objects and pass to each the ObjectIDs
	    // of objects NOT on the local node

	    for (int currObject = 0; currObject < numObjects; currObject++) {
		testObjID[oidNodes[currObject]].addElement(oids[currObject]);
	    }
	    Object[] wrapper = new Object[1];

	    for (int currNode = 0; currNode < numNodes; currNode++) {
		int slot = 0;
		Vector[] argsDetailDriver = new Vector[numNodes - 1];
		for (int thisNode = 0; thisNode < numNodes; thisNode++) {
		    if (thisNode != currNode) { //get all remote objs
			argsDetailDriver[slot++] = testObjID[thisNode];
		    }
		}
		wrapper[0]= argsDetailDriver;
		driverOids[currNode] =
		    KanSystem.newKanObject(DetailDriver.class, wrapper, true,
					   currNode);
	    }

	    for (int currNode = 0; currNode < numNodes; currNode++) {
		futs[currNode] =
		    KanSystem.invokeMethod(driverOids[currNode], (short)0,
					   argsSim);
	    }

	    // wait for these invoked methods to join
	    long[][][] allResults = new long[numNodes][][];
	    for (int i = 0; i < numNodes; i++) {
		allResults[i] = (long[][])futs[i].joinObject();
	    }

	    // Now we're all done, so go ahead and compute intensively
	    int numResults = allResults[0].length;
	    for (int i = 0; i < numNodes; i++) {
		System.out.println("Results for node " + i + ':');
		long[][] results = allResults[i];

		// For each time measured, do the following
		for (int j = 0; j < 12; j++) {

		    // Compute the mean
		    double mean = 0.0;
		    for (int k = 0; k < numResults; k++) {
			mean += results[k][j];
		    }
		    mean /= numResults;

		    // Compute the standard deviation
		    double stdDev = 0.0;
		    for (int k = 0; k < numResults; k++) {
			double diff = results[k][j] - mean;
			stdDev += diff * diff;
		    }
		    stdDev = Math.sqrt(stdDev / (numResults - 1));

		    // Compute the adjusted mean
		    double adjMean = 0.0;
		    int counted = 0;
		    double comparator = stdDev * 2;
		    for (int k = 0; k < numResults; k++) {
			double num = Math.abs(results[k][j] - mean);
			if (num < comparator) {
			    adjMean += results[k][j];
			    counted++;
			}
		    }
		    adjMean /= counted;
		    double MILL = 1000000.0;
		    System.out.println("Time " + j + " Mean          = " +
				       (mean / MILL));
		    System.out.println("Time " + j + " Standard Dev. = " +
				       (stdDev / MILL));
		    System.out.println("Time " + j + " Adjusted mean = " +
				       (adjMean / MILL));
		}
		System.out.println();
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }
}
