/*
 * @(#)$Id: Driver.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.io.*;
import java.util.Random;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.util.Timer;

/**
 * Run tests on the Kan system.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
public final class Driver implements Cloneable, GlobalObj, Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = 508536512190003919L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    publicmokan.obj.ObjectID objID; 

    // pseudo-final
   mokan.obj.ObjectID[] remoteObjects;

    transient final Random rand = new Random();

    // Deserializing constructor
    public Driver() {
	remoteObjects = null;
	objID = null;
    }

    public Driver(finalmokan.obj.ObjectID oid,
		  finalmokan.obj.ObjectID[] remoteObjects) {
	this.objID = oid;
	this.remoteObjects = remoteObjects;
    }

    privatemokan.obj.ObjectID chooseNext() {
	final int num = remoteObjects.length;
	return (num > 0) ? remoteObjects[rand.nextInt(num)] : null;
    }

    public long[] startSim(final Integer simType,
			   final Integer numMethodCalls) {
	final int numCalls = numMethodCalls.intValue();
	final long[] timings = new long[numCalls];
	Object[] args;

	long startTime = 0L;
	try {
	    for (int i = 0; i < numCalls; i++) {
		//choose object to invoke method on
	mokan.obj.ObjectID nextObj = chooseNext();
		if (nextObj != null) {
		    switch (simType.intValue()) {
		    case WorkLoad.INT_ARG:
			args = new Object[1];
			args[0] = new Integer(rand.nextInt());
			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)1, args).join();
			break;
		    case WorkLoad.INT_ARGS:
			args = new Object[32];

			// THE JIT CRASH IS INSIDE THIS LOOP!
			for (int j = 0; j < 32; j++) {
			    args[j] = new Integer(rand.nextInt());
			}
			// THE JIT CRASH IS INSIDE THIS LOOP!

			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)2, args).join();
			break;
		    case WorkLoad.INT_ARRAY:
			int[] array = new int[32];
			for (int j = 0; j < 32; j++) {
			    array[j] = rand.nextInt();
			}
			args = new Object[1];
			args[0] = array;
			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)3, args).join();
			break;
		    case WorkLoad.NODE_ARG:
			args = new Object[1];
			args[0] = new Tree(rand);
			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)4, args).join();
			break;
		    case WorkLoad.TREE_ARG:
			args = new Object[1];
			args[0] = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)4, args).join();
			break;
		    case WorkLoad.NODE_ARG_INT:
			args = new Object[1];
			args[0] = new Tree(rand);
			startTime = Timer.currentTimeNanos();
			int ret = KanSystem.invokeMethod(nextObj, (short)5,
							 args).joinInt();
			break;
		    case WorkLoad.TREE_ARG_INT:
			args = new Object[1];
			args[0] = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			ret = KanSystem.invokeMethod(nextObj, (short)5,
						     args).joinInt();
			break;
		    default:
			startTime = Timer.currentTimeNanos();
			KanSystem.invokeMethod(nextObj, (short)0, null).join();
			break;
		    }
		    timings[i] = Timer.currentTimeNanos() - startTime;
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	return timings;
    }

    public String toString() {
	return "Driver " + objID;
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// There's no state to copy, so this is a null method
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    return startSim((Integer)args[0], (Integer)args[1]);
	case 1:
	    return toString();
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
	out.writeObject(remoteObjects);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (kan.obj.ObjectID)in.readObject();
	remoteObjects = (kan.obj.ObjectID[])in.readObject();
    }
}
