/*
 * @(#)$Id: LocalWorkLoad.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.util.Vector;
import mokan.KanSystem;
import mokan.app.Application;
import mokan.thread.Future;

/**
 * Set up the system to run tests.  The actual tests are in the
 * <code>Driver</code> class.  This class just creates the distributed
 * objects and then launches one <code>Driver</code> per node.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
public class LocalWorkLoad {

    public static final int NULL_ARG     = 0;
    public static final int INT_ARG      = 1;
    public static final int INT_ARGS     = 2;
    public static final int INT_ARRAY    = 3;
    public static final int NODE_ARG     = 4;
    public static final int TREE_ARG     = 5;
    public static final int NODE_ARG_INT = 6;
    public static final int TREE_ARG_INT = 7;

    public static void main(final Application app, final String argv[]) {
	// Create the argument array for Driver.startSim
	Object[] argsSim = new Integer[2];

	// Parse the arguments
	int numObjects = Integer.parseInt(argv[0]);
	argsSim[0] = new Integer(argv[1]);	// Type of method to call
	argsSim[1] = new Integer(argv[2]);	// Number of method calls

	// Get the number of nodes involved in the test
	final int numNodes = app.numNodes();

	try {
	    // Create the TestObjects on all nodes
	    finalmokan.obj.ObjectID[][] objs = new mokan.obj.ObjectID[numNodes][];
	    for (int node = 0; node < numNodes; node++) {
		int numLocals = numObjects / numNodes +
		    (node < numObjects % numNodes ? 1 : 0);
		objs[node] = new mokan.obj.ObjectID[numLocals];
		for (int i = 0; i < numLocals; i++) {
		    objs[node][i] = KanSystem.newKanObject(TestObject.class,
							   null, true, node);
		}
	    }

	    // Create the driver objects and pass to each the ObjectIDs
	    // of objects on the local node ONLY
	    final Object[] wrapper = new Object[1];
	    finalmokan.obj.ObjectID[] driverOids =
		new mokan.obj.ObjectID[numNodes];
	    for (int node = 0; node < numNodes; node++) {
		int index = 0;
		wrapper[0] = objs[node];
		driverOids[node] =
		    KanSystem.newKanObject(Driver.class, wrapper, true, node);
	    }

	    // Start the drivers
	    final Future[] futs = new Future[numNodes];
	    for (int currNode = 0; currNode < numNodes; currNode++) {
		futs[currNode] =
		    KanSystem.invokeMethod(driverOids[currNode], (short)0,
					   argsSim/*, new long[15]*/ );
	    }

	    // Wait for the drivers to finish
	    long[][] results = new long[numNodes][];
	    for (int i = 0; i < numNodes; i++) {
		results[i] = (long[])futs[i].joinObject();
	    }

	    // Now we're all done, so go ahead and compute intensively
	    // Compute the raw mean
	    double[] averages = new double[numNodes];
	    for (int i = 0; i < numNodes; i++) {
		double average = 0.0;
		final int numResults = results[i].length;
		for (int j = 0; j < numResults; j++) {
		    average += results[i][j];
		}
		averages[i] = average / numResults;
	    }

	    // Compute the standard deviation
	    double[] stdDevs = new double[numNodes];
	    for (int i = 0; i < numNodes; i++) {
		double stdDev = 0.0;
		final int numResults = results[i].length;
		for (int j = 0; j < numResults; j++) {
		    double diff = results[i][j] - averages[i];
		    stdDev += diff * diff;
		}
		stdDevs[i] = Math.sqrt(stdDev / numResults);
	    }

	    // Compute the adjusted mean
	    double[] means = new double[numNodes];
	    for (int i = 0; i < numNodes; i++) {
		System.out.println("Results for node " + i);
		double mean = 0.0;
		int counted = 0;
		double comparator = stdDevs[i] * 2;
		final int numResults = results[i].length;
		for (int j = 0; j < numResults; j++) {
		    double num = Math.abs(results[i][j] - averages[i]);
		    if (num < comparator) {
			mean += results[i][j];
			counted++;
		    }
		}
		means[i] = mean / counted;
		double MICRO = 1000.0;
		System.out.println("Mean          = " + (averages[i] / MICRO));
		System.out.println("Standard Dev. = " + (stdDevs[i] / MICRO));
		System.out.println("Adjusted mean = " + (means[i] / MICRO));
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }
}
