/*
 * @(#)$Id: RMIDriver.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.util.Random;
import mokan.util.Timer;

/**
 * Run tests with RMI like those we run on Kan.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jerry James
 */
public final class RMIDriver {

    final RMITestObjectInterface[] remoteObjects;

    final Random rand = new Random();

    public RMIDriver(RMITestObjectInterface[] remoteObjects) {
	this.remoteObjects = remoteObjects;  
    }

    private RMITestObjectInterface chooseNext() {
	final int num = remoteObjects.length;
	return (num > 0) ? remoteObjects[rand.nextInt(num)] : null;
    }

    public long[] startSim(int simType, int numCalls) {
	long[] timings = new long[numCalls];

	try {
	    for (int i = 0; i < numCalls; i++) {
		//choose object to invoke method on
		RMITestObjectInterface nextObj = chooseNext();
		if (nextObj != null) {
		    long startTime;
		    switch (simType) {
		    case RMIWorkLoad.INT_ARG:
			int val = rand.nextInt();
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(val);
			break;
		    case RMIWorkLoad.INT_ARGS:
			int[] a = new int[32];
			for (int j = 0; j < 32; j++) {
			    a[j] = rand.nextInt();
			}
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(a[ 0], a[ 1], a[ 2], a[ 3],
					      a[ 4], a[ 5], a[ 6], a[ 7],
					      a[ 8], a[ 9], a[10], a[11],
					      a[12], a[13], a[14], a[15],
					      a[16], a[17], a[18], a[19],
					      a[20], a[21], a[22], a[23],
					      a[24], a[25], a[26], a[27],
					      a[28], a[29], a[30], a[31]);
			break;
		    case RMIWorkLoad.INT_ARRAY:
			int[] array = new int[32];
			for (int j = 0; j < 32; j++) {
			    array[j] = rand.nextInt();
			}
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(array);
			break;
		    case RMIWorkLoad.NODE_ARG:
			Tree tree = new Tree(rand);
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(tree);
			break;
		    case RMIWorkLoad.TREE_ARG:
			tree = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(tree);
			break;
		    case RMIWorkLoad.NODE_ARG_INT:
			tree = new Tree(rand);
			startTime = Timer.currentTimeNanos();
			int ret = nextObj.noCompute(tree);
			break;
		    case RMIWorkLoad.TREE_ARG_INT:
			tree = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			ret = nextObj.noCompute(tree);
			break;
		    case RMIWorkLoad.KAN_ARGS:
			ExecMsg msg = ExecMsg.makeExecMsg();
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation(msg);
			break;
		    default:
			startTime = Timer.currentTimeNanos();
			nextObj.noComputation();
			break;
		    }
		    timings[i] = Timer.currentTimeNanos() - startTime;
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return timings;
    }
}
