/*
 * @(#)$Id: RMITestObject.java,v 1.1.1.1 2002/11/06 22:10:27 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.io.IOException;
import java.net.InetAddress;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;

/**
 * A time-wasting object.  Waste varying amounts of time, depending on user
 * needs.  Also has methods taking various kinds of parameters.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:27 $
 * @author	Jerry James
 */
public class RMITestObject extends UnicastRemoteObject
    implements RMITestObjectInterface {

    private static final int MEDIUM_TIME = 100;
    private static final int BIG_TIME = 5000;
    private static int count = 0;
    private static final int serverPort = 5009;  // to talk with RMI

    private final int num;
    private int busy;

    public RMITestObject(final int numNodes, final String hostName)
	throws RemoteException {

	busy = numNodes;
	num = count++;
        try {
            Naming.rebind("rmi://" + hostName + ":" + serverPort +
			  "/RMITestObject" + num, this);
        } catch (IOException e) {
            e.printStackTrace();
	    System.out.println("RMITestObject::error registering -- exiting");
            System.exit(-1);
        }
    }

    public void noComputation() throws RemoteException {
	// No computation...remember!
    }

    public void noComputation(int dummy) throws RemoteException {
	// Hey look, he said dummy ... heh, heh, heh, heh
    }

    public void noComputation(int  a1, int  a2, int  a3, int  a4, int  a5,
			      int  a6, int  a7, int  a8, int  a9, int a10,
			      int a11, int a12, int a13, int a14, int a15,
			      int a16, int a17, int a18, int a19, int a20,
			      int a21, int a22, int a23, int a24, int a25,
			      int a26, int a27, int a28, int a29, int a30,
			      int a31, int a32) throws RemoteException {
	// All of the args are dummies, dummy!
    }

    public void noComputation(int[] dummy) throws RemoteException {
	// The arg is a dummy, dummy!
    }

    public void noComputation(Tree dummy) throws RemoteException {
	// The arg is still a dummy, dummy!
    }

    public int noCompute(Tree dummy) throws RemoteException {
	return 789734;
    }
    
    public void noComputation(ExecMsg dummy) throws RemoteException {
	// The dummy msg is a dummy
    }

    public synchronized void done() throws RemoteException {
	if (--busy == 0)
	    notifyAll();
    }

    public synchronized void waitUntilDone() throws RemoteException {
	while (busy > 0) {
	    try {
		wait();
	    } catch (InterruptedException intEx) {
		// This can't happen
	    }
	}
    }

    public String toString() {
	return ("RMITestObject" + num);
    }
}
