/*
 * @(#)$Id: TestObject.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.io.*;
import mokan.obj.GlobalObj;

/**
 * A time-wasting object.  Waste varying amounts of time, depending on user
 * needs.  Also has methods taking various kinds of parameters.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
public final class TestObject implements Cloneable, GlobalObj, Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = -4298841731354549551L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    publicmokan.obj.ObjectID objID;
    private int count = 0;

    private static final int MEDIUM_TIME = 100;
    private static final int BIG_TIME = 5000;

    public TestObject() {
	objID = null;
    }

    public TestObject(kan.obj.ObjectID oid) {
	objID = oid;
    }

    public void noComputation() {
	// No computation...remember!
    }

    public void noComputation(int dummy) {
	// Hey look, he said dummy ... heh, heh, heh, heh
    }

    public void noComputation(int  a1, int  a2, int  a3, int  a4, int  a5,
			      int  a6, int  a7, int  a8, int  a9, int a10,
			      int a11, int a12, int a13, int a14, int a15,
			      int a16, int a17, int a18, int a19, int a20,
			      int a21, int a22, int a23, int a24, int a25,
			      int a26, int a27, int a28, int a29, int a30,
			      int a31, int a32) {
	// All of the args are dummies, dummy!
    }

    public void noComputation(int[] dummy) {
	// The arg is a dummy, dummy!
    }

    public void noComputation(Tree dummy) {
	// The arg is still a dummy, dummy!
    }

    public int noCompute(Tree dummy) {
	return 789734;
    }

    public void mediumComputation() {
	// perform work for 100 ms
	long startTime = System.currentTimeMillis();
	long finishTime = startTime + MEDIUM_TIME;
	long currTime = 0;

	while ((currTime = System.currentTimeMillis()) < finishTime) {
	    currTime = System.currentTimeMillis();
	}
    }

    public void bigComputation() {
	// perform work for 5000 ms
	long startTime = System.currentTimeMillis();
	long finishTime = startTime + BIG_TIME;
	long currTime = 0;
	while ((currTime = System.currentTimeMillis()) < finishTime) {
	    currTime = System.currentTimeMillis();
	}
    }

    public String toString() {
	return ("TestObject[" + objID + ']');
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// No state to set
	count = ((TestObject)obj).count;
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    noComputation();
	    return null;
	case 1:
	    noComputation(((Integer)args[0]).intValue());
	    return null;
	case 2:
	    noComputation(((Integer)args[ 0]).intValue(),
			  ((Integer)args[ 1]).intValue(),
			  ((Integer)args[ 2]).intValue(),
			  ((Integer)args[ 3]).intValue(),
			  ((Integer)args[ 4]).intValue(),
			  ((Integer)args[ 5]).intValue(),
			  ((Integer)args[ 6]).intValue(),
			  ((Integer)args[ 7]).intValue(),
			  ((Integer)args[ 8]).intValue(),
			  ((Integer)args[ 9]).intValue(),
			  ((Integer)args[10]).intValue(),
			  ((Integer)args[11]).intValue(),
			  ((Integer)args[12]).intValue(),
			  ((Integer)args[13]).intValue(),
			  ((Integer)args[14]).intValue(),
			  ((Integer)args[15]).intValue(),
			  ((Integer)args[16]).intValue(),
			  ((Integer)args[17]).intValue(),
			  ((Integer)args[18]).intValue(),
			  ((Integer)args[19]).intValue(),
			  ((Integer)args[20]).intValue(),
			  ((Integer)args[21]).intValue(),
			  ((Integer)args[22]).intValue(),
			  ((Integer)args[23]).intValue(),
			  ((Integer)args[24]).intValue(),
			  ((Integer)args[25]).intValue(),
			  ((Integer)args[26]).intValue(),
			  ((Integer)args[27]).intValue(),
			  ((Integer)args[28]).intValue(),
			  ((Integer)args[29]).intValue(),
			  ((Integer)args[30]).intValue(),
			  ((Integer)args[31]).intValue());
	    return null;
	case 3:
	    noComputation((int[])args[0]);
	    return null;
	case 4:
	    noComputation((Tree)args[0]);
	    return null;
	case 5:
	    return new Integer(noCompute((Tree)args[0]));
	case 6:
	    mediumComputation();
	    return null;
	case 7:
	    bigComputation();
	    return null;
	case 8:
	    return toString();
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
	out.writeInt(count);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (kan.obj.ObjectID)in.readObject();
	count = in.readInt();
    }
}
