/*
 * @(#)$Id: VeryLocalDriver.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;

import java.io.*;
import java.util.Random;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.util.Timer;

/**
 * Run tests on the Kan system.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public class VeryLocalDriver implements Cloneable, GlobalObj, Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = -303897127194612689L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    // pseudo-final
    publicmokan.obj.ObjectID objID;

    // pseudo-final
   mokan.obj.ObjectID[] localObjects;

    transient final Random rand = new Random();

    // Deserializing constructor
    public VeryLocalDriver() {
	localObjects = null;
	objID = null;
    }

    public VeryLocalDriver(finalmokan.obj.ObjectID oid,
			   finalmokan.obj.ObjectID[] localObjects) {
	this.objID = oid;
	this.localObjects = localObjects;
    }

    privatemokan.obj.ObjectID chooseNext() {
	final int num = localObjects.length;
	return (num > 0) ? localObjects[rand.nextInt(num)] : null;
    }

    public long[] startSim(final Integer simType,
			   final Integer numMethodCalls) {
	final int numCalls = numMethodCalls.intValue();
	final long[] timings = new long[numCalls];
	Object[] args;

	long startTime = 0L;
	try {
	    for (int i = 0; i < numCalls; i++) {
		//choose object to invoke method on
	mokan.obj.ObjectID nextObj = chooseNext();
		if (nextObj != null) {
		    TestObject target =
			(TestObject)KanSystem.callDirectP(nextObj);
		    switch (simType.intValue()) {
		    case WorkLoad.INT_ARG:
			final int nextNum = rand.nextInt();
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Integer[1];
			    args[0] = new Integer(nextNum);
			    KanSystem.invokeMethod(nextObj, (short)1,
						   args).join();
			} else {
			    target.noComputation(nextNum);
			}
			break;
		    case WorkLoad.INT_ARGS:
			// Get 32 random integers
			final int  a0 = rand.nextInt();
			final int  a1 = rand.nextInt();
			final int  a2 = rand.nextInt();
			final int  a3 = rand.nextInt();
			final int  a4 = rand.nextInt();
			final int  a5 = rand.nextInt();
			final int  a6 = rand.nextInt();
			final int  a7 = rand.nextInt();
			final int  a8 = rand.nextInt();
			final int  a9 = rand.nextInt();
			final int a10 = rand.nextInt();
			final int a11 = rand.nextInt();
			final int a12 = rand.nextInt();
			final int a13 = rand.nextInt();
			final int a14 = rand.nextInt();
			final int a15 = rand.nextInt();
			final int a16 = rand.nextInt();
			final int a17 = rand.nextInt();
			final int a18 = rand.nextInt();
			final int a19 = rand.nextInt();
			final int a20 = rand.nextInt();
			final int a21 = rand.nextInt();
			final int a22 = rand.nextInt();
			final int a23 = rand.nextInt();
			final int a24 = rand.nextInt();
			final int a25 = rand.nextInt();
			final int a26 = rand.nextInt();
			final int a27 = rand.nextInt();
			final int a28 = rand.nextInt();
			final int a29 = rand.nextInt();
			final int a30 = rand.nextInt();
			final int a31 = rand.nextInt();
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Integer[32];
			    args[ 0] = new Integer( a0);
			    args[ 1] = new Integer( a1);
			    args[ 2] = new Integer( a2);
			    args[ 3] = new Integer( a3);
			    args[ 4] = new Integer( a4);
			    args[ 5] = new Integer( a5);
			    args[ 6] = new Integer( a6);
			    args[ 7] = new Integer( a7);
			    args[ 8] = new Integer( a8);
			    args[ 9] = new Integer( a9);
			    args[10] = new Integer(a10);
			    args[11] = new Integer(a11);
			    args[12] = new Integer(a12);
			    args[13] = new Integer(a13);
			    args[14] = new Integer(a14);
			    args[15] = new Integer(a15);
			    args[16] = new Integer(a16);
			    args[17] = new Integer(a17);
			    args[18] = new Integer(a18);
			    args[19] = new Integer(a19);
			    args[20] = new Integer(a20);
			    args[21] = new Integer(a21);
			    args[22] = new Integer(a22);
			    args[23] = new Integer(a23);
			    args[24] = new Integer(a24);
			    args[25] = new Integer(a25);
			    args[26] = new Integer(a26);
			    args[27] = new Integer(a27);
			    args[28] = new Integer(a28);
			    args[29] = new Integer(a29);
			    args[30] = new Integer(a30);
			    args[31] = new Integer(a31);
			    KanSystem.invokeMethod(nextObj, (short)2,
						   args).join();
			} else {
			    target.noComputation( a0,  a1,  a2,  a3,  a4,  a5,
						  a6,  a7,  a8,  a9, a10, a11,
						 a12, a13, a14, a15, a16, a17,
						 a18, a19, a20, a21, a22, a23,
						 a24, a25, a26, a27, a28, a29,
						 a30, a31);
			}
			break;
		    case WorkLoad.INT_ARRAY:
			final int[] array = new int[32];
			for (int j = 0; j < 32; j++) {
			    array[j] = rand.nextInt();
			}
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Object[1];
			    args[0] = array;
			    KanSystem.invokeMethod(nextObj, (short)3,
						   args).join();
			} else {
			    target.noComputation(array);
			}
			break;
		    case WorkLoad.NODE_ARG:
			final Tree node1 = new Tree(rand);
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Object[1];
			    args[0] = node1;
			    KanSystem.invokeMethod(nextObj, (short)4,
						   args).join();
			} else {
			    target.noComputation(node1);
			}
			break;
		    case WorkLoad.TREE_ARG:
			final Tree tree1 = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Object[1];
			    args[0] = tree1;
			    KanSystem.invokeMethod(nextObj, (short)4,
						   args).join();
			} else {
			    target.noComputation(tree1);
			}
			break;
		    case WorkLoad.NODE_ARG_INT:
			final Tree node2 = new Tree(rand);
			int ret;
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Object[1];
			    args[0] = node2;
			    ret = KanSystem.invokeMethod(nextObj, (short)5,
							 args).joinInt();
			} else {
			    ret = target.noCompute(node2);
			}
			break;
		    case WorkLoad.TREE_ARG_INT:
			final Tree tree2 = Tree.makeTree(rand);
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    args = new Object[1];
			    args[0] = tree2;
			    ret = KanSystem.invokeMethod(nextObj, (short)5,
							 args).joinInt();
			} else {
			    ret = target.noCompute(tree2);
			}
			break;
		    default:
			startTime = Timer.currentTimeNanos();
			if (target == null) {
			    KanSystem.invokeMethod(nextObj, (short)0,
						   null).join();
			} else {
			    target.noComputation();
			}
			break;
		    }
		    timings[i] = Timer.currentTimeNanos() - startTime;
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	return timings;
    }

    public String toString() {
	return "VeryLocalDriver " + objID;
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// There's no state to copy, so this is a null method
    }

    public Object kanCallMethod(short methNum, Object[] args) {
	switch (methNum) {
	case 0:
	    return startSim((Integer)args[0], (Integer)args[1]);
	case 1:
	    return toString();
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
	out.writeObject(localObjects);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (kan.obj.ObjectID)in.readObject();
	localObjects = (kan.obj.ObjectID[])in.readObject();
    }
}
