/*
 * @(#)$Id: asyncDriver.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.rmiTest;


import mokan.*;
import mokan.obj.*;
import mokan.thread.*;
import mokan.transact.*;
import java.util.Vector;
import java.util.Random;
import java.io.*;

/**
 * Run tests on the Kan system. It does async method calss in rounds
 */
public class asyncDriver implements GlobalObj, Cloneable {
 

    String DIR ="/fs/amazon3/dsl/jerry/kansrc/kan/test/rmiTest/results/data/";
    public ObjectID objID; 

    final int numRounds = 100;//number of async method calls to make each round

    final int numNodes;
    final Vector[] remoteObjects;


    Random nodeToCall = new Random(System.currentTimeMillis());
    Random objToCall  = new Random(System.currentTimeMillis());

    public asyncDriver( ObjectID oid,  Vector[] remoteObjects) {
	numNodes = remoteObjects.length;
	this.remoteObjects = remoteObjects;  
	this.objID = oid;
    }
    
    private ObjectID chooseNext()
    {
	int currNode = nodeToCall.nextInt(Math.abs(numNodes));
	int currObj  = objToCall.nextInt(remoteObjects[currNode].size());	
	return ((ObjectID)remoteObjects[currNode].elementAt(currObj));
    }

    public void startSim(Integer simType, Integer numMethodCalls) {

	
	// long startTime = System.currentTimeMillis();
	Thread thisThread = Thread.currentThread();
	String methodName = null;
	Future[] tids = new Future[numMethodCalls.intValue()];

	if (simType.intValue() == WorkLoad.SMALL_SIM) {
	    methodName = "noComputation";
	}
	else if (simType.intValue() == WorkLoad.MEDIUM_SIM) {
	    methodName = "mediumComputation";
	}
	else if (simType.intValue() == WorkLoad.BIG_SIM) {
	    methodName = "bigComputation";
	}
	else {
	    // whoops
	}

	long startTime = 0, finishTime = 0;
	Vector timings = new Vector(numMethodCalls.intValue());
	ObjectID nextOID;
	long tot =0;
	try {
	    	
	    for (int i= 0, t =0; i < numMethodCalls.intValue(); ) {
		
		nextOID = chooseNext(); //choose object to invoke method on
		startTime = System.currentTimeMillis();
		for (int j=0; j < numRounds; j++){		   
		    tids[i] = KanSystem.invokeMethod(nextOID, methodName,
						 null, true);
		    nextOID = chooseNext(); //choose object to invoke method on
		    i++;   //INCREMENT OUTER FOR LOOP
		}
		for (int j=0; j < numRounds; j++){ 
		    tids[t].join();
		    t=t+1; //INCREMENT tid counter
		}

		finishTime = System.currentTimeMillis();
		tot= (finishTime - startTime) / numRounds;//adjust timing
		timings.add(new Long(tot));
	    }
	}
	catch (ArrayIndexOutOfBoundsException a) {	 
	    System.out.print("asyncDriver.startSim:: numobjects%numRounds!=0 ");
	    System.out.println("so wont record this last timing, Its OK");
	    System.out.println("Dont worry. It does no harm");
	    a.printStackTrace();
	}
	catch (Exception e) {	    
	    e.printStackTrace();
	}

	//kill some time before writing to file so dont skew other nodes numbers
	try {
	    Thread.currentThread().sleep( 3 *60 * 1000);
	} catch (Exception e) {
	    e.printStackTrace();
	}

	writeResults( timings, objID); //write results to a file
    }
    //flush results to a file before quitting
    private void writeResults(Vector timings, ObjectID oid) {
	String s = oid.toString();
	String machineName = s.substring( s.indexOf('[') +1, 
					    s.indexOf(',')  );
	String fName = DIR + machineName;
	
	FileOutputStream fos = null;  
	ObjectOutputStream oos = null;
 
	try {
	    fos = new FileOutputStream(fName);
	    oos = new ObjectOutputStream( fos );
	} catch(IOException ioe) {
	    System.out.println("Driver.writeResults:: cant open file" + fName);
	    ioe.printStackTrace();
	}
	try {
	    for(int j = 0; j<timings.size(); j++) {
		oos.writeObject(timings);
	    }	   
	}
	    catch (IOException ioe) {
		System.out.println("Driver.writeResults:: cant Write to " + fName);
		ioe.printStackTrace();
	}
    }

    /*
     private void writeResults(Vector timings, ObjectID oid) {
 	String fName = DIR + oid;
 	FileWriter fw = null;
 	PrintWriter pw = null;
 
 	try {
 	    fw = new FileWriter( fName );
 	} catch(IOException ioe) {
 	    System.out.println("Driver.writeResults:: cant open file" + fName);
 	    ioe.printStackTrace();
 	}
 	pw = new PrintWriter(fw);

 	for(int j = 0; j<timings.size(); j++) {	    
	    //System.out.println("Time to invoke method # " + j + " " + (Long)timings.elementAt(j));
	     pw.println( ((Long)timings.elementAt(j)).longValue() );
 	}
    }
    */

	
    public String toString() {
	return "Driver ";
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// what's going on ?
    }
}
