/*
 * @(#)$Id: Context.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

/**
 * A test of the speed of thread creation and context switching
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Context extends Thread {

    private static long creationTime;
    private static long startTime;
    private static long realStartTime;
    private static boolean[] started;
    private static boolean[] done;

    private final int threads;
    private final int num;
    private final int loops;
    private int dummy;

    public static void main(String[] argv) throws InterruptedException {
	int threads = Integer.parseInt(argv[0]);
	int loops = Integer.parseInt(argv[1]);
	started = new boolean[threads];
	done = new boolean[threads];
	startTime = System.currentTimeMillis();
	for (int i = 0; i < threads; i++) {
	    new Context(threads, i, loops).start();
	}
	creationTime = System.currentTimeMillis();
	try {
	    Thread.sleep(3600000L);  // Sleep for one hour; should be enough!
	} catch (InterruptedException intEx) {
	    // Do nothing; this can't happen
	}
    }

    Context(int threads, int num, int loops) {
	this.threads = threads;
	this.num = num;
	this.loops = loops;
    }

    public void run() {

	// First barrier: make sure all have started
	started[num] = true;
	boolean wakeup = true;
	for (int i = 0; i < threads; i++) {
	    if (!started[i]) {
		try {
		    synchronized (started) {
			started.wait();
		    }
		} catch (InterruptedException intEx) {
		}
		wakeup = false;
	    }
	}
	if (wakeup) {
	    realStartTime = System.currentTimeMillis();
	    synchronized (started) {
		started.notifyAll();
	    }
	}

	// Here we go!  Do a bunch of nothing.
	for (int i = 0; i < loops; i++) {
	    dummy += i;
	}

	// Second barrier; make sure all have finished
	done[num] = true;
	for (int i = 0; i < threads; i++) {
	    if (!done[i]) {
		try {
		    synchronized (done) {
			done.wait();
		    }
		} catch (InterruptedException intEx) {
		}
	    }
	}

	long endTime = System.currentTimeMillis();
	System.out.println("Creation time (milliseconds): " +
			   (creationTime - startTime));
	System.out.println("Startup time (milliseconds): " +
			   (realStartTime - creationTime));
	System.out.println("Value of dummy to fool optimizer: " + dummy);
	long totalTime = endTime - realStartTime;

	// How long does the bunch of nothing take?
	dummy = 0;
	long loopStartTime = System.currentTimeMillis();
	for (int i = 0; i < loops; i++) {
	    dummy += i;
	}
	long loopTime = System.currentTimeMillis() - loopStartTime;

	// Each thread did the bunch of nothing
	long adjustedTime = totalTime - loopTime * threads;

	System.out.println("Value of dummy (again) to fool optimizer: " +
			   dummy);
	System.out.println("Total time (milliseconds): " + totalTime);
	System.out.println("Adjusted time (milliseconds): " + adjustedTime);
	System.out.println("Milliseconds per thread per operation: " +
			   ((double)adjustedTime / threads));
	System.exit(0);
    }
}
