/*
 * @(#)$Id: Context2.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

/**
 * A test of the speed of thread creation and context switching
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Context2 extends Thread {

    private static long startTime;
    private static MyBoolean[] signal;

    private final int threads;
    private final int num;
    private final int loops;
    private int dummy;

    public static void main(String[] argv) throws InterruptedException {
	int threads = Integer.parseInt(argv[0]);
	int loops = Integer.parseInt(argv[1]);

	signal = new MyBoolean[threads];
	for (int i = 0; i < threads; i++) {
	    signal[i] = new MyBoolean(false);
	}

	for (int i = 0; i < threads; i++) {
	    new Context2(threads, i, loops).start();
	}
	startTime = System.currentTimeMillis();
	synchronized (signal[0]) {
	    signal[0].value = true;
	    signal[0].notifyAll();
	}
	try {
	    Thread.sleep(3600000L);  // Sleep for one hour; should be enough!
	} catch (InterruptedException intEx) {
	    // Do nothing; this can't happen
	}
    }

    Context2(int threads, int num, int loops) {
	this.threads = threads;
	this.num = num;
	this.loops = loops;
    }

    public void run() {
	int next = (num + 1) % threads;
	try {
	    for (int i = 0; i < loops; i++) {
		synchronized (signal[num]) {
		    if (!signal[num].value)
			signal[num].wait();
		}
		signal[num].value = false;
		synchronized (signal[next]) {
		    signal[next].value = true;
		    signal[next].notifyAll();
		}
	    }
	} catch (InterruptedException intEx) {
	}

	if (num == threads - 1) {
	    long endTime = System.currentTimeMillis();
	    long totalTime = endTime - startTime;
	    System.out.println("Total time (milliseconds): " + totalTime);
	    System.out.println("Milliseconds per thread per operation: " +
			       ((double)totalTime / (threads * loops)));
	    System.exit(0);
	}
    }
}
