/*
 * @(#)$Id: Context3.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

/**
 * A test of the speed of thread creation and context switching
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Context3 extends Thread {

    private static long startTime;
    private static MyInteger turn = new MyInteger(-1);

    private final int threads;
    private final int num;
    private final int loops;
    private int dummy = 0;

    public static void main(String[] argv) throws InterruptedException {
	int threads = Integer.parseInt(argv[0]);
	int loops = Integer.parseInt(argv[1]);

	for (int i = 0; i < threads; i++) {
	    new Context3(threads, i, loops).start();
	}
	startTime = System.currentTimeMillis();
	synchronized (turn) {
	    turn.value = 0;
	    turn.notifyAll();
	}
	try {
	    Thread.sleep(3600000L);  // Sleep for one hour; should be enough!
	} catch (InterruptedException intEx) {
	    // Do nothing; this can't happen
	}
    }

    Context3(int threads, int num, int loops) {
	this.threads = threads;
	this.num = num;
	this.loops = loops;
    }

    public void run() {
	int next = (num + 1) % threads;
	try {
	    for (int i = 0; i < loops; i++) {
		synchronized (turn) {
		    while (turn.value != num)
			turn.wait();
		    turn.value = next;
		    turn.notifyAll();
		}
		synchronized (this) {
		    dummy++;
		}
	    }
	} catch (InterruptedException intEx) {
	}

	if (num == threads - 1) {
	    long endTime = System.currentTimeMillis();
	    long totalTime = endTime - startTime;
	    System.out.println("Total time (milliseconds): " + totalTime);
	    System.out.println("Milliseconds per thread per operation: " +
			       ((double)totalTime / (threads * loops)));
	    System.exit(0);
	}
    }
}
