/*
 * @(#)$Id: Inlining.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import java.io.*;
import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.GlobalObj;
import mokan.obj.NoSuchKanObjectException;
import mokan.obj.ObjectID;
import mokan.thread.Future;
import mokan.transact.TransactionException;
import mokan.util.Timer;

/**
 * A test of Kan's thread inlining mechanism
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Inlining implements Cloneable, Externalizable, GlobalObj {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = -1214269084259802110L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    public ObjectID objID;

    public static void main(final Application app, final String[] argv)
	throws NoSuchKanObjectException, NoSuchMethodException,
	TransactionException {

	final int loops = (argv.length > 0) ? Integer.parseInt(argv[0]) : 1000;
	// Only use 1 node!  All calls are local, so inlining has a chance
	final ObjectID obj1 =
	    KanSystem.newKanObject(Inlining.class, null, true, 1);
	final ObjectID obj2 =
	    KanSystem.newKanObject(Inlining.class, null, true, 1);
	final Integer wrappedLoops = new Integer(loops);
	final Object[] args = new Object[2];
	args[0] = obj2;
	args[1] = wrappedLoops;
	KanSystem.invokeMethod(obj1, (short)0, args).join();
	System.gc();	// Clean up as much as possible
	args[0] = new Integer(1);
	final long startTime = Timer.currentTimeNanos();
	KanSystem.invokeMethod(obj1, (short)2, args).join();
	final long totalTime = Timer.currentTimeNanos() - startTime;
	System.out.println("Total time for serial calls: " + totalTime);
	System.out.println("Nanoseconds per method call: " +
			   ((double)totalTime / loops));
    }

    public Inlining() {
	objID = null;
    }

    public Inlining(final ObjectID objID) {
	this.objID = objID;
    }

    public void tree2Level(final ObjectID anObjID, final Integer wrappedLoops)
	throws NoSuchKanObjectException, NoSuchMethodException,
	TransactionException {

	final int loops = wrappedLoops.intValue();
	final long startTime = Timer.currentTimeNanos();
	final Future[] f = new Future[loops];
	for (int i = 0; i < loops; i++) {
	    f[i] = KanSystem.invokeMethod(anObjID, (short)1, null);
	}
	for (int i = 0; i < loops; i++) {
	    f[i].join();
	}
	final long totalTime = Timer.currentTimeNanos() - startTime;

	System.out.println("Total time for 2-level tree: " + totalTime);
	System.out.println("Nanoseconds per method call: " +
			   ((double)totalTime / loops));
    }

    public void doNothing() {
    }

    public void serial(final Integer wrappedCurrent, final Integer wrappedLast)
	throws NoSuchKanObjectException, NoSuchMethodException,
	TransactionException {

	final int current = wrappedCurrent.intValue();
	final int last = wrappedLast.intValue();
	if (current < last) {
	    final Object[] args = new Object[2];
	    args[0] = new Integer(current + 1);
	    args[1] = wrappedLast;
	    KanSystem.invokeMethod(objID, (short)2, args).join();
	}
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(final GlobalObj obj) {
    }

    public Object kanCallMethod(final short methNum, final Object[] args)
	throws NoSuchKanObjectException, NoSuchMethodException,
	TransactionException {

	switch (methNum) {
	case 0:
	    tree2Level((ObjectID)args[0], (Integer)args[1]);
	    return null;
	case 1:
	    doNothing();
	    return null;
	case 2:
	    serial((Integer)args[0], (Integer)args[1]);
	    return null;
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (kan.obj.ObjectID)in.readObject();
    }
}
