/*
 * @(#)$Id: MethodTest.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import mokan.util.Solaris;

/**
 * A test of the speed of Java method calls, thread creation, and thread run
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class MethodTest extends Thread {

    static boolean isTrue = true;
    boolean last;
    int loops;
    static long forTime;
    static long startTime4;

    public static void main(String[] argv) {
	int loops = Integer.parseInt(argv[0]);
	MethodTest[] methds = new MethodTest[loops];

	// Measure the time it takes to spin around an empty for loop
	long startTime0 = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++)
	    ;
	forTime = Solaris.currentTimeNanos() - startTime0;

	// Measure the cost of a static method call
	long startTime1 = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++)
	    staticEmptyMethod();
	long totalTime1 = Solaris.currentTimeNanos() - startTime1 - forTime;
	System.out.println("Total time: " + totalTime1);
	System.out.println("Nanoseconds per empty method call (static): " +
			   ((double)totalTime1 / loops));

	// Measure the cost of a non-static method call
	Empty[] empties = new Empty[loops];
	for (int i = 0; i < loops; i++)
	    empties[i] = new Empty();
	long startTime2 = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++)
	    empties[i].emptyMethod();
	long totalTime2 = Solaris.currentTimeNanos() - startTime2 - forTime;
	System.out.println("Total time: " + totalTime1);
	System.out.println("Nanoseconds per empty method call: " +
			   ((double)totalTime2 / loops));
	// Now clear out the memory we used to avoid later GC costs
	empties = null;
	System.gc();

	// Measure thread creation costs
	long startTime3 = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++)
	    methds[i] = new MethodTest(loops, i == loops - 1);
	long totalTime3 = Solaris.currentTimeNanos() - startTime3 - forTime;
	System.out.println("Total time: " + totalTime3);
	System.out.println("Nanoseconds per thread creation: " +
			   ((double)totalTime3 / loops));

	// Measure thread startup costs
	startTime4 = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++)
	    methds[i].start();
    }

    MethodTest(int loops, boolean last) {
	this.loops = loops;
	this.last = last;
    }

    public void run() {
	if (last) {
	    long totalTime = Solaris.currentTimeNanos() - startTime4 - forTime;
	    System.out.println("Total time: " + totalTime);
	    System.out.println("Nanoseconds per thread start: " +
			       ((double)totalTime / loops));
	}
    }

    public static void staticEmptyMethod() {
	isTrue = !isTrue;
    }
}
