/*
 * @(#)$Id: Reflection.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * A test of the speed of Java method calls.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Reflection {

    public static void main(String[] argv) throws IllegalAccessException,
	InvocationTargetException, NoSuchMethodException {

	int loops = Integer.parseInt(argv[0]);
	MethodCall obj = new MethodCall();
	doEmptyMethods(obj, loops);
	doMostlyEmptyMethods(obj, loops);
	doOneParam(obj, loops);
    }

    private static void doEmptyMethods(MethodCall obj, int loops)
	throws IllegalAccessException, InvocationTargetException,
	NoSuchMethodException {

	long startTime = System.currentTimeMillis();
	Method meth;
	for (int i = 0; i < loops; i++) {
	    meth = MethodCall.class.getMethod("emptyMethod1", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("emptyMethod2", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("emptyMethod3", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("emptyMethod4", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("emptyMethod5", null);
	    meth.invoke(obj, null);
	}
	long endTime = System.currentTimeMillis();
	long totalTime = endTime - startTime;

	startTime = System.currentTimeMillis();
	for (int i = 0; i < loops; i++) {
	    // Do nothing; just time the loop
	}
	endTime = System.currentTimeMillis();
	long adjustedTime = totalTime - endTime + startTime;
	System.out.println("Adjusted time: " + adjustedTime);
	System.out.println("Milliseconds per reflection invocation: " +
			   (((double)adjustedTime / loops) / 5));
    }

    private static void doMostlyEmptyMethods(MethodCall obj, int loops)
	throws IllegalAccessException, InvocationTargetException,
	NoSuchMethodException {

	long startTime = System.currentTimeMillis();
	Method meth;
	for (int i = 0; i < loops; i++) {
	    meth = MethodCall.class.getMethod("mostlyEmptyMethod1", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("mostlyEmptyMethod2", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("mostlyEmptyMethod3", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("mostlyEmptyMethod4", null);
	    meth.invoke(obj, null);
	    meth = MethodCall.class.getMethod("mostlyEmptyMethod5", null);
	    meth.invoke(obj, null);
	}
	long endTime = System.currentTimeMillis();
	long totalTime = endTime - startTime;

	startTime = System.currentTimeMillis();
	for (int i = 0; i < loops; i++) {
	    // Do nothing; just time the loop
	}
	endTime = System.currentTimeMillis();
	long adjustedTime = totalTime - endTime + startTime;
	System.out.println("Adjusted time: " + adjustedTime);
	System.out.println("Milliseconds per reflection invocation: " +
			   (((double)adjustedTime / loops) / 5));
    }

    private static void doOneParam(MethodCall obj, int loops)
	throws IllegalAccessException, InvocationTargetException,
	NoSuchMethodException {

	long startTime = System.currentTimeMillis();
	Method meth;
	for (int i = 0; i < loops; i++) {
	    Class[] argClass = new Class[1];
	    argClass[0] = int.class;
	    meth = MethodCall.class.getMethod("oneParamMethod1", argClass);
	    Object[] args = new Object[1];
	    args[0] = new Integer(1);
	    meth.invoke(obj, args);
	    argClass = new Class[1];
	    argClass[0] = int.class;
	    meth = MethodCall.class.getMethod("oneParamMethod2", argClass);
	    args = new Object[1];
	    args[0] = new Integer(2);
	    meth.invoke(obj, args);
	    argClass = new Class[1];
	    argClass[0] = int.class;
	    meth = MethodCall.class.getMethod("oneParamMethod3", argClass);
	    args = new Object[1];
	    args[0] = new Integer(3);
	    meth.invoke(obj, args);
	    argClass = new Class[1];
	    argClass[0] = int.class;
	    meth = MethodCall.class.getMethod("oneParamMethod4", argClass);
	    args = new Object[1];
	    args[0] = new Integer(4);
	    meth.invoke(obj, args);
	    argClass = new Class[1];
	    argClass[0] = int.class;
	    meth = MethodCall.class.getMethod("oneParamMethod5", argClass);
	    args = new Object[1];
	    args[0] = new Integer(5);
	    meth.invoke(obj, args);
	}
	long endTime = System.currentTimeMillis();
	long totalTime = endTime - startTime;

	startTime = System.currentTimeMillis();
	for (int i = 0; i < loops; i++) {
	    // Do nothing; just time the loop
	}
	endTime = System.currentTimeMillis();
	long adjustedTime = totalTime - endTime + startTime;
	System.out.println("Adjusted time: " + adjustedTime);
	System.out.println("Milliseconds per reflection invocation: " +
			   (((double)adjustedTime / loops) / 5));
    }
}
