/*
 * @(#)$Id: ReplDriver.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.speed;

import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.transact.TransactionException;

/**
 * Do lots of reads, then lots of writes
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public class ReplDriver implements GlobalObj, Cloneable {

    public ObjectID objID; 

    private final int calls;

    private final static Integer junk = new Integer(42);

    public ReplDriver(ObjectID oid, Integer callsInt) {
	objID = oid;
	calls = callsInt.intValue();
    }

    public void startSim(final ObjectID[] objIDs) {
	try {
	    // First do lots of reads, and watch the replicas appear!
	    for (int i = 0; i < objIDs.length; i++) {
		for (int j = 0; j < calls; j++) {
		    ReplDriver target =
			(ReplDriver)KanSystem.callDirectP(objIDs[i]);
		    if (target == null) {
			int num = KanSystem.invokeMethod(objIDs[i], "read",
							 null, true).joinInt();
		    } else {
			System.out.println("Reading local[" + i + "]: " + j);
			target.read();
		    }
		}
	    }

	    // Now do lots of writes, and watch those replicas vanish!
	    for (int i = 0; i < objIDs.length; i++) {
		for (int j = 0; j < calls; j++) {
		    ReplDriver target =
			(ReplDriver)KanSystem.callDirectP(objIDs[i]);
		    if (target == null) {
			KanSystem.invokeMethod(objIDs[i], "write", null,
					       true).join();
		    } else {
			System.out.println("Writing local[" + i + "]: " + j);
			target.write();
		    }
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public Integer read() {
	try {
	    KanSystem.guardBegin(objID, null, false, false, false);
	    KanSystem.guardEnd();
	} catch (TransactionException transEx) {
	}
	return junk;
    }

    public void write() {
	try {
	    KanSystem.guardBegin(objID, null, true, false, false);
	    KanSystem.guardEnd();
	} catch (TransactionException transEx) {
	}
    }

    public String toString() {
	return "ReplDriver " + objID;
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// There's no state to copy, so this is a null method
    }
}
