/*
 * @(#)$Id: Replication.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.speed;

import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.GlobalObj;
import mokan.obj.ObjectID;
import mokan.thread.Future;

/**
 * Test the replication behavior of Kan
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Replication implements GlobalObj, Cloneable {

    /**
     * The default number of method calls to make on each object
     */
    private static final int NUM_CALLS = 1000;

    /**
     * Total number of nodes in the system.
     */
    public static int numNodes;

    public ObjectID objID;

    public static void main(Application app, String argv[]) {
	numNodes = app.numNodes();
	Object[] argsSim = new Object[1];
	argsSim[0] = (argv.length != 1)
	    ? new Integer(NUM_CALLS)
	    : new Integer(argv[0]);

	// Create the ReplDrivers on all nodes in a round robin fashion
	try {
	    // Object IDs of all 'ReplDrivers' in the system
	    ObjectID driverOids[] = new ObjectID[numNodes];
	    for (int currNode = 0; currNode < numNodes; currNode++) {
		driverOids[currNode] =
		    KanSystem.newKanObject(ReplDriver.class, argsSim, true);
	    }

	    // Future for each 'startSim' method to be invoked
	    Future tids[] = new Future[numNodes];
	    Object[] args = new Object[1];
	    args[0] = driverOids;
	    for (int currNode = 0; currNode < numNodes; currNode++) {
		tids[currNode] =
		    KanSystem.invokeMethod(driverOids[currNode], "startSim",
					   args, true);
	    }

	    // Wait for these invoked methods to join
	    for (int i = 0; i < numNodes; i++)
		tids[i].join();
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	// Do nothing
    }
}
