/*
 * @(#)$Id: Synchronization.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import mokan.util.Solaris;

/**
 * A test of the speed of synchronization operations
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Synchronization extends Thread {

    private static long startTime;
    private static MyInteger turn = new MyInteger(-1);
    private static Boolean wakeup = new Boolean(true);

    private final int threads;
    private final int num;
    private final int loops;

    public static void main(String[] argv) throws InterruptedException {
	int lowThreads = Integer.parseInt(argv[0]);
	int highThreads = Integer.parseInt(argv[1]);
	int loops = Integer.parseInt(argv[2]);
	Boolean b = new Boolean(true);
	int junk = 0;

	long myStartTime = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++) {
	    synchronized (b) {
		junk += i;
	    }
	}
	long endTime = Solaris.currentTimeNanos();
	long totalTime = endTime - myStartTime;

	myStartTime = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++) {
	    junk += i;
	}
	endTime = Solaris.currentTimeNanos();
	long adjustedTime = totalTime - endTime + myStartTime;

	System.out.println("Use junk to avoid optimizing it away: " + junk);
	System.out.println("Total time (nanoseconds): " + totalTime);
	System.out.println("Adjusted time (nanoseconds): " + adjustedTime);
	System.out.println
	    ("Nanoseconds per synchronization with no contention: " +
	     ((double)adjustedTime / loops));

	// Now start the multithreaded test with low threads
	Synchronization[] synch = new Synchronization[lowThreads];
	for (int i = 0; i < lowThreads; i++) {
	    synch[i] = new Synchronization(lowThreads, i, loops);
	    synch[i].start();
	}
	startTime = Solaris.currentTimeNanos();
	synchronized (turn) {
	    turn.value = 0;
	    turn.notifyAll();
	}
	for (int i = 0; i < lowThreads; i++) {
	    synch[i].join();
	}

	// And do it again with high threads
	synch = new Synchronization[highThreads];
	for (int i = 0; i < highThreads; i++) {
	    synch[i] = new Synchronization(highThreads, i, loops);
	    synch[i].start();
	}
	startTime = Solaris.currentTimeNanos();
	synchronized (turn) {
	    turn.value = 0;
	    turn.notifyAll();
	}
	for (int i = 0; i < highThreads; i++) {
	    synch[i].join();
	}
    }

    Synchronization(int threads, int num, int loops) {
	this.threads = threads;
	this.num     = num;
	this.loops   = loops;
    }

    public void run() {
	long totalTime;

	// The loop with one synchronized block
	int next = (num + 1) % threads;
	try {
	    for (int i = 0; i < loops; i++) {
		synchronized (turn) {
		    while (turn.value != num)
			turn.wait();
		    turn.value = next;
		    turn.notify();
		}
	    }
	} catch (InterruptedException intEx) {
	    // Can't happen
	}

	totalTime = Solaris.currentTimeNanos() - startTime;

	// The loop with two synchronized blocks
	int junk = 0;
	long newStartTime = Solaris.currentTimeNanos();
	try {
	    for (int i = 0; i < loops; i++) {
		synchronized (turn) {
		    junk++;
		}
		synchronized (turn) {
		    while (turn.value != num)
			turn.wait();
		    turn.value = next;
		    turn.notify();
		}
	    }
	} catch (InterruptedException intEx) {
	    // Can't happen
	}
	long newEndTime = Solaris.currentTimeNanos();

	if (num == threads - 1) {
	    long adjustedTime = newEndTime - newStartTime - totalTime;
	    System.out.println("Use junk so it isn't optimized away: " + junk);
	    System.out.println("Total time (nanoseconds): " + totalTime);
	    System.out.println("Adjusted time: " + adjustedTime);
	    System.out.println("Nanoseconds per thread per operation: " +
			       ((double)adjustedTime / (threads * loops)));
	}
    }
}
