/*
 * @(#)$Id: ThreadPoolSize.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import java.io.*;
import mokan.KanSystem;
import mokan.app.Application;
import mokan.obj.*;
import mokan.thread.*;
import mokan.transact.TransactionException;

/**
 * A test of the size of the thread pool
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class ThreadPoolSize implements GlobalObj, Cloneable {

    public ObjectID objID;

    public int junk = 0;

    private static int total;
    public static int counter = 0;
    public static int[] numCalls;
    public static int[] poolSize;
    public static int[] poolCapacity;

    public static void main(Application app, String[] argv)
	throws InterruptedException, IOException, NoSuchKanObjectException,
	NoSuchMethodException, TransactionException {

	int threads = (argv.length > 0) ? Integer.parseInt(argv[0]) : 500;
	total = 2 * threads + 1;
	numCalls = new int[total + 1];
	poolSize = new int[total + 1];
	poolCapacity = new int[total + 1];

	// Only use 1 node!  All calls are local, so thread pool will work
	ObjectID myObj = KanSystem.newKanObject(ThreadPoolSize.class, null,
						true, 1);

	// Make a whole bunch of calls, to make the thread pool change size
	Future[] f = new Future[threads];
	for (int i = 0; i < threads; i++) {
	    f[i] = KanSystem.invokeMethod(myObj, "wasteTime", null, true);
	}

	// Wait for them to finish, without explicitly joining (tricky!)
	synchronized (ThreadPoolSize.class) {
	    ThreadPoolSize.class.wait();
	}

	// Dump statistics to a file
	PrintWriter out = new PrintWriter(new FileWriter("Poolsize.txt"));
	for (int i = 0; i < total; i++) {
	    out.print(numCalls[i]);
	    out.print('\t');
	    out.print(poolSize[i]);
	    out.print('\t');
	    out.println(poolCapacity[i]);
	}
	out.close();
    }

    public static synchronized void addEntry(boolean end, int size, int cap) {
	// Make sure the application has started
	if (numCalls != null) {
	    counter++;
	    numCalls[counter] = numCalls[counter - 1] + (end ? -1 : 1);
	    poolSize[counter] = size;
	    poolCapacity[counter] = cap;
	    if (counter == total - 1) {
		ThreadPoolSize.class.notify();
		counter = 0;
	    }
	}
    }

    public ThreadPoolSize(ObjectID myObjID) {
	objID = myObjID;
    }

    public void wasteTime() {
	for (int i = 0; i < 1000000; i++) {
	    junk += i;
	}
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	junk = ((ThreadPoolSize)obj).junk;
    }
}
