/*
 * @(#)$Id: ThreadPoolTest.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 */

package mokan.test.speed;

import mokan.comm.LogicalNode;
import mokan.thread.*;
import mokan.util.Solaris;

/**
 * A test of the speed of the thread pool
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class ThreadPoolTest {

    private static final int THREADS = 40;
    private static final int BIG_THREADS = 100;

    public static void main(String[] argv) {
	int loops = Integer.parseInt(argv[0]);
	LogicalNode fakeNode = new LogicalNode(null, "Node", true);

	// First, create the thread pool
	long startTime = Solaris.currentTimeNanos();
	ThreadPool pool = new ThreadPool(true);
	long totalTime = Solaris.currentTimeNanos() - startTime;
	System.out.println("Time to create pool (nanoseconds): " + totalTime);

	// Pattern 1: remove 40, add 40, remove 40, add 40, ...
	KanThread[] threads = new KanThread[BIG_THREADS];
	startTime = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++) {
	    for (int j = 0; j < THREADS; j++) {
		threads[j] = pool.getThreadFromPool(fakeNode);
	    }
	    for (int j = 0; j < THREADS; j++) {
		pool.returnThreadToPool(threads[j]);
	    }
	}
	totalTime = Solaris.currentTimeNanos() - startTime;
	System.out.println("Time for pattern 1 (nanoseconds): " + totalTime);
	System.out.println("Time for each action: " +
			   (totalTime / (float)(loops * THREADS)));

	// Pattern 2: remove 40, then remove 1, add 1, remove 1, add 1, ...
	for (int i = 0; i < THREADS; i++) {
	    threads[i] = pool.getThreadFromPool(fakeNode);
	}
	startTime = Solaris.currentTimeNanos();
	for (int i = 0; i < loops; i++) {
	    KanThread thrd = pool.getThreadFromPool(fakeNode);
	    pool.returnThreadToPool(thrd);
	}
	totalTime = Solaris.currentTimeNanos() - startTime;
	System.out.println("Time for pattern 2 (nanoseconds): " + totalTime);
	System.out.println("Time for each action: " +
			   (totalTime / (float)(loops * THREADS)));
	for (int i = 0; i < THREADS; i++) {
	    pool.returnThreadToPool(threads[i]);
	}

	// Pattern 3: Remove a whole bunch
	startTime = Solaris.currentTimeNanos();
	for (int i = 0; i < BIG_THREADS; i++) {
	    threads[i] = pool.getThreadFromPool(fakeNode);
	}
	totalTime = Solaris.currentTimeNanos() - startTime;
	System.out.println("Time for pattern 3 (nanoseconds): " + totalTime);
	System.out.println("Time for each action: " +
			   (totalTime / (float)BIG_THREADS));

	// Pattern 4: Return a whole bunch
	startTime = Solaris.currentTimeNanos();
	for (int i = 0; i < BIG_THREADS; i++) {
	    pool.returnThreadToPool(threads[i]);
	}
	totalTime = Solaris.currentTimeNanos() - startTime;
	System.out.println("Time for pattern 4 (nanoseconds): " + totalTime);
	System.out.println("Time for each action: " +
			   (totalTime / (float)BIG_THREADS));

	// Done: we have to do this because there are live non-daemon threads
	System.exit(0);
    }
}
