/*
 * @(#)$Id: Target.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 * Copyright (c)1999 Regents of the University of California
 *
 */

package mokan.test.trans;

import java.io.*;
import mokan.KanSystem;
import mokan.obj.GlobalObj;
import mokan.obj.NoSuchKanObjectException;
import mokan.obj.ObjectID;
import mokan.transact.TransactionException;

/**
 * Do various kinds of transactions in support of testing
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class Target implements Cloneable, GlobalObj, Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    private static final long serialVersionUID = 3227708310600009221L;

    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    public ObjectID objID;
    public int dummy;

    public Target() {
	objID = null;
    }

    public Target(final ObjectID oid) {
	objID = oid;
    }

    public int read() throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, false, false, false);
	    return dummy;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void write(final int val) throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, true, false, false);
	    dummy = val;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public int readBlock() throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, false, true, false);
	    return dummy;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void writeBlock(final int val) throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, true, true, false);
	    dummy = val;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public int readNest(final ObjectID[] list, final int index)
	throws NoSuchKanObjectException, TransactionException {

	try {
	    KanSystem.guardBegin(objID, null, false, true, true);
	    if (index == list.length)
		return dummy;
	    final ObjectID nextObj = list[index];
	    Target target = (Target)KanSystem.callDirectP(nextObj);
	    if (target == null) {
		Object[] args = new Object[2];
		args[0] = list;
		args[1] = new Integer(index + 1);
		return KanSystem.invokeMethod(nextObj, (short)4,
					      args).joinInt();
	    } else {
		return target.readNest(list, index + 1);
	    }
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void writeNest(final int val, final ObjectID[] list,
			  final int index)
	throws NoSuchKanObjectException, TransactionException {

	try {
	    KanSystem.guardBegin(objID, null, true, true, true);
	    if (index == list.length) {
		dummy = val;
		return;
	    }
	    final ObjectID nextObj = list[index];
	    Target target = (Target)KanSystem.callDirectP(nextObj);
	    if (target == null) {
		Object[] args = new Object[3];
		args[0] = new Integer(val);
		args[1] = list;
		args[2] = new Integer(index + 1);
		KanSystem.invokeMethod(nextObj, (short)5, args).join();
	    } else {
		target.writeNest(val, list, index + 1);
	    }
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public int readNest(final ObjectID[] list)
	throws NoSuchKanObjectException, TransactionException {

	try {
	    KanSystem.guardBegin(objID, null, false, true, true);
	    int sum = 0;
	    for (int i = 0; i < list.length; i++) {
		final ObjectID nextObj = list[i];
		Target target = (Target)KanSystem.callDirectP(nextObj);
		sum += (target == null)
		    ? KanSystem.invokeMethod(nextObj, (short)0, null).joinInt()
		    : target.read();
	    }
	    return sum;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void writeNest(final int val, final ObjectID[] list)
	throws NoSuchKanObjectException, TransactionException {

	try {
	    KanSystem.guardBegin(objID, null, true, true, true);
	    for (int i = 0; i < list.length; i++) {
		final ObjectID nextObj = list[i];
		Target target = (Target)KanSystem.callDirectP(nextObj);
		if (target == null) {
		    Object[] args = new Object[1];
		    args[0] = new Integer(val);
		    KanSystem.invokeMethod(nextObj, (short)1, args).join();
		} else {
		    target.write(val);
		}
	    }
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public int abortRead() throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, false, false, false);
	    KanSystem.abort(new Exception("Bang!  You're dead!"));
	    return dummy;
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public void abortWrite(final int val) throws TransactionException {
	try {
	    KanSystem.guardBegin(objID, null, true, false, false);
	    dummy = val;
	    KanSystem.abort(new Exception("Bang!  You're dead!"));
	} finally {
	    KanSystem.guardEnd();
	}
    }

    public String toString() {
	return ("Target[" + objID + "]: " + dummy);
    }

    public GlobalObj kanClone() {
	try {
	    return (GlobalObj)clone();
	} catch (CloneNotSupportedException cloneEx) {
	    return null;
	}
    }

    public void kanSetObject(GlobalObj obj) {
	dummy = ((Target)obj).dummy;
    }

    public Object kanCallMethod(short methNum, Object[] args)
	throws Exception {

	switch (methNum) {
	case 0:
	    return new Integer(read());
	case 1:
	    write(((Integer)(args[0])).intValue());
	    return null;
	case 2:
	    return new Integer(readBlock());
	case 3:
	    writeBlock(((Integer)(args[0])).intValue());
	    return null;
	case 4:
	    return new Integer(readNest((ObjectID[])args[0],
					((Integer)args[1]).intValue()));
	case 5:
	    writeNest(((Integer)(args[0])).intValue(), (ObjectID[])args[1],
		      ((Integer)(args[2])).intValue());
	    return null;
	case 6:
	    return new Integer(readNest((ObjectID[])args[0]));
	case 7:
	    writeNest(((Integer)(args[0])).intValue(), (ObjectID[])args[1]);
	    return null;
	case 8:
	    return new Integer(abortRead());
	case 9:
	    abortWrite(((Integer)(args[0])).intValue());
	    return null;
	case 10:
	    return toString();
	default:
	    System.err.println("Invalid method called on " + toString());
	    return null;
	}
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	out.writeObject(objID);
	out.writeInt(dummy);
    }

    public void readExternal(ObjectInput in)
	throws IOException, ClassNotFoundException {

	objID = (ObjectID)in.readObject();
	dummy = in.readInt();
    }
}
