/*
 * @(#)ExecMsg.java ExecMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.thread;

import java.io.*;
import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.debug.Event;
import mokan.obj.KanObject;
import mokan.obj.NoSuchKanObjectException;
import mokan.obj.ObjectID;
import mokan.transact.TransactionAbortException;

/**
 * Request execution of a method on a remote node
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
final class ExecMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -1845861076159647647L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;


    /**
     * The method calling information.  This is a pseudo-final field.
     *
     * @serial information on the method call to make
     */
    private MethodInfo info;

    /**
     * The no-args constructor, needed for deserialization
     */
    public ExecMsg() {
        super(true, true);
    }

    /**
     * Create a new <code>ExecMsg</code> to request execution of a method on a
     * remote node
     *
     * @param node the originating logical node for this message
     * @param info information on the method call to make
     */
    ExecMsg(final LogicalNode node, final MethodInfo info) {
        super(node, info.args != null, true);
        this.info = info;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * Take the receive action for this method, which is to execute a method
     * on a local object
     */
    public void handle () {
        super.handle();

        // Pseudo-final handling
        final MethodInfo myInfo = info;
        final FutureID futID = myInfo.futID;
        final ObjectID objID = myInfo.objID;
        //KK  If we are unable to start the call (either because we don't
        //KK  Have a valid object or the copy we have has been invalidated.
        //KK  Then Send A ObjectException
        if (!objID.isLocalValid()) {
            new DoneMsg(to, futID,
                        new NoSuchKanObjectException(objID.toString()), false,
                        null).send(from);
            return;
        }
        //KK I've re-enabled the code above with some reservations.
        //KK is this a race?  Local becomes invalid before execution begins?
        try {
            final KanObject kObj = objID.getLocKanObject();
            if (kObj.executeHere(from, to, this)) {
                final ThreadScheduler scheduler =
                    to.getLocalHandler().scheduler;
                scheduler.makeLocalCall(objID, myInfo);
            } else {
                send(from);
            }
        } catch (TransactionAbortException e) {
            // return the exception
            new DoneMsg(to, futID, e, false, null).send(from);
        }
    }

    /**
     * Convert the <code>ExecMsg</code> to a string
     *
     * @return a string representation of this <code>ExecMsg</code>
     */
    public String toString() {
        return "ExecMsg[" + info + ']';
    }

    /**
     * Save the state of the <code>ExecMsg</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by information
     *             on how to invoke the method (MethodInfo).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        info.writeCompactly(out);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        info = MethodInfo.readCompactly(in);
    }
}
