/**
 * @(#)$Id: FutureID.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.thread;

import java.io.*;
import mokan.comm.LogicalNode;
import mokan.util.ID;
import mokan.util.CanonicalizingIDSet;

/**
 * A unique identifier for a Future (a handle to an asynchronous method call)
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 * @seemokan.thread.Future
 */
final class FutureID extends ID implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 2594163157073033147L;






    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;


    /**
     * A container for canonicalizing <code>FutureID</code>s
     */
    private static final CanonicalizingIDSet allFut =
	new CanonicalizingIDSet();

    /**
     * A unique sequence number generator for each node.
     */
    private static long futCounter = 0L;

    /**
     * A pointer to the actual Future object, if it is local
     */
    final transient Future theFuture; // Don't go back to it
    // Well, if you must know, that      ^^^^^^^^^^^^^^^^^^^ was a joke, made
    // by someone who has watched far too many 1980s movies.

    /**
     * The no-args constructor, needed for deserialization
     */
    public FutureID() {
	super();
	theFuture = null;
    }

    /** 
     * Construct a new <code>FutureID</code> for a <code>Future</code>
     *
     * @param node the originating node for the future
     * @param fut the future
     */
    FutureID(final LogicalNode node, final Future fut) {
	super(node, futCounter++);
	theFuture = fut;
	allFut.add(this);
    }

    /** 
     * Deserialization constructor
     *
     * @param node the originating node for the future
     * @param count the count portion of the <code>ID</code>
     */
    FutureID(final LogicalNode node, final long count) {
	super(node, count);
	theFuture = null;
    }

    /**
     * Remove this future from the set of all futures
     */
    void remove() {
	allFut.remove(this);
    }

    /**
     * Determine the <code>ID</code> of the originating thread for this future
     *
     * @return the <code>ID</code> of the originating thread for this future
     */
    ThreadID getParentThread() {
	return theFuture == null ? null : theFuture.tid;
    }

    /**
     * Save the state of the <code>FutureID</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out.
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
    }

    public void readExternal(final ObjectInput in)
	throws ClassNotFoundException, IOException {

	super.readExternal(in);
	allFut.add(this);
    }

    /**
     * A shortcut to reduce the cost of serialization.  Read from an object
     * input stream, but skip some object handles that we don't have to read
     *
     * @param in the object input stream to read from
     * @param add <code>true</code> if the <code>FutureID</code> to read has
     *        never been deserialized on this node before, and so needs to be
     *        added to the set of all <code>FutureID</code>s
     * @return the <code>FutureID</code> read from the stream
     * @exception ClassNotFoundException if some class identified in the
     *        stream cannot be found locally
     * @exception IOException if a stream error occurs
     */
    static FutureID readCompactly(final ObjectInput in, final boolean add)
	throws ClassNotFoundException, IOException {

	final long count = in.readLong();
	final LogicalNode node = (LogicalNode)in.readObject();
	return (node == null)
	    ? null
	    : (add ? new FutureID(node, count)
		   : (FutureID)allFut.find(node, count));
    }

    public String toString() {
	return "Future" + super.toString();
    }
}
