/*
 * @(#)AbortMsg.java AbortMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.transact;

import java.io.*;
import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.debug.Event;

/**
 * Notify a remote transaction of an abort.  This is used in the nested
 * transaction code to notify children that they are dead when the parent
 * transaction aborts.
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Sean Brydon
 * @author      Jerry James
 */
final public class AbortMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -1504053262001960088L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;



    /**
     * The <code>TransactionID</code> of the transaction to abort.  This is a
     * pseudo-final field.
     *
     * @serial the <code>TransactionID</code> of the transaction to abort
     */
    private TransactionID victim;

    /**
     * The no-args constructor, needed for deserialization
     */
    public AbortMsg() {
        super(false, false);
    }

    /**
     * Make a new <code>AbortMsg</code>
     *
     * @param node the originating node for this message
     * @param transID the <code>ID</code> of the transaction to abort
     */
    public AbortMsg(final LogicalNode node, final TransactionID transID) {
        super(node, false, false);
        victim = transID;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * Handle this message by aborting the target transaction
     */
    public void handle() {
        super.handle();
        victim.abort(new TransactionAbortException("Parent killed"));
    }

    /**
     * Convert the <code>AbortMsg</code> to a string
     *
     * @return a string representation of the <code>AbortMsg</code>
     */
    public String toString() {
        return "AbortMsg(" + victim + ')';
    }

    /**
     * Save the state of the <code>AbortMsg</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             identifier for the transaction to be aborted
     *             (TransactionID).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(victim);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        victim = (TransactionID)in.readObject();
    }
}
