/**
 * ConcurrencyScheduler
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.transact;

import mokan.comm.LogicalNode;
import mokan.obj.GuardCondition;
import mokan.obj.ObjectID;
import mokan.obj.GlobalObj;
import mokan.obj.KanObject;

import mokan.thread.AsynchCall;
import mokan.thread.Future;




/**
 *  The <code>ConcurrencyScheduler</code> schedules transaction and locks.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public interface ConcurrencyScheduler {

    public static final int NONBLOCKING = 1;
    public static final int BLOCKING = 2;
    public static final int NESTED = 3;


    //////////////////////////////////////////////////////////
    // Transactions
    
    /** Create a Transaction type for this concurrency scheduler */
    public Transaction makeTransaction (final GuardCondition gc, 
                                 final ObjectID objID, 
                                 final boolean write,
                                 final AsynchCall call, 
                                 final int type);

    /** Initiate the transaction 
     *  @param t  The transation 
     */
    public void beginTransaction (Transaction t);

    /** 
     * End the transtaction
     * @param t the transaction to finish
     */
    public void    endTransaction(Transaction t, boolean install);
    public void    abortTransaction(TransactionID t, TransactionException te);

    //IsolationRequest  lookup(TransactionID tid, IsolationPrimitive lock);

    /////////////////////////////////////////////////////////
    // Lock Requests

//     public IsolationRequest makeIsolationRequest (TransactionID tid,
//                                            LogicalNode   ln,
//                                            GuardCondition gc,
//                                            ObjectID oid,
//                                            int flags);

    public IsolationRequest beginIsolation(IsolationPrimitive lock,
                                           GuardCondition guardCond, 
                                           ObjectID objID,
                                           int   flags);

    public void beginIsolation(IsolationRequest ir, IsolationPrimitive lock);

    public void endIsolation(IsolationRequest lockReq,
                      IsolationPrimitive lock,
                      GlobalObj oldObj,
                      boolean install);

//     public IsolationRequest upgrade(IsolationRequest ir, 
//                                     IsolationPrimitive lock);



    /////////////////////////////////////////////////////////
    // Lock Objects


    public IsolationPrimitive makePrimitive (KanObject kobj, int flags);

    /////////////////////////////////////////////////////////
    // Objects
//    public void install(GlobalObj obj, long version);
//    public GlobalObjCopy() copy();


    /**
     * Return a protocol number used by the object based on the 
     * type of the object.    Object Types are either MOBILE
     * or IMMOBILE depending on whether they can use replica
     * change home, etc.    
     * @param type Either <code>KanSystem.MOBILE</code> or <code>KanSystem.IMMOBILE </code>
     * @return A value from <code>Protocol</code>
     */
    public int protocol (int type);



        /**
     * Create a thread to invoke a method on some <code>KanObject</code>
     *
     * @param objID the {@linkmokan.obj.ObjectID ObjectID} for the object on
     *        which to invoke the method 
     * @param fut a {@linkmokan.thread.Future Future} to hold the results of
     *        the method call
     * @param num the method number to invoke
     * @param args an array of objects holding the method arguments
     * @exception TransactionException if the containing transaction (if any)
     *        is aborted
     */
    public void invokeMethod(final ObjectID objID, final Future fut,
                             final int num, final Object[] args);



}
