/**
 * InvalidateMsg
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.transact;


import java.io.Externalizable;
import java.io.ObjectStreamField;
import java.io.ObjectStreamClass;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;


import mokan.obj.ObjectID;
import mokan.obj.KanObject;
import mokan.obj.GlobalObj;


import mokan.comm.LogicalNode;
import mokan.comm.AppMessage;
import mokan.comm.AppReply;
import mokan.debug.Event;

import mokan.nameservice.*;

/**
 *  The <code>InvalidateMsg</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public class InvalidateMsg extends AppMessage  implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -1504053262001960088L;


    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The <code>ObjectID</code> for the copied object.  This is a
     * pseudo-final field.
     *
     * @serial the <code>ObjectID</code> for the copied object
     */
    private ObjectID objID;

    /**
     * A copy of a global object.  This is a pseudo-final field.
     *
     * @serial a copy of a global object
     */
    GlobalObj gObj;

    /**
     * The version number of this object.  This is a pseudo-final field.
     *
     * @serial the version number of this object
     */
    long version;


    public InvalidateMsg (){
        super(false, false);
    }

    public InvalidateMsg (ObjectID objID){
        super(false, false);
    }

    public InvalidateMsg (ObjectID objID, GlobalObj gobj, long version){
        super(false, false);
        this.gObj = gobj;
        this.version = version;
    }


    /**
     * Handle this message by removing the sender from the list of replica
     * holders for the indicated object
     */
    public void handle() {
        super.handle();
        objID.invalidate();
    }

    /**
     * Convert the <code>ReplicaDropMsg</code> to a string
     *
     * @return a string representation of this <code>ReplicaDropMsg</code>
     */
    public String toString() {
        return "ReplicaDropMsg[" + objID + ']';
    }

    /**
     * Save the state of the <code>ReplicaDropMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the
     *             identifier for the object whose replica is being deleted
     *             (ObjectID).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();
    }
}
