/**
 * IsolationPrimitive
 * Copyright kkvilekval@acm.org.   See COPYRIGHT for details.
 */
package mokan.transact;


import mokan.obj.GlobalObj;
import mokan.obj.GlobalObjCopy;


/**
 *  The <code>IsolationPrimitive</code> is an interface to
 *  any class of locks that allow the concurrency manager to
 *  guarantee isolation.  Expected types include strict
 *  and optimistic locks.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval (kkvilekval@acm.org)
 */
public interface IsolationPrimitive {
    /* Private/Public Variables */

    public void lock(IsolationRequest ir);
    public void unlock(IsolationRequest ir, boolean install, GlobalObj gob);
    public void unlock(IsolationRequest ir, boolean install, GlobalObj gob, long version);
    public boolean idle();
    public void nudge();

    public IsolationRequest lockForCopy();
    public void unlockForCopy(IsolationRequest lr);

    //  Iffy these are here because <code>Protocol</code> expects them
    //public GlobalObjCopy copy()  throws InterruptedException;
    public LockRequest upgrade(LockRequest lockReq);
    public LockRequest lookup (TransactionID tid);

    //Added by Huahai
    public void abortWaiters();
}
