/*
 * @(#)LockMsg.java LockMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */ 

package mokan.transact;

import java.io.*;

import mokan.obj.ObjectID;
import mokan.obj.GlobalObj;
import mokan.obj.KanObject;

import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.comm.YesNoReply;
import mokan.debug.Event;
//import mokan.transact.TransactionAbortException;

/**
 * Ask the home or owner node to lock an object
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 */
public final class LockMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = 5385069810208699620L;


    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * All the information needed to lock the object.  This is a pseudo-final
     * field.
     *
     * @serial the information needed to lock the object
     * @see mokan.obj.LockRequest
     */
    private LockRequest request;

    /**
     * The no-args constructor, needed for deserialization
     */
    public LockMsg() {
        super(true, true);
    }

    /**
     * Create a new <code>LockMsg</code>
     *
     * @param node the originating logical node for this request
     * @param request the type of lock desired
     */
    public LockMsg(final LogicalNode node, final LockRequest request) {
        super(node, true, true);
        this.request = request;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * Lock requests can block, so handle this message asynchronously.
     */
    public void handle() {
        super.handle();

        //Pseudo-final handling
        final LockRequest myRequest = request;
        final KanObject kObj = myRequest.objID.getLocKanObject();
        YesNoReply reply;
        try {
            if (kObj != null) {
                if (myRequest.isUpgrade()) 
                    kObj.upgrade(myRequest);
                else
                    kObj.lock(myRequest);
                reply = new YesNoReply(this, true);
            } else {
                reply = new YesNoReply(this, false);
            }
        } catch (TransactionAbortException abortEx) {
            Event.DEBUG(Event.LOCK,"Remote lock denied for ",kObj);
            reply = new YesNoReply(this, false);
        }
        reply.send(from);
    }

    /**
     * Convert the <code>LockMsg</code> to a string
     *
     * @return a string representation of this <code>LockMsg</code>
     */
    public String toString() {
        return "LockMsg[" + request + ']';
    }

    /**
     * Save the state of the <code>LockMsg</code> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The superclass data is written out, followed by the lock
     *             request (LockRequest).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(request);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        request = (LockRequest)in.readObject();
    }
}
