/*
 * @(#)$Id: NonblockingTransaction.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 *
 */

package mokan.transact;

import mokan.comm.LogicalNode;
import mokan.obj.GuardCondition;
import mokan.obj.ObjectID;
import mokan.thread.AsynchCall;

/**
 * A representation of an atomic transaction that cannot abort.  Since aborts
 * are impossible, we can take some shortcuts, such as not making a copy of
 * the object to work on.  This also means that we panic if an abort actually
 * <em>does</em> happen.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public final class NonblockingTransaction extends Transaction {

    /**
     * Create a nonblocking transaction
     *
     * @param node the <code>LogicalNode</code> on which this transaction is
     *        initiated
     * @param gc the guard that has to be satisfied for this transaction to
     *        begin
     * @param objID the object on which this transaction will operate
     * @param write <code>true</code> if this transaction might change the
     *        state of the object; <code>false</code> otherwise
     * @param call the creating thread for this transaction
     */
    protected NonblockingTransaction(final LogicalNode node,
				     final GuardCondition gc,
				     final ObjectID objID,
				     final boolean write,
				     final AsynchCall call) {
	super(node, gc, objID, write, call);
    }

    /**
     * Begin the transaction by locking the object in a state satisfying the
     * guard, and storing a copy of its original state
     *
     * @exception TransactionException if this transaction is aborted (e.g.,
     *            due to the wound-wait algorithm)
     */
    public void begin() throws TransactionException {
	// Update the access counts and lock the object
	// Lock in nonpreemptible mode, since we cannot abort
	super.begin(false);
    }

    /**
     * Convert the <code>NonblockingTransaction</code> to a string
     *
     * @return a string representation of the
     *         <code>NonblockingTransaction</code>
     */
    public String toString() {
	return "NonBlockTrans" + transID;
    }
}
