/*
 * @(#)OptimisticCommitMsg.java CommitMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.transact;

import java.io.*;
import java.util.Iterator;
import java.util.LinkedList;
import mokan.comm.AppMessage;
import mokan.comm.AppReply;
import mokan.comm.LogicalNode;
import mokan.debug.Event;

/**
 * A message to tell child transactions to commit.  This happens in the second 
 * phase of a 2-phase commit.
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 */
final class OptimisticCommitMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -9067916762535678909L;



    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when 
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The set of child transaction IDs for the target node.  This is a
     * pseudo-final field.
     *
     * @serial a linked list of child transaction IDs for the target node
     */
    private LinkedList setOfID;
    private boolean    isfinal;

    /**
     * The no-args constructor, needed for deserialization
     */
    public OptimisticCommitMsg() {
	super(false, false);
    }

    /**
     * Create a new commit-all message
     *
     * @param node the originating node for this message
     * @param setOfID the IDs of the set of child transactions for the target
     *        node
     */
    OptimisticCommitMsg(final LogicalNode node, final LinkedList setOfID, boolean isfinal) {
	super(node, false, false);
	this.setOfID = setOfID;
        this.isfinal = isfinal;
	Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }

    /**
     * Process the commit message by committing the transaction
     */
    public void handle() {
	super.handle();
	NestedOptimisticTransaction.commitAll(to, setOfID, isfinal);
	new AppReply(this, false, false).send(from);
    }

    /**
     * Convert the <code>CommitMsg</code> to a string
     *
     * @return a string representation of the <code>CommitMsg</code>
     */
    public String toString() {
	return "CommitMsg: " + setOfID;
    }

    /**
     * Save the state of the <code>CommitMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, followed by the IDs of
     *             the transactions to commit (LinkedList).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
	super.writeExternal(out);
	out.writeInt(setOfID.size());
	final Iterator iter = setOfID.listIterator();
	while (iter.hasNext())
	    out.writeObject(iter.next());
        out.writeBoolean (isfinal);
    }

    public void readExternal(final ObjectInput in)
	throws IOException, ClassNotFoundException {

	super.readExternal(in);
	final int size = in.readInt();
	setOfID = new LinkedList();
	for (int i = size; i > 0; i--)
	    setOfID.add(in.readObject());
        isfinal = in.readBoolean ();
    }
}
