/*
 * @(#)$Id: TransactionUserException.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 *
 */

package mokan.transact;

/**
 * Thrown if the user code throws an exception during a transaction.  This
 * indicates termination of the associated method, and the exception is passed
 * back as the result of the associated thread.  This class allows
 * encapsulation of the thrown exception so that we can avoid having to catch
 * all exceptions in many places in our code.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Jerry James
 */
public class TransactionUserException extends TransactionException {

    /**
     * The exception thrown by the user code
     *
     * @serial the exception thrown by the user code
     */
    private Exception userException;

    /**
     * Constructs a <code>TransactionUserException</code> with no detail
     * message.
     *
     * @param ex the user exception
     */
    public TransactionUserException(final Exception ex) {
	super();
	userException = ex;
    }

    /**
     * Constructs a <code>TransactionUserException</code> with the specified
     * detail message.
     *
     * @param ex the user exception
     * @param s  the detail message
     */
    public TransactionUserException(final Exception ex, final String s) {
	super(s);
	userException = ex;
    }

    /**
     * Get the user's exception
     *
     * @return the exception thrown by the user
     */
    public Exception fetchException() {
	return userException;
    }

    /**
     * Rethrow the user's exception
     *
     * @exception Exception the exception thrown by the user
     */
    public void throwException() throws Exception {
	throw userException;
    }
}
