/*
 * @(#)UnlockMsg.java UnlockMsg.java,v 1.2 2002/11/22 23:26:07 kris Exp $
 *
 *
 */

package mokan.transact;

import java.io.*;

import mokan.obj.ObjectID;
import mokan.obj.GlobalObj;
import mokan.obj.KanObject;

import mokan.comm.AppMessage;
import mokan.comm.LogicalNode;
import mokan.comm.YesNoReply;
import mokan.debug.Event;
import mokan.transact.TransactionException;
import mokan.transact.TransactionID;

import mokan.nameservice.*;

/**
 * Ask the home or owner node to unlock an object
 *
 * @version	$Revision: 1.2 $, $Date: 2002/11/22 23:26:07 $
 * @author	Jerry James
 */
public class UnlockMsg extends AppMessage implements Externalizable {

    /**
     * The <var>serialVersionUID</var> for this class.  If you change the
     * signature of this class (field names or types, method names, parameter
     * names or types, adding or removing fields or methods, etc.), you
     * <strong>must</strong> change this value.  The recommended way is to use
     * the <code>serialver</code> tool.
     */
    static final long serialVersionUID = -8454079410023378534L;







    /**
     * The list of persistent fields for this class.  This is empty, because
     * that is the fastest option for <code>Externalizable</code> classes when
     * we are not worried about versioning.
     */
    private static final ObjectStreamField[] serialPersistentFields =
	ObjectStreamClass.NO_FIELDS;

    /**
     * The identity of the locked object.  This is a pseudo-final field.
     *
     * @serial the identity of the locked object
     */
    private ObjectID objID;
    // FIXME: It should not be necessary to send this.

    /**
     * The identity of the locking transaction.  This is a pseudo-final field.
     *
     * @serial the identity of the locking transaction
     */
    private TransactionID locker;

    /**
     * Whether to install a new version of the object.  This is a pseudo-final
     * field.
     *
     * @serial an indication of whether to install a new version of the object
     */
    private boolean install;

    /**
     * The new version of the object to install (for write locks).  This is a
     * pseudo-final field.
     *
     * @serial the new version of the object to install
     */
    private GlobalObj obj;

    /**
     * The no-args constructor, needed for deserialization
     */
    public UnlockMsg() {
        super(false, true);
    }

    /**
     * Create a new <code>UnlockMsg</code>
     *
     * @param node the originating logical node for this request
     * @param locker the transaction that holds the lock
     * @param install <code>true</code> to install a new version of the object
     * @param objID the object to unlock
     * @param obj the new version of the object if <var>install</var> is
     *        <code>true</code>, or <code>null</code> otherwise
     */
    public UnlockMsg(final LogicalNode node, final TransactionID locker,
                     final boolean install, final ObjectID objID,
                     final GlobalObj obj) {
        super(node, obj != null, true);
        this.locker = locker;
        this.install = install;
        this.objID = objID;
        this.obj = obj;
        Event.DEBUG(Event.MSG_CREATE, "Created ", this);
    }


    /**
     * Unlock the object
     */
    public void handle() {

        super.handle();
        final KanObject kObj = objID.getLocKanObject();
        YesNoReply reply;
        try {
            if (kObj != null) {
                kObj.unlock(locker, install, obj);
                reply = new YesNoReply(this, true);
            } else {
                reply = new YesNoReply(this, false);
            }
        } catch (TransactionException abortEx) {
            reply = new YesNoReply(this, false);
        }
        reply.send(from);
    }

    /**
     * Convert the <code>UnlockMsg</code> to a string
     *
     * @return a string representation of this <code>UnlockMsg</code>
     */
    public String toString() {
        return "UnlockMsg[" + locker + "]: " + install;
    }

    /**
     * Save the state of the <code>UnlockMsg</code> instance to a stream
     * (i.e., serialize it).
     *
     * @serialData The superclass data is written out, then the ID of the
     *             object being unlocked (ObjectID), then ID of the
     *             transaction that held the lock (TransactionID), then an
     *             indication of whether a new version of the object should be
     *             installed (boolean), followed by a new version of the
     *             object (GlobalObj).
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(objID);
        out.writeObject(locker);
        out.writeBoolean(install);
        out.writeObject(obj);
    }

    public void readExternal(final ObjectInput in)
        throws IOException, ClassNotFoundException {

        super.readExternal(in);
        objID = (ObjectID)in.readObject();

        locker = (TransactionID)in.readObject();
        install = in.readBoolean();
        obj = (GlobalObj)in.readObject();
    }
}
