/*
 * @(#)$Id: AppChoice.java,v 1.1.1.1 2002/11/06 22:10:28 kris Exp $
 *
 *
 */

package mokan.ui;

import java.net.InetAddress;

/**
 * An application selected by the user, along with information on how to start
 * that application
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:28 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
final class AppChoice {

    /**
     * The application name
     */
    private final String appName;

    /**
     * A shell command to start Kan; e.g.,
     * <samp>javamokan.Kan -someflag</samp>
     */
    private final String kanCommand;

    /**
     * The name of the machine on which to run the system
     */
    private final String machineName;

    /**
     * The IP address of the machine on which to run the system
     *
     * @see java.net.InetAddress
     */
    private final InetAddress machineAddress;

    /**
     * The constructor for running applications
     *
     * @param application the name of the application to run
     * @param kanStart the shell command to start Kan
     * @param mach the <code>InetAddress</code> of the target machine
     */
    AppChoice(final String application, final String kanStart,
	      final InetAddress mach) { 
	appName = application;
	kanCommand = kanStart;
	machineAddress = mach;
	machineName = mach.getHostName();
    }

    /**
     * The constructor for NON-running applications
     *
     * @param application the name of the application to run
     * @param kanStart the shell command to start Kan
     */
    AppChoice(final String application, final String kanStart) {
	appName = application;
	kanCommand = kanStart;
	machineAddress = null;
	machineName = null;
    }

    /**
     * Get the name of the application
     *
     * @return the name of the application to run
     */
    String getAppName() {
	return appName;
    }

    /**
     * Get the command used to start a Kan application remotely
     *
     * @return a string containing the command to start a remote Kan
     *         application
     */
    String getKanCommand() {
	return kanCommand + ' ' + appName;
    }

    /**
     * Get the <code>InetAddress</code> of the remote node
     *
     * @return the <code>InetAddress</code> of the remote node
     */
    InetAddress getMachineAddress() {
	return machineAddress;
    }

    /**
     * Get the name of the remote node
     *
     * @return a string containing the name of the remote node
     */
    String getMachineName() {
	return machineName;
    }

    /**
     * Convert the <code>AppChoice</code> to a string
     *
     * @return a string representation of the <code>AppChoice</code>
     */
    public String toString() {
	return kanCommand + " on " + machineName;
    }
}
