/*
 * @(#)$Id: Commander.java,v 1.1.1.1 2002/11/06 22:10:29 kris Exp $
 *
 *
 */

package mokan.ui;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import mokan.Kan;
import mokan.comm.MachinePort;

/**
 * The main element of the GUI front-end for Kan
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:29 $
 * @author	Sean Brydon
 * @author	Jerry James
 */
public final class Commander {

    /**
     * Kan Virtual Machines that are running on other nodes
     */
    private final static Vector runningKVMs = new Vector();

    /**
     * A mapping from machine names to remote telnet sessions
     */
    private final static Hashtable telnetList = new Hashtable();

    /**
     * The number of telnet sessions currently open
     */
    private static int curNumTelnets = 0;

    /**
     * The user's username, for logging into remote machines
     */
    private static String user;

    /**
     * The user's password, for logging into remote machines
     */
    private static String password;

    /**
     * The starting point for the Kan GUI
     *
     * @param apps the list of applications to choose from
     * @param machines the list of machines to run on
     * @param numMachines the number of machines to run on (zero if the user
     *        made no selection)
     */
    public static void doIt(final String[] apps, final MachinePort[] machines,
			    final int numMachines) {
	final String kanCommand = Kan.systemProps.getProperty("run");

	// This is a blocking constructor that sets the username & password
	new LoginGUI();

	new StartupGUI(apps, machines, runningKVMs, kanCommand);

	// Make this parent daemon stay alive forever for now-need to change
	// later pop up window here
	try {
	    synchronized (apps) {
		apps.wait();		// No matching notify; sleep forever
	    }
	} catch (InterruptedException intEx) {
	    // This won't happen, so we don't care
	}
    }

    /**
     * Start a remote KVM.  This is called by the GUI.  NOTE: the user can
     * <em>not</em> telnet to the current machine; we need to spawn a thread
     * locally instead.  This is <em>only</em> used to fire up KVMs and is
     * <em>not</em> used to start user applications on running KVMs.
     *
     * @param ap an object describing how to start a remote KVM
     */
    static void startTelnetApp(final AppChoice ap) {
	final RemoteTelnet rt =
	    new RemoteTelnet(user, password, ap.getKanCommand());
	telnetList.put(ap.getMachineName(), rt);
	if (rt.doTelnet(ap.getMachineAddress())) {
	    curNumTelnets++;
	    System.out.println("******** curNumTelnets = " + curNumTelnets);
	}
    }

    /**
     * Stop a remote KVM.  This is called by the GUI.  NOTE: we must kill
     * remote telnet clients differently than local threads.  CURRENTLY this
     * method doesn't work - so DON'T call it, but feel free to make it work.
     * Maybe send a suicide message.
     *
     * @param ap an object describing how to kill an application
     */
    static void killApp(final AppChoice ap) {
	final String mach = ap.getMachineName();
	final InetAddress addr = ap.getMachineAddress();
	System.out.println("Commander.killApp: killing " + ap.getAppName()
			   + " on machine " + mach);
	final RemoteTelnet rt =
	    (RemoteTelnet) telnetList.get(ap.getMachineName());
	rt.killRemoteTelnet(ap.getMachineName());
    }

    /**
     * Called from the GUI when the user chooses to turn off the monitor.
     * NOTE: we must kill remote telnet clients differently than local
     * threads.
     *
     * @param ap an object describing how to contact a remote machine
     */
    static void toggleMonitor(final AppChoice ap) {
	final String mach = ap.getMachineName();
	final InetAddress addr = ap.getMachineAddress();
	System.out.println("Commander.toggleMonitor: toggle window on machine "
			   + mach);
	final RemoteTelnet rt =
	    (RemoteTelnet)telnetList.get(ap.getMachineName());
	rt.toggleMonitor();
    }

    /**
     * Record the user's login name and password
     *
     * @param u the user's login name
     * @param pass the user's password
     */
    static void setLogin(final String u, final String pass) {
	user = u;
	password = pass;
    }

    /**
     * Stop printing output to the telnet screen.  For debugging, if trying to
     * read some output that is scrolling by too fast.  I recommend SCRAPPING
     * this method since it is a HACK.
     *
     * @param mach the machine for which we should freeze output
     * @exception UnknownHostException if that machine cannot be located
     */
    static void freezeOutput(String mach) throws UnknownHostException {
	final InetAddress addr = InetAddress.getByName(mach);
	System.out.println("Commander.freezeApp: Freezing output on machine "
			   + mach);
	final RemoteTelnet rt = (RemoteTelnet) telnetList.get(mach);
	rt.togglePrint();
    }
}
