/*
 * @(#)$Id
 *
 *
 */

package mokan.ui;

import java.awt.*;
import java.awt.event.*; 
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JOptionPane;
import mokan.comm.MachinePort;

/**
 * A window for launching Kan Virtual Machines (KVMs)
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:29 $
 * @author	Sean Brydon
 * @author	Sandeep Bhatia
 */
final class KVMLauncherGUI extends Frame implements ActionListener {
    private final Checkbox cbxBatch = new Checkbox("Batch");
    private final Checkbox cbxDebug = new Checkbox("Debug");
    private final Checkbox cbxVerbose = new Checkbox("Verbose");

    private final List lstAvailMachines = new List(6, true);
    private final List lstUsedMachines = new List(6, true);

    private final Vector thisKVM;
    private final KanStatus thisKVMStatus;

    /**
     * The machines to choose from
     */
    private final MachinePort[] machines;

    /**
     * Create a new <code>KVMLauncherGUI</code>
     *
     * @param KVMInfo a list of running Kan installations
     * @param machines a user-defined list of machines for running Kan
     *        applications
     * @param isKVMRunning <code>true</code> if Kan is running on the machines
     *        already
     * @param thisKVMStatus the status of the local Kan installation
     */
    KVMLauncherGUI(final Vector KVMInfo, final MachinePort[] machines,
		   final boolean isKVMRunning, final KanStatus thisKVMStatus) {
	super("KVM Launcher");
	thisKVM = KVMInfo;
	this.thisKVMStatus = thisKVMStatus;

	for (int i = 0; i < machines.length; i++)
	    lstAvailMachines.add(machines[i].toString());
	this.machines = machines;

	final Panel pnlCenter = new Panel(new BorderLayout());
	final Panel pnlChoice = new Panel(new GridLayout(3,1));

	setLayout(new BorderLayout());
	pnlChoice.add(cbxBatch);
	pnlChoice.add(cbxDebug);
	pnlChoice.add(cbxVerbose);

	final Panel pnlTexts = new Panel (new BorderLayout());
	final Panel pnlLabels = new Panel (new GridLayout(2, 1));
	final Panel pnlTxtFlds = new Panel (new GridLayout(2, 1));

	if (isKVMRunning) {
	    final AppChoice KVMDefault = (AppChoice)thisKVM.firstElement();
	    processDefaults(KVMDefault.getKanCommand());

	    // remove the selected machines from this list
	    // and add to UsedMachines list
	    for (int i = 0; i < thisKVM.size(); i++) {
		final String appName =
		    ((AppChoice)thisKVM.elementAt(i)).getMachineName();
		lstAvailMachines.remove(appName);
		lstUsedMachines.add(appName);
	    }
	} else {
	    processDefaults(null);
	    // set defaults the machine list
	    for (int i = 0; i <= 3; i++)
		lstAvailMachines.select(i);
	}

	pnlLabels.add(new Label("ClassPath"));
	pnlLabels.add(new Label("Directory"));

	pnlTexts.add("West", pnlLabels);
	pnlTexts.add("Center", pnlTxtFlds);

	final Panel pnlChTxt = new Panel(new FlowLayout());
	pnlChTxt.add(pnlChoice);
	pnlChTxt.add(pnlTexts);

	final Panel pnlBtns = new Panel(new GridLayout(1,4));
	final Button btnStart = new Button("Start");
	final Button btnCancel = new Button("Cancel");
	final Button btnJoin = new Button("Join");
	final Button btnToggle = new Button("Toggle Windows");
	final Button btnFreeze = new Button("Freeze Pane");
	pnlBtns.add(btnStart);
	pnlBtns.add(btnCancel);
	pnlBtns.add(btnJoin);
	pnlBtns.add(btnToggle);
	pnlBtns.add(btnFreeze);

	final Panel pnlLists1 = new Panel(new BorderLayout());
	final Panel pnlLists2 = new Panel(new BorderLayout());
	final Panel pnlLists  = new Panel(new GridLayout(2,1));

	pnlLists1.add("North", new Label("Available Machines"));
	pnlLists1.add("Center", lstAvailMachines);
	pnlLists2.add("North", new Label("Used Machines"));
	pnlLists2.add("Center", lstUsedMachines);
	pnlLists.add(pnlLists1);
	pnlLists.add(pnlLists2);

	final Panel pnlStart = new Panel(new BorderLayout());
	pnlStart.add("Center", pnlChTxt);
	pnlStart.add("South", pnlBtns);
	pnlStart.add("East", pnlLists);

	add(pnlStart);

	pnlChoice.setEnabled(!isKVMRunning);
	pnlTexts.setEnabled(!isKVMRunning);
	btnStart.setEnabled(!isKVMRunning);
	btnJoin.setEnabled(isKVMRunning);
	btnToggle.setEnabled(isKVMRunning);

	btnStart.addActionListener(this);
	btnCancel.addActionListener(this);
	btnJoin.addActionListener(this);
	btnToggle.addActionListener(this);
	btnFreeze.addActionListener(this);

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		KVMLauncherGUI.this.dispose();
		System.exit(0);
	    }
	});

	pack();
	show();
    }

    private void processDefaults(final String defaultOptions) {
	// what is needed is to parse the string for the
	// -b : batch
	// -d : debug
	// -v : verbose
	// -j : join
	// options so that we can set the window as user specified in the
	// config file
	if (defaultOptions != null) {
	    final StringTokenizer st =
		new StringTokenizer(defaultOptions, "-");
	    String nextOption = st.nextToken();
	    while (st.hasMoreTokens()) {
		if (nextOption.equals("b")) {
		    cbxBatch.setState(true);
		} else if (nextOption.equals("d")) {
		    cbxDebug.setState(true);
		} else if (nextOption.equals("v")) {
		    cbxVerbose.setState(true);
		}
		nextOption = st.nextToken();
	    }
	} else {
	    // assign the checkboxes some pre-defined defaults (all off)
	    cbxBatch.setState(false);
	    cbxDebug.setState(false);
	    cbxVerbose.setState(false);
	}
    }

    private String constructCommand(final String[] connectToMachines) {
	// checks all the settings on the checkboxes and stuff
	// and constructs the string to encapsulate the settings
	String javaCommand = "javamokan.Kan ";
	if (cbxBatch.getState()) {
	    javaCommand += "-b ";
	}
	if (cbxDebug.getState()) {
	    javaCommand += "-d ";
	}
	if (cbxVerbose.getState()) {
	    javaCommand += "-v ";
	}
	javaCommand += " -u";
	final int lastMachine = connectToMachines.length - 1;
	if (lastMachine >= 0) {
	    for (int i = 0; i < connectToMachines.length - 1; i++) {
		javaCommand += connectToMachines[i] + ",";
	    }
	    javaCommand += connectToMachines[connectToMachines.length - 1];
	} else {
	    JOptionPane.showMessageDialog(this, "No machines selected!",
					  "Choose a machine",
					  JOptionPane.ERROR_MESSAGE);
	}
	return (javaCommand);
    }

    private String constructJoinCommand(final String[] joinMachines) {
	String javaCommand = "javamokan.Kan ";
	if (cbxBatch.getState()) {
	    javaCommand += "-b ";
	}
	if (cbxDebug.getState()) {
	    javaCommand += "-d ";
	}
	if (cbxVerbose.getState()) {
	    javaCommand += "-v ";
	}
	javaCommand += " -u";
	for (int i = 0; i < thisKVM.size() - 1; i++) {
	    javaCommand += ((AppChoice)thisKVM.elementAt(i)).getMachineName()
		+ "," ;
	}
	javaCommand += ((AppChoice)thisKVM.lastElement()).getMachineName();
	javaCommand += " -j";
	for (int i = 0; i < joinMachines.length - 1; i++) {
	    javaCommand += joinMachines[i] + ",";
	}
	javaCommand += joinMachines[joinMachines.length - 1];
	return (javaCommand);
    }

    private MachinePort getMachinePort(final String machineName) {
	for (int i = machines.length - 1; i >= 0; i--) {
	    if (machines[i].equals(machineName))
		return machines[i];
	}
	return null;
    }

    public void actionPerformed(final ActionEvent ae) {
	if (ae.getActionCommand().equals("Start")) {
	    // just rebuild the string to pass to the Commander
	    final String[] selectedMachines =
		lstAvailMachines.getSelectedItems();
	    final String currJavaCommand = constructCommand(selectedMachines);

	    System.out.println("Processing following selections :");
	    // Hacked!!
	    lstUsedMachines.setEnabled(true);
	    for (int i = 0; i < selectedMachines.length; i++) {
		// test only
		System.out.print(selectedMachines[i] +"\t");
		final MachinePort mp = getMachinePort(selectedMachines[i]);
		final AppChoice newApp =
		    new AppChoice(null, currJavaCommand, mp.addr);
		thisKVM.addElement(newApp);
		thisKVMStatus.addMachine(selectedMachines[i]);
		lstAvailMachines.remove(selectedMachines[i]);
		lstUsedMachines.add(selectedMachines[i]);
		Commander.startTelnetApp(newApp);
		System.out.println("Application Started");
		dispose();
	    }
	} else if (ae.getActionCommand().equals("Join")) {
	    final String[] selectedMachines =
		lstAvailMachines.getSelectedItems();
	    if (selectedMachines.length < 1) {
		JOptionPane.showMessageDialog
		    (this, "Please select a machine to join",
		     "Choose a machine", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    final String currJavaCommand =
		constructJoinCommand(selectedMachines);

	    System.out.println("Java Command is : " + currJavaCommand);
	    System.out.println("Processing following selections for join:");

	    for (int i = 0; i < selectedMachines.length; i++) {
		// test only
		System.out.print(selectedMachines[i] +"\t");
		try {
		    final AppChoice newApp = new AppChoice
			(null, currJavaCommand,
			 InetAddress.getByName(selectedMachines[i]));
		    thisKVM.addElement(newApp);
		    thisKVMStatus.addMachine(selectedMachines[i]);
		    lstAvailMachines.remove(selectedMachines[i]);
		    lstUsedMachines.add(selectedMachines[i]);
		    Commander.startTelnetApp(newApp);
		} catch (UnknownHostException uhe) {
		    JOptionPane.showMessageDialog
			(this, "Can't find " + selectedMachines[i],
		     "Can't find machine", JOptionPane.ERROR_MESSAGE);
		}
	    }
	} else if (ae.getActionCommand().equals("Toggle Windows")) {
	    final String[] selectedMachines =
		lstUsedMachines.getSelectedItems();
	    if (selectedMachines.length < 1) {
		JOptionPane.showMessageDialog
		    (this, "Please select a machine to toggle",
		     "Choose a machine", JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    System.out.println("Processing following selections for toggle:");

	    for (int i = 0; i < selectedMachines.length; i++) {
		// test only
		System.out.print(selectedMachines[i] +"\t");
		try {
		    AppChoice newApp =
			new AppChoice(null, null,
				      InetAddress.getByName(selectedMachines[i]));
		    Commander.toggleMonitor(newApp);
		} catch (UnknownHostException uhe) {
		    JOptionPane.showMessageDialog
			(this, "Can't find " + selectedMachines[i],
		     "Can't find machine", JOptionPane.ERROR_MESSAGE);
		}
	    }
	} else if (ae.getActionCommand().equals("Cancel")) {
	    // dunno if dispose does the trick
	    dispose();
	} else if (ae.getActionCommand().equals("Freeze Pane")) {
	    final String[] selectedMachines =
		lstUsedMachines.getSelectedItems();
	    if (selectedMachines.length < 1) {
		JOptionPane.showMessageDialog
		    (this, "Please select a machine to toggle",
		     "Choose a machine", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    try {
		for (int i = 0; i < selectedMachines.length; i++) {
		    Commander.freezeOutput(selectedMachines[i]);
		}
	    } catch (UnknownHostException unkEx) {
		JOptionPane.showMessageDialog
		    (this, unkEx.getMessage(),
		     "Can't find machine", JOptionPane.ERROR_MESSAGE);
	    }
	}
    }
}
