/*
 * @(#)$Id
 *
 *
 */

package mokan.ui;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import javax.swing.text.Document;

/**
 * A simple class for showing the data that is being printed by a distributed
 * application (i.e., it traps System.out and System.err)
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:29 $
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
final class MonitorClient extends JFrame {

    /**
     * The mouse listener for implementing copy &amp; paste
     */
    private final MouseAdapter mouse;

    /**
     * The text display for the characters that have been written to
     * System.out or System.err
     */
    private final JTextArea clientDisplay = new JTextArea(25, 80);

    /**
     * Provide the capability of scrolling
     */
    private final JScrollPane scroller =
	new JScrollPane(clientDisplay, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

    /**
     * The actual text displayed on <var>clientDisplay</var>
     */
    private final Document doc = clientDisplay.getDocument();

    /**
     * Create a new monitor client to display standard out and standard error
     * for a particular host
     *
     * @param addr the IP address of the host
     */
    MonitorClient(final InetAddress addr) {
	// Create the JFrame and set it up
	super(addr.getHostName());
	setContentPane(scroller);
	setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

	// Set up the JTextArea
	clientDisplay.setAutoscrolls(true);
	clientDisplay.setBackground(Color.lightGray);
	clientDisplay.setFont(new Font("monospaced", Font.PLAIN, 11));

	// Turn on backing store
	final JViewport port = scroller.getViewport();
	port.setBackingStoreEnabled(true);

	// Set up the mouse listener
	mouse = new MouseAdapter() {
	    public void mouseReleased(MouseEvent e) {
		clientDisplay.copy();
	    }
	};
	clientDisplay.addMouseListener(mouse);

	// Do it!!
	pack();
	show();
    }

    /**
     * Display one character on the monitor
     *
     * @param newChar the character to display
     */
    void displaych(final char newChar) {
	clientDisplay.append(String.valueOf(newChar));
	if (newChar == '\n' || newChar == '\r') {
	    clientDisplay.setCaretPosition(doc.getLength());
	    // scroller.validate();
	}
    }

    /**
     * Display a string on the monitor
     *
     * @param str the string to display
     */
    void display(final String str) {
	clientDisplay.append(str);
	clientDisplay.setCaretPosition(doc.getLength());
	// scroller.validate();
    }

    /**
     * Terminate the connection
     */
    void shutdown() {
	display("\nConnection closed\n");
    }

    /**
     * Clear the monitor
     */
    void clearch() {
	clientDisplay.setText(null);
	clientDisplay.setCaretPosition(0);
    }

    /**
     * Disconnect the connection
     */
    void disconnect() {
	display("Disconnected!");
    }

    /**
     * Release the resources used by this monitor client
     */
    void killMe() {
	dispose();
	clientDisplay.removeMouseListener(mouse);
    }
}
