/*
 * @(#)$Id
 *
 *
 */

package mokan.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;
import mokan.comm.KanComm;
import mokan.comm.MachinePort;

/**
 * The main control panel for Kan.  Offer the user various choices.
 *
 * @version	$Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:29 $
 * @author	Sandeep Bhatia
 * @author	Jerry James
 */
final class StartupGUI extends JFrame implements ActionListener {

    /**
     * The user-defined list of Kan applications
     */
    private final String[] availableApps;

    private final Vector runningApps = new Vector();

    /**
     * The user-defined list of machines on which to run Kan applications
     */
    private final MachinePort[] machines;

    private final Vector KVMInfos;

    private final String kanCommand;

    private boolean isKVMRunning = false;

    // status info
    private final KanStatus currKVMStatus = new KanStatus(1);

    /**
     * Create a new initial window for the Kan system
     *
     * @param availableApps a user-defined list of Kan application names
     * @param machines a user-defined list of machines for running Kan
     *        applications
     * @param KVMInfos a list of running Kan installations
     * @param kanCommand the command to start Kan on a remote node
     */
    StartupGUI(final String[] availableApps, final MachinePort[] machines,
	       final Vector KVMInfos, final String kanCommand) {

	// Call the superclass constructor with the frame title
	super("The Wrath of Kan");

	// Prepare the content pane for adding components
	final Container pane = getContentPane();
	pane.setBackground(Color.lightGray);

	// Set fields according to the constructor arguments
	this.availableApps = availableApps;
	this.machines      = machines;
	this.KVMInfos      = KVMInfos;
	this.kanCommand    = kanCommand;

	// Update the status
	if (KVMInfos.size() != 0) {
	    isKVMRunning = true;
	    for (int i = 0; i < KVMInfos.size(); i++) {
		final AppChoice currApp = (AppChoice)KVMInfos.elementAt(i);
		currKVMStatus.addMachine(currApp.getMachineName());
	    }
	}

	// Create the buttons
	final JButton btnStartup     = new JButton("Startup");
	final JButton btnStatus      = new JButton("Status");
	final JButton btnAppLauncher = new JButton("Application Launcher");
	final JButton btnExit        = new JButton("Exit KAN");

	// Add the buttons
	pane.setLayout(new GridLayout(4, 1, 0, 20));
	pane.add(btnStartup);
	pane.add(btnStatus);
	pane.add(btnAppLauncher);
	pane.add(btnExit);

	btnStartup.addActionListener(this);
	btnStatus.addActionListener(this);
	btnAppLauncher.addActionListener(this); 
	btnExit.addActionListener(this);

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		StartupGUI.this.dispose();
		System.exit(0);
	    }
	});

	// Set up the frame the rest of the way
	setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	pack();
	setVisible(true);
    }

    public void actionPerformed(final ActionEvent ae) {
	if (ae.getActionCommand().equals("Startup")) {
	    isKVMRunning = (KVMInfos.size() != 0);
	    new KVMLauncherGUI(KVMInfos, machines, isKVMRunning,currKVMStatus);
	} else if (ae.getActionCommand().equals("Status")) {
	    System.out.println("Checking Status");
	    // new StatusGUI();
	} else if (ae.getActionCommand().equals("Application Launcher")) {
	    new AppLauncherGUI(availableApps, runningApps, currKVMStatus,
			       kanCommand);
	} else if (ae.getActionCommand().equals("Exit KAN")) {
//KK2001-03-01	    KanComm.shutdownAllKVMs();	// This method does not return
	}
    }
}
