/*
 * @(#)$Id: Telnet.java,v 1.1.1.1 2002/11/06 22:10:29 kris Exp $
 *
 *
 */

package mokan.ui;

import java.io.*;
import java.net.*;

/**
 * Simple telnet client in Java
 *
 * @version    $Revision: 1.1.1.1 $, $Date: 2002/11/06 22:10:29 $
 * @author     Jing Wang
 * @author     Sean Brydon
 */
final class Telnet {
    // FIXME: Get first 2 values from a configuration file
    private static final int port = 23;	 // remote port
    private final String term = "vt100"; // the terminal type
    private boolean connecting = false;
    private final InputStream in;	 // data from other end
    private final OutputStream out;	 // data to other end
    private final PrintWriter output;
    private final InetAddress  host;	 // remote host
    private final String user, passwd;	 // remot user and passwd
    private final Socket socket;	 // connection to remote host

    Telnet(final InetAddress host, final String user, final String passwd)
	throws IOException, UnknownHostException {

	this.host = host;
	this.user = user;
	this.passwd = passwd;
	// Open connection
	try {
	    if ((socket = new Socket(host, port)) == null)
		System.err.println("Failed to connect to host " + host);
	} catch(UnknownHostException e) {
	    System.out.println("Host " + host + " not found");
	    throw e;
	} catch(IOException e) {
	    System.out.println("Telnet:constructor::Failed to connect to host " + host);
	    throw e;
	}
       
	try {
	    in = socket.getInputStream();
	    out = socket.getOutputStream();
	    output = new PrintWriter(out,true);
	    connect();
	} catch(IOException e) {
	    System.out.println("Telnet:constructor::Failed to get stream from socket");
	    throw e;
	}
	connect();
	System.out.println("Telnet:constructor::Connected to host " + host);
    }

    void connect() {
	while (!connecting) {
	    final char c = readch();
	    if (c == 255) {
		// Telnet IAC
		final char cmd = readch();
		final char opt = readch();
		switch(opt) {
		case 1:         // echo
		    if (cmd == 251) {
			sendComm(user);
		    }
		    break;
		case 3:         // supress go-ahead
		    break;
		case 24:        // terminal type
		    if (cmd == 253) {
                                // IAC WILL terminal-type
			transmitch((char)255);
			transmitch((char)251);
			transmitch((char)24);
                                // IAC SB terminal-type IS <term> IAC SE
			transmitch((char)255);
			transmitch((char)250);
			transmitch((char)24);
			transmitch((char)0);
			transmit(term);
			transmitch((char)255);
			transmitch((char)240);
		    } else if (cmd == 250) {
			while(readch() != 240) {
			    Thread.yield();
			}
		    }
		    break;
		default:        // some other command
		    if (cmd == 253) {
                                // IAC DONT whatever
			transmitch((char)255);
			transmitch((char)252);
			transmitch((char)opt);
		    }
		    break;
		}
	    } else if (c == 'P') {
		final String ss ="Password:";
		for (int i = 1; i < 9; i++) {
		    if (readch() != ss.charAt(i))
			continue;
		}
		sendComm(passwd);
		connecting = true;
	    }
	}
    }

    char readch() {
	try {
	    while (in.available() == 0) {
		Thread.yield();
	    }
	    final int c = in.read();
	    if (c == -1)                              
		disconnect();	    
	    return (char)c;
	} catch (IOException e){
	    e.printStackTrace();
	    disconnect();
	    return (char)-1;
	}
    }

    // disconnect
    // Close the connection 
    void disconnect() {
	try {
	    socket.close();
	} catch (IOException e){
	    System.err.println("Connection refused!");
	    e.printStackTrace();
	}
	connecting = false;
    }

    void sendComm(final String[] comm) {
	if (output != null)
	    for (int i = 0; i < comm.length; i++)
		output.println(comm[i]);
    }

    void sendComm(final String comm) {
	if (output != null)
	    output.println(comm);
    }

    //transmit a character
    void transmitch(final char c) {
	if (c == '\n')
	    transmitch('\r');
	try {
	    out.write((int)c);
	    out.flush();
	} catch (IOException e) {
	}
    }

    void transmit(final String str) {  // send a String to client
	for (int i = 0; i < str.length(); i++)
	    transmitch(str.charAt(i));
    }
}

